@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -19,51 +19,69 @@
 
 package net.sf.freecol.client;
 
+import java.io.BufferedInputStream;
+import java.io.File;
+import java.io.FileInputStream;
+import java.io.FileNotFoundException;
+import java.io.InputStream;
 import java.util.ArrayList;
 import java.util.Comparator;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.logging.Level;
 import java.util.logging.Logger;
 
-import net.sf.freecol.common.Specification;
-import net.sf.freecol.common.io.Mods;
+import net.sf.freecol.FreeCol;
+import net.sf.freecol.common.io.FreeColModFile;
 import net.sf.freecol.common.io.FreeColModFile.ModInfo;
+import net.sf.freecol.common.io.Mods;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.Europe;
 import net.sf.freecol.common.model.FreeColGameObject;
+import net.sf.freecol.common.model.Game;
 import net.sf.freecol.common.model.ModelMessage;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Unit;
-import net.sf.freecol.common.option.AudioMixerOption;
 import net.sf.freecol.common.option.BooleanOption;
 import net.sf.freecol.common.option.ListOption;
 import net.sf.freecol.common.option.ListOptionSelector;
+import net.sf.freecol.common.option.Option;
 import net.sf.freecol.common.option.OptionGroup;
-import net.sf.freecol.common.option.OptionMap;
-import net.sf.freecol.common.option.PercentageOption;
 import net.sf.freecol.common.option.SelectOption;
 
-import org.w3c.dom.Element;
+import javax.xml.stream.XMLInputFactory;
+import javax.xml.stream.XMLStreamConstants;
+import javax.xml.stream.XMLStreamException;
+import javax.xml.stream.XMLStreamReader;
+
 
 /**
  * Defines how available client options are displayed on the Setting dialog from
  * File>Preferences Also contains several Comparators used for display purposes.
- * 
+ *
  * <br>
  * <br>
- * 
- * New options should be added to {@link #addDefaultOptions()} and each option
- * should be given an unique identifier (defined as a constant in this class,
- * then as fully-qualified strings in FreeColMessages.properties). Identifiers
- * like "guiShowSonsOfLiberty" actually REQUIRE string keys of:
- * clientOptions.messages.guiShowSonsOfLiberty.name
- * clientOptions.messages.guiShowSonsOfLiberty.shortDescription So be sure to
- * include both.
+ *
+ * Most available client options and their default values are defined
+ * in the file <code>base/client-options.xml</code> in the FreeCol
+ * data directory. They are overridden by the player's personal
+ * settings in the file <code>options.xml</code> in the user
+ * directory. Options that can not be defined in this way because they
+ * are generated dynamically (such as a list of available mods) must
+ * be added to {@link #addDefaultOptions()}.
+ *
+ * Each option should be given an unique identifier (defined as a
+ * constant in this class). In general, the options are called
+ * something like "model.option.UNIQUENAME". Since the options must
+ * also be represented by the GUI, they following two keys must be
+ * added to the file <code>FreeColMessages.properties</code>:
+ *
+ * <ul><li>model.option.UNIQUENAME.name</li>
+ * <li>model.option.UNIQUENAME.shortDescription</li></ul>
  */
-public class ClientOptions extends OptionMap {
+public class ClientOptions extends OptionGroup {
 
-    @SuppressWarnings("unused")
     private static final Logger logger = Logger.getLogger(ClientOptions.class.getName());
 
     /**
@@ -74,16 +92,16 @@ public class ClientOptions extends OptionMap {
     /**
      * If this option is enabled, the display will recenter in order to display
      * the active unit if it is not
-     * {@link net.sf.freecol.client.gui.GUI#onScreen(Map.Position)}).
-     * 
+     * {@link net.sf.freecol.client.gui.GUI#onScreen(Tile)}).
+     *
      * @see net.sf.freecol.client.gui.GUI
      */
     public static final String JUMP_TO_ACTIVE_UNIT = "model.option.jumpToActiveUnit";
 
     /**
      * Selected tiles always gets centered if this option is enabled (even if
-     * the tile is {@link net.sf.freecol.client.gui.GUI#onScreen(Map.Position)}).
-     * 
+     * the tile is {@link net.sf.freecol.client.gui.GUI#onScreen(Tile)}).
+     *
      * @see net.sf.freecol.client.gui.GUI
      */
     public static final String ALWAYS_CENTER = "model.option.alwaysCenter";
@@ -91,7 +109,7 @@ public class ClientOptions extends OptionMap {
     /**
      * Used by GUI, this is the minimum number of goods a colony must possess for
      * the goods to show up at the bottom of the colony panel.
-     * 
+     *
      * @see net.sf.freecol.client.gui.GUI
      */
     public static final String MIN_NUMBER_FOR_DISPLAYING_GOODS = "model.option.guiMinNumberToDisplayGoods";
@@ -99,7 +117,7 @@ public class ClientOptions extends OptionMap {
     /**
      * Used by GUI, the number will be displayed when a group of goods are
      * higher than this number.
-     * 
+     *
      * @see net.sf.freecol.client.gui.GUI
      */
     public static final String MIN_NUMBER_FOR_DISPLAYING_GOODS_COUNT = "model.option.guiMinNumberToDisplayGoodsCount";
@@ -107,7 +125,7 @@ public class ClientOptions extends OptionMap {
     /**
      * Used by GUI, this is the most repetitions drawn of a goods image for a
      * single goods grouping.
-     * 
+     *
      * @see net.sf.freecol.client.gui.GUI
      */
     public static final String MAX_NUMBER_OF_GOODS_IMAGES = "model.option.guiMaxNumberOfGoodsImages";
@@ -129,11 +147,11 @@ public class ClientOptions extends OptionMap {
 
     /**
      * Whether to delay on a unit's last move or not.
-     * 
+     *
      * TODO: Add this option's name and short description to languages other than English.
      */
     public static final String UNIT_LAST_MOVE_DELAY = "model.option.unitLastMoveDelay";
-    
+
     /**
      * Whether to display borders by default or not.
      */
@@ -144,7 +162,8 @@ public class ClientOptions extends OptionMap {
      */
     public static final String DISPLAY_TILE_TEXT = "model.option.displayTileText";
 
-    public static final int DISPLAY_TILE_TEXT_EMPTY = 0, DISPLAY_TILE_TEXT_NAMES = 1, DISPLAY_TILE_TEXT_OWNERS = 2, DISPLAY_TILE_TEXT_REGIONS = 3;
+    public static final int DISPLAY_TILE_TEXT_EMPTY = 0, DISPLAY_TILE_TEXT_NAMES = 1,
+        DISPLAY_TILE_TEXT_OWNERS = 2, DISPLAY_TILE_TEXT_REGIONS = 3;
 
     /**
      * Animation speed for friendly units.
@@ -159,7 +178,7 @@ public class ClientOptions extends OptionMap {
     /**
      * Used by GUI, this defines the grouping of ModelMessages. Possible values
      * include nothing, type and source.
-     * 
+     *
      * @see net.sf.freecol.client.gui.GUI
      * @see net.sf.freecol.common.model.ModelMessage
      */
@@ -171,47 +190,23 @@ public class ClientOptions extends OptionMap {
 
     public static final int MESSAGES_GROUP_BY_SOURCE = 2;
 
-    public static final String AUDIO_MIXER = "audioMixer";
+    public static final String AUDIO_MIXER = "model.option.audioMixer";
 
-    public static final String MUSIC_VOLUME = "musicVolume";
-
-    public static final String SFX_VOLUME = "sfxVolume";
+    public static final String AUDIO_VOLUME = "model.option.audioVolume";
 
     /**
      * Used by GUI, this defines whether SoL messages will be displayed.
-     * 
+     *
      * @see net.sf.freecol.client.gui.GUI
      */
-    public static final String SHOW_SONS_OF_LIBERTY = "model.option.guiShowSonsOfLiberty";
-
-    public static final String SHOW_WARNING = "model.option.guiShowWarning";
-
-    public static final String SHOW_GOVERNMENT_EFFICIENCY = "model.option.guiShowGovernmentEfficiency";
-
-    public static final String SHOW_WAREHOUSE_CAPACITY = "model.option.guiShowWarehouseCapacity";
-
-    public static final String SHOW_UNIT_IMPROVED = "model.option.guiShowUnitImproved";
-
-    public static final String SHOW_UNIT_DEMOTED = "model.option.guiShowUnitDemoted";
-
-    public static final String SHOW_UNIT_ADDED = "model.option.guiShowUnitAdded";
-
-    public static final String SHOW_UNIT_LOST = "model.option.guiShowUnitLost";
-
-    public static final String SHOW_BUILDING_COMPLETED = "model.option.guiShowBuildingCompleted";
-
-    public static final String SHOW_FOREIGN_DIPLOMACY = "model.option.guiShowForeignDiplomacy";
-
-    public static final String SHOW_MARKET_PRICES = "model.option.guiShowMarketPrices";
-
-    public static final String SHOW_MISSING_GOODS = "model.option.guiShowMissingGoods";
-
-    public static final String SHOW_TUTORIAL = "model.option.guiShowTutorial";
-
     public static final String SHOW_COLONY_WARNINGS = "model.option.guiShowColonyWarnings";
 
     public static final String SHOW_PRECOMBAT = "model.option.guiShowPreCombat";
 
+    public static final String SHOW_NOT_BEST_TILE = "model.option.guiShowNotBestTile";
+
+    public static final String SHOW_GOODS_MOVEMENT = "model.option.guiShowGoodsMovement";
+
     /**
      * Use default values for savegames instead of displaying a dialog. <br>
      * <br>
@@ -258,9 +253,18 @@ public class ClientOptions extends OptionMap {
     public static final String AUTOSAVE_GENERATIONS = "model.option.autosaveGenerations";
 
     /**
-     * Option for setting wether or not the fog of war should be displayed.
+     * Option for setting the number of days autosaves are keep (valid time). If set to 0,
+     * valid time is not checked.
      */
-    public static final String DISPLAY_FOG_OF_WAR = "model.option.displayFogOfWar";
+    public static final String AUTOSAVE_VALIDITY = "model.option.autosaveValidity";
+
+
+
+    /**
+     * Option for deleting autosaves when a new game is started. If set to
+     * true, old autosaves will be deleted if a new game is started.
+     */
+    public static final String AUTOSAVE_DELETE = "model.option.autosaveDelete";
 
     /**
      * Option for activating autoscroll when dragging units on the mapboard.
@@ -272,7 +276,7 @@ public class ClientOptions extends OptionMap {
      */
     public static final String AUTO_SCROLL = "model.option.autoScroll";
 
-	
+
     /**
      * Option for autoload emigrants on saling to america.
      */
@@ -297,21 +301,6 @@ public class ClientOptions extends OptionMap {
     public static final String MINIMAP_BACKGROUND_COLOR = "model.option.color.background";
 
     public static final String USER_MODS ="userMods";
-    
-    /** 
-    public static final String COLOR_BLACK = "black";
-
-    public static final String COLOR_VERY_DARK_GRAY = "gray.dark.very";
-
-    public static final String COLOR_DARK_GRAY = "gray.dark";
-
-    public static final String COLOR_GRAY = "gray";
-
-    public static final String COLOR_LIGHT_GRAY = "gray.light";
-
-    public static final String COLOR_VERY_LIGHT_GRAY = "gray.light.very";
-
-    public static final String COLOR_LIGHT_BLUE = "blue.light";
 
     /**
      * The Stock the custom house should keep when selling goods.
@@ -364,13 +353,28 @@ public class ClientOptions extends OptionMap {
     public static final int UNLOAD_OVERFLOW_RESPONSE_NEVER = 1;
     public static final int UNLOAD_OVERFLOW_RESPONSE_ALWAYS = 2;
 
+    /**
+     * Style of colony labels.
+     */
+    public static final String COLONY_LABELS = "model.option.displayColonyLabels";
+    public static final int COLONY_LABELS_NONE = 0;
+    public static final int COLONY_LABELS_CLASSIC = 1;
+    public static final int COLONY_LABELS_MODERN = 2;
+
+
     /**
      * Comparators for sorting colonies.
      */
     private static Comparator<Colony> colonyAgeComparator = new Comparator<Colony>() {
-        // ID should indicate age
         public int compare(Colony s1, Colony s2) {
-            return s1.getIntegerID().compareTo(s2.getIntegerID());
+            if (s1.getEstablished().getNumber() > 0
+                && s2.getEstablished().getNumber() > 0) {
+                return s1.getEstablished().getNumber() - s2.getEstablished().getNumber();
+            } else {
+                // TODO: remove 0.9.x compatibility code
+                // ID should indicate age
+                return s1.getIntegerID().compareTo(s2.getIntegerID());
+            }
         }
     };
 
@@ -419,11 +423,14 @@ public class ClientOptions extends OptionMap {
     private Comparator<ModelMessage> messageSourceComparator = new Comparator<ModelMessage>() {
         // sort according to message source
         public int compare(ModelMessage message1, ModelMessage message2) {
-            Object source1 = message1.getSource();
-            Object source2 = message2.getSource();
-            if (source1 == source2) {
+            String sourceId1 = message1.getSourceId();
+            String sourceId2 = message2.getSourceId();
+            if (sourceId1 == sourceId2) {
                 return messageTypeComparator.compare(message1, message2);
             }
+            Game game = FreeCol.getFreeColClient().getGame();
+            FreeColGameObject source1 = game.getMessageSource(message1);
+            FreeColGameObject source2 = game.getMessageSource(message2);
             int base = getClassIndex(source1) - getClassIndex(source2);
             if (base == 0) {
                 if (source1 instanceof Colony) {
@@ -454,67 +461,38 @@ public class ClientOptions extends OptionMap {
     private Comparator<ModelMessage> messageTypeComparator = new Comparator<ModelMessage>() {
         // sort according to message type
         public int compare(ModelMessage message1, ModelMessage message2) {
-            int dtype = message1.getType().ordinal() - message2.getType().ordinal();
-            if (dtype == 0 && message1.getSource() != message2.getSource()) {
-                return messageSourceComparator.compare(message1, message2);
-            } else {
-                return dtype;
-            }
+            return message1.getMessageType().ordinal() - message2.getMessageType().ordinal();
         }
     };
 
 
     /**
      * Creates a new <code>ClientOptions</code>.
+     * Unlike other OptionGroup classes, ClientOptions can not supply a
+     * specification as it is needed before the specification is available.
      */
     public ClientOptions() {
         super(getXMLElementTagName());
-    }
-
-    /**
-     * Creates a <code>ClientOptions</code> from an XML representation.
-     * 
-     * <br>
-     * <br>
-     * 
-     * @param element The XML <code>Element</code> from which this object
-     *            should be constructed.
-     */
-    public ClientOptions(Element element) {
-        super(element, getXMLElementTagName());
+        addDefaultOptions();
     }
 
     /**
      * Adds the options to this <code>GameOptions</code>.
      */
     protected void addDefaultOptions() {
-        Specification spec = Specification.getSpecification();
-        
-        OptionGroup guiGroup = spec.getOptionGroup("clientOptions.gui");
-        guiGroup.add(spec.getOptionGroup("clientOptions.minimap"));
-        add(guiGroup);
-        
-        add(spec.getOptionGroup("clientOptions.messages"));
+        load(new File(new File(FreeCol.getDataDirectory(), "base"),
+                      "client-options.xml"), false);
 
-        OptionGroup audioGroup = new OptionGroup("clientOptions.audio");
-        new AudioMixerOption(AUDIO_MIXER, audioGroup);
-        new PercentageOption(MUSIC_VOLUME, audioGroup, 100).setPreviewEnabled(true);
-        new PercentageOption(SFX_VOLUME, audioGroup, 100).setPreviewEnabled(true);
-        add(audioGroup);
-
-        add(spec.getOptionGroup("clientOptions.savegames"));
-        add(spec.getOptionGroup("clientOptions.warehouse"));
-        add(spec.getOptionGroup("clientOptions.other"));
-        
         final OptionGroup modsGroup = new OptionGroup("clientOptions.mods");
         final ListOptionSelector<ModInfo> selector = new ListOptionSelector<ModInfo>() {
-            private Map<String, ModInfo> mods = null; 
+            private Map<String, ModInfo> mods = null;
             private void init() {
                 if (mods == null) {
-                    final List<ModInfo> modInfos = Mods.getModInfos();
+                    List<FreeColModFile> fcmfs = Mods.getAllMods();
                     mods = new HashMap<String, ModInfo>();
-                    for (ModInfo mi : modInfos) {
-                        mods.put(mi.getId(), mi);
+                    for (FreeColModFile f : fcmfs) {
+                        ModInfo modInfo = f.getModInfo();
+                        mods.put(modInfo.getId(), modInfo);
                     }
                 }
             }
@@ -537,6 +515,122 @@ public class ClientOptions extends OptionMap {
         add(modsGroup);
     }
 
+    /**
+     * Gets a list of active mods in this ClientOptions.
+     *
+     * @return A list of active mods.
+     */
+    public List<FreeColModFile> getActiveMods() {
+        final List<FreeColModFile> fcmfs = Mods.getAllMods();
+        List<FreeColModFile> active = new ArrayList<FreeColModFile>();
+        ListOption<?> options = (ListOption<?>) getOption(ClientOptions.USER_MODS);
+        for (Object o : options.getValue()) {
+            ModInfo modInfo = (ModInfo) o;
+            if (modInfo == null) continue;
+            for (FreeColModFile f : fcmfs) {
+                if (modInfo.getId().equals(f.getModInfo().getId())) {
+                    active.add(f);
+                    break;
+                }
+            }
+        }
+        return active;
+    }
+
+    /**
+     * Reads the options from the given file.
+     *
+     * @param loadFile The <code>File</code> to read the
+     *            options from.
+     */
+    public void load(File loadFile) {
+        load(loadFile, true);
+    }
+
+    /**
+     * Reads the options from the given file.
+     *
+     * @param loadFile The <code>File</code> to read the
+     *            options from.
+     * @param update a <code>boolean</code> value
+     */
+    public void load(File loadFile, boolean update) {
+        if (loadFile == null || !loadFile.exists()) {
+            logger.warning("Could not find the client options file.");
+            return;
+        }
+        logger.finest((update ? "Updating" : "Loading") + " client options from " + loadFile.getPath());
+        try {
+            load(new BufferedInputStream(new FileInputStream(loadFile)), update);
+        } catch(FileNotFoundException e) {
+            logger.warning("Could not find the client options file.");
+        }
+    }
+
+    /**
+     * Reads the options from the given file.
+     *
+     * @param in <code>InputStream</code> from XML resource
+     * @param update a <code>boolean</code> value
+     */
+    public void load(InputStream in, boolean update) {
+        try {
+            XMLStreamReader xsr = XMLInputFactory.newInstance().createXMLStreamReader(in, "UTF-8");
+            xsr.nextTag();
+            if (update) {
+                logger.finest("Updating " + getId());
+                updateFromXML(xsr);
+            } else {
+                logger.finest("Loading " + getId());
+                readFromXML(xsr);
+            }
+        } catch (Exception e) {
+            logger.log(Level.WARNING, "Exception while loading options.", e);
+        } finally {
+            try {
+                if (in != null) {
+                    in.close();
+                }
+            } catch (Exception e) {
+                logger.log(Level.WARNING, "Exception while closing stream.", e);
+            }
+        }
+    }
+
+    /**
+     * Initialize this object from an XML-representation of this object.
+     * @param in The input stream with the XML.
+     * @throws XMLStreamException if a problem was encountered
+     *      during parsing.
+     */
+    private void updateFromXML(XMLStreamReader in) throws XMLStreamException {
+        while (in.nextTag() != XMLStreamConstants.END_ELEMENT) {
+            if (in.getLocalName().equals(OptionGroup.getXMLElementTagName())) {
+                updateFromXML(in);
+            } else {
+                String idStr = in.getAttributeValue(null, ID_ATTRIBUTE_TAG);
+                if (idStr == null) {
+                    idStr = in.getLocalName();
+                }
+                Option o = getOption(idStr);
+
+                if (o != null) {
+                    o.readFromXML(in);
+                } else {
+                    // Normal only if this option is from an old save game:
+                    logger.info("Option \"" + idStr + "\" (" + in.getLocalName() + ") could not be found.");
+
+                    // Ignore the option:
+                    final String ignoredTag = in.getLocalName();
+                    while (in.nextTag() != XMLStreamConstants.END_ELEMENT
+                           || !in.getLocalName().equals(ignoredTag)) {}
+                }
+            }
+        }
+        // DONE BY while-loop: in.nextTag();
+    }
+
+
     /**
      * Return the client's preferred tile text type.
      *
@@ -548,7 +642,7 @@ public class ClientOptions extends OptionMap {
 
     /**
      * Return the client's preferred comparator for colonies.
-     * 
+     *
      * @return a <code>Comparator</code> value
      */
     public Comparator<Colony> getColonyComparator() {
@@ -557,7 +651,7 @@ public class ClientOptions extends OptionMap {
 
     /**
      * Return the colony comparator identified by type.
-     * 
+     *
      * @param type an <code>int</code> value
      * @return a <code>Comparator</code> value
      */
@@ -580,7 +674,7 @@ public class ClientOptions extends OptionMap {
 
     /**
      * Return the client's preferred comparator for ModelMessages.
-     * 
+     *
      * @return a <code>Comparator</code> value
      */
     public Comparator<ModelMessage> getModelMessageComparator() {
@@ -596,40 +690,12 @@ public class ClientOptions extends OptionMap {
 
     /**
      * Returns the boolean option associated with a ModelMessage.
-     * 
+     *
      * @param message a <code>ModelMessage</code> value
      * @return a <code>BooleanOption</code> value
      */
     public BooleanOption getBooleanOption(ModelMessage message) {
-        switch (message.getType()) {
-        case WARNING:
-            return (BooleanOption) getObject(ClientOptions.SHOW_WARNING);
-        case SONS_OF_LIBERTY:
-            return (BooleanOption) getObject(ClientOptions.SHOW_SONS_OF_LIBERTY);
-        case GOVERNMENT_EFFICIENCY:
-            return (BooleanOption) getObject(ClientOptions.SHOW_GOVERNMENT_EFFICIENCY);
-        case WAREHOUSE_CAPACITY:
-            return (BooleanOption) getObject(ClientOptions.SHOW_WAREHOUSE_CAPACITY);
-        case UNIT_IMPROVED:
-            return (BooleanOption) getObject(ClientOptions.SHOW_UNIT_IMPROVED);
-        case UNIT_DEMOTED:
-            return (BooleanOption) getObject(ClientOptions.SHOW_UNIT_DEMOTED);
-        case UNIT_LOST:
-            return (BooleanOption) getObject(ClientOptions.SHOW_UNIT_LOST);
-        case UNIT_ADDED:
-            return (BooleanOption) getObject(ClientOptions.SHOW_UNIT_ADDED);
-        case BUILDING_COMPLETED:
-            return (BooleanOption) getObject(ClientOptions.SHOW_BUILDING_COMPLETED);
-        case FOREIGN_DIPLOMACY:
-            return (BooleanOption) getObject(ClientOptions.SHOW_FOREIGN_DIPLOMACY);
-        case MARKET_PRICES:
-            return (BooleanOption) getObject(ClientOptions.SHOW_MARKET_PRICES);
-        case MISSING_GOODS:
-            return (BooleanOption) getObject(ClientOptions.SHOW_MISSING_GOODS);
-        case DEFAULT:
-        default:
-            return null;
-        }
+        return (BooleanOption) getOption(message.getMessageType().getOptionName());
     }
 
     protected boolean isCorrectTagName(String tagName) {
@@ -638,7 +704,7 @@ public class ClientOptions extends OptionMap {
 
     /**
      * Gets the tag name of the root element representing this object.
-     * 
+     *
      * @return "clientOptions".
      */
     public static String getXMLElementTagName() {
