@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -20,26 +20,31 @@
 package net.sf.freecol.client;
 
 import java.awt.Dimension;
+import java.awt.Font;
 import java.awt.GraphicsDevice;
 import java.awt.GraphicsEnvironment;
+import java.awt.Image;
+import java.awt.Point;
 import java.awt.Rectangle;
+import java.awt.Toolkit;
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
 import java.io.File;
+import java.io.FileNotFoundException;
 import java.util.ArrayList;
-import java.util.LinkedList;
 import java.util.List;
 import java.util.Locale;
-import java.util.Queue;
-import java.util.Random;
+import java.util.logging.Level;
 import java.util.logging.Logger;
 
+import javax.swing.ImageIcon;
 import javax.swing.JFrame;
+import javax.swing.JLabel;
 import javax.swing.JMenuBar;
+import javax.swing.JWindow;
 import javax.swing.SwingUtilities;
 
 import net.sf.freecol.FreeCol;
-import net.sf.freecol.client.control.ClientModelController;
 import net.sf.freecol.client.control.ConnectController;
 import net.sf.freecol.client.control.InGameController;
 import net.sf.freecol.client.control.InGameInputHandler;
@@ -54,23 +59,24 @@ import net.sf.freecol.client.gui.ImageLibrary;
 import net.sf.freecol.client.gui.WindowedFrame;
 import net.sf.freecol.client.gui.action.ActionManager;
 import net.sf.freecol.client.gui.i18n.Messages;
-import net.sf.freecol.client.gui.sound.MusicLibrary;
-import net.sf.freecol.client.gui.sound.SfxLibrary;
-import net.sf.freecol.client.gui.sound.SoundLibrary;
+import net.sf.freecol.client.gui.plaf.FreeColLookAndFeel;
 import net.sf.freecol.client.gui.sound.SoundPlayer;
 import net.sf.freecol.client.networking.Client;
-import net.sf.freecol.common.PseudoRandom;
+import net.sf.freecol.common.FreeColException;
+import net.sf.freecol.common.io.FreeColDataFile;
 import net.sf.freecol.common.io.FreeColModFile;
-import net.sf.freecol.common.io.FreeColModFile.ModInfo;
+import net.sf.freecol.common.io.FreeColSavegameFile;
+import net.sf.freecol.common.io.FreeColTcFile;
 import net.sf.freecol.common.model.Game;
 import net.sf.freecol.common.model.Player;
+import net.sf.freecol.common.model.StringTemplate;
+import net.sf.freecol.common.model.Unit;
 import net.sf.freecol.common.networking.Message;
 import net.sf.freecol.common.option.AudioMixerOption;
 import net.sf.freecol.common.option.LanguageOption;
-import net.sf.freecol.common.option.ListOption;
+import net.sf.freecol.common.option.LanguageOption.Language;
 import net.sf.freecol.common.option.Option;
 import net.sf.freecol.common.option.PercentageOption;
-import net.sf.freecol.common.option.LanguageOption.Language;
 import net.sf.freecol.common.resources.ResourceManager;
 import net.sf.freecol.common.resources.ResourceMapping;
 import net.sf.freecol.server.FreeColServer;
@@ -85,6 +91,14 @@ import org.w3c.dom.Element;
 public final class FreeColClient {
 
     private static final Logger logger = Logger.getLogger(FreeColClient.class.getName());
+    private static FreeColClient instance;
+
+    public static FreeColClient get () {return instance;}
+
+    /**
+     * The space not being used in windowed mode.
+     */
+    private static final int DEFAULT_WINDOW_SPACE = 100;
 
     // Control:
     private ConnectController connectController;
@@ -97,10 +111,8 @@ public final class FreeColClient {
 
     private InGameInputHandler inGameInputHandler;
 
-    private ClientModelController modelController;
-    
     private MapEditorController mapEditorController;
-    
+
 
     // Gui:
     private GraphicsDevice gd;
@@ -113,13 +125,7 @@ public final class FreeColClient {
 
     private ImageLibrary imageLibrary;
 
-    private MusicLibrary musicLibrary;
-
-    private SfxLibrary sfxLibrary;
-
-    private SoundPlayer musicPlayer;
-
-    private SoundPlayer sfxPlayer;
+    private SoundPlayer soundPlayer;
 
     // Networking:
     /**
@@ -131,22 +137,27 @@ public final class FreeColClient {
     // Model:
     private Game game;
 
-    private final PseudoRandom _random = new ClientPseudoRandom();
-
     /** The player "owning" this client. */
     private Player player;
 
+    private boolean isRetired = false;
+
+    /** Indicates if the game has started, has nothing to do with whether or not the
+    client is logged in. */
+    private boolean inGame = false;
+
+
     /** The Server that has been started from the client-GUI. */
     private FreeColServer freeColServer = null;
 
     private boolean windowed;
-    
+
     private boolean mapEditor;
 
     private boolean singleplayer;
 
     private final ActionManager actionManager;
-    
+
     private ClientOptions clientOptions;
 
     public final Worker worker;
@@ -157,7 +168,7 @@ public final class FreeColClient {
      * it is an indication of an approved login to a server.
      */
     private boolean loggedIn = false;
-    
+
     private Rectangle windowBounds;
 
     /**
@@ -169,73 +180,158 @@ public final class FreeColClient {
     /**
      * Creates a new <code>FreeColClient</code>. Creates the control objects
      * and starts the GUI.
-     * 
-     * @param windowed Determines if the <code>Canvas</code> should be
-     *            displayed within a <code>JFrame</code> (when
-     *            <code>true</code>) or in fullscreen mode (when
-     *            <code>false</code>).
-     * @param innerWindowSize The inner size of the window (borders not included).
-     * @param imageLibrary The object holding the images.
-     * @param musicLibrary The object holding the music.
-     * @param sfxLibrary The object holding the sound effects.
+     *
+     * @param savegameFile a <code>File</code> value
+     * @param size a <code>Dimension</code> value
+     * @param sound True if sounds should be played
+     * @param splashFilename a <code>String</code> value
      * @param showOpeningVideo Display the opening video.
+     * @param fontName a <code>String</code> value
      */
-    public FreeColClient(boolean windowed, final Dimension innerWindowSize, 
-                         ImageLibrary imageLibrary, MusicLibrary musicLibrary,
-                         SfxLibrary sfxLibrary, final boolean showOpeningVideo) {
+    public FreeColClient(final File savegameFile, Dimension size,
+                         final boolean sound,
+                         final String splashFilename,
+                         final boolean showOpeningVideo, String fontName) {
+
         headless = "true".equals(System.getProperty("java.awt.headless", "false"));
-        this.windowed = windowed;
-        this.imageLibrary = imageLibrary;
-        this.musicLibrary = musicLibrary;
-        this.sfxLibrary = sfxLibrary;
-        
+
+        // look for data base directory
+        File baseDirectory = new File(FreeCol.getDataDirectory(), "base");
+        if (!baseDirectory.exists() || !baseDirectory.isDirectory()) {
+            System.err.println("Could not find base data directory: "
+                               + baseDirectory.getName());
+            System.exit(1);
+        }
+
+        // display the splash screen
+        JWindow splash = null;
+        if (splashFilename != null) {
+            splash = displaySplash(splashFilename);
+        }
+
+        // load the resource mappings (whatever that is!)
+        FreeColDataFile baseData = new FreeColDataFile(baseDirectory);
+        ResourceManager.setBaseMapping(baseData.getResourceMapping());
+
+        FreeColTcFile tcData = new FreeColTcFile("classic");
+        ResourceManager.setTcMapping(tcData.getResourceMapping());
+
+        imageLibrary = new ImageLibrary();
+
+        // calculate a window size (if "windowed" mode is opted)
+        windowed = (size != null);
+        if (size != null && size.width < 0) {
+            final Rectangle bounds = GraphicsEnvironment.getLocalGraphicsEnvironment()
+                                     .getMaximumWindowBounds();
+            size = new Dimension(bounds.width - DEFAULT_WINDOW_SPACE,
+                                 bounds.height - DEFAULT_WINDOW_SPACE);
+        }
+        final Dimension windowSize = size;
+        logger.info("Window size is " + windowSize);
+
+        // Swing system + LAF init
+        Font font = null;
+        if (fontName != null) {
+            font = Font.decode(fontName);
+            if (font == null) {
+                System.err.println("Font not found: " + fontName);
+            }
+        }
+        if (font == null) font = ResourceManager.getFont("NormalFont");
+        try {
+            FreeColLookAndFeel fclaf
+                = new FreeColLookAndFeel(FreeCol.getDataDirectory(), windowSize);
+            try {
+                FreeColLookAndFeel.install(fclaf, font);
+            } catch (FreeColException e) {
+                e.printStackTrace();
+                System.err.println("Unable to install FreeCol look-and-feel.");
+                System.exit(1);
+            }
+        } catch (FreeColException e) {
+            removeSplash(splash);
+            e.printStackTrace();
+            System.err.println("\nThe data files could not be found by FreeCol. Please make sure");
+            System.err.println("they are present. If FreeCol is looking in the wrong directory");
+            System.err.println("then run the game with a command-line parameter:\n");
+            System.exit(1);
+        }
+
+        // TODO: don't use same datafolder for both images and
+        // music because the images are best kept inside the .JAR
+        // file.
+        logger.info("Now starting to load images.");
+
         mapEditor = false;
-        
+
+        // load options
         clientOptions = new ClientOptions();
-        if (FreeCol.getClientOptionsFile() != null
-                && FreeCol.getClientOptionsFile().exists()) {
-            clientOptions.load(FreeCol.getClientOptionsFile());
+        logger.info("Loaded default client options.");
+        try {
+            FreeColSavegameFile savegame = new FreeColSavegameFile(savegameFile);
+            clientOptions.load(savegame.getInputStream(FreeColSavegameFile.CLIENT_OPTIONS), false);
+            logger.info("Loaded client options from savegame file.");
+        } catch(Exception e) {
+            logger.warning("Failed to read client options from savegame file.");
         }
-        
-        // TODO: Move this to a more suitable place:
-        final List<ModInfo> mods = new ArrayList<ModInfo>();
-        final List<ResourceMapping> modResources = new ArrayList<ResourceMapping>(mods.size());
-        for (Object object : ((ListOption) getClientOptions()
-                              .getObject(ClientOptions.USER_MODS)).getValue()) {
-            mods.add((ModInfo) object);
-            modResources.add(new FreeColModFile((ModInfo) object).getResourceMapping());
-        }
-        ResourceManager.setModMappings(modResources);
-        ResourceManager.preload(innerWindowSize);
-        
+
         actionManager = new ActionManager(this);
-        if (!headless) {
-            actionManager.initializeActions();
+        File preferences = FreeCol.getClientOptionsFile();
+        if (preferences != null && preferences.exists()) {
+            clientOptions.load(preferences);
+            logger.info("Loaded client options from file " + preferences.getPath());
+        } else {
+            logger.warning("Unable to load client options.");
         }
+
+        List<ResourceMapping> modMappings = new ArrayList<ResourceMapping>();
+        for (FreeColModFile f : clientOptions.getActiveMods()) {
+            modMappings.add(f.getResourceMapping());
+        }
+        ResourceManager.setModMappings(modMappings);
+        ResourceManager.preload(size);
+
+        //If modMappings change any UI resources related to actions,
+        //these would not become visible without a 2nd call to initializeActions()
+        //potential TODO: find a better way to untangle this.
+
         // Control:
         connectController = new ConnectController(this);
         preGameController = new PreGameController(this);
         preGameInputHandler = new PreGameInputHandler(this);
         inGameController = new InGameController(this);
         inGameInputHandler = new InGameInputHandler(this);
-        modelController = new ClientModelController(this);
         mapEditorController = new MapEditorController(this);
-        
+
+        removeSplash(splash);
+
         // Gui:
         if (!headless) {
             SwingUtilities.invokeLater(new Runnable() {
                     public void run() {
-                        startGUI(innerWindowSize, showOpeningVideo);
+                        startGUI(windowSize, sound, showOpeningVideo, savegameFile != null);
                     }
                 });
         }
+
+        // Job deamon
         worker = new Worker();
         worker.start();
-        
+
+        // loads an optional savegame from a file (client parameter)
+        if (savegameFile != null) {
+            SwingUtilities.invokeLater(new Runnable() {
+                    public void run() {
+                        connectController.loadGame(savegameFile);
+                    }
+                });
+        }
+
+        // -- I wonder if this has its right location here!
         if (FreeCol.getClientOptionsFile() != null
                 && FreeCol.getClientOptionsFile().exists()) {
             if (!headless) {
-                Option o = clientOptions.getObject(ClientOptions.LANGUAGE);
+                Option o = clientOptions.getOption(ClientOptions.LANGUAGE);
                 o.addPropertyChangeListener(new PropertyChangeListener() {
                         public void propertyChange(PropertyChangeEvent e) {
                             if (((Language) e.getNewValue()).getKey().equals(LanguageOption.AUTO)) {
@@ -243,44 +339,87 @@ public final class FreeColClient {
                             } else {
                                 Locale l = ((Language) e.getNewValue()).getLocale();
                                 Messages.setMessageBundle(l);
-                                canvas.showInformationMessage("newLanguageSelected", "%language%", l.getDisplayName());
+                                canvas.showInformationMessage(StringTemplate.template("newLanguageSelected")
+                                                              .addName("%language%", l.getDisplayName()));
                             }
                         }
                     });
             }
         }
+
+        // remember the first instance as (quasi) singleton
+        if ( instance == null )
+                instance = this;
+    }
+
+    /**
+     * Displays a splash screen.
+     * @return The splash screen. It should be removed by the caller
+     *      when no longer needed by a call to removeSplash().
+     */
+    private static JWindow displaySplash(String filename) {
+        try {
+            Image im = Toolkit.getDefaultToolkit().getImage(filename);
+            JWindow f = new JWindow();
+            f.getContentPane().add(new JLabel(new ImageIcon(im)));
+            f.pack();
+            Point center = GraphicsEnvironment.getLocalGraphicsEnvironment().getCenterPoint();
+            f.setLocation(center.x - f.getWidth() / 2, center.y - f.getHeight() / 2);
+            f.setVisible(true);
+            return f;
+        } catch (Exception e) {
+            logger.warning(e.toString());
+            return null;
+        }
+    }
+
+    /**
+     * Removes splash screen.
+     */
+    private static void removeSplash(JWindow splash) {
+        if (splash != null) {
+            splash.setVisible(false);
+            splash.dispose();
+        }
     }
 
     /**
      * Starts the GUI by creating and displaying the GUI-objects.
      */
-    private void startGUI(Dimension innerWindowSize, final boolean showOpeningVideo) {
-        final AudioMixerOption amo = (AudioMixerOption) getClientOptions().getObject(ClientOptions.AUDIO_MIXER);
-        if (musicLibrary != null) {
-            musicPlayer = new SoundPlayer(amo,
-                    (PercentageOption) getClientOptions().getObject(ClientOptions.MUSIC_VOLUME),
-                    false,
-                    true);
-            //playMusic("intro");
+    private void startGUI(Dimension innerWindowSize,
+                          final boolean sound,
+                          final boolean showOpeningVideo,
+                          final boolean loadGame) {
+       // start the sound system
+        if (sound) {
+            final ClientOptions opts = getClientOptions();
+            final AudioMixerOption amo
+                = (AudioMixerOption) opts.getOption(ClientOptions.AUDIO_MIXER);
+            final PercentageOption volume
+                = (PercentageOption) opts.getOption(ClientOptions.AUDIO_VOLUME);
+            try {
+                soundPlayer = new SoundPlayer(amo, volume);
+            } catch (Exception e) {
+                // #3168279 reports an undocumented NPE thrown by
+                // AudioSystem.getMixer(null).  Workaround this and other
+                // such failures by just disabling sound.
+                soundPlayer = null;
+                logger.log(Level.WARNING, "Unexpected sound failure", e);
+            }
         } else {
-            musicPlayer = null;
+            soundPlayer = null;
         }
-        if (sfxLibrary != null) {
-            sfxPlayer = new SoundPlayer(amo,
-                    (PercentageOption) getClientOptions().getObject(ClientOptions.SFX_VOLUME),
-                    true,
-                    false);
-        } else {
-            sfxPlayer = null;
-        }
-        
+
         if (GraphicsEnvironment.isHeadless()) {
             logger.info("It seems that the GraphicsEnvironment is headless!");
         }
         gd = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice();
         if (!windowed) {
             if (!gd.isFullScreenSupported()) {
-                String fullscreenNotSupported = "\nIt seems that full screen mode is not fully supported for this\nGraphicsDevice. Please try the \"--windowed\" option if you\nexperience any graphical problems while running FreeCol.";
+                String fullscreenNotSupported =
+                   "\nIt seems that full screen mode is not fully supported for this" +
+                   "\nGraphicsDevice. Please try the \"--windowed\" option if you\nexperience" +
+                   "any graphical problems while running FreeCol.";
                 logger.info(fullscreenNotSupported);
                 System.out.println(fullscreenNotSupported);
                 /*
@@ -297,17 +436,20 @@ public final class FreeColClient {
         gui = new GUI(this, innerWindowSize, imageLibrary);
         canvas = new Canvas(this, innerWindowSize, gui);
         changeWindowedMode(windowed);
-
         frame.setIconImage(ResourceManager.getImage("FrameIcon.image"));
-        if (showOpeningVideo) {
+
+        // run opening video or main panel
+        if (showOpeningVideo && !loadGame) {
             SwingUtilities.invokeLater(new Runnable() {
                 public void run() {
-                    canvas.showOpeningVideoPanel();
+            canvas.showOpeningVideoPanel();
                 }
             });
         } else {
-            canvas.showMainPanel();
-            playMusic("intro");
+            if (!loadGame) {
+                canvas.showMainPanel();
+            }
+            playSound("sound.intro.general");
         }
         gui.startCursorBlinking();
     }
@@ -329,7 +471,7 @@ public final class FreeColClient {
     public void setHeadless(final boolean newHeadless) {
         this.headless = newHeadless;
     }
-    
+
     /**
      *  Verifies if this client can save the current game
      *  Clients that do not have the server running, or that have not the required permissions
@@ -405,7 +547,7 @@ public final class FreeColClient {
         canvas.updateSizes();
         frame.setVisible(true);
     }
-    
+
     /**
      * Checks if the application is displayed in a window.
      * @return <code>true</code> if the application is currently
@@ -419,34 +561,36 @@ public final class FreeColClient {
 
     /**
      * Writes the client options to the default location.
-     * 
+     *
+     * @throws FileNotFoundException
      * @see ClientOptions
      */
-    public void saveClientOptions() {
+    public void saveClientOptions() throws FileNotFoundException {
         saveClientOptions(FreeCol.getClientOptionsFile());
     }
 
     public void setMapEditor(boolean mapEditor) {
         this.mapEditor = mapEditor;
     }
-    
+
     public boolean isMapEditor() {
         return mapEditor;
     }
-    
+
     /**
      * Writes the client options to the given file.
-     * 
+     *
      * @param saveFile The file where the client options should be written.
+     * @throws FileNotFoundException
      * @see ClientOptions
      */
-    public void saveClientOptions(File saveFile) {
+    public void saveClientOptions(File saveFile) throws FileNotFoundException {
         getClientOptions().save(saveFile);
     }
 
     /**
      * Gets the <code>ImageLibrary</code>.
-     * 
+     *
      * @return The <code>ImageLibrary</code>.
      */
     public ImageLibrary getImageLibrary() {
@@ -462,7 +606,7 @@ public final class FreeColClient {
 
     /**
      * Reads the {@link ClientOptions} from the given file.
-     * 
+     *
      * @param loadFile The <code>File</code> to read the
      *            <code>ClientOptions</code> from.
      */
@@ -472,7 +616,7 @@ public final class FreeColClient {
 
     /**
      * Gets the object responsible for keeping and updating the actions.
-     * 
+     *
      * @return The <code>ActionManager</code>.
      */
     public ActionManager getActionManager() {
@@ -481,20 +625,20 @@ public final class FreeColClient {
 
     /**
      * Returns the object keeping the current client options.
-     * 
+     *
      * @return The <code>ClientOptions</code>.
      */
     public ClientOptions getClientOptions() {
         return clientOptions;
     }
-    
+
     public MapEditorController getMapEditorController() {
         return mapEditorController;
     }
 
     /**
      * Gets the <code>Player</code> that uses this client.
-     * 
+     *
      * @return The <code>Player</code> made to represent this clients user.
      * @see #setMyPlayer(Player)
      */
@@ -504,7 +648,7 @@ public final class FreeColClient {
 
     /**
      * Sets the <code>Player</code> that uses this client.
-     * 
+     *
      * @param player The <code>Player</code> made to represent this clients
      *            user.
      * @see #getMyPlayer()
@@ -516,7 +660,7 @@ public final class FreeColClient {
     /**
      * Sets the <code>FreeColServer</code> which has been started by the
      * client gui.
-     * 
+     *
      * @param freeColServer The <code>FreeColServer</code>.
      * @see #getFreeColServer()
      */
@@ -526,7 +670,7 @@ public final class FreeColClient {
 
     /**
      * Gets the <code>FreeColServer</code> started by the client.
-     * 
+     *
      * @return The <code>FreeColServer</code> or <code>null</code> if no
      *         server has been started.
      */
@@ -536,7 +680,7 @@ public final class FreeColClient {
 
     /**
      * Sets the <code>Game</code> that we are currently playing.
-     * 
+     *
      * @param game The <code>Game</code>.
      * @see #getGame
      */
@@ -546,7 +690,7 @@ public final class FreeColClient {
 
     /**
      * Gets the <code>Game</code> that we are currently playing.
-     * 
+     *
      * @return The <code>Game</code>.
      * @see #setGame
      */
@@ -557,7 +701,7 @@ public final class FreeColClient {
     /**
      * Gets the <code>Canvas</code> this client uses to display the
      * GUI-components.
-     * 
+     *
      * @return The <code>Canvas</code>.
      */
     public Canvas getCanvas() {
@@ -567,53 +711,84 @@ public final class FreeColClient {
     /**
      * Gets the <code>GUI</code> that is being used to draw the map on the
      * {@link Canvas}.
-     * 
+     *
      * @return The <code>GUI</code>.
      */
     public GUI getGUI() {
         return gui;
     }
 
+    private void exitActions () {
+       try {
+          // action: delete outdated autosave files
+          int validDays = getClientOptions().getInteger(ClientOptions.AUTOSAVE_VALIDITY);
+          long validPeriod = (long)validDays * 86400 * 1000;  // millisecond equivalent of valid days
+          long timeNow = System.currentTimeMillis();
+          File autosaveDir = FreeCol.getAutosaveDirectory();
+
+          if (validPeriod != 0) {
+             // analyse all files in autosave directory
+             String[] flist = autosaveDir.list();
+             for ( int i = 0; flist != null && i < flist.length; i++ ) {
+                String filename = flist[i];
+                // delete files which are older than valid period set by user option
+                if (filename.endsWith(".fsg")) {
+                   File saveGameFile = new File(autosaveDir, filename);
+                   if (saveGameFile.lastModified() + validPeriod < timeNow) {
+                       saveGameFile.delete();
+                   }
+                }
+             }
+          }
+       } catch (Exception e) {
+          e.printStackTrace();
+       }
+    }
+
+
     /**
      * Quits the application without any questions.
      */
     public void quit() {
-        getConnectController().quitGame(true);
+        getConnectController().quitGame(isSingleplayer());
+        exitActions();
         if (!windowed) {
             try {
                 gd.setFullScreenWindow(null);
             } catch(Exception e) {
                 // this can fail, but who cares?
                 // we are quitting anyway
+                System.exit(1);
             }
         }
         System.exit(0);
     }
 
     /**
-     * Retires the player from the game. Returns true if the player
-     * achieved a new high score.
+     * Retires the player from the game.
      *
-     * @return a <code>boolean</code> value
+     * @return True if the player achieved a new high score.
      */
     public boolean retire() {
         Element retireElement = Message.createNewRootElement("retire");
         Element reply = client.ask(retireElement);
-        return ("true".equals(reply.getAttribute("highScore")));
+        boolean result = reply != null
+            && "true".equals(reply.getAttribute("highScore"));
+        return result;
     }
 
 
     /**
-     * Continue playing after win the game
+     * Continue playing after winning the game.
      */
     public void continuePlaying() {
-        Element continueElement = Message.createNewRootElement("continuePlaying");
-        client.send(continueElement);
+        client.send(Message.createNewRootElement("continuePlaying"));
     }
 
+
     /**
      * Checks if this client is the game admin.
-     * 
+     *
      * @return <i>true</i> if the client is the game admin and <i>false</i>
      *         otherwise. <i>false</i> is also returned if a game have not yet
      *         been started.
@@ -627,7 +802,7 @@ public final class FreeColClient {
 
     /**
      * Sets whether or not this game is a singleplayer game.
-     * 
+     *
      * @param singleplayer Indicates whether or not this game is a singleplayer
      *            game.
      * @see #isSingleplayer
@@ -638,7 +813,7 @@ public final class FreeColClient {
 
     /**
      * Is the user playing in singleplayer mode.
-     * 
+     *
      * @return <i>true</i> if the user is playing in singleplayer mode and
      *         <i>false</i> otherwise.
      * @see #setSingleplayer
@@ -647,10 +822,29 @@ public final class FreeColClient {
         return singleplayer;
     }
 
+    /**
+     * Sets whether or not the user has retired the game.
+     *
+     * @param isRetired Indicates whether or not the user has retired the game.
+     */
+    public void setIsRetired(boolean isRetired) {
+        this.isRetired = isRetired;
+    }
+
+    /**
+     * Has the user retired the game.
+     *
+     * @return <i>true</i> if the user has retired the game and
+     *         <i>false</i> otherwise.
+     */
+    public boolean isRetired() {
+        return isRetired;
+    }
+
     /**
      * Gets the controller responsible for starting a server and connecting to
      * it.
-     * 
+     *
      * @return The <code>ConnectController</code>.
      */
     public ConnectController getConnectController() {
@@ -659,7 +853,7 @@ public final class FreeColClient {
 
     /**
      * Gets the controller that will be used before the game has been started.
-     * 
+     *
      * @return The <code>PreGameController</code>.
      */
     public PreGameController getPreGameController() {
@@ -669,7 +863,7 @@ public final class FreeColClient {
     /**
      * Gets the input handler that will be used before the game has been
      * started.
-     * 
+     *
      * @return The <code>PreGameInputHandler</code>.
      */
     public PreGameInputHandler getPreGameInputHandler() {
@@ -678,7 +872,7 @@ public final class FreeColClient {
 
     /**
      * Gets the controller that will be used when the game has been started.
-     * 
+     *
      * @return The <code>InGameController</code>.
      */
     public InGameController getInGameController() {
@@ -687,26 +881,17 @@ public final class FreeColClient {
 
     /**
      * Gets the input handler that will be used when the game has been started.
-     * 
+     *
      * @return The <code>InGameInputHandler</code>.
      */
     public InGameInputHandler getInGameInputHandler() {
         return inGameInputHandler;
     }
 
-    /**
-     * Gets the <code>ClientModelController</code>.
-     * 
-     * @return The <code>ClientModelController</code>.
-     */
-    public ClientModelController getModelController() {
-        return modelController;
-    }
-
     /**
      * Sets the <code>Client</code> that shall be used to send messages to the
      * server.
-     * 
+     *
      * @param client the <code>Client</code>
      * @see #getClient
      */
@@ -717,7 +902,7 @@ public final class FreeColClient {
     /**
      * Gets the <code>Client</code> that can be used to send messages to the
      * server.
-     * 
+     *
      * @return the <code>Client</code>
      * @see #setClient
      */
@@ -725,69 +910,45 @@ public final class FreeColClient {
         return client;
     }
 
-    /**
-     * Plays the music.
-     */
-    public void playMusic(String music) {
-        if (musicPlayer != null) {
-            musicPlayer.play(musicLibrary.get(music));
-        }
+    public SoundPlayer getSoundPlayer ()
+    {
+        return soundPlayer;
     }
-    
+
     /**
-     * Plays a random music from the given playlist.
-     */
-    public void playMusicOnce(String music) {
-        if (musicPlayer != null) {
-            musicPlayer.playOnce(musicLibrary.get(music));
-        }
-    }
-    
-    /**
-     * Plays a random music from the given playlist.
-     * @param delay A delay before playing the sound (ms).
-     */
-    public void playMusicOnce(String music, int delay) {
-        if (musicPlayer != null) {
-            musicPlayer.playOnce(musicLibrary.get(music), delay);
-        }
-    }
-    
-    /**
-     * Plays the given sound effect.
-     * 
-     * @param sound The key sound effect given by {@link SfxLibrary}.
+     * Plays some sound. Parameter == null stops playing a sound.
+     *
+     * @param sound The sound resource to play or <b>null</b>
      */
     public void playSound(String sound) {
-        if (sfxPlayer != null) {
-            sfxPlayer.play(sfxLibrary.get(sound));
+        if (canPlaySound()) {
+            if (sound == null)
+               soundPlayer.fade();
+            else {
+               File file = ResourceManager.getAudio(sound);
+               if (file != null) {
+                   soundPlayer.playOnce(file);
+               }
+               logger.finest(((file == null) ? "Could not load" : "Playing")
+                             + " sound: " + sound);
+            }
         }
     }
 
     /**
-     * Plays the given sound effect.
-     * 
-     * @param sound The key sound effect given by {@link SfxLibrary}.
+     * Verifies if the client can play sounds.
+     * @return boolean <b>true</b> if and only if client sound player has an instance
      */
-    public void playSound(SoundLibrary.SoundEffect sound) {
-        if (sfxPlayer != null) {
-            sfxPlayer.play(sfxLibrary.get(sound));
-        }
-    }
-    
-    /**
-     * Verifies if the client can play music
-     */
-    public boolean canPlayMusic(){
-        return musicPlayer != null;
+    public boolean canPlaySound() {
+        return soundPlayer != null;
     }
 
     /**
-     * Returns <i>true</i> if this client is logged in to a server or <i>false</i>
-     * otherwise.
-     * 
-     * @return <i>true</i> if this client is logged in to a server or <i>false</i>
-     *         otherwise.
+     * Returns <i>true</i> if this client is logged in to a server or
+     * <i>false</i> otherwise.
+     *
+     * @return <i>true</i> if this client is logged in to a server or
+     *         <i>false</i> otherwise.
      */
     public boolean isLoggedIn() {
         return loggedIn;
@@ -795,7 +956,7 @@ public final class FreeColClient {
 
     /**
      * Sets whether or not this client is logged in to a server.
-     * 
+     *
      * @param loggedIn An indication of whether or not this client is logged in
      *            to a server.
      */
@@ -804,105 +965,49 @@ public final class FreeColClient {
     }
 
     /**
-     * Get the pseudo-random number generator for the client.
-     * 
-     * @return random number generator.
+     * Set the game-wide next active unit if one can be found.
+     *
+     * @param unitId A unit id for the unit to make active.
      */
-    public PseudoRandom getPseudoRandom() {
-        return _random;
+    public void setActiveUnit(String unitId) {
+        if (unitId != null && getGame() != null) {
+            Unit active = (Unit) getGame().getFreeColGameObject(unitId);
+            if (active != null) {
+                active.getOwner().resetIterators();
+                active.getOwner().setNextActiveUnit(active);
+                getGUI().setActiveUnit(active);
+            }
+        }
     }
 
 
+
     /**
-     * This class provides server-generated random numbers for client-side use.
-     * It requires a server connection. If the connection is unavailable it will
-     * generate random numbers on the client side rather than failing.
+     * Notifies this GUI that the game has started or ended.
+     * @param inGame Indicates whether or not the game has started.
      */
-    private class ClientPseudoRandom implements PseudoRandom {
-        ClientPseudoRandom() {
-            values = new LinkedList<Integer>();
-            offlineRandom = new Random();
-        }
+    public void setInGame(boolean inGame) {
+        this.inGame = inGame;
+    }
 
-        /**
-         * Get the next pseudo-random integer between 0 and n.
-         * 
-         * @param n The upper bound (exclusive).
-         * @return random number between 0 (inclusive) and n (exclusive).
-         */
-        public int nextInt(int n) {
-            if (n <= 0) {
-                throw new IllegalArgumentException("n must be positive!");
-            }
-            // TODO (Erik): get random int in given range.
-            // This may not be good enough, as the low bits may be less
-            // random than the entire range. See the Random class for a
-            // more advanced implementation.
-            return Math.abs(nextInt() % n);
-        }
+    /**
+     * Checks if the game has started.
+     * @return <i>true</i> if the game has started.
+     * @see #setInGame
+     */
+    public boolean isInGame() {
+        return inGame;
+    }
 
-        /**
-         * Get the next pseudo-random integer.
-         * <p>
-         * The method requires one network call per {@link #VALUES_PER_CALL}
-         * requests. If multiple threads are active a single thread may
-         * theoretically have to wait indefinitely, but in practice this is not
-         * very likely.
-         * 
-         * @return next random integer.
-         */
-        private int nextInt() {
-            Integer i = pop();
-            while (i == null) {
-                getNewNumbers();
-                i = pop();
-            }
-            return i.intValue();
-        }
-
-        /**
-         * Get new numbers to the queue from the server, generate on client side
-         * if not connected. The method is guaranteed to generate at least one
-         * new number.
-         */
-        private void getNewNumbers() {
-            int valuesAdded = 0;
-            if (isLoggedIn()) {
-                Element query = Message.createNewRootElement("getRandomNumbers");
-                query.setAttribute("n", String.valueOf(VALUES_PER_CALL));
-                // We expect client != null when logged in
-                Element answer = getClient().ask(query);
-                if (answer != null && "getRandomNumbersConfirmed".equals(answer.getTagName())) {
-                    for (String s : answer.getAttribute("result").split(",")) {
-                        push(new Integer(s));
-                        ++valuesAdded;
-                    }
-                } else {
-                    logger.warning("Expected getRandomNumbersConfirmed, got "
-                            + (answer != null ? answer.getTagName() : "null"));
-                }
-            }
-            // Fallback solution on errors (we don't want to crash the game)
-            if (valuesAdded < 1) {
-                logger.fine("Generating random number on client side");
-                push(Integer.valueOf(offlineRandom.nextInt()));
-            }
-        }
-
-        private synchronized void push(Integer i) {
-            values.offer(i);
-        }
-
-        private synchronized Integer pop() {
-            return values.poll();
-        }
-
-
-        private final Random offlineRandom;
-
-        private final Queue<Integer> values;
-
-        private static final int VALUES_PER_CALL = 100;
+    /**
+     * Start the game skipping turns.
+     *
+     * @param turns The number of turns to skip.
+     */
+    public void skipTurns(int turns) {
+        if (freeColServer == null) return;
+        freeColServer.getInGameController().setSkippedTurns(turns);
+        getCanvas().closeMenus();
+        inGameController.endTurn();
     }
 }
-
