@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -45,19 +45,19 @@ import net.sf.freecol.client.gui.panel.LoadingSavegameDialog;
 import net.sf.freecol.client.networking.Client;
 import net.sf.freecol.common.FreeColException;
 import net.sf.freecol.common.ServerInfo;
-import net.sf.freecol.common.Specification;
 import net.sf.freecol.common.io.FreeColSavegameFile;
 import net.sf.freecol.common.model.Game;
-import net.sf.freecol.common.model.GameOptions;
-import net.sf.freecol.common.model.NationOptions;
+import net.sf.freecol.common.model.NationOptions.Advantages;
 import net.sf.freecol.common.model.Player;
+import net.sf.freecol.common.model.Specification;
+import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.networking.Connection;
 import net.sf.freecol.common.networking.Message;
 import net.sf.freecol.common.networking.NoRouteToServerException;
+import net.sf.freecol.common.option.OptionGroup;
 import net.sf.freecol.common.resources.ResourceManager;
 import net.sf.freecol.common.util.XMLStream;
 import net.sf.freecol.server.FreeColServer;
-import net.sf.freecol.server.generator.MapGeneratorOptions;
 
 import org.w3c.dom.Element;
 import org.w3c.dom.NodeList;
@@ -82,17 +82,16 @@ public final class ConnectController {
         this.freeColClient = freeColClient;
     }
 
-    
+
     /**
      * Starts a multiplayer server and connects to it.
      *
-     * @param publicServer Should this server be listed at the meta server.
      * @param username The name to use when logging in.
      * @param port The port in which the server should listen for new clients.
-     * @param nationOptions a <code>NationOptions</code> value
+     * @param level a <code>DifficultyLevel</code> value
      */
-    public void startMultiplayerGame(boolean publicServer, String username, int port,
-                                     NationOptions nationOptions) {
+    public void startMultiplayerGame(Specification specification, boolean publicServer, String username, int port,
+                                     Advantages advantages, OptionGroup level) {
 
         freeColClient.setMapEditor(false);
 
@@ -100,7 +99,7 @@ public final class ConnectController {
             logout(true);
         }
 
-        if (freeColClient.getFreeColServer() != null && 
+        if (freeColClient.getFreeColServer() != null &&
             freeColClient.getFreeColServer().getServer().getPort() == port) {
             if (freeColClient.getCanvas().showConfirmDialog("stopServer.text",
                                                             "stopServer.yes",
@@ -112,7 +111,7 @@ public final class ConnectController {
         }
 
         try {
-            FreeColServer freeColServer = new FreeColServer(publicServer, false, port, null, nationOptions);
+            FreeColServer freeColServer = new FreeColServer(specification, publicServer, false, port, null, advantages);
             freeColClient.setFreeColServer(freeColServer);
         } catch (NoRouteToServerException e) {
             freeColClient.getCanvas().errorMessage("server.noRouteToServer");
@@ -129,13 +128,14 @@ public final class ConnectController {
     /**
      * Starts a new singleplayer game by connecting to the server.
      *
+     * @param specification a <code>Specification</code> value
      * @param username The name to use when logging in.
-     * @param nationOptions a <code>NationOptions</code> value
+     * @param advantages an <code>Advantages</code> value
      */
-    public void startSingleplayerGame(String username, NationOptions nationOptions) {
+    public void startSingleplayerGame(Specification specification, String username, Advantages advantages) {
 
         freeColClient.setMapEditor(false);
-        
+
         if (freeColClient.isLoggedIn()) {
             logout(true);
         }
@@ -155,7 +155,10 @@ public final class ConnectController {
         }
 
         try {
-            FreeColServer freeColServer = new FreeColServer(false, true, port, null, nationOptions);
+            FreeColServer freeColServer = new FreeColServer(specification, false, true, port, null, advantages);
+            if (freeColClient.getClientOptions().getBoolean(ClientOptions.AUTOSAVE_DELETE)) {
+                FreeColServer.removeAutosaves(Messages.message("clientOptions.savegames.autosave.fileprefix"));
+            }
             freeColClient.setFreeColServer(freeColServer);
         } catch (NoRouteToServerException e) {
             logger.warning("Illegal state: An exception occured that can only appear in public multiplayer games.");
@@ -171,7 +174,7 @@ public final class ConnectController {
             freeColClient.getPreGameController().setReady(true);
             freeColClient.getCanvas().showStartGamePanel(freeColClient.getGame(), freeColClient.getMyPlayer(),
                                                          true);
-                                                         
+
         }
     }
 
@@ -186,17 +189,17 @@ public final class ConnectController {
     public void joinMultiplayerGame(String username, String host, int port) {
         final Canvas canvas = freeColClient.getCanvas();
         freeColClient.setMapEditor(false);
-        
+
         if (freeColClient.isLoggedIn()) {
             logout(true);
         }
 
         List<String> vacantPlayers = getVacantPlayers(host, port);
         if (vacantPlayers != null) {
-            String choice = canvas
-                .showSimpleChoiceDialog(Messages.message("connectController.choicePlayer"),
-                                        Messages.message("cancel"),
-                                        vacantPlayers);
+            String choice = canvas.showSimpleChoiceDialog(null,
+                                                          "connectController.choicePlayer",
+                                                          "cancel",
+                                                          vacantPlayers);
             if (choice != null) {
                 username = choice;
             } else {
@@ -205,7 +208,7 @@ public final class ConnectController {
         }
 
         freeColClient.setSingleplayer(false);
-        if (login(username, host, port) && !freeColClient.getGUI().isInGame()) {
+        if (login(username, host, port) && !freeColClient.isInGame()) {
             canvas.showStartGamePanel(freeColClient.getGame(), freeColClient.getMyPlayer(), false);
         }
     }
@@ -222,10 +225,8 @@ public final class ConnectController {
     public boolean login(String username, String host, int port) {
         Client client = freeColClient.getClient();
         Canvas canvas = freeColClient.getCanvas();
-        NationOptions nationOptions = new NationOptions();
-
         freeColClient.setMapEditor(false);
-        
+
         if (client != null) {
             client.disconnect();
         }
@@ -256,37 +257,42 @@ public final class ConnectController {
                 boolean startGame = (startGameStr != null) && Boolean.valueOf(startGameStr).booleanValue();
                 boolean singleplayer = Boolean.valueOf(in.getAttributeValue(null, "singleplayer")).booleanValue();
                 boolean isCurrentPlayer = Boolean.valueOf(in.getAttributeValue(null, "isCurrentPlayer")).booleanValue();
+                String activeUnitId = in.getAttributeValue(null, "activeUnit");
 
                 in.nextTag();
-                Game game = new Game(freeColClient.getModelController(), in, username);
-                
+                Game game = new Game(in, username);
+
                 // this completes the client's view of the spec with options obtained from the server difficulty
                 // it should not be required in the client, to be removed later, when newTurn() only runs in the server
-                Specification.getSpecification().applyDifficultyLevel(game.getGameOptions().getInteger(GameOptions.DIFFICULTY));
-                
+
                 Player thisPlayer = game.getPlayerByName(username);
 
                 freeColClient.setGame(game);
                 freeColClient.setMyPlayer(thisPlayer);
 
-                final MapGeneratorOptions mgo;
-                if (in.getLocalName().equals(MapGeneratorOptions.getXMLElementTagName())) {
-                    mgo = new MapGeneratorOptions(in);
-                } else {
-                    mgo = new MapGeneratorOptions();
-                }
-                freeColClient.getPreGameController().setMapGeneratorOptions(mgo);
-                
+                freeColClient.getActionManager().addSpecificationActions(game.getSpecification());
+
                 c.endTransmission(in);
-                
+
                 // If (true) --> reconnect
                 if (startGame) {
+                    Tile entryTile = thisPlayer.getEntryLocation().getTile();
                     freeColClient.setSingleplayer(singleplayer);
                     freeColClient.getPreGameController().startGame();
 
                     if (isCurrentPlayer) {
-                        freeColClient.getInGameController().setCurrentPlayer(thisPlayer);
+                        freeColClient.getInGameController()
+                            .setCurrentPlayer(thisPlayer);
+                        if (activeUnitId != null) {
+                            freeColClient.setActiveUnit(activeUnitId);
+                        } else {
+                            freeColClient.getGUI().setSelectedTile(entryTile, false);
+                        }
+                    } else {
+                        freeColClient.getGUI().setSelectedTile(entryTile, false);
                     }
+                    freeColClient.getGUI().setSelectedTile(thisPlayer
+                        .getEntryLocation().getTile(), false);
                 }
             } else if (in.getLocalName().equals("error")) {
                 canvas.errorMessage(in.getAttributeValue(null, "messageID"), in.getAttributeValue(null, "message"));
@@ -297,7 +303,7 @@ public final class ConnectController {
                 logger.warning("Unkown message received: " + in.getLocalName());
                 c.endTransmission(in);
                 return false;
-            }            
+            }
         } catch (Exception e) {
             StringWriter sw = new StringWriter();
             e.printStackTrace(new PrintWriter(sw));
@@ -323,7 +329,7 @@ public final class ConnectController {
         final String username = freeColClient.getMyPlayer().getName();
         final String host = freeColClient.getClient().getHost();
         final int port = freeColClient.getClient().getPort();
-        
+
         freeColClient.getCanvas().removeInGameComponents();
         logout(true);
         login(username, host, port);
@@ -352,7 +358,7 @@ public final class ConnectController {
         final File theFile = file;
 
         freeColClient.setMapEditor(false);
-        
+
         class ErrorJob implements Runnable {
             private final  String  message;
             ErrorJob( String message ) {
@@ -364,7 +370,6 @@ public final class ConnectController {
             }
         }
 
-        final boolean publicServer;
         final boolean singleplayer;
         final String name;
         final int port;
@@ -372,7 +377,7 @@ public final class ConnectController {
         try {
             // Get suggestions for "singleplayer" and "public game" settings from the file:
             final FreeColSavegameFile fis = new FreeColSavegameFile(theFile);
-            xs = FreeColServer.createXMLStreamReader(fis);
+            xs = new XMLStream(fis.getSavegameInputStream());
             final XMLStreamReader in = xs.getXMLStreamReader();
             in.nextTag();
             final boolean defaultSingleplayer = Boolean.valueOf(in.getAttributeValue(null, "singleplayer")).booleanValue();
@@ -384,12 +389,18 @@ public final class ConnectController {
                 defaultPublicServer = false;
             }
             xs.close();
+            try {
+                freeColClient.getClientOptions().load(fis.getInputStream(FreeColSavegameFile.CLIENT_OPTIONS), true);
+                freeColClient.getActionManager().update();
+            } catch(IOException e) {
+                // old savegame format, we don't care
+                logger.info("No saved client options available.");
+            }
             final int sgo = freeColClient.getClientOptions().getInteger(ClientOptions.SHOW_SAVEGAME_SETTINGS);
             if (sgo == ClientOptions.SHOW_SAVEGAME_SETTINGS_ALWAYS
                     || !defaultSingleplayer && sgo == ClientOptions.SHOW_SAVEGAME_SETTINGS_MULTIPLAYER) {
                 if (canvas.showLoadingSavegameDialog(defaultPublicServer, defaultSingleplayer)) {
                     LoadingSavegameDialog lsd = canvas.getLoadingSavegameDialog();
-                    publicServer = lsd.isPublic();
                     singleplayer = lsd.isSingleplayer();
                     name = lsd.getName();
                     port = lsd.getPort();
@@ -397,7 +408,6 @@ public final class ConnectController {
                     return;
                 }
             } else {
-                publicServer = defaultPublicServer;
                 singleplayer = defaultSingleplayer;
                 name = null;
                 port = FreeCol.getDefaultPort();
@@ -414,13 +424,15 @@ public final class ConnectController {
         } catch (XMLStreamException e) {
             StringWriter sw = new StringWriter();
             e.printStackTrace(new PrintWriter(sw));
-            logger.warning(sw.toString());                    
+            logger.warning(sw.toString());
             SwingUtilities.invokeLater( new ErrorJob("server.couldNotStart") );
             return;
         } finally {
-            xs.close();
+            if (xs != null) {
+                xs.close();
+            }
         }
-        
+
         if (freeColClient.getFreeColServer() != null && freeColClient.getFreeColServer().getServer().getPort() == port) {
             if (freeColClient.getCanvas().showConfirmDialog("stopServer.text", "stopServer.yes", "stopServer.no")) {
                 freeColClient.getFreeColServer().getController().shutdown();
@@ -430,23 +442,24 @@ public final class ConnectController {
         }
 
         canvas.showStatusPanel(Messages.message("status.loadingGame"));
-        
+
         Runnable loadGameJob = new Runnable() {
             public void run() {
                 FreeColServer freeColServer = null;
                 try {
                     final FreeColSavegameFile savegame = new FreeColSavegameFile(theFile);
-                    freeColServer = new FreeColServer(savegame, publicServer, singleplayer, port, name);
+                    freeColServer = new FreeColServer(savegame, port, name);
                     freeColClient.setFreeColServer(freeColServer);
                     final String username = freeColServer.getOwner();
                     freeColClient.setSingleplayer(singleplayer);
+                    freeColClient.getInGameController().setGameConnected();
                     SwingUtilities.invokeLater( new Runnable() {
                         public void run() {
                             ResourceManager.setScenarioMapping(savegame.getResourceMapping());
                             login(username, "127.0.0.1", FreeCol.getDefaultPort());
                             canvas.closeStatusPanel();
                         }
-                    } );                    
+                    } );
                 } catch (NoRouteToServerException e) {
                     SwingUtilities.invokeLater(new Runnable() {
                         public void run() {
@@ -478,11 +491,11 @@ public final class ConnectController {
                             freeColClient.getCanvas().showMainPanel();
                         }
                     });
-                    SwingUtilities.invokeLater( new ErrorJob(e.getMessage()) );                    
+                    SwingUtilities.invokeLater( new ErrorJob(e.getMessage()) );
                 }
             }
         };
-        freeColClient.worker.schedule( loadGameJob );        
+        freeColClient.worker.schedule( loadGameJob );
     }
 
     /**
@@ -508,15 +521,15 @@ public final class ConnectController {
 
         ResourceManager.setScenarioMapping(null);
         ResourceManager.setCampaignMapping(null);
-        
+
         if (!freeColClient.isHeadless()) {
-            freeColClient.getGUI().setInGame(false);
+            freeColClient.setInGame(false);
         }
         freeColClient.setGame(null);
         freeColClient.setMyPlayer(null);
         freeColClient.setClient(null);
 
-        freeColClient.setLoggedIn(false);        
+        freeColClient.setLoggedIn(false);
     }
 
 
@@ -536,20 +549,21 @@ public final class ConnectController {
     */
     public void quitGame(boolean bStopServer, boolean notifyServer) {
         final FreeColServer server = freeColClient.getFreeColServer();
-        if (bStopServer && server != null) {            
+        if (bStopServer && server != null) {
             server.getController().shutdown();
             freeColClient.setFreeColServer(null);
 
             ResourceManager.setScenarioMapping(null);
             ResourceManager.setCampaignMapping(null);
-            freeColClient.getGUI().setInGame(false);
+            freeColClient.setInGame(false);
             freeColClient.setGame(null);
             freeColClient.setMyPlayer(null);
-            freeColClient.setClient(null);                
-            freeColClient.setLoggedIn(false);            
+            freeColClient.setIsRetired(false);
+            freeColClient.setClient(null);
+            freeColClient.setLoggedIn(false);
         } else if (freeColClient.isLoggedIn()) {
             logout(notifyServer);
-        }          
+        }
     }
 
 
@@ -595,7 +609,7 @@ public final class ConnectController {
                 logger.warning("The reply has an unknown type: " + reply.getTagName());
                 return null;
             }
-                        
+
             NodeList nl = reply.getChildNodes();
             for (int i=0; i<nl.getLength(); i++) {
                 items.add(((Element) nl.item(i)).getAttribute("username"));
@@ -609,7 +623,7 @@ public final class ConnectController {
                 logger.warning("Could not close connection.");
             }
         }
-                        
+
         return items;
     }
 
