@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -41,10 +41,9 @@ import net.sf.freecol.client.gui.panel.RiverStylePanel;
 import net.sf.freecol.common.model.IndianSettlement;
 import net.sf.freecol.common.model.Map;
 import net.sf.freecol.common.model.Map.Direction;
-import net.sf.freecol.common.model.Map.Position;
 import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.TileImprovement;
-import net.sf.freecol.common.model.TileType;
+import net.sf.freecol.common.resources.ResourceManager;
 import net.sf.freecol.server.generator.TerrainGenerator;
 
 /**
@@ -69,7 +68,7 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
 
     /**
      * The constructor to use.
-     * 
+     *
      * @param canvas The component this object gets created for.
      * @param g The GUI that holds information such as screen resolution.
      */
@@ -78,8 +77,8 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
         gui = g;
         scrollThread = null;
     }
-    
-    
+
+
     /**
      * This method can be called to make sure the map is loaded
      * There is no point executing mouse events if the map is not loaded
@@ -93,7 +92,7 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
 
     /**
      * Invoked when a mouse button was clicked.
-     * 
+     *
      * @param e The MouseEvent that holds all the information.
      */
     public void mouseClicked(MouseEvent e) {
@@ -102,8 +101,8 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
         }
         try {
             if (e.getClickCount() > 1) {
-                Position p = gui.convertToMapCoordinates(e.getX(), e.getY());
-                gui.showColonyPanel(p);
+                Tile tile = gui.convertToMapTile(e.getX(), e.getY());
+                canvas.showColonyPanel(tile);
             } else {
                 canvas.requestFocus();
             }
@@ -114,7 +113,7 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
 
     /**
      * Invoked when the mouse enters the component.
-     * 
+     *
      * @param e The MouseEvent that holds all the information.
      */
     public void mouseEntered(MouseEvent e) {
@@ -123,17 +122,17 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
 
     /**
      * Invoked when the mouse exits the component.
-     * 
+     *
      * @param e The MouseEvent that holds all the information.
      */
     public void mouseExited(MouseEvent e) {
         // Ignore for now.
     }
-    
+
 
     /**
      * Invoked when a mouse button was pressed.
-     * 
+     *
      * @param e The MouseEvent that holds all the information.
      */
     public void mousePressed(MouseEvent e) {
@@ -145,8 +144,7 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
         }
         try {
             if (e.getButton() == MouseEvent.BUTTON3 || e.isPopupTrigger()) {
-                Position p = gui.convertToMapCoordinates(e.getX(), e.getY());
-                Tile tile = getMap().getTile(p);
+                Tile tile = gui.convertToMapTile(e.getX(), e.getY());
                 if (tile != null) {
                     if (tile.hasRiver()) {
                         TileImprovement river = tile.getRiver();
@@ -155,18 +153,17 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
                             // user canceled
                         } else if (style == 0) {
                             tile.getTileItemContainer().removeTileItem(river);
-                        } else if (0 < style && style < ImageLibrary.RIVER_STYLES) {
+                        } else if (0 < style && style < ResourceManager.RIVER_STYLES) {
                             river.setStyle(style);
                         } else {
                             logger.warning("Unknown river style: " + style);
                         }
                     }
-                    if (tile.getSettlement() instanceof IndianSettlement) {
-                        IndianSettlement settlement = (IndianSettlement) tile.getSettlement();
-                        canvas.showFreeColDialog(new EditSettlementDialog(canvas, settlement));
+                    if (tile.getIndianSettlement() != null) {
+                        canvas.showFreeColDialog(new EditSettlementDialog(canvas, tile.getIndianSettlement()));
                     }
                 } else {
-                    gui.setSelectedTile(p, true);
+                    gui.setSelectedTile(null, true);
                 }
             } else if (e.getButton() == MouseEvent.BUTTON1) {
                 startPoint = e.getPoint();
@@ -181,7 +178,7 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
 
     /**
      * Invoked when a mouse button was released.
-     * 
+     *
      * @param e The MouseEvent that holds all the information.
      */
     public void mouseReleased(MouseEvent e) {
@@ -189,10 +186,10 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
             return;
         }
         JComponent component = (JComponent)e.getSource();
-        
+
         MapEditorController controller = canvas.getClient().getMapEditorController();
-        boolean isTransformActive = controller.getMapTransform() != null; 
-        
+        boolean isTransformActive = controller.getMapTransform() != null;
+
         if(startPoint == null){
         	startPoint = e.getPoint();
         }
@@ -200,37 +197,37 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
         	oldPoint = e.getPoint();
         }
         drawBox(component, startPoint, oldPoint);
-        if (gui.getFocus() != null) {	
-            Position start = gui.convertToMapCoordinates(startPoint.x, startPoint.y);
-            Position end = start;
+        if (gui.getFocus() != null) {
+            Tile start = gui.convertToMapTile(startPoint.x, startPoint.y);
+            Tile end = start;
             //Optimization, only check if the points are different
             if(startPoint.x != oldPoint.x || startPoint.y != oldPoint.y){
-            	end = gui.convertToMapCoordinates(oldPoint.x, oldPoint.y);
+            	end = gui.convertToMapTile(oldPoint.x, oldPoint.y);
             }
-            
+
             // no option selected, just center map
             if(!isTransformActive){
             	gui.setFocus(end);
             	return;
             }
-            
+
             // find the area to transform
             int min_x, max_x, min_y, max_y;
-            if (start.x < end.x) {
-                min_x = start.x;
-                max_x = end.x;
+            if (start.getX() < end.getX()) {
+                min_x = start.getX();
+                max_x = end.getX();
             } else {
-                min_x = end.x;
-                max_x = start.x;
+                min_x = end.getX();
+                max_x = start.getX();
             }
-            if (start.y < end.y) {
-                min_y = start.y;
-                max_y = end.y;
+            if (start.getY() < end.getY()) {
+                min_y = start.getY();
+                max_y = end.getY();
             } else {
-                min_y = end.y;
-                max_y = start.y;
+                min_y = end.getY();
+                max_y = start.getY();
             }
-            
+
             // apply transformation to all tiles in the area
             Tile t = null;
             for (int x = min_x; x <= max_x; x++) {
@@ -258,15 +255,15 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
 
     /**
      * Invoked when the mouse has been moved.
-     * 
+     *
      * @param e The MouseEvent that holds all the information.
      */
     public void mouseMoved(MouseEvent e) {
         if (getMap() == null) {
             return;
         }
-		
-        if (e.getComponent().isEnabled() && 
+
+        if (e.getComponent().isEnabled() &&
             canvas.getClient().getClientOptions().getBoolean(ClientOptions.AUTO_SCROLL)) {
             auto_scroll(e.getX(), e.getY());
         } else if (scrollThread != null) {
@@ -274,23 +271,21 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
             scrollThread = null;
         }
     }
-	
+
     /**
      * Invoked when the mouse has been dragged.
-     * 
+     *
      * @param e The MouseEvent that holds all the information.
      */
     public void mouseDragged(MouseEvent e) {
         if (getMap() == null) {
             return;
         }
-		
+
         JComponent component = (JComponent)e.getSource();
         drawBox(component, startPoint, oldPoint);
         oldPoint = e.getPoint();
         drawBox(component, startPoint, oldPoint);
-        Map.Position p = gui.convertToMapCoordinates(e.getX(), e.getY());
-
         if (e.getComponent().isEnabled() &&
             canvas.getClient().getClientOptions().getBoolean(ClientOptions.MAP_SCROLL_ON_DRAG)) {
             drag_scroll(e.getX(), e.getY());
@@ -300,7 +295,7 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
         }
         canvas.refresh();
     }
-	
+
     private void drawBox(JComponent component, Point startPoint, Point endPoint) {
         if(startPoint == null || endPoint == null){
         	return;
@@ -308,13 +303,13 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
         if(startPoint.distance(endPoint) == 0){
         	return;
         }
-        
+
         // only bother to draw if a transformation is active
         MapEditorController controller = canvas.getClient().getMapEditorController();
-        if(controller.getMapTransform() == null){ 
+        if(controller.getMapTransform() == null){
         	return;
         }
-    	
+
     	Graphics2D graphics = (Graphics2D) component.getGraphics ();
         graphics.setColor(Color.WHITE);
         int x = Math.min(startPoint.x, endPoint.x);
@@ -327,7 +322,7 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
     private void auto_scroll(int x, int y){
         scroll(x, y, AUTO_SCROLLSPACE);
     }
-	
+
     private void drag_scroll(int x, int y){
         scroll(x, y, DRAG_SCROLLSPACE);
     }
@@ -336,14 +331,14 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
         if (getMap() == null) {
             return;
         }
-		
+
         /*
          * if (y < canvas.getMenuBarHeight()) { if (scrollThread != null) {
          * scrollThread.stopScrolling(); scrollThread = null; } return; } else
          * if (y < canvas.getMenuBarHeight() + SCROLLSPACE) { y -=
          * canvas.getMenuBarHeight(); }
          */
-		 
+
         Direction direction;
         if ((x < scrollspace) && (y < scrollspace)) {
             // Upper-Left
@@ -383,7 +378,7 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
             scrollThread.setDirection(direction);
         } else {
             // start scrolling in a direction
-            scrollThread = new ScrollThread(getMap(), gui);
+            scrollThread = new ScrollThread(gui);
             scrollThread.setDirection(direction);
             scrollThread.start();
         }
@@ -395,8 +390,6 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
      */
     private class ScrollThread extends Thread {
 
-        private final Map map;
-
         private final GUI gui;
 
         private Direction direction;
@@ -406,20 +399,18 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
 
         /**
          * The constructor to use.
-         * 
-         * @param m The Map that needs to be scrolled.
+         *
          * @param g The GUI that holds information such as screen resolution.
          */
-        public ScrollThread(Map m, GUI g) {
+        public ScrollThread(GUI g) {
             super(FreeCol.CLIENT_THREAD+"Mouse scroller");
-            map = m;
             gui = g;
             cont = true;
         }
 
         /**
          * Sets the direction in which this ScrollThread will scroll.
-         * 
+         *
          * @param d The direction in which this ScrollThread will scroll.
          */
         public void setDirection(Direction d) {
@@ -436,6 +427,7 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
         /**
          * Performs the actual scrolling.
          */
+        @Override
         public void run() {
             do {
                 try {
@@ -448,12 +440,12 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
                             public void run() {
                                 try {
                                     int x, y;
-                                    Tile t = map.getTile(gui.getFocus().getX(), gui.getFocus().getY());
+                                    Tile t = gui.getFocus();
                                     if (t == null) {
                                         return;
                                     }
 
-                                    t = map.getNeighbourOrNull(direction, t);
+                                    t = t.getNeighbourOrNull(direction);
                                     if (t == null) {
                                         return;
                                     }
@@ -504,7 +496,7 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
                                         x = t.getX();
                                     }
 
-                                    gui.setFocus(x, y);
+                                    gui.setFocus(getMap().getTile(x,y));
                                 } catch (Exception e) {
                                     logger.log(Level.WARNING, "Exception while scrolling!", e);
                                 }
