@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -40,31 +40,40 @@ import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.gui.action.UnloadAction;
 import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.client.gui.panel.ChoiceItem;
-import net.sf.freecol.client.gui.panel.IndianSettlementPanel;
 import net.sf.freecol.client.gui.panel.ReportPanel;
 import net.sf.freecol.client.gui.panel.TilePanel;
 import net.sf.freecol.common.model.Colony;
+import net.sf.freecol.common.model.CombatModel.CombatOdds;
 import net.sf.freecol.common.model.Game;
 import net.sf.freecol.common.model.Goods;
+import net.sf.freecol.common.model.GoodsType;
 import net.sf.freecol.common.model.IndianSettlement;
+import net.sf.freecol.common.model.LostCityRumour.RumourType;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Settlement;
+import net.sf.freecol.common.model.Specification;
+import net.sf.freecol.common.model.StringTemplate;
+import net.sf.freecol.common.model.Tension;
 import net.sf.freecol.common.model.Tile;
+import net.sf.freecol.common.model.TradeRoute;
 import net.sf.freecol.common.model.Unit;
-import net.sf.freecol.common.model.CombatModel.CombatOdds;
-import net.sf.freecol.common.model.LostCityRumour.RumourType;
 import net.sf.freecol.common.model.Unit.MoveType;
+import net.sf.freecol.common.model.Unit.UnitState;
+import net.sf.freecol.common.model.UnitType;
+import net.sf.freecol.common.networking.Message;
 import net.sf.freecol.server.ai.AIColony;
 import net.sf.freecol.server.ai.AIGoods;
 import net.sf.freecol.server.ai.AIUnit;
 import net.sf.freecol.server.ai.TileImprovementPlan;
 import net.sf.freecol.server.ai.Wish;
 import net.sf.freecol.server.ai.mission.TransportMission;
+import net.sf.freecol.server.model.ServerUnit;
 
 
 /**
- * Allows the user to obtain more info about a certain tile
- * or to activate a specific unit on the tile.
+ * Allows the user to obtain more info about a certain tile or to
+ * activate a specific unit on the tile, or perform various debug mode
+ * actions.
  */
 public final class TilePopup extends JPopupMenu {
 
@@ -89,33 +98,38 @@ public final class TilePopup extends JPopupMenu {
      * @param canvas The component containing the map.
      * @param gui An object with methods used for making the popup.
      */
-    public TilePopup(final Tile tile, final FreeColClient freeColClient, final Canvas canvas, final GUI gui) {
-        super(Messages.message("tile",
-                               "%x%", String.valueOf(tile.getX()),
-                               "%y%", String.valueOf(tile.getY())));
+    public TilePopup(final Tile tile, final FreeColClient freeColClient,
+                     final Canvas canvas, final GUI gui) {
+        super(Messages.message(StringTemplate.template("tile")
+                               .addAmount("%x%", tile.getX())
+                               .addAmount("%y%", tile.getY())));
 
         this.canvas = canvas;
         this.freeColClient = freeColClient;
         this.gui = gui;
 
+        final Player player = freeColClient.getMyPlayer();
         final Unit activeUnit = gui.getActiveUnit();
         if (activeUnit != null) {
+            Tile unitTile = activeUnit.getTile();
             JMenuItem gotoMenuItem = null;
             if (activeUnit.isOffensiveUnit() &&
-                activeUnit.getTile().isAdjacent(tile) && 
+                unitTile.isAdjacent(tile) &&
                 activeUnit.getMoveType(tile) == MoveType.ATTACK) {
                 CombatOdds combatOdds = activeUnit.getGame().getCombatModel()
                     .calculateCombatOdds(activeUnit, tile.getDefendingUnit(activeUnit));
-                    
+
                 String victoryPercent;
-                //If attacking a settlement, the true odds are never known because units may be hidden within
+                // If attacking a settlement, the true odds are never known because units
+                // may be hidden within
                 if (tile.getSettlement() != null || combatOdds.win == CombatOdds.UNKNOWN_ODDS) {
                     victoryPercent = "??";
                 } else {
                     victoryPercent = Integer.toString((int)(combatOdds.win * 100));
                 }
-                gotoMenuItem = new JMenuItem(Messages.message("attackTileOdds", "%chance%", victoryPercent));
-            } else if (activeUnit.getSimpleMoveType(null, tile, false).isLegal()) {
+                gotoMenuItem = new JMenuItem(Messages.message(StringTemplate.
+                         template("attackTileOdds").addName("%chance%", victoryPercent)));
+            } else if (activeUnit.getSimpleMoveType(unitTile, tile, false).isLegal()) {
                 //final Image gotoImage = (Image) UIManager.get("cursor.go.image");
                 //JMenuItem gotoMenuItem = new JMenuItem(Messages.message("gotoThisTile"), new ImageIcon(gotoImage));
                 gotoMenuItem = new JMenuItem(Messages.message("gotoThisTile"));
@@ -123,7 +137,7 @@ public final class TilePopup extends JPopupMenu {
             if (gotoMenuItem != null) {
                 gotoMenuItem.addActionListener(new ActionListener() {
                         public void actionPerformed(ActionEvent event) {
-                            if (freeColClient.getGame().getCurrentPlayer() != freeColClient.getMyPlayer()) {
+                            if (freeColClient.getGame().getCurrentPlayer() != player) {
                             	return;
                             }
                             Tile currTile = activeUnit.getTile();
@@ -134,7 +148,7 @@ public final class TilePopup extends JPopupMenu {
 
                             freeColClient.getInGameController().setDestination(activeUnit, tile);
                             freeColClient.getInGameController().moveToDestination(activeUnit);
-                            
+
                             //if unit did not move, we should show the goto path
                             if(activeUnit.getTile() == currTile){
                             	gui.updateGotoPathForActiveUnit();
@@ -143,13 +157,32 @@ public final class TilePopup extends JPopupMenu {
                     });
                 add(gotoMenuItem);
                 hasAnItem = true;
-                addSeparator();
             }
+
+            // Add move to Europe entry if the unit can do so
+            if (unitTile == tile && activeUnit.isNaval()
+                && activeUnit.canMoveToEurope()) {
+                JMenuItem europeMenuItem
+                    = new JMenuItem(Messages.message(StringTemplate.template("gotoEurope")));
+                europeMenuItem.addActionListener(new ActionListener() {
+                        public void actionPerformed(ActionEvent event) {
+                            if (freeColClient.getGame().getCurrentPlayer()
+                                != player) {
+                                return;
+                            }
+                            freeColClient.getInGameController()
+                                .moveToEurope(activeUnit);
+                        }
+                    });
+                add(europeMenuItem);
+                hasAnItem = true;
+            }
+            if (hasAnItem) addSeparator();
         }
 
         Settlement settlement = tile.getSettlement();
         if (settlement != null) {
-            if (settlement.getOwner() == freeColClient.getMyPlayer()) {
+            if (settlement.getOwner() == player) {
                 addColony(((Colony) settlement));
             } else if (settlement instanceof IndianSettlement) {
                 addIndianSettlement((IndianSettlement) settlement);
@@ -192,7 +225,7 @@ public final class TilePopup extends JPopupMenu {
             activateAllItem.addActionListener(new ActionListener() {
                     public void actionPerformed(ActionEvent event) {
                         Unit lastUnit = null;
-                        for (Unit unit: tile.getUnitList()) {
+                        for (Unit unit: new ArrayList<Unit>(tile.getUnitList())) {
                             freeColClient.getInGameController().clearOrders(unit);
                             lastUnit = unit;
                         }
@@ -205,19 +238,21 @@ public final class TilePopup extends JPopupMenu {
         // START DEBUG
         if (FreeCol.isInDebugMode()
             && freeColClient.getFreeColServer() != null) {
+            final Game serverGame = freeColClient.getFreeColServer().getGame();
+            final Player serverPlayer = (Player)
+                serverGame.getFreeColGameObject(player.getId());
+            boolean notEmpty = false;
             addSeparator();
             JMenu takeOwnership = new JMenu("Take ownership");
             takeOwnership.setOpaque(false);
+
             JMenu transportLists = new JMenu("Transport lists");
             transportLists.setOpaque(false);
-            boolean notEmpty = false;
             for (final Unit currentUnit : tile.getUnitList()) {
                 JMenuItem toMenuItem = new JMenuItem(currentUnit.toString());
                 toMenuItem.addActionListener(new ActionListener() {
                         public void actionPerformed(ActionEvent event) {
                             // Server:
-                            final Game serverGame = freeColClient.getFreeColServer().getGame();
-                            final Player serverPlayer = (Player) serverGame.getFreeColGameObject(freeColClient.getMyPlayer().getId());
                             final Unit serverUnit = (Unit) serverGame.getFreeColGameObject(currentUnit.getId());
                             serverUnit.setOwner(serverPlayer);
                             for (Unit serverChildUnit : currentUnit.getUnitList()) {
@@ -229,7 +264,8 @@ public final class TilePopup extends JPopupMenu {
                 takeOwnership.add(toMenuItem);
                 notEmpty = true;
                 if (currentUnit.isCarrier()) {
-                    final AIUnit au = (AIUnit) freeColClient.getFreeColServer().getAIMain().getAIObject(currentUnit);
+                    final AIUnit au = freeColClient.getFreeColServer()
+                        .getAIMain().getAIUnit(currentUnit);
                     if (au.getMission() != null && au.getMission() instanceof TransportMission) {
                         JMenuItem menuItem = new JMenuItem(currentUnit.toString());
                         menuItem.addActionListener(new ActionListener() {
@@ -252,22 +288,23 @@ public final class TilePopup extends JPopupMenu {
                 toMenuItem.addActionListener(new ActionListener() {
                     public void actionPerformed(ActionEvent event) {
                         // Server:
-                        final Game serverGame = freeColClient.getFreeColServer().getGame();
-                        final Player serverPlayer = (Player) serverGame.getFreeColGameObject(freeColClient.getMyPlayer().getId());
-                        final Tile serverTile = (Tile) serverGame.getFreeColGameObject(tile.getId());
-                        serverTile.getSettlement().setOwner(serverPlayer);
+                        final Tile serverTile = (Tile)
+                            serverGame.getFreeColGameObject(tile.getId());
+                        serverTile.getSettlement().changeOwner(serverPlayer);
                         freeColClient.getConnectController().reconnect();
                     }
                 });
                 takeOwnership.add(toMenuItem);
+                notEmpty = true;
+
                 JMenuItem displayColonyPlan = new JMenuItem("Display Colony Plan");
                 displayColonyPlan.addActionListener(new ActionListener() {
                     public void actionPerformed(ActionEvent event) {
                         // Server:
-                        final Game serverGame = freeColClient.getFreeColServer().getGame();
-                        final Player serverPlayer = (Player) serverGame.getFreeColGameObject(freeColClient.getMyPlayer().getId());
-                        final Tile serverTile = (Tile) serverGame.getFreeColGameObject(tile.getId());
-                        final AIColony ac = (AIColony) freeColClient.getFreeColServer().getAIMain().getAIObject(serverTile.getSettlement());
+                        final Tile serverTile = (Tile)
+                            serverGame.getFreeColGameObject(tile.getId());
+                        final AIColony ac = freeColClient.getFreeColServer()
+                            .getAIMain().getAIColony(serverTile.getColony());
                         StringBuilder info = new StringBuilder(ac.getColonyPlan().toString());
                         info.append("\n\nTILE IMPROVEMENTS:\n");
                         Iterator<TileImprovementPlan> tipIt = ac.getTileImprovementPlanIterator();
@@ -291,31 +328,57 @@ public final class TilePopup extends JPopupMenu {
                     }
                 });
                 add(displayColonyPlan);
-                notEmpty = true;
+            }
+            if (tile.getIndianSettlement() != null) {
+                JMenuItem displayGoods = new JMenuItem("Examine Settlement");
+                final IndianSettlement is = tile.getIndianSettlement();
+                displayGoods.addActionListener(new ActionListener() {
+                        public void actionPerformed(ActionEvent event) {
+                            final IndianSettlement sis = (IndianSettlement)
+                                serverGame.getFreeColGameObject(is.getId());
+                            canvas.showInformationMessage(
+                                debugSummarizeSettlement(serverGame, sis));
+                        }
+                    });
+                add(displayGoods);
             }
             if (notEmpty) {
                 add(takeOwnership);
                 hasAnItem = true;
             }
-        }
-        if (FreeCol.isInDebugMode() && tile.hasLostCityRumour()) {
-            JMenuItem rumourItem = new JMenuItem("Set Lost City Rumour type");
-            rumourItem.setOpaque(false);
-            rumourItem.addActionListener(new ActionListener() {
-                    public void actionPerformed(ActionEvent event) {
-                        List<ChoiceItem<RumourType>> rumours = new ArrayList<ChoiceItem<RumourType>>();
-                        for (RumourType rumour : RumourType.values()) {
-                            if (rumour == RumourType.NO_SUCH_RUMOUR) continue;
-                            rumours.add(new ChoiceItem<RumourType>(rumour.toString(), rumour));
+
+            if (tile.hasLostCityRumour()) {
+                JMenuItem rumourItem = new JMenuItem("Set Lost City Rumour type");
+                rumourItem.setOpaque(false);
+                rumourItem.addActionListener(new ActionListener() {
+                        public void actionPerformed(ActionEvent event) {
+                            debugSetRumourType(serverGame, tile);
                         }
-                        RumourType rumourChoice = freeColClient.getCanvas()
-                            .showChoiceDialog("Select Lost City Rumour", "Cancel", rumours);
-                        tile.getTileItemContainer().getLostCityRumour().setType(rumourChoice);
-                        final Tile serverTile = (Tile) freeColClient.getFreeColServer().getGame().getFreeColGameObject(tile.getId());
-                        serverTile.getTileItemContainer().getLostCityRumour().setType(rumourChoice);
+                    });
+                add(rumourItem);
+            }
+
+            JMenuItem addu = new JMenuItem("Add unit");
+            addu.setOpaque(false);
+            addu.addActionListener(new ActionListener() {
+                    public void actionPerformed(ActionEvent event) {
+                        debugAddNewUnitToTile(serverGame, tile);
                     }
                 });
-            add(rumourItem);
+            add(addu);
+
+            JMenuItem dumpItem = new JMenuItem("Dump tile");
+            dumpItem.setOpaque(false);
+            dumpItem.addActionListener(new ActionListener() {
+                    public void actionPerformed(ActionEvent event) {
+                        System.err.println("\nClient side:\n");
+                        tile.dumpObject();
+                        System.err.println("\nServer side:\n");
+                        serverGame.getFreeColGameObject(tile.getId())
+                            .dumpObject();
+                    }
+                });
+            add(dumpItem);
         }
         // END DEBUG
         Component lastComponent = getComponent(getComponentCount() - 1);
@@ -334,13 +397,42 @@ public final class TilePopup extends JPopupMenu {
      * @return an <code>int</code> value
      */
     private int addUnit(Container menu, final Unit unit, boolean enabled, boolean indent) {
-        String occ = unit.getDetailedOccupationIndicator();
-        if (occ.length() > 0) occ = " (" + occ + ")";
+        StringTemplate occ;
+        TradeRoute tradeRoute = unit.getTradeRoute();
+
+        if (unit.getState() == Unit.UnitState.ACTIVE
+            && unit.getMovesLeft() == 0) {
+            if (unit.isUnderRepair()) {
+                occ = StringTemplate.label(": ")
+                    .add("model.unit.occupation.underRepair")
+                    .add(Integer.toString(unit.getTurnsForRepair()));
+            } else if (tradeRoute != null) {
+                occ = StringTemplate.label(": ")
+                    .add("model.unit.occupation.inTradeRoute")
+                    .addName(tradeRoute.getName());
+            } else {
+                occ = StringTemplate.key("model.unit.occupation.activeNoMovesLeft");
+            }
+        } else if (unit.getState() == Unit.UnitState.IMPROVING
+                   && unit.getWorkImprovement() != null) {
+            occ = StringTemplate.label(": ")
+                .add(unit.getWorkImprovement().getType() + ".occupationString")
+                .add(Integer.toString(unit.getWorkTurnsLeft()));
+        } else if (tradeRoute != null) {
+            occ = StringTemplate.label(": ")
+                .add("model.unit.occupation.inTradeRoute")
+                .add(tradeRoute.getName());
+        } else if (unit.getDestination() != null) {
+            occ = StringTemplate.key("model.unit.occupation.goingSomewhere");
+        } else {
+            occ = StringTemplate.key("model.unit.occupation." + unit.getState().toString().toLowerCase());
+        }
+
         String text = (indent ? "    " : "")
-            + Messages.message("model.unit.nationUnit",
-                               "%nation%", Messages.getNationAsString(unit.getOwner()),
-                               "%unit%", Messages.getLabel(unit))
-            + occ;
+            + Messages.message(StringTemplate.template("model.unit.nationUnit")
+                               .addStringTemplate("%nation%", unit.getOwner().getNationName())
+                               .addStringTemplate("%unit%", Messages.getLabel(unit)))
+            + " (" + Messages.message(occ) + ")";
         JMenuItem menuItem = new JMenuItem(text);
         menuItem.addActionListener(new ActionListener() {
                 public void actionPerformed(ActionEvent event) {
@@ -360,7 +452,8 @@ public final class TilePopup extends JPopupMenu {
 
         boolean hasGoods = false;
         for (Goods goods: unit.getGoodsList()) {
-            text = (indent ? "         " : "     ") + Messages.getLabel(goods);
+            text = (indent ? "         " : "     ")
+                + Messages.message(goods.getLabel(true));
             menuItem = new JMenuItem(text);
             menuItem.setFont(menuItem.getFont().deriveFont(Font.ITALIC));
             menuItem.setEnabled(false);
@@ -371,15 +464,10 @@ public final class TilePopup extends JPopupMenu {
 
         if (hasGoods) {
             JMenuItem dumpItem = new JMenuItem(Messages.message("dumpCargo"));
-            dumpItem.setAction(new UnloadAction(freeColClient));
+            dumpItem.setAction(new UnloadAction(freeColClient, unit));
             menu.add(dumpItem);
             lineCount++;
         }
-        if (menu instanceof JMenu) {
-            ((JMenu) menu).addSeparator();
-        } else {
-            ((JPopupMenu) menu).addSeparator();
-        }
         hasAnItem = true;
         return lineCount;
     }
@@ -389,7 +477,8 @@ public final class TilePopup extends JPopupMenu {
      * @param colony The colony that will be represented on the popup.
      */
     private void addColony(final Colony colony) {
-        JMenuItem menuItem = new JMenuItem(colony.toString());
+        String name = colony.getNameFor(freeColClient.getMyPlayer());
+        JMenuItem menuItem = new JMenuItem(Messages.message(name));
         menuItem.addActionListener(new ActionListener() {
                 public void actionPerformed(ActionEvent event) {
                     canvas.showColonyPanel(colony);
@@ -413,13 +502,16 @@ public final class TilePopup extends JPopupMenu {
 
     /**
      * Adds an indian settlement entry to this popup.
-     * @param settlement The Indian settlement that will be represented on the popup.
+     *
+     * @param settlement The Indian settlement that will be
+     *     represented on the popup.
      */
     private void addIndianSettlement(final IndianSettlement settlement) {
-        JMenuItem menuItem = new JMenuItem(settlement.getName());
+        String name = settlement.getNameFor(freeColClient.getMyPlayer());
+        JMenuItem menuItem = new JMenuItem(Messages.message(name));
         menuItem.addActionListener(new ActionListener() {
                 public void actionPerformed(ActionEvent event) {
-                    canvas.showPanel(new IndianSettlementPanel(canvas, settlement));
+                    canvas.showIndianSettlementPanel(settlement);
                 }
             });
         add(menuItem);
@@ -428,10 +520,11 @@ public final class TilePopup extends JPopupMenu {
 
     /**
      * Adds a tile entry to this popup.
+     *
      * @param tile The tile that will be represented on the popup.
      */
     private void addTile(final Tile tile) {
-        JMenuItem menuItem = new JMenuItem(Messages.getName(tile));
+        JMenuItem menuItem = new JMenuItem(Messages.message(tile.getNameKey()));
         menuItem.addActionListener(new ActionListener() {
                 public void actionPerformed(ActionEvent event) {
                     canvas.showPanel(new TilePanel(canvas, tile));
@@ -447,12 +540,144 @@ public final class TilePopup extends JPopupMenu {
     }
 
     /**
-     * Returns true if this popup has at least one menuitem so that we know that we can
-     * show it to the user. Returns false if there are no menuitems.
-     * @return true if this popup has at least one menuitem, false otherwise.
+     * Returns true if this popup has at least one menuitem so that we
+     * know that we can show it to the user. Returns false if there
+     * are no menuitems.
+     *
+     * @return True if this popup has at least one menuitem, false otherwise.
      */
     public boolean hasItem() {
-        return hasAnItem;
+        return hasAnItem || FreeCol.isInDebugMode();
     }
 
+    /**
+     * Debug action to set the lost city rumour type on a tile.
+     *
+     * @param serverGame The server <code>Game</code> containing the tile.
+     * @param tile The <code>Tile</code> to operate on.
+     */
+    private void debugSetRumourType(final Game serverGame, Tile tile) {
+        List<ChoiceItem<RumourType>> rumours
+            = new ArrayList<ChoiceItem<RumourType>>();
+        for (RumourType rumour : RumourType.values()) {
+            if (rumour == RumourType.NO_SUCH_RUMOUR) continue;
+            rumours.add(new ChoiceItem<RumourType>(rumour.toString(), rumour));
+        }
+
+        RumourType rumourChoice = freeColClient.getCanvas()
+            .showChoiceDialog(null, "Select Lost City Rumour", "Cancel",
+                              rumours);
+        tile.getTileItemContainer().getLostCityRumour().setType(rumourChoice);
+        final Tile serverTile = (Tile) serverGame
+            .getFreeColGameObject(tile.getId());
+        serverTile.getTileItemContainer().getLostCityRumour()
+            .setType(rumourChoice);
+    }
+
+    /**
+     * Debug action to add a new unit to a tile.
+     *
+     * @param serverGame The server <code>Game</code> containing the tile.
+     * @param tile The <code>Tile</code> to add to.
+     */
+    private void debugAddNewUnitToTile(final Game serverGame, Tile tile) {
+        Specification spec = serverGame.getSpecification();
+        List<ChoiceItem<UnitType>> uts = new ArrayList<ChoiceItem<UnitType>>();
+        for (UnitType t : spec.getUnitTypeList()) {
+            uts.add(new ChoiceItem<UnitType>(Messages.message(t.toString()
+                                                              + ".name"), t));
+        }
+        UnitType unitChoice = freeColClient.getCanvas()
+            .showChoiceDialog(null, "Select Unit Type", "Cancel", uts);
+        if (unitChoice == null) return;
+
+        Player player = freeColClient.getMyPlayer();
+        Player serverPlayer = (Player) serverGame
+            .getFreeColGameObject(player.getId());
+        Tile serverTile = (Tile) serverGame
+            .getFreeColGameObject(tile.getId());
+        ServerUnit serverUnit
+            = new ServerUnit(serverGame, serverTile, serverPlayer,
+                             unitChoice, UnitState.ACTIVE,
+                             unitChoice.getDefaultEquipment());
+        serverUnit.setMovesLeft(serverUnit.getInitialMovesLeft());
+        Unit unit = new Unit(freeColClient.getGame(),
+                serverUnit.toXMLElement(Message.createNewDocument()));
+        tile.add(unit);
+        gui.setActiveUnit(unit);
+        player.invalidateCanSeeTiles();
+        canvas.refresh();
+    }
+
+    /**
+     * Debug action to summarize information about a native settlement
+     * that is normally hidden.
+     *
+     * @param serverGame The server <code>Game</code> containing the
+     *     settlement.
+     * @param sis The server version of the <code>IndianSettlement</code>
+     *     to summarize.
+     * @return A string summarizing the settlement.
+     */
+    private String debugSummarizeSettlement(final Game serverGame,
+                                            final IndianSettlement sis) {
+        Specification spec = serverGame.getSpecification();
+        StringBuilder sb = new StringBuilder(sis.getName() + "\n");
+
+        sb.append("\nAlarm\n");
+        for (Player p : serverGame.getLiveEuropeanPlayers()) {
+            Tension tension = sis.getAlarm(p);
+            sb.append(Messages.message(p.getNationName())
+                      + " " + ((tension == null) ? "(none)"
+                               : Integer.toString(tension.getValue()))
+                      + " " + Messages.message(sis.getShortAlarmLevelMessageId(p))
+                      + " " + ((sis.hasSpokenToChief(p)) ? "(spoke to chief)"
+                               : "")
+                      + "\n");
+        }
+
+        sb.append("\nGoods Present\n");
+        for (Goods goods : sis.getCompactGoods()) {
+            sb.append(Messages.message(goods.getLabel(true)) + "\n");
+        }
+
+        sb.append("\nGoods Production\n");
+        for (GoodsType type : spec.getGoodsTypeList()) {
+            int prod = sis.getProductionOf(type);
+            if (prod > 0) {
+                sb.append(Messages.message(type.getNameKey())
+                          + " " + prod + "\n");
+            }
+        }
+
+        sb.append("\nPrices (buy 1/100 / sell 1/100)\n");
+        GoodsType[] wanted = sis.getWantedGoods();
+        for (GoodsType type : spec.getGoodsTypeList()) {
+            if (!type.isStorable()) continue;
+            int i;
+            for (i = wanted.length - 1; i >= 0; i--) {
+                if (type == wanted[i]) break;
+            }
+            sb.append(Messages.message(type.getNameKey())
+                      + ": " + sis.getPriceToBuy(type, 1)
+                      + "/" + sis.getPriceToBuy(type, 100)
+                      + " / " + sis.getPriceToSell(type, 1)
+                      + "/" + sis.getPriceToSell(type, 100)
+                      + ((i < 0) ? "" : " wanted[" + Integer.toString(i) + "]")
+                      + "\n");
+        }
+
+        sb.append("\nOwned Units\n");
+        for (Unit u : sis.getOwnedUnits()) {
+            sb.append(u + "\n");
+            sb.append("  at " + u.getTile() + "\n");
+        }
+
+        sb.append("\nTiles\n");
+        for (Tile t : sis.getOwnedTiles()) {
+            sb.append(t + "\n");
+        }
+
+        return sb.toString();
+    }
 }
