@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -19,13 +19,8 @@
 
 package net.sf.freecol.client.gui.panel;
 
-import java.awt.AlphaComposite;
-import java.awt.Color;
 import java.awt.Component;
-import java.awt.Composite;
 import java.awt.Dimension;
-import java.awt.Graphics;
-import java.awt.Graphics2D;
 import java.awt.datatransfer.DataFlavor;
 import java.awt.datatransfer.Transferable;
 import java.awt.datatransfer.UnsupportedFlavorException;
@@ -35,6 +30,7 @@ import java.awt.event.ActionListener;
 import java.awt.event.ItemListener;
 import java.awt.event.MouseAdapter;
 import java.awt.event.MouseEvent;
+import java.beans.PropertyChangeEvent;
 
 import java.util.ArrayList;
 import java.util.HashMap;
@@ -62,12 +58,13 @@ import javax.swing.ListModel;
 import javax.swing.ListSelectionModel;
 import javax.swing.SwingConstants;
 import javax.swing.TransferHandler;
+import javax.swing.plaf.PanelUI;
 
 import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.client.gui.Canvas;
 import net.sf.freecol.client.gui.plaf.FreeColComboBoxRenderer;
+import net.sf.freecol.client.gui.plaf.FreeColSelectedPanelUI;
 
-import net.sf.freecol.FreeCol;
 import net.sf.freecol.common.model.Ability;
 import net.sf.freecol.common.model.AbstractGoods;
 import net.sf.freecol.common.model.BuildableType;
@@ -76,6 +73,8 @@ import net.sf.freecol.common.model.BuildingType;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.FeatureContainer;
 import net.sf.freecol.common.model.FreeColGameObjectType;
+import net.sf.freecol.common.model.Limit;
+import net.sf.freecol.common.model.StringTemplate;
 import net.sf.freecol.common.model.UnitType;
 import net.sf.freecol.common.resources.ResourceManager;
 import net.sf.freecol.common.util.Utils;
@@ -89,6 +88,8 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
 
     private static final String BUY = "buy";
 
+    private static final int UNABLE_TO_BUILD = -1;
+
     private final BuildQueueTransferHandler buildQueueHandler = new BuildQueueTransferHandler();
 
     private ListCellRenderer cellRenderer;
@@ -97,11 +98,12 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
     private JList buildQueueList;
     private JList unitList;
     private JList buildingList;
+    private ConstructionPanel constructionPanel;
     private JButton buyBuilding;
     private Colony colony;
     private int unitCount;
 
-    private FeatureContainer featureContainer = new FeatureContainer();
+    private FeatureContainer featureContainer;
 
     private Map<BuildableType, String> lockReasons = new HashMap<BuildableType, String>();
     private Set<BuildableType> unbuildableTypes = new HashSet<BuildableType>();
@@ -110,22 +112,22 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
      * A list of unit types that can be build. Most unit types are
      * human and can never be built.
      */
-    private static final List<UnitType> buildableUnits = new ArrayList<UnitType>();
+    private List<UnitType> buildableUnits = new ArrayList<UnitType>();
 
-    static {
-        for (UnitType unitType : FreeCol.getSpecification().getUnitTypeList()) {
-            if (!unitType.getGoodsRequired().isEmpty()) {
+    public BuildQueuePanel(Colony colony, Canvas parent) {
+
+        super(parent, new MigLayout("wrap 3", "[260:][390:, fill][260:]", "[][][300:400:][]"));
+        this.colony = colony;
+        this.unitCount = colony.getUnitCount();
+        featureContainer = new FeatureContainer(getSpecification());
+
+        for (UnitType unitType : getSpecification().getUnitTypeList()) {
+            if (!(unitType.getGoodsRequired().isEmpty()
+                  || unitType.hasAbility("model.ability.bornInColony"))) {
                 // can be built
                 buildableUnits.add(unitType);
             }
         }
-    }
-
-    public BuildQueuePanel(Colony colony, Canvas parent) {
-
-        super(parent, new MigLayout("wrap 3", "[260:][260:][260:]", "[][][300:400:][]"));
-        this.colony = colony;
-        this.unitCount = colony.getUnitCount();
 
         DefaultListModel current = new DefaultListModel();
         for (BuildableType type : colony.getBuildQueue()) {
@@ -135,15 +137,23 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
 
         cellRenderer = getCellRenderer();
 
+        // remove previous listeners
+        for (ItemListener listener : compact.getItemListeners()) {
+            compact.removeItemListener(listener);
+        }
         compact.setText(Messages.message("colonyPanel.compactView"));
         compact.addItemListener(this);
 
+        // remove previous listeners
+        for (ItemListener listener : showAll.getItemListeners()) {
+            showAll.removeItemListener(listener);
+        }
         showAll.setText(Messages.message("colonyPanel.showAll"));
         showAll.addItemListener(this);
 
         buildQueueList = new JList(current);
         buildQueueList.setTransferHandler(buildQueueHandler);
-        buildQueueList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
+        buildQueueList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
         buildQueueList.setDragEnabled(true);
         buildQueueList.setCellRenderer(cellRenderer);
         buildQueueList.addMouseListener(new BuildQueueMouseAdapter(false));
@@ -201,13 +211,20 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
         buyBuilding.setActionCommand(BUY);
         buyBuilding.addActionListener(this);
 
+        constructionPanel = new ConstructionPanel(getCanvas(), colony, false);
+        constructionPanel.setOpaque(false);
+        StringTemplate buildingNothing = StringTemplate.template("colonyPanel.currentlyBuilding")
+            .add("%buildable%", "nothing");
+        constructionPanel.setDefaultLabel(buildingNothing);
+
         updateAllLists();
 
         add(headLine, "span 3, align center, wrap 40");
-        add(new JLabel(Messages.message("menuBar.colopedia.unit")), "align center");
+        add(new JLabel(Messages.message("colonyPanel.units")), "align center");
         add(new JLabel(Messages.message("colonyPanel.buildQueue")), "align center");
-        add(new JLabel(Messages.message("menuBar.colopedia.building")), "align center");
+        add(new JLabel(Messages.message("colonyPanel.buildings")), "align center");
         add(new JScrollPane(unitList), "grow");
+        add(constructionPanel, "split 2, flowy");
         add(new JScrollPane(buildQueueList), "grow");
         add(new JScrollPane(buildingList), "grow, wrap 20");
         add(buyBuilding, "span, split 4");
@@ -227,21 +244,29 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
             }
 
             if (unitType.getPopulationRequired() > unitCount) {
-                lockReason.add(Messages.message("colonyPanel.populationTooSmall", "%number%",
-                                                Integer.toString(unitType.getPopulationRequired())));
+                lockReason.add(Messages.message(StringTemplate.template("colonyPanel.populationTooSmall")
+                                                .addAmount("%number%", unitType.getPopulationRequired())));
+            }
+
+            if (unitType.getLimits() != null) {
+                for (Limit limit : unitType.getLimits()) {
+                    if (!limit.evaluate(colony)) {
+                        lockReason.add(Messages.message(limit.getDescriptionKey()));
+                    }
+                }
             }
 
             if (!(colony.getFeatureContainer()
                   .hasAbility("model.ability.build", unitType, getGame().getTurn())
                   || featureContainer.hasAbility("model.ability.build", unitType))) {
                 boolean builderFound = false;
-                for (Ability ability : FreeCol.getSpecification().getAbilities("model.ability.build")) {
+                for (Ability ability : getSpecification().getAbilities("model.ability.build")) {
                     if (ability.appliesTo(unitType)
                         && ability.getValue()
                         && ability.getSource() != null
                         && !unbuildableTypes.contains(ability.getSource())) {
                         builderFound = true;
-                        lockReason.add(Messages.getName(ability.getSource()));
+                        lockReason.add(Messages.message(ability.getSource().getNameKey()));
                         break;
                     }
                 }
@@ -255,22 +280,22 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
             for (Entry<String, Boolean> entry : requiredAbilities.entrySet()) {
                 if (colony.hasAbility(entry.getKey()) != entry.getValue()
                     && featureContainer.hasAbility(entry.getKey()) != entry.getValue()) {
-                    List<FreeColGameObjectType> sources = FreeCol.getSpecification()
+                    List<FreeColGameObjectType> sources = getSpecification()
                         .getTypesProviding(entry.getKey(), entry.getValue());
                     if (sources.isEmpty()) {
                         // no type provides the required ability
                         unbuildableTypes.add(unitType);
                         continue loop;
                     } else {
-                        lockReason.add(Messages.getName(sources.get(0)));
+                        lockReason.add(Messages.message(sources.get(0).getNameKey()));
                     }
                 }
             }
             if (lockReason.isEmpty()) {
                 lockReasons.put(unitType, null);
             } else {
-                lockReasons.put(unitType, Messages.message("colonyPanel.requires", "%string%",
-                                                           Utils.join("/", lockReason)));
+                lockReasons.put(unitType, Messages.message(StringTemplate.template("colonyPanel.requires")
+                                                           .addName("%string%", Utils.join("/", lockReason))));
             }
             if (lockReason.isEmpty() || showAll.isSelected()) {
                 units.addElement(unitType);
@@ -282,7 +307,7 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
         DefaultListModel buildings = (DefaultListModel) buildingList.getModel();
         DefaultListModel current = (DefaultListModel) buildQueueList.getModel();
         buildings.clear();
-        loop: for (BuildingType buildingType : FreeCol.getSpecification().getBuildingTypeList()) {
+        loop: for (BuildingType buildingType : getSpecification().getBuildingTypeList()) {
             // compare colony.getNoBuildReason()
             List<String> lockReason = new ArrayList<String>();
             Building colonyBuilding = colony.getBuilding(buildingType);
@@ -301,22 +326,30 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
             }
 
             if (buildingType.getPopulationRequired() > unitCount) {
-                lockReason.add(Messages.message("colonyPanel.populationTooSmall", "%number%",
-                                                Integer.toString(buildingType.getPopulationRequired())));
+                lockReason.add(Messages.message(StringTemplate.template("colonyPanel.populationTooSmall")
+                                                .addAmount("%number%", buildingType.getPopulationRequired())));
             }
 
             Map<String, Boolean> requiredAbilities = buildingType.getAbilitiesRequired();
             for (Entry<String, Boolean> entry : requiredAbilities.entrySet()) {
                 if (colony.hasAbility(entry.getKey()) != entry.getValue()
                     && featureContainer.hasAbility(entry.getKey()) != entry.getValue()) {
-                    List<FreeColGameObjectType> sources = FreeCol.getSpecification()
+                    List<FreeColGameObjectType> sources = getSpecification()
                         .getTypesProviding(entry.getKey(), entry.getValue());
                     if (sources.isEmpty()) {
                         // no type provides the required ability
                         unbuildableTypes.add(buildingType);
                         continue loop;
                     } else {
-                        lockReason.add(Messages.getName(sources.get(0)));
+                        lockReason.add(Messages.message(sources.get(0).getNameKey()));
+                    }
+                }
+            }
+
+            if (buildingType.getLimits() != null) {
+                for (Limit limit : buildingType.getLimits()) {
+                    if (!limit.evaluate(colony)) {
+                        lockReason.add(Messages.message(limit.getDescriptionKey()));
                     }
                 }
             }
@@ -325,14 +358,14 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
                 && !current.contains(buildingType.getUpgradesFrom())) {
                 if (colonyBuilding == null
                     || colonyBuilding.getType() != buildingType.getUpgradesFrom()) {
-                    lockReason.add(Messages.getName(buildingType.getUpgradesFrom()));
+                    lockReason.add(Messages.message(buildingType.getUpgradesFrom().getNameKey()));
                 }
             }
             if (lockReason.isEmpty()) {
                 lockReasons.put(buildingType, null);
             } else {
-                lockReasons.put(buildingType, Messages.message("colonyPanel.requires", "%string%",
-                                                               Utils.join("/", lockReason)));
+                lockReasons.put(buildingType, Messages.message(StringTemplate.template("colonyPanel.requires")
+                                                               .addName("%string%", Utils.join("/", lockReason))));
             }
             if (lockReason.isEmpty() || showAll.isSelected()) {
                 buildings.addElement(buildingType);
@@ -342,7 +375,7 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
 
     private void updateAllLists() {
         DefaultListModel current = (DefaultListModel) buildQueueList.getModel();
-        featureContainer = new FeatureContainer();
+        featureContainer = new FeatureContainer(getSpecification());
         for (Object type: current.toArray()) {
             if (getMinimumIndex((BuildableType) type) >= 0) {
                 featureContainer.add(((BuildableType) type).getFeatureContainer());
@@ -356,6 +389,10 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
         updateBuildingList();
         updateUnitList();
         updateBuyBuildingButton();
+        // work-around to re-initialize construction panel
+        PropertyChangeEvent event = new PropertyChangeEvent(colony, ConstructionPanel.EVENT, null,
+                                                            getBuildableTypes(buildQueueList));
+        constructionPanel.propertyChange(event);
     }
 
     private void updateBuyBuildingButton() {
@@ -436,24 +473,103 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
                 }
             }
         }
-        return -1;
+        return UNABLE_TO_BUILD;
     }
 
+    private int getMaximumIndex(BuildableType buildableType) {
+        ListModel buildQueue = buildQueueList.getModel();
+        final int buildQueueLastPos = buildQueue.getSize();
+
+        boolean canBuild = false;
+        if (colony.canBuild(buildableType)) {
+            canBuild = true;
+        }
+
+        if (buildableType instanceof UnitType) {
+            // does not depend on anything, nothing depends on it
+            // can be built at any time
+            if(canBuild){
+                return buildQueueLastPos;
+            }
+            // check for building in queue that allows builting this unit
+            for (int index = 0; index < buildQueue.getSize(); index++) {
+                BuildableType toBuild = (BuildableType) buildQueue.getElementAt(index);
+
+                if(toBuild == buildableType){
+                    continue;
+                }
+
+                if (toBuild.hasAbility("model.ability.build", buildableType)) {
+                    return buildQueueLastPos;
+                }
+            }
+
+            return UNABLE_TO_BUILD;
+        }
+
+        if (buildableType instanceof BuildingType) {
+            BuildingType upgradesFrom = ((BuildingType) buildableType).getUpgradesFrom();
+            BuildingType upgradesTo = ((BuildingType) buildableType).getUpgradesTo();
+
+            // does not depend on nothing, but still cannot be built
+            if (!canBuild && upgradesFrom == null) {
+                return UNABLE_TO_BUILD;
+            }
+
+            // if can be built and does not have any upgrade,
+            //then it can be built at any time
+            if(canBuild && upgradesTo == null){
+                return buildQueueLastPos;
+            }
+
+            // if can be built, does not depend on anything, mark upgradesfrom as found
+            boolean foundUpgradesFrom = canBuild? true:false;
+            for (int index = 0; index < buildQueue.getSize(); index++) {
+                BuildableType toBuild = (BuildableType) buildQueue.getElementAt(index);
+
+                if(toBuild == buildableType){
+                    continue;
+                }
+
+                if (!canBuild && !foundUpgradesFrom && upgradesFrom.equals(toBuild)) {
+                    foundUpgradesFrom = true;
+                    // nothing else to upgrade this building to
+                    if(upgradesTo == null){
+                        return buildQueueLastPos;
+                    }
+                }
+                // found a building it upgrades to, cannot go to or beyond this position
+                if (foundUpgradesFrom && upgradesTo != null && upgradesTo.equals(toBuild)) {
+                    return index;
+                }
+
+                // Don't go past a unit this building can build.
+                if (buildableType.hasAbility("model.ability.build", toBuild)) {
+                    return index;
+                }
+            }
+
+            return buildQueueLastPos;
+        }
+
+        return UNABLE_TO_BUILD;
+    }
 
     /**
      * This function analyses an event and calls the right methods to take
      * care of the user's requests.
      * @param event The incoming ActionEvent.
      */
+    @Override
     public void actionPerformed(ActionEvent event) {
         if (colony.getOwner() == getMyPlayer()) {
             String command = event.getActionCommand();
             List<BuildableType> buildables = getBuildableTypes(buildQueueList);
             if (!buildables.isEmpty() && lockReasons.get(buildables.get(0)) != null) {
-                getCanvas().showInformationMessage("colonyPanel.unbuildable",
-                                                   buildables.get(0),
-                                                   "%colony%", colony.getName(),
-                                                   "%object%", Messages.getName(buildables.get(0)));
+                getCanvas().showInformationMessage(buildables.get(0),
+                                                   StringTemplate.template("colonyPanel.unbuildable")
+                                                   .addName("%colony%", colony.getName())
+                                                   .add("%object%", buildables.get(0).getNameKey()));
                 return;
             }
             getController().setBuildQueue(colony, buildables);
@@ -461,7 +577,6 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
                 // do nothing?
             } else if (BUY.equals(command)) {
                 getController().payForBuilding(colony);
-                getCanvas().updateGoldLabel();
             } else {
                 logger.warning("Unsupported command " + command);
             }
@@ -492,7 +607,7 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
         } else if (cellRenderer == null || cellRenderer instanceof SimpleBuildQueueCellRenderer) {
             return new DefaultBuildQueueCellRenderer();
         }
-                
+
         // return current one
         return cellRenderer;
     }
@@ -508,7 +623,6 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
 
         JList source = null;
         int[] indices = null;
-        int targetIndex = -1;  // preferred index of target list
         int numberOfItems = 0;  // number of items to be added
 
         /**
@@ -518,8 +632,8 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
          * @param data The build queue to import.
          * @return Whether the import was successful.
          */
+        @Override
         public boolean importData(JComponent comp, Transferable data) {
-
             if (!canImport(comp, data.getTransferDataFlavors())) {
                 return false;
             }
@@ -532,61 +646,138 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
                 target = (JList) comp;
                 targetModel = (DefaultListModel) target.getModel();
                 Object transferData = data.getTransferData(buildQueueFlavor);
-                if (transferData instanceof List) {
-                    for (Object object : (List) transferData) {
-                        if (object instanceof BuildableType) {
-                            if ((object instanceof BuildingType
-                                 && target == unitList)
-                                || (object instanceof UnitType
-                                    && target == buildingList)) {
-                                return false;
-                            }
-                            buildQueue.add((BuildableType) object);
-                        }
+
+                if (!(transferData instanceof List<?>)) {
+                    return false;
+                }
+
+                for (Object object : (List<?>) transferData) {
+                    // first we need to make sure all items are compatible
+                    //with the drop zone
+                    if (!(object instanceof BuildableType)) {
+                        return false;
                     }
+
+                    // check if trying to drop units in the Building list
+                    //or buildings in the Unit List
+                    if ((object instanceof BuildingType && target == unitList)
+                        || (object instanceof UnitType && target == buildingList)) {
+                        return false;
+                    }
+
+                    // Object check complete
+                    // Add object to the to-add list
+                    buildQueue.add((BuildableType) object);
                 }
             } catch (Exception e) {
                 logger.warning(e.toString());
                 return false;
             }
 
+            // check if every Buildable can be added to the queue
             for (BuildableType type : buildQueue) {
                 if (getMinimumIndex(type) < 0) {
                     return false;
                 }
             }
 
+            // This is where the dragged target was dropped.
             int preferredIndex = target.getSelectedIndex();
 
+            boolean isOrderingQueue = false;
+            int prevPos = -1;
             if (source.equals(target)) {
-                if (target == buildQueueList) {
-                    // don't drop selection on itself
-                    if (indices != null &&
-                        preferredIndex >= indices[0] - 1 &&
-                        preferredIndex <= indices[indices.length - 1]) {
-                        indices = null;
-                        return true;
-                    }
-                    numberOfItems = buildQueue.size();
-                } else {
+                // only the build queue allows ordering
+                if (target != buildQueueList && target.getParent() != buildQueueList) {
                     return false;
                 }
+
+                // find previous position
+                for(int i=0; i < targetModel.getSize(); i++){
+                    if(targetModel.getElementAt(i) == buildQueue.get(0)){
+                        prevPos = i;
+                        break;
+                    }
+                }
+
+                // don't drop selection on itself
+                if(preferredIndex != -1 && prevPos == preferredIndex){
+                    indices = null;
+                    return false;
+                }
+
+                int maximumIndex = getMaximumIndex(buildQueue.get(0));
+                if(preferredIndex > maximumIndex){
+                    indices = null;
+                    return false;
+                }
+
+                isOrderingQueue = true;
+                numberOfItems = buildQueue.size();
+            }
+            else if (source == buildQueueList) {
+                // Dragging out of build queue - just remove the element.
+                // updateAllLists takes care of the rest.
+                DefaultListModel sourceModel = (DefaultListModel) source.getModel();
+
+                for(Object obj : buildQueue) {
+                    sourceModel.removeElement(obj);
+                }
+
+                return true;
             }
 
             int maxIndex = targetModel.size();
+            // Set index to add to the last position, if not set or out-of-bounds
             if (preferredIndex < 0 || preferredIndex > maxIndex) {
                 preferredIndex = maxIndex;
             }
-            targetIndex = preferredIndex;
 
-            for (int index = 0; index < buildQueue.size(); index++) {
-                int minimumIndex = getMinimumIndex(buildQueue.get(index));
-                if (minimumIndex < targetIndex + index) {
-                    minimumIndex = targetIndex + index;
+            if(isOrderingQueue){
+                // Remove all elements in the build queue from the target.
+                for(Object obj : buildQueue){
+                    targetModel.removeElement(obj);
                 }
-                targetModel.insertElementAt(buildQueue.get(index), minimumIndex);
             }
 
+            for (int index = 0; index < buildQueue.size(); index++) {
+                BuildableType toBuild = (BuildableType) buildQueue.get(index);
+
+                int minimumIndex = getMinimumIndex(toBuild);
+
+                // minimumIndex == targetModel.size() means it has to go at the
+                // end.
+                if (minimumIndex < targetModel.size())
+                    {
+                        int maximumIndex = getMaximumIndex(toBuild);
+
+                        // minimumIndex == maximumIndex means there is only one
+                        // place it can go.
+                        if (minimumIndex != maximumIndex)
+                            {
+                                if (minimumIndex < preferredIndex + index) {
+                                    minimumIndex = preferredIndex + index;
+                                }
+
+                                if (minimumIndex > targetModel.size())
+                                    {
+                                        minimumIndex = targetModel.size();
+                                    }
+
+                                if (minimumIndex > maximumIndex)
+                                    {
+                                        minimumIndex = maximumIndex;
+                                    }
+                            }
+                    }
+
+                targetModel.add(minimumIndex,buildQueue.get(index));
+            }
+
+            // update selected index to new position
+            if(isOrderingQueue){
+                buildQueueList.setSelectedIndex(preferredIndex);
+            }
             return true;
         }
 
@@ -596,27 +787,10 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
          * @param data The data exported.
          * @param action The transfer action, e.g. MOVE.
          */
+        @Override
         protected void exportDone(JComponent source, Transferable data, int action) {
-
-            if ((action == MOVE) && (indices != null)) {
-                DefaultListModel model = (DefaultListModel) ((JList) source).getModel();
-
-                // adjust indices if necessary
-                if (numberOfItems > 0) {
-                    for (int i = 0; i < indices.length; i++) {
-                        if (indices[i] > targetIndex) {
-                            indices[i] += numberOfItems;
-                        }
-                    }
-                }
-                // has to be done backwards
-                for (int i = indices.length -1; i >= 0; i--) {
-                    model.remove(indices[i]);
-                }
-            }
             // clean up
             indices = null;
-            targetIndex = -1;
             numberOfItems = 0;
             updateAllLists();
         }
@@ -627,6 +801,7 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
          * @param comp The component to import data.
          * @param flavors An array of data flavors.
          */
+        @Override
         public boolean canImport(JComponent comp, DataFlavor[] flavors) {
             if (flavors == null) {
                 return false;
@@ -647,6 +822,7 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
          * @return A Transferable suitable for wrapping the build
          * queue.
          */
+        @Override
         protected Transferable createTransferable(JComponent comp) {
             if (comp instanceof JList) {
                 source = (JList) comp;
@@ -663,6 +839,7 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
          * @param comp The source component.
          * @return The possible source actions of the component.
          */
+        @Override
         public int getSourceActions(JComponent comp) {
             if (comp == unitList) {
                 return COPY;
@@ -734,29 +911,11 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
         }
     }
 
-    /**
-     * See FreeColComboBoxRenderer.
-     */
-    class SelectedPanel extends JPanel {
-
-        public void paintComponent(Graphics g) {
-            Graphics2D g2d = (Graphics2D) g;
-            Composite oldComposite = g2d.getComposite();
-            Color oldColor = g2d.getColor();
-            g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.1f));
-            g2d.setColor(Color.BLACK);
-            g2d.fillRect(0, 0, getWidth(), getHeight());
-            g2d.setComposite(oldComposite);
-            g2d.setColor(oldColor);
-
-            super.paintComponent(g);
-        }
-    }
-
     class SimpleBuildQueueCellRenderer extends FreeColComboBoxRenderer {
 
+        @Override
         public void setLabelValues(JLabel c, Object value) {
-            c.setText(Messages.getName(((BuildableType) value)));
+            c.setText(Messages.message(((BuildableType) value).getNameKey()));
         }
 
     }
@@ -764,7 +923,7 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
     class DefaultBuildQueueCellRenderer implements ListCellRenderer {
 
         JPanel itemPanel = new JPanel();
-        JPanel selectedPanel = new SelectedPanel();
+        JPanel selectedPanel = new JPanel();
         JLabel imageLabel = new JLabel(new ImageIcon());
         JLabel nameLabel = new JLabel();
 
@@ -778,6 +937,7 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
             itemPanel.setLayout(new MigLayout());
             selectedPanel.setOpaque(false);
             selectedPanel.setLayout(new MigLayout());
+            selectedPanel.setUI((PanelUI) FreeColSelectedPanelUI.createUI(selectedPanel));
         }
 
         public Component getListCellRendererComponent(JList list,
@@ -792,7 +952,7 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
             ((ImageIcon) imageLabel.getIcon()).setImage(ResourceManager.getImage(item.getId() + ".image",
                                                                                  buildingDimension));
 
-            nameLabel.setText(Messages.getName(item));
+            nameLabel.setText(Messages.message(item.getNameKey()));
             panel.setToolTipText(lockReasons.get(item));
             panel.add(imageLabel, "span 1 2");
             if (lockReasons.get(item) == null) {
@@ -822,36 +982,45 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
     class BuildQueueMouseAdapter extends MouseAdapter {
 
         private boolean add = true;
+        
+        private boolean enabled = false;
 
         public BuildQueueMouseAdapter(boolean add) {
             this.add = add;
         }
 
+        @Override
         public void mousePressed(MouseEvent e) {
-            JList source = (JList) e.getSource();
-            if ((e.getButton() == MouseEvent.BUTTON3 || e.isPopupTrigger())) {
-                int index = source.locationToIndex(e.getPoint());
-                BuildableType type = (BuildableType) source.getModel().getElementAt(index);
-                if (type instanceof BuildingType) {
-                    getCanvas().showPanel(new ColopediaPanel(getCanvas(),
-                                                             ColopediaPanel.PanelType.BUILDINGS, type));
-                } else if (type instanceof UnitType) {
-                    getCanvas().showPanel(new ColopediaPanel(getCanvas(),
-                                                             ColopediaPanel.PanelType.UNITS, type));
-                }
-            } else if ((e.getClickCount() > 1 && !e.isConsumed())) {
-                DefaultListModel model = (DefaultListModel) buildQueueList.getModel();
-                if (source.getSelectedIndex() == -1) {
-                    source.setSelectedIndex(source.locationToIndex(e.getPoint()));
-                }
-                for (Object type : source.getSelectedValues()) {
-                    if (add) {
-                        model.addElement(type);
-                    } else {
-                        model.removeElement(type);
+            if (!enabled && e.getClickCount() == 1 && !e.isConsumed()) {
+                enabled = true;
+            }
+
+            if (enabled) {
+                JList source = (JList) e.getSource();
+                if ((e.getButton() == MouseEvent.BUTTON3 || e.isPopupTrigger())) {
+                    int index = source.locationToIndex(e.getPoint());
+                    BuildableType type = (BuildableType) source.getModel().getElementAt(index);
+                    if (type instanceof BuildingType) {
+                        getCanvas().showPanel(new ColopediaPanel(getCanvas(),
+                                                                 ColopediaPanel.PanelType.BUILDINGS, type));
+                    } else if (type instanceof UnitType) {
+                        getCanvas().showPanel(new ColopediaPanel(getCanvas(),
+                                                                 ColopediaPanel.PanelType.UNITS, type));
                     }
+                } else if ((e.getClickCount() > 1 && !e.isConsumed())) {
+                        DefaultListModel model = (DefaultListModel) buildQueueList.getModel();
+                        if (source.getSelectedIndex() == -1) {
+                            source.setSelectedIndex(source.locationToIndex(e.getPoint()));
+                        }
+                        for (Object type : source.getSelectedValues()) {
+                            if (add) {
+                                model.addElement(type);
+                            } else {
+                                model.removeElement(type);
+                            }
+                        }
+                        updateAllLists();
                 }
-                updateAllLists();
             }
         }
     }
