@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -24,7 +24,7 @@ import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.common.model.Building;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.GoodsType;
-import net.sf.freecol.common.model.Player;
+import net.sf.freecol.common.model.StringTemplate;
 import net.sf.freecol.common.model.Unit;
 import net.sf.freecol.common.model.UnitType;
 
@@ -51,8 +51,6 @@ import net.miginfocom.swing.MigLayout;
  */
 public final class CompactLabourReport extends ReportPanel {
 
-    private static final int HEADER_ROWS = 1;
-
     private int COLONY_COLUMN = 0;
     private int UNIT_TYPE_COLUMN = 1;
     private int WORKING_COLUMN = 2;
@@ -76,7 +74,12 @@ public final class CompactLabourReport extends ReportPanel {
 
     private boolean showBuildings;
 
-    private final JPanel headerRow = new JPanel();
+    private final JPanel headerRow = new JPanel() {
+        @Override
+        public String getUIClassID() {
+            return "ReportPanelUI";
+        }
+    };
 
     /**
      * The constructor that will add the items to this panel.
@@ -96,10 +99,12 @@ public final class CompactLabourReport extends ReportPanel {
      * @param parent The parent of this panel.
      */
     private CompactLabourReport(Canvas parent, LabourData.UnitData data) {
-        super(parent, data == null ? Messages.message("report.labour") : Messages.message("report.labour.details"));
+        super(parent, data == null ? Messages.message("reportLabourAction.name")
+              : Messages.message("report.labour.details"));
         this.unitData = data;
 
         headerRow.setBorder(new EmptyBorder(20, 20, 0, 20));
+        headerRow.setOpaque(true);
         scrollPane.setColumnHeaderView(headerRow);
     }
 
@@ -186,15 +191,11 @@ public final class CompactLabourReport extends ReportPanel {
 
         row = addLocationData(labourData.getSummary().getTotal(), null, row);
 
-        int minHeight = allColonistsButton.getPreferredSize().height;
-        //HIGLayout higLayout = (HIGLayout) reportPanel.getLayout();
-        
         for (UnitType unitType : LabourData.getLabourTypes(getMyPlayer())) {
             LabourData.UnitData unitData = labourData.getUnitData(unitType);
 
             JButton unitButton = createUnitNameButton(unitData.getUnitName(), unitData);
             int rows = unitData.getUnitSummaryRowCount();
-            //ensureHeight(higLayout, unitButton.getPreferredSize().height, row, rows, minHeight);
             reportPanel.add(unitButton, "cell " + COLONY_COLUMN + " " + row + " " + 1 + " " + rows);
 
             if (unitData.hasDetails()) {
@@ -211,31 +212,14 @@ public final class CompactLabourReport extends ReportPanel {
         }
     }
 
-    /**
-     * ensures that {@code rows} rows, starting from {@code startRow}, are at least {@code height} heigh
-     *
-     * @param higLayout
-     * @param height    the height we need
-     * @param startRow  from this row on
-     * @param rows      number of rows where the height has to be met
-     * @param minHeight minimum height for each row
-     */
     /*
-    private void ensureHeight(HIGLayout higLayout, int height, int startRow, int rows, int minHeight) {
-        for (int delta : distribute(height, rows)) {
-            higLayout.setRowHeight(startRow, Math.max(delta, minHeight));
-            startRow++;
-        }
-    }
-    */
-
-    /**
      * distributes {@code value}, amount the number of {@code pocketCount}
      *
      * @param value
      * @param pocketCount
      * @return distribution of {@code value}
      */
+    /*
     private int[] distribute(int value, int pocketCount) {
         int[] pockets = new int[pocketCount];
 
@@ -248,6 +232,7 @@ public final class CompactLabourReport extends ReportPanel {
         }
         return pockets;
     }
+     */
 
     private JLabel createEmptyLabel() {
         JLabel empty = new JLabel("");
@@ -280,18 +265,19 @@ public final class CompactLabourReport extends ReportPanel {
         }
         LabourData.LocationData europe = unitData.getUnitsInEurope();
         if (europe.getRowCount() > 0) {
-            JButton button = createButton(getMyPlayer().getEurope().getName(), new ActionListener() {
-                public void actionPerformed(ActionEvent e) {
-                    getCanvas().showEuropePanel();
-                }
-            });
+            JButton button = createButton(Messages.message(getMyPlayer().getEurope().getNameKey()),
+                                          new ActionListener() {
+                                              public void actionPerformed(ActionEvent e) {
+                                                  getCanvas().showEuropePanel();
+                                              }
+                                          });
             reportPanel.add(button, "cell " + COLONY_COLUMN + " " + row + " " + 1 + " " + europe.getRowCount());
             row = addLocationData(europe, null, row);
         }
         row = addNonLinkedLocation(unitData.getUnitsOnLand(), "report.onLand", row);
         row = addNonLinkedLocation(unitData.getUnitsAtSea(), "report.atSea", row);
 
-        reportPanel.add(new JLabel(Messages.message("report.labour.canTrain")), 
+        reportPanel.add(new JLabel(Messages.message("report.labour.canTrain")),
                         "cell " + 1 + " " + row + " " + COLUMNS + " " + 1);
     }
 
@@ -387,7 +373,7 @@ public final class CompactLabourReport extends ReportPanel {
         Building productionBuilding = null;
 
         if (!allColonists) {
-            workingAs = Messages.message(unitType.getId() + ".workingAs");
+            workingAs = Messages.message(unitType.getWorkingAsKey());
             if (colony != null) {
                 GoodsType expert = unitType.getExpertProduction();
                 if (expert != null) {
@@ -422,7 +408,7 @@ public final class CompactLabourReport extends ReportPanel {
         int notProducingStartRow = row;
 
         if (showBuildings && productionBuilding != null && row > buildingStartRow) {
-            JLabel buildingLabel = new JLabel(Messages.getName(productionBuilding));
+            JLabel buildingLabel = localizedLabel(productionBuilding.getNameKey());
             buildingLabel.setBorder(CELLBORDER);
             reportPanel.add(buildingLabel, "cell " + buildingStartRow + " " + BUILDING_COLUMN
                             + " 1 " + (row - buildingStartRow));
@@ -462,9 +448,8 @@ public final class CompactLabourReport extends ReportPanel {
         if (!allColonists) {
             row = addRow(data,
                          data.getOtherStudentsName(),
-                         Messages.message("report.labour.learning",
-                                          "%unit%",
-                                          data.getUnitData().getUnitName()),
+                         Messages.message(StringTemplate.template("report.labour.learning")
+                                          .addName("%unit%", data.getUnitData().getUnitName())),
                          data.getOtherStudents(),
                          0, row);
         }
@@ -485,9 +470,8 @@ public final class CompactLabourReport extends ReportPanel {
                 }
 
                 String student = resultOfTraining.size() == 1 ?
-                    Messages.message("report.labour.learning",
-                                     "%unit%",
-                                     Messages.getName(resultOfTraining.iterator().next())) :
+                    Messages.message(StringTemplate.template("report.labour.learning")
+                                     .addName("%unit%", resultOfTraining.iterator().next())) :
                     Messages.message("report.labour.learningOther");
                 addRow(data,
                        data.getUnitData().getUnitName(),
@@ -499,7 +483,8 @@ public final class CompactLabourReport extends ReportPanel {
         }
 
         if (showBuildings && row > buildingStartRow) {
-            JLabel buildingLabel = new JLabel(schoolhouse != null ? Messages.getName(schoolhouse) : "");
+            JLabel buildingLabel = new JLabel(schoolhouse != null ?
+                                              Messages.message(schoolhouse.getNameKey()) : "");
             buildingLabel.setBorder(CELLBORDER);
             reportPanel.add(buildingLabel, "cell " + buildingStartRow + " " + BUILDING_COLUMN
                             + " 1 " + (row - buildingStartRow));
@@ -523,7 +508,8 @@ public final class CompactLabourReport extends ReportPanel {
                                 + " " + 1 + " " + rows);
             }
         } else {
-            colonistsLabel.setToolTipText(Messages.message("report.labour.unitTotal.tooltip", "%unit%", data.getUnitData().getUnitName()));
+            colonistsLabel.setToolTipText(Messages.message(StringTemplate.template("report.labour.unitTotal.tooltip")
+                                                           .addName("%unit%", data.getUnitData().getUnitName())));
         }
 
         reportPanel.add(colonistsLabel, "cell " + COLONIST_SUMMARY_COLUMN + " " + row
@@ -616,7 +602,7 @@ public final class CompactLabourReport extends ReportPanel {
         }
     }
 
-    private JButton createColonyButton(final Colony colony) {
+    public JButton createColonyButton(final Colony colony) {
         String text = colony.getName();
         if (!unitData.isSummary()) {
 //            int unitIndex = unitData.getUnitType().getIndex();
