@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -28,6 +28,7 @@ import java.awt.event.MouseEvent;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.logging.Logger;
 
 import javax.swing.ImageIcon;
 import javax.swing.JComponent;
@@ -40,7 +41,6 @@ import net.sf.freecol.client.gui.Canvas;
 import net.sf.freecol.client.gui.ImageLibrary;
 import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.client.gui.panel.UnitLabel.UnitAction;
-import net.sf.freecol.common.Specification;
 import net.sf.freecol.common.model.AbstractGoods;
 import net.sf.freecol.common.model.Building;
 import net.sf.freecol.common.model.Colony;
@@ -50,6 +50,7 @@ import net.sf.freecol.common.model.Europe;
 import net.sf.freecol.common.model.GameOptions;
 import net.sf.freecol.common.model.Goods;
 import net.sf.freecol.common.model.GoodsType;
+import net.sf.freecol.common.model.StringTemplate;
 import net.sf.freecol.common.model.Unit;
 import net.sf.freecol.common.model.Unit.UnitState;
 import net.sf.freecol.common.model.UnitType;
@@ -63,12 +64,14 @@ import net.sf.freecol.common.model.WorkLocation;
  */
 public final class DragListener extends MouseAdapter {
 
+    private static final Logger logger = Logger.getLogger(DragListener.class.getName());
+
     private final FreeColPanel parentPanel;
     private final Canvas canvas;
 
     /**
      * The constructor to use.
-     * 
+     *
      * @param parentPanel The layered pane that contains the components to which
      *            a DragListener might be attached.
      */
@@ -80,7 +83,7 @@ public final class DragListener extends MouseAdapter {
     /**
      * Gets called when the mouse was pressed on a Swing component that has this
      * object as a MouseListener.
-     * 
+     *
      * @param e The event that holds the information about the mouse click.
      */
     public void mousePressed(MouseEvent e) {
@@ -96,9 +99,13 @@ public final class DragListener extends MouseAdapter {
                     menu = getUnitMenu((UnitLabel) comp);
                 } else if (comp instanceof GoodsLabel) {
                     menu = getGoodsMenu((GoodsLabel) comp);
-                } else if (comp instanceof MarketLabel &&
-                           parentPanel instanceof EuropePanel) {
-                    ((EuropePanel) parentPanel).payArrears(((MarketLabel) comp).getType());
+                } else if (comp instanceof MarketLabel
+                           && parentPanel instanceof EuropePanel) {
+                    GoodsType goodsType = ((MarketLabel) comp).getType();
+                    if (canvas.getClient().getInGameController().payArrears(goodsType)) {
+                        ((EuropePanel) parentPanel).revalidate();
+                        ((EuropePanel) parentPanel).refresh();
+                    }
                 }
                 if (menu != null) {
                     int elements = menu.getSubElements().length;
@@ -141,7 +148,7 @@ public final class DragListener extends MouseAdapter {
                     ((MarketLabel) comp).setAmount(100);
                 }
             }
-        
+
             if ((comp instanceof UnitLabel) && (((UnitLabel) comp).getUnit().isCarrier())) {
                 Unit u = ((UnitLabel) comp).getUnit();
                 if (parentPanel instanceof EuropePanel) {
@@ -168,11 +175,11 @@ public final class DragListener extends MouseAdapter {
         ImageLibrary imageLibrary = parentPanel.getLibrary();
         final Unit tempUnit = unitLabel.getUnit();
         JPopupMenu menu = new JPopupMenu("Unit");
-        ImageIcon unitIcon = imageLibrary.getUnitImageIcon(tempUnit);
+        ImageIcon unitIcon = imageLibrary.getUnitImageIcon(tempUnit, 0.66);
 
-        JMenuItem name = new JMenuItem(Messages.getLabel(tempUnit) + " (" +
-                                       Messages.message("menuBar.colopedia") + ")", 
-                                       imageLibrary.getScaledImageIcon(unitIcon, 0.66f));
+        JMenuItem name = new JMenuItem(Messages.message(tempUnit.getLabel()) + " (" +
+                                       Messages.message("menuBar.colopedia") + ")",
+                                       unitIcon);
         name.setActionCommand(UnitAction.COLOPEDIA.toString());
         name.addActionListener(unitLabel);
         menu.add(name);
@@ -182,9 +189,9 @@ public final class DragListener extends MouseAdapter {
             if (addCarrierItems(unitLabel, menu)) {
                 menu.addSeparator();
             }
-        }                
+        }
 
-        if (tempUnit.getLocation().getTile() != null && 
+        if (tempUnit.getLocation().getTile() != null &&
             tempUnit.getLocation().getTile().getColony() != null) {
             if (addWorkItems(unitLabel, menu)) {
                 menu.addSeparator();
@@ -227,12 +234,12 @@ public final class DragListener extends MouseAdapter {
             menu.add(cargo);
 
             for (Unit passenger : tempUnit.getUnitList()) {
-                JMenuItem menuItem = new JMenuItem("    " + Messages.getLabel(passenger));
+                JMenuItem menuItem = new JMenuItem("    " + Messages.message(passenger.getLabel()));
                 menuItem.setFont(menuItem.getFont().deriveFont(Font.ITALIC));
                 menu.add(menuItem);
             }
             for (Goods goods : tempUnit.getGoodsList()) {
-                JMenuItem menuItem = new JMenuItem("    " + Messages.getLabel(goods));
+                JMenuItem menuItem = new JMenuItem("    " + Messages.message(goods.getLabel(true)));
                 menuItem.setFont(menuItem.getFont().deriveFont(Font.ITALIC));
                 menu.add(menuItem);
             }
@@ -249,16 +256,15 @@ public final class DragListener extends MouseAdapter {
         Colony colony = tempUnit.getLocation().getColony();
         boolean separatorNeeded = false;
 
-        List<GoodsType> farmedGoods = Specification.getSpecification().getFarmedGoodsTypeList();
+        List<GoodsType> farmedGoods = canvas.getSpecification().getFarmedGoodsTypeList();
         // Work in Field - automatically find the best location
         for (GoodsType goodsType : farmedGoods) {
             ColonyTile bestTile = colony.getVacantColonyTileFor(tempUnit, false, goodsType);
             if (bestTile != null) {
                 int maxpotential = bestTile.getProductionOf(tempUnit, goodsType);
-                UnitType expert = Specification.getSpecification().getExpertForProducing(goodsType);
-                JMenuItem menuItem = new JMenuItem(Messages.message(goodsType.getId() + ".workAs",
-                                                                    "%amount%",
-                                                                    Integer.toString(maxpotential)),
+                String text = Messages.message(StringTemplate.template(goodsType.getId() + ".workAs")
+                                               .addAmount("%amount%", maxpotential));
+                JMenuItem menuItem = new JMenuItem(text,
                                                    imageLibrary.getScaledGoodsImageIcon(goodsType, 0.66f));
                 menuItem.setActionCommand(UnitAction.WORK_TILE.toString() + ":" + goodsType.getId());
                 menuItem.addActionListener(unitLabel);
@@ -266,13 +272,13 @@ public final class DragListener extends MouseAdapter {
                 separatorNeeded = true;
             }
         }
-                    
+
         // Work at Building - show both max potential and realistic projection
         for (Building building : colony.getBuildings()) {
             if (tempUnit.getWorkLocation() != building) { // Skip if currently working at this location
                 if (building.canAdd(tempUnit)) {
                     GoodsType goodsType = building.getGoodsOutputType();
-                    String locName = Messages.getName(building);
+                    String locName = Messages.message(building.getNameKey());
                     JMenuItem menuItem = new JMenuItem(locName);
                     if (goodsType != null) {
                         menuItem.setIcon(imageLibrary.getScaledGoodsImageIcon(goodsType, 0.66f));
@@ -283,10 +289,10 @@ public final class DragListener extends MouseAdapter {
                             // Not reaching full potential, show full potential
                             locName += "/" + potential;
                         }
-                        locName +=  " " + Messages.getName(goodsType)+")";
+                        locName +=  " " + Messages.message(goodsType.getNameKey()) +")";
                         menuItem.setText(locName);
                         if (addOutput == 0) {
-                            menuItem.setForeground(FreeColPanel.LINK_COLOR);
+                            menuItem.setForeground(FreeColPanel.WARNING_COLOR);
                         }
                     }
                     menuItem.setActionCommand(UnitAction.WORK_BUILDING.toString() + ":" +
@@ -320,40 +326,61 @@ public final class DragListener extends MouseAdapter {
 
         return separatorNeeded;
     }
-    
+
     private boolean addEducationItems(final UnitLabel unitLabel, final JPopupMenu menu) {
-        Unit tempUnit = unitLabel.getUnit();
-        ImageLibrary imageLibrary = parentPanel.getLibrary();
         boolean separatorNeeded = false;
-        if (!tempUnit.getGame().getGameOptions().getBoolean(GameOptions.EDUCATE_LEAST_SKILLED_UNIT_FIRST)) {
-            for (Unit teacher : tempUnit.getColony().getTeachers()) {
-                if (tempUnit.canBeStudent(teacher) &&
-                    tempUnit.getLocation() instanceof WorkLocation &&
-                    teacher.getStudent() != tempUnit) {
-                    JMenuItem menuItem = new JMenuItem(Messages.message("assignToTeacher"),
-                                                       imageLibrary.getScaledImageIcon(imageLibrary.getUnitImageIcon(teacher), 0.5f));
-                    menuItem.setActionCommand(UnitAction.ASSIGN.toString() + ":" + teacher.getId());
-                    menuItem.addActionListener(unitLabel);
+        Unit unit = unitLabel.getUnit();
+        ImageLibrary imageLibrary = parentPanel.getLibrary();
+
+        if (unit.getSpecification().getBoolean(GameOptions.ALLOW_STUDENT_SELECTION)) {
+            for (Unit teacher : unit.getColony().getTeachers()) {
+                if (unit.canBeStudent(teacher) && (unit.getLocation() instanceof WorkLocation)) {
+                    JMenuItem menuItem = null;
+                    ImageIcon teacherIcon = imageLibrary.getUnitImageIcon(teacher, 0.5);
+                    if (teacher.getStudent() != unit) {
+                        menuItem = new JMenuItem(Messages.message("assignToTeacher"), teacherIcon);
+                        menuItem.setActionCommand(UnitAction.ASSIGN.toString() + ":" + teacher.getId());
+                        menuItem.addActionListener(unitLabel);
+                    } else {
+                        String teacherName = Messages.message(teacher.getType().getNameKey());
+                        menuItem = new JMenuItem(Messages.message(StringTemplate.template("menu.unit.apprentice")
+                                                                  .addName("%unit%", teacherName)),
+                                                 teacherIcon);
+                        menuItem.setEnabled(false);
+                    }
                     menu.add(menuItem);
                     separatorNeeded = true;
                 }
             }
         }
-        if (tempUnit.getTurnsOfTraining() > 0 && tempUnit.getStudent() != null) {
-            JMenuItem teaching = new JMenuItem(Messages.message("menuBar.teacher") +
-                                               ": " + tempUnit.getTurnsOfTraining() +
-                                               "/" + tempUnit.getNeededTurnsOfTraining());
-            teaching.setEnabled(false);
-            menu.add(teaching);
+
+        if ((unit.getTurnsOfTraining() > 0) && (unit.getStudent() != null)) {
+            JMenuItem menuItem = new JMenuItem(Messages.message("menuBar.teacher") +
+                                               ": " + unit.getTurnsOfTraining() +
+                                               "/" + unit.getNeededTurnsOfTraining());
+            menuItem.setEnabled(false);
+            menu.add(menuItem);
             separatorNeeded = true;
         }
-        int experience = Math.min(tempUnit.getExperience(), 200);
-        if (experience > 0 && tempUnit.getWorkType() != null) {
-            UnitType workType = Specification.getSpecification()
-                .getExpertForProducing(tempUnit.getWorkType());
-            if (tempUnit.getType().canBeUpgraded(workType, ChangeType.EXPERIENCE)) {
-                JMenuItem experienceItem = new JMenuItem(Messages.message("menuBar.experience") +
-                                                         ": " + experience + "/5000");
+
+        int experience = unit.getExperience();
+        GoodsType goods = unit.getExperienceType();
+        if (goods == null) {
+            goods = unit.getWorkType();
+        }
+        if (experience > 0 && goods != null) {
+            UnitType expertType = canvas.getSpecification().getExpertForProducing(goods);
+            if (unit.getType().canBeUpgraded(expertType, ChangeType.EXPERIENCE)) {
+                int maxExperience = unit.getType().getMaximumExperience();
+                double probability = unit.getType().getUnitTypeChange(expertType)
+                    .getProbability(ChangeType.EXPERIENCE) * experience / (double) maxExperience;
+                String jobName = Messages.message(goods.getWorkingAsKey());
+                ImageIcon expertIcon = imageLibrary.getUnitImageIcon(expertType, 0.5);
+                JMenuItem experienceItem = new JMenuItem(Messages.message(StringTemplate.template("menu.unit.experience")
+                                                                          .addName("%job%", jobName))
+                                                         + " " + experience + "/" + maxExperience + " ("
+                                                         + FreeColPanel.getModifierFormat().format(probability) + "%)",
+                                                         expertIcon);
                 experienceItem.setEnabled(false);
                 menu.add(experienceItem);
                 separatorNeeded = true;
@@ -365,14 +392,14 @@ public final class DragListener extends MouseAdapter {
 
 
     private boolean addCommandItems(final UnitLabel unitLabel, final JPopupMenu menu) {
-    	final Unit tempUnit = unitLabel.getUnit();
+        final Unit tempUnit = unitLabel.getUnit();
         final boolean isUnitBetweenEuropeAndNewWorld = tempUnit.isBetweenEuropeAndNewWorld();
-    	
-    	JMenuItem menuItem = new JMenuItem(Messages.message("activateUnit"));
+
+        JMenuItem menuItem = new JMenuItem(Messages.message("activateUnit"));
         menuItem.setActionCommand(UnitAction.ACTIVATE_UNIT.toString());
         menuItem.addActionListener(unitLabel);
         menuItem.setEnabled(tempUnit.getState() != UnitState.ACTIVE
-        						&& !isUnitBetweenEuropeAndNewWorld);
+                                && !isUnitBetweenEuropeAndNewWorld);
         menu.add(menuItem);
 
         if (!(tempUnit.getLocation() instanceof Europe)) {
@@ -384,21 +411,21 @@ public final class DragListener extends MouseAdapter {
                                      tempUnit.getState() == UnitState.FORTIFYING));
             menu.add(menuItem);
         }
- 
+
         UnitState unitState = tempUnit.getState();
         menuItem = new JMenuItem(Messages.message("sentryUnit"));
         menuItem.setActionCommand(UnitAction.SENTRY.toString());
         menuItem.addActionListener(unitLabel);
         menuItem.setEnabled(unitState != UnitState.SENTRY
-        						&& !isUnitBetweenEuropeAndNewWorld);
+                                && !isUnitBetweenEuropeAndNewWorld);
         menu.add(menuItem);
-        
+
         boolean hasTradeRoute = tempUnit.getTradeRoute() != null;
         menuItem = new JMenuItem(Messages.message("clearUnitOrders"));
         menuItem.setActionCommand(UnitAction.CLEAR_ORDERS.toString());
         menuItem.addActionListener(unitLabel);
         menuItem.setEnabled((unitState != UnitState.ACTIVE || hasTradeRoute)
-        						&& !isUnitBetweenEuropeAndNewWorld);
+                                && !isUnitBetweenEuropeAndNewWorld);
         menu.add(menuItem);
 
         if (tempUnit.canCarryTreasure() && !tempUnit.getColony().isLandLocked()) {
@@ -417,6 +444,7 @@ public final class DragListener extends MouseAdapter {
 
     private boolean addEquipmentItems(final UnitLabel unitLabel, final JPopupMenu menu) {
         final Unit tempUnit = unitLabel.getUnit();
+        final InGameController igc = canvas.getClient().getInGameController();
         ImageLibrary imageLibrary = parentPanel.getLibrary();
         boolean separatorNeeded = false;
         if (tempUnit.getEquipment().size() > 1) {
@@ -426,8 +454,7 @@ public final class DragListener extends MouseAdapter {
                         Map<EquipmentType, Integer> equipment =
                             new HashMap<EquipmentType, Integer>(tempUnit.getEquipment().getValues());
                         for (Map.Entry<EquipmentType, Integer> entry: equipment.entrySet()) {
-                            canvas.getClient().getInGameController()
-                                .equipUnit(tempUnit, entry.getKey(), -entry.getValue());
+                            igc.equipUnit(tempUnit, entry.getKey(), -entry.getValue());
                         }
                         unitLabel.updateIcon();
                     }
@@ -437,7 +464,7 @@ public final class DragListener extends MouseAdapter {
 
         EquipmentType horses = null;
         EquipmentType muskets = null;
-        for (EquipmentType equipmentType : Specification.getSpecification().getEquipmentTypeList()) {
+        for (EquipmentType equipmentType : canvas.getSpecification().getEquipmentTypeList()) {
             int count = tempUnit.getEquipment().getCount(equipmentType);
             if (count > 0) {
                 // "remove current equipment" action
@@ -447,11 +474,10 @@ public final class DragListener extends MouseAdapter {
                     newItem.setIcon(imageLibrary.getScaledGoodsImageIcon(goodsType, 0.66f));
                 }
                 final int items = count;
-                final EquipmentType type = equipmentType; 
+                final EquipmentType type = equipmentType;
                 newItem.addActionListener(new ActionListener() {
                         public void actionPerformed(ActionEvent e) {
-                            canvas.getClient().getInGameController()
-                                .equipUnit(tempUnit, type, -items);
+                            igc.equipUnit(tempUnit, type, -items);
                             unitLabel.updateIcon();
                         }
                     });
@@ -473,12 +499,12 @@ public final class DragListener extends MouseAdapter {
                                                                              goodsRequired.getAmount());
                         newItem.setIcon(imageLibrary.getScaledGoodsImageIcon(goodsRequired.getType(), 0.66f));
                     }
-                    while (count * price > tempUnit.getOwner().getGold()) {
+                    while (!tempUnit.getOwner().checkGold(count * price)) {
                         count--;
                     }
                     newItem.setText(Messages.message(equipmentType.getId() + ".add") + " (" +
-                                    Messages.message("goldAmount", "%amount%", 
-                                                     String.valueOf(count * price)) +
+                                    Messages.message(StringTemplate.template("goldAmount")
+                                                     .addAmount("%amount%", count * price)) +
                                     ")");
                 } else if (tempUnit.getColony() != null &&
                            tempUnit.getColony().canBuildEquipment(equipmentType)) {
@@ -501,11 +527,10 @@ public final class DragListener extends MouseAdapter {
                         muskets = equipmentType;
                     }
                     final int items = count;
-                    final EquipmentType type = equipmentType; 
+                    final EquipmentType type = equipmentType;
                     newItem.addActionListener(new ActionListener() {
                             public void actionPerformed(ActionEvent e) {
-                                canvas.getClient().getInGameController()
-                                    .equipUnit(tempUnit, type, items);
+                                igc.equipUnit(tempUnit, type, items);
                                 unitLabel.updateIcon();
                             }
                         });
@@ -520,15 +545,13 @@ public final class DragListener extends MouseAdapter {
             JMenuItem newItem = new JMenuItem(Messages.message("model.equipment.dragoon"));
             newItem.addActionListener(new ActionListener() {
                     public void actionPerformed(ActionEvent e) {
-                        canvas.getClient().getInGameController()
-                            .equipUnit(tempUnit, horseType, 1);
-                        canvas.getClient().getInGameController()
-                            .equipUnit(tempUnit, musketType, 1);
+                        igc.equipUnit(tempUnit, horseType, 1);
+                        igc.equipUnit(tempUnit, musketType, 1);
                         unitLabel.updateIcon();
                     }
                 });
             menu.add(newItem);
-        }            
+        }
 
         separatorNeeded = true;
 
@@ -537,7 +560,7 @@ public final class DragListener extends MouseAdapter {
             separatorNeeded = false;
         }
 
-        UnitType newUnitType = tempUnit.getType().getUnitTypeChange(ChangeType.CLEAR_SKILL, tempUnit.getOwner());
+        UnitType newUnitType = tempUnit.getType().getTargetType(ChangeType.CLEAR_SKILL, tempUnit.getOwner());
         if (newUnitType != null) {
             JMenuItem menuItem = new JMenuItem(Messages.message("clearSpeciality"));
             menuItem.setActionCommand(UnitAction.CLEAR_SPECIALITY.toString());
@@ -559,8 +582,8 @@ public final class DragListener extends MouseAdapter {
         final InGameController inGameController = canvas.getClient().getInGameController();
         ImageLibrary imageLibrary = parentPanel.getLibrary();
         JPopupMenu menu = new JPopupMenu("Cargo");
-        JMenuItem name = new JMenuItem(Messages.getName(goods) + " (" +
-                                       Messages.message("menuBar.colopedia") + ")", 
+        JMenuItem name = new JMenuItem(Messages.message(goods.getNameKey()) + " (" +
+                                       Messages.message("menuBar.colopedia") + ")",
                                        imageLibrary.getScaledGoodsImageIcon(goods.getType(), 0.66f));
         name.addActionListener(new ActionListener() {
                 public void actionPerformed(ActionEvent e) {
@@ -569,25 +592,43 @@ public final class DragListener extends MouseAdapter {
                 }
             });
         menu.add(name);
-                
+
         if (!(goods.getLocation() instanceof Colony)) {
-            JMenuItem unload = new JMenuItem(Messages.message("unload"));
-            unload.addActionListener(new ActionListener() {
-                public void actionPerformed(ActionEvent e) {
-                    inGameController.unloadCargo(goods);
-                    if (parentPanel instanceof CargoPanel) {
-                        CargoPanel cargoPanel = (CargoPanel) parentPanel;
-                        cargoPanel.initialize();
-                        /*
-                        if (cargoPanel.getParentPanel() instanceof ColonyPanel) {
-                            ((ColonyPanel) cargoPanel.getParentPanel()).updateWarehouse();
+            if (canvas.getClient().getMyPlayer().canTrade(goods)) {
+                JMenuItem unload = new JMenuItem(Messages.message("unload"));
+                unload.addActionListener(new ActionListener() {
+                        public void actionPerformed(ActionEvent e) {
+                            inGameController.unloadCargo(goods, false);
+                            if (parentPanel instanceof CargoPanel) {
+                                CargoPanel cargoPanel = (CargoPanel) parentPanel;
+                                cargoPanel.initialize();
+                                /*
+                                  if (cargoPanel.getParentPanel() instanceof ColonyPanel) {
+                                  ((ColonyPanel) cargoPanel.getParentPanel()).updateWarehouse();
+                                  }
+                                */
+                            }
+                            parentPanel.revalidate();
                         }
-                         */
-                    }
-                    parentPanel.revalidate();
+                    });
+                menu.add(unload);
+            } else {
+                if (goods.getLocation() instanceof Unit
+                    && ((Unit)goods.getLocation()).isInEurope()) {
+                    JMenuItem pay = new JMenuItem(Messages.message("boycottedGoods.payArrears"));
+                    pay.addActionListener(new ActionListener() {
+                            public void actionPerformed(ActionEvent e) {
+                                inGameController.payArrears(goods);
+                                if (parentPanel instanceof CargoPanel) {
+                                    CargoPanel cargoPanel = (CargoPanel) parentPanel;
+                                    cargoPanel.initialize();
+                                }
+                                parentPanel.revalidate();
+                            }
+                        });
+                    menu.add(pay);
                 }
-            });
-            menu.add(unload);
+            }
             JMenuItem dump = new JMenuItem(Messages.message("dumpCargo"));
             dump.addActionListener(new ActionListener() {
                 public void actionPerformed(ActionEvent e) {
@@ -600,7 +641,7 @@ public final class DragListener extends MouseAdapter {
             });
             menu.add(dump);
         }
-                
+
         return menu;
     }
 }
\ No newline at end of file
