@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -51,7 +51,6 @@ import net.miginfocom.swing.MigLayout;
 import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.gui.Canvas;
 import net.sf.freecol.client.gui.i18n.Messages;
-import net.sf.freecol.server.generator.MapGeneratorOptions;
 
 
 /**
@@ -62,7 +61,7 @@ public class FreeColDialog<T> extends FreeColPanel {
 
     private static final Logger logger = Logger.getLogger(FreeColDialog.class.getName());
 
-    private static final String CANCEL = "CANCEL";
+    protected static final String CANCEL = "CANCEL";
 
     // Stores the response from the user:
     private T response = null;
@@ -74,6 +73,7 @@ public class FreeColDialog<T> extends FreeColPanel {
 
     /**
      * Constructor.
+     * @param parent <code>Canvas</code>
      */
     public FreeColDialog(Canvas parent) {
         super(parent);
@@ -153,25 +153,26 @@ public class FreeColDialog<T> extends FreeColPanel {
     * Creates a new <code>FreeColDialog</code> with a text and a cancel-button,
     * in addition to buttons for each of the objects in the given array.
     *
-    * @param text The text that explains the choice for the user.
-    * @param cancelText The text displayed on the "cancel"-button.
-    * @param choices The ChoiceItems.
-    * @return The <code>FreeColDialog</code>.
+    * @param text String, text that explains the choice for the user
+    * @param cancelText String, text displayed on the "cancel"-button
+    * @param choices List of <code>ChoiceItem<T> <code> the choice items
+    * @return <code>FreeColDialog</code>
     * @see ChoiceItem
     */
-    public static <T> FreeColDialog<ChoiceItem<T>> createChoiceDialog(String text, String cancelText, 
+    public static <T> FreeColDialog<ChoiceItem<T>> createChoiceDialog(String text, String cancelText,
                                                                       List<ChoiceItem<T>> choices) {
 
         if (choices.isEmpty()) {
             throw new IllegalArgumentException("Can not create choice dialog with 0 choices!");
         }
 
-        
+
         final List<JButton> choiceBtnLst = new ArrayList<JButton>();
 
         final FreeColDialog<ChoiceItem<T>> choiceDialog =
             new FreeColDialog<ChoiceItem<T>>(FreeCol.getFreeColClient().getCanvas()) {
-            public void requestFocus() {
+           @Override
+           public void requestFocus() {
             	for(JButton b : choiceBtnLst){
             		if(b.isEnabled()){
             			b.requestFocus();
@@ -190,12 +191,12 @@ public class FreeColDialog<T> extends FreeColPanel {
              if ((choices.size() % 4) == 0 && choices.size() > 12) columns = 4;
         else if ((choices.size() % 3) == 0 && choices.size() > 6)  columns = 3;
         else if ((choices.size() % 2) == 0 && choices.size() > 4)  columns = 2;
-        
+
         else if (choices.size() > 21) columns = 4;
         else if (choices.size() > 10) columns = 2;
-        
+
         JPanel choicesPanel = new JPanel(new GridLayout(0, columns, 10, 10));
-        choicesPanel.setBorder(new CompoundBorder(choicesPanel.getBorder(), 
+        choicesPanel.setBorder(new CompoundBorder(choicesPanel.getBorder(),
                                                   new EmptyBorder(10, 20, 10, 20)));
 
         /*
@@ -314,9 +315,10 @@ public class FreeColDialog<T> extends FreeColPanel {
                                                           String okText, String cancelText) {
 
         final JTextField input = new JTextField(defaultValue);
-        
+
         final FreeColDialog<String> inputDialog =
             new FreeColDialog<String>(FreeCol.getFreeColClient().getCanvas())  {
+            @Override
             public void requestFocus() {
                 input.requestFocus();
             }
@@ -336,7 +338,7 @@ public class FreeColDialog<T> extends FreeColPanel {
                 inputDialog.setResponse(input.getText());
             }
         });
-        
+
         input.selectAll();
 
         inputDialog.add(getDefaultTextArea(text));
@@ -357,24 +359,21 @@ public class FreeColDialog<T> extends FreeColPanel {
     }
 
 
-    public static FreeColDialog<Dimension> createMapSizeDialog() {
+    public static FreeColDialog<Dimension> createMapSizeDialog(final Canvas canvas) {
 
-        final int defaultSize = FreeCol.getSpecification().getRangeOption("model.option.mapSize")
-            .getValue();
-        final int defaultHeight = MapGeneratorOptions.getHeight(defaultSize);
-        final int defaultWidth = MapGeneratorOptions.getWidth(defaultSize);
+        final int defaultHeight = 100;
+        final int defaultWidth = 40;
         final int COLUMNS = 5;
-        
+
         final String widthText = Messages.message("width");
         final String heightText = Messages.message("height");
-        
+
         final JTextField inputWidth = new JTextField(Integer.toString(defaultWidth), COLUMNS);
         final JTextField inputHeight = new JTextField(Integer.toString(defaultHeight), COLUMNS);
 
-        final FreeColDialog<Dimension> mapSizeDialog =
-            new FreeColDialog<Dimension>(FreeCol.getFreeColClient().getCanvas());
+        final FreeColDialog<Dimension> mapSizeDialog = new FreeColDialog<Dimension>(canvas);
 
-        mapSizeDialog.setLayout(new MigLayout("wrap 2", "", ""));
+        mapSizeDialog.setLayout(new MigLayout("wrap 2"));
 
         mapSizeDialog.okButton.setText(Messages.message("ok"));
         mapSizeDialog.okButton.addActionListener(new ActionListener() {
@@ -387,12 +386,12 @@ public class FreeColDialog<T> extends FreeColPanel {
                         }
                         mapSizeDialog.setResponse(new Dimension(width, height));
                     } catch (NumberFormatException nfe) {
-                        FreeCol.getFreeColClient().getCanvas().errorMessage("integerAboveZero");
+                        canvas.errorMessage("integerAboveZero");
                     }
                 }
             });
 
-        
+
         JLabel widthLabel = new JLabel(widthText);
         widthLabel.setLabelFor(inputWidth);
         JLabel heightLabel = new JLabel(heightText);
@@ -517,7 +516,7 @@ public class FreeColDialog<T> extends FreeColPanel {
 
         return new FreeColFileFilter( ".fsg", "filter.savedGames" );
     }
-    
+
 
     /**
     * Returns a filter accepting "*.fgo".
@@ -527,13 +526,13 @@ public class FreeColDialog<T> extends FreeColPanel {
 
         return new FreeColFileFilter( ".fgo", "filter.gameOptions" );
     }
-    
-    
+
+
     /**
     * Returns a filter accepting all files containing a
     * {@link net.sf.freecol.common.model.GameOptions}.
     * That is; both "*.fgo" and "*.fsg".
-    * 
+    *
     * @return The filter.
     */
     public static FileFilter getGameOptionsFileFilter() {
@@ -584,9 +583,10 @@ public class FreeColDialog<T> extends FreeColPanel {
     /**
      * This function analyses an event and calls the right methods to take care
      * of the user's requests.
-     * 
+     *
      * @param event The incoming ActionEvent.
      */
+    @Override
     public void actionPerformed(ActionEvent event) {
         String command = event.getActionCommand();
         if (CANCEL.equals(command)) {
