@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -24,9 +24,9 @@ import java.awt.Component;
 import java.awt.Dimension;
 import java.awt.FlowLayout;
 import java.awt.Font;
-import java.awt.Image;
 import java.awt.Insets;
 import java.awt.LayoutManager;
+import java.awt.Point;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.awt.event.KeyEvent;
@@ -34,6 +34,8 @@ import java.awt.event.MouseAdapter;
 import java.awt.event.MouseEvent;
 import java.awt.event.MouseListener;
 import java.awt.event.MouseMotionListener;
+import java.beans.PropertyChangeEvent;
+import java.beans.PropertyChangeListener;
 import java.text.DecimalFormat;
 import java.util.ArrayList;
 import java.util.EnumMap;
@@ -49,33 +51,39 @@ import javax.swing.Icon;
 import javax.swing.InputMap;
 import javax.swing.JButton;
 import javax.swing.JComponent;
+import javax.swing.JInternalFrame;
 import javax.swing.JLabel;
 import javax.swing.JPanel;
 import javax.swing.JTextArea;
 import javax.swing.JTextPane;
 import javax.swing.KeyStroke;
-import javax.swing.UIManager;
+import javax.swing.SwingUtilities;
 import javax.swing.border.Border;
+import javax.swing.text.AttributeSet;
 import javax.swing.text.DefaultStyledDocument;
 import javax.swing.text.Style;
 import javax.swing.text.StyleConstants;
 import javax.swing.text.StyleContext;
-import javax.swing.SwingUtilities;
 
 import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.control.InGameController;
 import net.sf.freecol.client.gui.Canvas;
 import net.sf.freecol.client.gui.ImageLibrary;
 import net.sf.freecol.client.gui.i18n.Messages;
-import net.sf.freecol.common.resources.ResourceManager;
+import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.Game;
 import net.sf.freecol.common.model.Modifier;
 import net.sf.freecol.common.model.Player;
+import net.sf.freecol.common.model.Specification;
+import net.sf.freecol.common.model.StringTemplate;
+import net.sf.freecol.common.option.Option;
+import net.sf.freecol.common.option.IntegerOption;
+import net.sf.freecol.common.resources.ResourceManager;
 
 /**
  * Superclass for all panels in FreeCol.
  */
-public class FreeColPanel extends JPanel implements ActionListener {
+public abstract class FreeColPanel extends JPanel implements ActionListener {
 
     @SuppressWarnings("unused")
     private static final Logger logger = Logger.getLogger(FreeColPanel.class.getName());
@@ -95,41 +103,43 @@ public class FreeColPanel extends JPanel implements ActionListener {
     private Canvas canvas = null;
 
     // Font to use for text areas
-    protected static final Font defaultFont = new Font("Dialog", Font.BOLD, 12);
+    protected static final Font defaultFont = ResourceManager.getFont("NormalFont", 13f);
 
     // Fonts to use for report headers, etc.
-    protected static final Font headerFont = ((Font) UIManager.get("HeaderFont")).deriveFont(0, 12);
-
-    protected static final Font smallHeaderFont = ((Font) UIManager.get("HeaderFont")).deriveFont(0, 24);
-
-    protected static final Font mediumHeaderFont = ((Font) UIManager.get("HeaderFont")).deriveFont(0, 36);
-
-    protected static final Font bigHeaderFont = ((Font) UIManager.get("HeaderFont")).deriveFont(0, 48);
+    protected static final Font  smallHeaderFont = ResourceManager.getFont("HeaderFont", 24f);
+    protected static final Font mediumHeaderFont = ResourceManager.getFont("HeaderFont", 36f);
+    protected static final Font    bigHeaderFont = ResourceManager.getFont("HeaderFont", 48f);
 
     // How many columns (em-widths) to use in the text area
     protected static final int COLUMNS = 20;
 
-    // The margin to use for HIGLayout
+    // The margin to use.
     protected static final int margin = 3;
 
-    // The color to use for links
-    protected static final Color LINK_COLOR = new Color(122, 109, 82);
+    // The color to use for things the player probably shouldn't do
+    protected static final Color WARNING_COLOR
+        = ResourceManager.getColor("lookAndFeel.warning.color");
 
-    // The color to use for selected items in lists
-    protected static final Color LIST_SELECT_COLOR = new Color(216, 194, 145);
+    // The color to use for links
+    protected static final Color LINK_COLOR
+        = ResourceManager.getColor("lookAndFeel.link.color");
+
+    // The color to use for borders
+    protected static final Color BORDER_COLOR
+        = ResourceManager.getColor("lookAndFeel.border.color");
 
     // The borders to use for table cells
     public static final Border TOPCELLBORDER =
-        BorderFactory.createCompoundBorder(BorderFactory.createMatteBorder(1, 0, 1, 1, LINK_COLOR),
+        BorderFactory.createCompoundBorder(BorderFactory.createMatteBorder(1, 0, 1, 1, BORDER_COLOR),
                                            BorderFactory.createEmptyBorder(2, 2, 2, 2));
-    public static final Border CELLBORDER = 
-        BorderFactory.createCompoundBorder(BorderFactory.createMatteBorder(0, 0, 1, 1, LINK_COLOR),
+    public static final Border CELLBORDER =
+        BorderFactory.createCompoundBorder(BorderFactory.createMatteBorder(0, 0, 1, 1, BORDER_COLOR),
                                            BorderFactory.createEmptyBorder(2, 2, 2, 2));
-    public static final Border LEFTCELLBORDER = 
-        BorderFactory.createCompoundBorder(BorderFactory.createMatteBorder(0, 1, 1, 1, LINK_COLOR),
+    public static final Border LEFTCELLBORDER =
+        BorderFactory.createCompoundBorder(BorderFactory.createMatteBorder(0, 1, 1, 1, BORDER_COLOR),
                                            BorderFactory.createEmptyBorder(2, 2, 2, 2));
-    public static final Border TOPLEFTCELLBORDER = 
-        BorderFactory.createCompoundBorder(BorderFactory.createMatteBorder(1, 1, 1, 1, LINK_COLOR),
+    public static final Border TOPLEFTCELLBORDER =
+        BorderFactory.createCompoundBorder(BorderFactory.createMatteBorder(1, 1, 1, 1, BORDER_COLOR),
                                            BorderFactory.createEmptyBorder(2, 2, 2, 2));
 
 
@@ -137,16 +147,17 @@ public class FreeColPanel extends JPanel implements ActionListener {
 
     protected JButton okButton = new JButton(Messages.message("ok"));
 
+    protected static final String okCommand = "OK";
+
     protected static StyleContext styleContext = new StyleContext();
 
     static {
-        Style defaultStyle = styleContext.getDefaultStyleContext()
+        Style defaultStyle = StyleContext.getDefaultStyleContext()
             .getStyle(StyleContext.DEFAULT_STYLE);
 
         Style regular = styleContext.addStyle("regular", defaultStyle);
-        StyleConstants.setFontFamily(regular, "Dialog");
-        StyleConstants.setBold(regular, true);
-        StyleConstants.setFontSize(regular, 12);
+        StyleConstants.setFontFamily(regular, "NormalFont");
+        StyleConstants.setFontSize(regular, 13);
 
         Style buttonStyle = styleContext.addStyle("button", regular);
         StyleConstants.setForeground(buttonStyle, LINK_COLOR);
@@ -157,6 +168,8 @@ public class FreeColPanel extends JPanel implements ActionListener {
 
     /**
      * Constructor.
+     *
+     * @param parent <code>Canvas</code>
      */
     public FreeColPanel(Canvas parent) {
         this(parent, new FlowLayout());
@@ -164,7 +177,7 @@ public class FreeColPanel extends JPanel implements ActionListener {
 
     /**
      * Default constructor.
-     * 
+     *
      * @param parent The <code>Canvas</code> all panels belong to.
      * @param layout The <code>LayoutManager</code> to be used.
      */
@@ -181,24 +194,107 @@ public class FreeColPanel extends JPanel implements ActionListener {
         addMouseListener(new MouseAdapter() {
         });
 
-        okButton.setActionCommand(OK);
+        okButton.setActionCommand(okCommand);
         okButton.addActionListener(this);
         enterPressesWhenFocused(okButton);
         setCancelComponent(okButton);
+
+    }
+
+
+    /**
+     * Return an <code>int</code> associated with the name of the
+     * panel's class plus the given key from the saved ClientOptions.
+     *
+     * @param key a <code>String</code> value
+     * @return an <code>int</code> value
+     */
+    private int getInteger(String key) {
+        return getClient().getClientOptions().getInteger(getClass().getName() + key);
     }
 
     /**
-     * Set the <code>SavedSize</code> value.
+     * Returns the saved size of this panel, null by default.
      *
-     * @param newSavedSize The new SavedSize value.
+     * @return a <code>Dimension</code> value
      */
-    public void setSavedSize(final Dimension newSavedSize) {
-        // override this if you want a panel to remember its size
+    public final Dimension getSavedSize() {
+        try {
+            return new Dimension(getInteger(".w"), getInteger(".h"));
+        } catch(Exception e) {
+            return null;
+        }
     }
 
+
+    /**
+     * Save an <code>int</code> value to the saved ClientOptions,
+     * using the name of the panel's class plus the given key as and
+     * ID.
+     *
+     * @param key a <code>String</code> value
+     * @param value an <code>int</code> value
+     */
+    private void saveInteger(String key, int value) {
+        Option o = getClient().getClientOptions().getOption(getClass().getName() + key);
+        if (o == null) {
+            IntegerOption io = new IntegerOption(getClass().getName() + key);
+            io.setValue(value);
+            getClient().getClientOptions().add(io);
+        } else if (o instanceof IntegerOption) {
+            ((IntegerOption) o).setValue(value);
+        }
+    }
+
+    /**
+     * Set preferred size to saved size, or to the given
+     * <code>Dimension</code> if no saved size was found. Call this
+     * method in the constructor of a FreeColPanel in order to
+     * remember its size and position.
+     *
+     * @param d a <code>Dimension</code> value
+     */
+    protected void restoreSavedSize(Dimension d) {
+        Dimension size = getSavedSize();
+        if (size == null) {
+            size = d;
+            saveSize(size);
+        }
+        if (!getPreferredSize().equals(size)) {
+            setPreferredSize(size);
+        }
+    }
+
+    /**
+     * Set preferred size to saved size, or to [w, h] if no saved size
+     * was found. Call this method in the constructor of a
+     * FreeColPanel in order to remember its size and position.
+     *
+     * @param w an <code>int</code> value
+     * @param h an <code>int</code> value
+     */
+    protected void restoreSavedSize(int w, int h) {
+        restoreSavedSize(new Dimension(w, h));
+    }
+
+
+    /**
+     * Returns the saved position of this panel, null by default.
+     *
+     * @return a <code>Point</code> value
+     */
+    public Point getSavedPosition() {
+        try {
+            return new Point(getInteger(".x"), getInteger(".y"));
+        } catch(Exception e) {
+            return null;
+        }
+    }
+
+
     /**
      * Get the <code>Canvas</code> value.
-     * 
+     *
      * @return a <code>Canvas</code> value
      */
     public final Canvas getCanvas() {
@@ -232,6 +328,15 @@ public class FreeColPanel extends JPanel implements ActionListener {
         return canvas.getClient().getGame();
     }
 
+    /**
+     * Describe <code>getSpecification</code> method here.
+     *
+     * @return a <code>Specification</code> value
+     */
+    public Specification getSpecification() {
+        return canvas.getSpecification();
+    }
+
     /**
      * Describe <code>getController</code> method here.
      *
@@ -252,15 +357,26 @@ public class FreeColPanel extends JPanel implements ActionListener {
 
     /**
      * Checks if this panel is editable
+     * @return boolean
      */
     public boolean isEditable() {
         return editable;
     }
 
+    /**
+     * Set the <code>Editable</code> value.
+     *
+     * @param newEditable boolean, the new Editable value
+     */
+    public void setEditable(boolean newEditable) {
+        this.editable = newEditable;
+    }
+
     /**
      * The OK button requests focus.
      *
      */
+    @Override
     public void requestFocus() {
         okButton.requestFocus();
     }
@@ -281,7 +397,24 @@ public class FreeColPanel extends JPanel implements ActionListener {
      * @return a <code>JTextPane</code> value
      */
     public static JTextPane getDefaultTextPane(String text) {
-        JTextPane textPane = new JTextPane(new DefaultStyledDocument(styleContext));
+
+        DefaultStyledDocument document = new DefaultStyledDocument(styleContext) {
+             @Override
+             public Font getFont(AttributeSet attr) {
+                 Font font = ResourceManager.getFont(StyleConstants.getFontFamily(attr),
+                                                     StyleConstants.getFontSize(attr));
+                 if (font == null) {
+                     return super.getFont(attr);
+                 } else {
+                     int fontStyle = Font.PLAIN;
+                     if (StyleConstants.isBold(attr)) fontStyle |= Font.BOLD;
+                     if (StyleConstants.isItalic(attr)) fontStyle |= Font.ITALIC;
+                     return (fontStyle == Font.PLAIN) ? font : font.deriveFont(fontStyle);
+                 }
+             }
+         };
+
+        JTextPane textPane = new JTextPane(document);
         textPane.setOpaque(false);
         textPane.setEditable(false);
         textPane.setLogicalStyle(styleContext.getStyle("regular"));
@@ -295,7 +428,7 @@ public class FreeColPanel extends JPanel implements ActionListener {
     /**
      * Returns a text area with standard settings suitable for use in FreeCol
      * dialogs.
-     * 
+     *
      * @param text The text to display in the text area.
      * @return a text area with standard settings suitable for use in FreeCol
      *         dialogs.
@@ -307,7 +440,7 @@ public class FreeColPanel extends JPanel implements ActionListener {
     /**
      * Returns a text area with standard settings suitable for use in FreeCol
      * dialogs.
-     * 
+     *
      * @param text The text to display in the text area.
      * @param columns an <code>int</code> value
      * @return a text area with standard settings suitable for use in FreeCol
@@ -346,10 +479,16 @@ public class FreeColPanel extends JPanel implements ActionListener {
         return button;
     }
 
+    public JButton createColonyButton(Colony colony) {
+        JButton button = getLinkButton(colony.getName(), null, colony.getId());
+        button.addActionListener(this);
+        return button;
+    }
+
 
     /**
      * Returns the default header for panels.
-     * 
+     *
      * @param text a <code>String</code> value
      * @return a <code>JLabel</code> value
      */
@@ -360,6 +499,27 @@ public class FreeColPanel extends JPanel implements ActionListener {
         return header;
     }
 
+    /**
+     * Return a JLabel with Messages.message(key) as text.
+     *
+     * @param key a <code>String</code> value
+     * @return a <code>JLabel</code> value
+     */
+    public JLabel localizedLabel(String key) {
+        return new JLabel(Messages.message(key));
+    }
+
+    /**
+     * Return a JLabel with Messages.localize(template) as text.
+     *
+     * @param template a <code>StringTemplate</code> value
+     * @return a <code>JLabel</code> value
+     */
+    public JLabel localizedLabel(StringTemplate template) {
+        return new JLabel(Messages.message(template));
+    }
+
+
     /**
      * Make the given button the CANCEL button.
      *
@@ -369,17 +529,17 @@ public class FreeColPanel extends JPanel implements ActionListener {
         if (cancelButton == null) {
             throw new NullPointerException();
         }
-        
+
         InputMap inputMap = getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
         inputMap.put(KeyStroke.getKeyStroke(cancelKeyCode, 0, true), "release");
-                
+
         Action cancelAction = cancelButton.getAction();
         getActionMap().put("release", cancelAction);
     }
 
     /**
      * Registers enter key for a JButton.
-     * 
+     *
      * @param button
      */
     public static void enterPressesWhenFocused(JButton button) {
@@ -403,7 +563,8 @@ public class FreeColPanel extends JPanel implements ActionListener {
     /**
      * Sort the given modifiers according to type.
      *
-     * @return a sorted Set of Modifiers
+     * @param result Set of <code>Modifier</code>
+     * @return a sorted Set of <code>Modifier</code>
      */
     public Set<Modifier> sortModifiers(Set<Modifier> result) {
         EnumMap<Modifier.Type, List<Modifier>> modifierMap =
@@ -424,21 +585,76 @@ public class FreeColPanel extends JPanel implements ActionListener {
     /**
      * This function analyses an event and calls the right methods to take care
      * of the user's requests.
-     * 
+     *
      * @param event The incoming ActionEvent.
      */
     public void actionPerformed(ActionEvent event) {
         String command = event.getActionCommand();
-        if (OK.equals(command)) {
+        if (command.equals(okCommand)) {
             getCanvas().remove(this);
         }
     }
 
+    /**
+     * Notify this panel that it is being removed from the
+     * canvas. Saves the current size and position of the panel to the
+     * ClientOptions, which are included in the savegame file.
+     *
+     * @see Canvas#remove(Component)
+     */
+    public void notifyClose() {
+        firePropertyChange("closing", false, true);
+        Component frame = SwingUtilities.getAncestorOfClass(JInternalFrame.class, this);
+        if (frame != null) {
+            saveInteger(".x", frame.getLocation().x);
+            saveInteger(".y", frame.getLocation().y);
+        }
+        saveSize();
+    }
+
+    /**
+     * Save the current size of the panel.
+     *
+     */
+    private void saveSize() {
+        saveSize(getSize());
+    }
+
+    /**
+     * Save the given Dimension as size of the panel.
+     *
+     * @param size a <code>Dimension</code> value
+     */
+    private void saveSize(Dimension size) {
+        saveInteger(".w", size.width);
+        saveInteger(".h", size.height);
+    }
+
+
+
+
+    /**
+     * Add a routine to be called when this panel closes.
+     * Triggered by notifyClose() above.
+     *
+     * @param runnable Some code to run on close.
+     */
+    public void addClosingCallback(final Runnable runnable) {
+        addPropertyChangeListener(new PropertyChangeListener() {
+                public void propertyChange(PropertyChangeEvent e) {
+                    if ("closing".equals(e.getPropertyName())) {
+                        runnable.run();
+                    }
+                }
+            });
+    }
+
     /**
      * Creates a <code>MouseListener</code> which forwards events
      * to the given <code>Component</code>.
      *
      * @param c The <code>Component</code> the events should be forwarded to.
+     * @return <code>MouseListener</code>
      */
     public static MouseListener createEventForwardingMouseListener(final Component c) {
         final MouseListener ml = new MouseListener() {
@@ -469,7 +685,8 @@ public class FreeColPanel extends JPanel implements ActionListener {
      * Creates a <code>MouseMotionListener</code> which forwards events
      * to the given <code>Component</code>.
      *
-     * @param c The <code>Component</code> the events should be forwarded to.
+     * @param c The <code>Component</code> the events should be forwarded to
+     * @return <code>MouseMotionListener</code>
      */
     public static MouseMotionListener createEventForwardingMouseMotionListener(final Component c) {
         final MouseMotionListener ml = new MouseMotionListener() {
