@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -21,14 +21,12 @@ package net.sf.freecol.client.gui.panel;
 
 import java.awt.BorderLayout;
 import java.awt.Dimension;
-import java.awt.FlowLayout;
 import java.awt.Font;
 import java.awt.Graphics;
 import java.awt.Image;
 import java.awt.event.MouseAdapter;
 import java.awt.event.MouseEvent;
 import java.awt.image.BufferedImage;
-import java.util.ArrayList;
 import java.util.List;
 import java.util.logging.Logger;
 
@@ -36,20 +34,20 @@ import javax.swing.ImageIcon;
 import javax.swing.JButton;
 import javax.swing.JLabel;
 import javax.swing.JPanel;
-import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.gui.GUI;
 import net.sf.freecol.client.gui.ViewMode;
 import net.sf.freecol.client.gui.i18n.Messages;
+import net.sf.freecol.client.gui.action.EndTurnAction;
 import net.sf.freecol.client.gui.panel.MapEditorTransformPanel.MapTransform;
 import net.sf.freecol.common.model.AbstractGoods;
 import net.sf.freecol.common.model.EquipmentType;
 import net.sf.freecol.common.model.Goods;
+import net.sf.freecol.common.model.StringTemplate;
 import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.TileImprovement;
 import net.sf.freecol.common.model.Unit;
 import net.sf.freecol.common.resources.ResourceManager;
-import net.sf.freecol.common.util.Utils;
 
 import net.miginfocom.swing.MigLayout;
 
@@ -59,23 +57,23 @@ import net.miginfocom.swing.MigLayout;
  * left and stuff like that.
  */
 public final class InfoPanel extends FreeColPanel {
-    
+
     @SuppressWarnings("unused")
     private static final Logger logger = Logger.getLogger(InfoPanel.class.getName());
-    
+
     private static final int PANEL_WIDTH = 256;
-    
+
     private static final int PANEL_HEIGHT = 128;
-    
+
     private final EndTurnPanel endTurnPanel = new EndTurnPanel();
-    
+
     private final UnitInfoPanel unitInfoPanel;
-    
+
     private final TileInfoPanel tileInfoPanel = new TileInfoPanel();
-    
+
     private final JPanel mapEditorPanel;
-    
-    
+
+
     /**
      * The constructor that will add the items to this panel.
      *
@@ -83,10 +81,10 @@ public final class InfoPanel extends FreeColPanel {
      */
     public InfoPanel(final FreeColClient freeColClient) {
         super(freeColClient.getCanvas());
-        
+
         unitInfoPanel = new UnitInfoPanel();
         setLayout(null);
-        
+
         int internalPanelTop = 0;
         int internalPanelHeight = 128;
         Image skin = ResourceManager.getImage("InfoPanel.skin");
@@ -99,27 +97,28 @@ public final class InfoPanel extends FreeColPanel {
             internalPanelTop = 75;
             internalPanelHeight = 100;
         }
-        
+
         mapEditorPanel = new JPanel(null);
         mapEditorPanel.setSize(130, 100);
         mapEditorPanel.setOpaque(false);
-        
+
         add(unitInfoPanel, internalPanelTop, internalPanelHeight);
         add(endTurnPanel, internalPanelTop, internalPanelHeight);
         add(tileInfoPanel, internalPanelTop, internalPanelHeight);
         add(mapEditorPanel, internalPanelTop, internalPanelHeight);
 
         addMouseListener(new MouseAdapter() {
-                public void mousePressed(MouseEvent e) {
-                    GUI gui = getClient().getGUI();
-                    Unit activeUnit = gui.getActiveUnit();
-                    if (activeUnit != null && activeUnit.getTile() != null) {
-                        gui.setFocus(activeUnit.getTile().getPosition());
-                    }
-                }
+           @Override
+           public void mousePressed(MouseEvent e) {
+              GUI gui = getClient().getGUI();
+              Unit activeUnit = gui.getActiveUnit();
+              if (activeUnit != null && activeUnit.getTile() != null) {
+                  gui.setFocus(activeUnit.getTile());
+              }
+          }
             });
     }
-    
+
     /**
      * Adds a panel to show information
      */
@@ -129,7 +128,7 @@ public final class InfoPanel extends FreeColPanel {
                 + (internalPanelHeight - panel.getHeight()) / 2);
         add(panel);
     }
-    
+
     /**
      * Updates this <code>InfoPanel</code>.
      *
@@ -138,7 +137,7 @@ public final class InfoPanel extends FreeColPanel {
     public void update(Unit unit) {
         unitInfoPanel.update(unit);
     }
-    
+
     /**
      * Updates this <code>InfoPanel</code>.
      *
@@ -159,8 +158,8 @@ public final class InfoPanel extends FreeColPanel {
             }
         }
     }
-    
-    
+
+
     /**
      * Updates this <code>InfoPanel</code>.
      *
@@ -169,7 +168,7 @@ public final class InfoPanel extends FreeColPanel {
     public void update(Tile tile) {
         tileInfoPanel.update(tile);
     }
-    
+
     /**
      * Gets the <code>Unit</code> in which this <code>InfoPanel</code> is
      * displaying information about.
@@ -180,7 +179,7 @@ public final class InfoPanel extends FreeColPanel {
     public Unit getUnit() {
         return unitInfoPanel.getUnit();
     }
-    
+
     /**
      * Gets the <code>Tile</code> in which this <code>InfoPanel</code> is
      * displaying information about.
@@ -191,12 +190,13 @@ public final class InfoPanel extends FreeColPanel {
     public Tile getTile() {
         return tileInfoPanel.getTile();
     }
-    
+
     /**
      * Paints this component.
      *
      * @param graphics The Graphics context in which to draw this component.
      */
+    @Override
     public void paintComponent(Graphics graphics) {
         int viewMode = getClient().getGUI().getViewMode().getView();
         if (!getClient().isMapEditor()) {
@@ -233,32 +233,32 @@ public final class InfoPanel extends FreeColPanel {
                 tileInfoPanel.setVisible(false);
             }
         }
-        
+
         Image skin = ResourceManager.getImage("InfoPanel.skin");
         if (skin != null) {
             graphics.drawImage(skin, 0, 0, null);
         }
-        
+
         super.paintComponent(graphics);
     }
-    
-    
+
+
     /**
      * Panel for displaying <code>Tile</code>-information.
      */
     public class TileInfoPanel extends JPanel {
-        
+
         private Tile tile;
         private Font font = new JLabel().getFont().deriveFont(9f);
-        
+
         public TileInfoPanel() {
             super(null);
-            
+
             setSize(226, 100);
             setOpaque(false);
             setLayout(new MigLayout("fill, wrap 2, gap 0 0", "", "[][][][][nogrid]"));
         }
-        
+
         /**
          * Updates this <code>InfoPanel</code>.
          *
@@ -267,29 +267,28 @@ public final class InfoPanel extends FreeColPanel {
         public void update(Tile tile) {
 
             this.tile = tile;
-            
+
             removeAll();
 
             if (tile != null) {
                 int width = getLibrary().getTerrainImageWidth(tile.getType());
                 int height = getLibrary().getTerrainImageHeight(tile.getType());
                 BufferedImage image = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
-                getClient().getGUI().displayTerrain(image.createGraphics(), 
-                                                      getClient().getGame().getMap(),
-                                                      tile, 0, 0);
+                getClient().getGUI().displayTerrain(image.createGraphics(),
+                                                      tile);
                 if (tile.isExplored()) {
-                    List<String> items = new ArrayList<String>();
-                    items.add(Messages.getName(tile));
+                    StringTemplate items = StringTemplate.label(", ");
+                    items.add(tile.getNameKey());
                     for (TileImprovement tileImprovement : tile.getCompletedTileImprovements()) {
-                        items.add(Messages.getDescription(tileImprovement.getType()));
+                        items.add(tileImprovement.getType().getDescriptionKey());
                     }
-                    add(new JLabel(Utils.join(", ", items)), "span, align center");
+                    add(new JLabel(Messages.message(items)), "span, align center");
 
                     add(new JLabel(new ImageIcon(image)), "span 1 3");
                     if (tile.getOwner() == null) {
                         add(new JLabel());
                     } else {
-                        JLabel ownerLabel = new JLabel(Messages.getNationAsString(tile.getOwner()));
+                        JLabel ownerLabel = localizedLabel(tile.getOwner().getNationName());
                         ownerLabel.setFont(font);
                         add(ownerLabel);
                     }
@@ -310,7 +309,7 @@ public final class InfoPanel extends FreeColPanel {
                         JLabel goodsLabel = new JLabel(String.valueOf(tile.potential(goods.getType(), null)),
                                                        getLibrary().getScaledGoodsImageIcon(goods.getType(), 0.50f),
                                                        JLabel.RIGHT);
-                        goodsLabel.setToolTipText(Messages.getName(goods.getType()));
+                        goodsLabel.setToolTipText(Messages.message(goods.getType().getNameKey()));
                         goodsLabel.setFont(font);
                         add(goodsLabel);
                     }
@@ -322,7 +321,7 @@ public final class InfoPanel extends FreeColPanel {
                 repaint();
             }
         }
-        
+
         /**
          * Gets the <code>Tile</code> in which this <code>InfoPanel</code>
          * is displaying information about.
@@ -334,22 +333,22 @@ public final class InfoPanel extends FreeColPanel {
             return tile;
         }
     }
-    
+
     /**
      * Panel for displaying <code>Unit</code>-information.
      */
     public class UnitInfoPanel extends JPanel {
-        
+
         private Unit unit;
-        
+
         public UnitInfoPanel() {
 
             super(new MigLayout("wrap 6, fill, gap 0 0", "", ""));
-            
+
             setSize(226, 100);
             setOpaque(false);
         }
-        
+
         /**
          * Updates this <code>InfoPanel</code>.
          *
@@ -357,11 +356,11 @@ public final class InfoPanel extends FreeColPanel {
          */
         public void update(Unit unit) {
             this.unit = unit;
-            
+
             removeAll();
             if (unit != null) {
                 add(new JLabel(getLibrary().getUnitImageIcon(unit)), "spany, gapafter 5px");
-                String name = Messages.getLabel(unit);
+                String name = Messages.message(Messages.getLabel(unit));
                 // TODO: this is too brittle!
                 int index = name.indexOf(" (");
                 if (index < 0) {
@@ -371,32 +370,36 @@ public final class InfoPanel extends FreeColPanel {
                     add(new JLabel(name.substring(index + 1)), "span");
                 }
                 add(new JLabel(Messages.message("moves") + " " + unit.getMovesAsString()), "span");
-                
+
                 // Handle the special cases. TODO: make this more generic
                 if (unit.canCarryTreasure()) {
                     add(new JLabel(unit.getTreasureAmount() + " " + Messages.message("gold")), "span");
                 } else if (unit.isCarrier()) {
                     for (Goods goods : unit.getGoodsList()) {
                         JLabel goodsLabel = new JLabel(getLibrary().getScaledGoodsImageIcon(goods.getType(), 0.66f));
-                        goodsLabel.setToolTipText(Messages.message("model.goods.goodsAmount",
-                                                                   "%amount%", String.valueOf(goods.getAmount()),
-                                                                   "%goods%", Messages.getName(goods)));
+                        goodsLabel.setToolTipText(Messages.message(StringTemplate.template("model.goods.goodsAmount")
+                                                                   .addAmount("%amount%", goods.getAmount())
+                                                                   .add("%goods%", goods.getNameKey())));
                         add(goodsLabel);
                     }
                     for (Unit carriedUnit : unit.getUnitList()) {
-                        ImageIcon unitIcon = getLibrary().getUnitImageIcon(carriedUnit);
-                        JLabel unitLabel = new JLabel(getLibrary().getScaledImageIcon(unitIcon, 0.5f));
-                        unitLabel.setToolTipText(Messages.getLabel(carriedUnit));
+                        ImageIcon unitIcon = getLibrary().getUnitImageIcon(carriedUnit, 0.5);
+                        JLabel unitLabel = new JLabel(unitIcon);
+                        unitLabel.setToolTipText(Messages.message(carriedUnit.getLabel()));
                         add(unitLabel);
                     }
                 } else {
                     for (EquipmentType equipment : unit.getEquipment().keySet()) {
                         for (AbstractGoods goods : equipment.getGoodsRequired()) {
-                        	int amount = goods.getAmount() * unit.getEquipment().getCount(equipment);
-                        	String amountStr = String.valueOf(amount);
-                        	JLabel equipmentLabel = 
-                                new JLabel(amountStr,getLibrary().getScaledGoodsImageIcon(goods.getType(), 0.66f), JLabel.CENTER);
-                            equipmentLabel.setToolTipText(amountStr + " " + Messages.getName(goods.getType()));
+                            int amount = goods.getAmount() * unit.getEquipment().getCount(equipment);
+                            JLabel equipmentLabel =
+                                new JLabel(Integer.toString(amount),
+                                           getLibrary().getScaledGoodsImageIcon(goods.getType(), 0.66f),
+                                           JLabel.CENTER);
+                            equipmentLabel
+                                .setToolTipText(Messages.message(StringTemplate.template("model.goods.goodsAmount")
+                                                                 .addAmount("%amount%", amount)
+                                                                 .add("%goods%", goods.getNameKey())));
                             add(equipmentLabel);
                         }
                     }
@@ -405,7 +408,7 @@ public final class InfoPanel extends FreeColPanel {
             revalidate();
             repaint();
         }
-        
+
         /**
          * Gets the <code>Unit</code> in which this <code>InfoPanel</code>
          * is displaying information about.
@@ -416,23 +419,22 @@ public final class InfoPanel extends FreeColPanel {
         public Unit getUnit() {
             return unit;
         }
-        
+
     }
-    
+
     /**
      * Panel for ending the turn.
      */
     public class EndTurnPanel extends JPanel {
-        
-        private JButton endTurnButton = new JButton(Messages.message("infoPanel.endTurnPanel.endTurnButton"));
-        
+
         public EndTurnPanel() {
             super(new MigLayout("wrap 1, center", "[center]", ""));
 
             String labelString = Messages.message("infoPanel.endTurnPanel.text");
             int width = getFontMetrics(getFont()).stringWidth(labelString);
             if (width > 150 ) {
-                int index = getCanvas().getGUI().getBreakingPoint(labelString);
+                getCanvas().getGUI();
+                int index = GUI.getBreakingPoint(labelString);
                 if (index > 0) {
                     add(new JLabel(labelString.substring(0, index)));
                     add(new JLabel(labelString.substring(index + 1)));
@@ -443,26 +445,10 @@ public final class InfoPanel extends FreeColPanel {
                 add(new JLabel(labelString));
             }
 
-            add(endTurnButton);
+            add(new JButton(getClient().getActionManager().getFreeColAction(EndTurnAction.id)));
             setOpaque(false);
             setSize(getPreferredSize());
-            
-            /*
-             * TODO: The action listener does not work, because this button
-             * looses it's focus. The reason why the focus gets lost should be
-             * found, in order to use the actionlistener.
-             */
-            /*
-             * endTurnButton.addActionListener(new ActionListener() { public
-             * void actionPerformed(ActionEvent e) {
-             * getClient().getInGameController().endTurn(); } });
-             */
-            
-            endTurnButton.addMouseListener(new MouseAdapter() {
-                public void mousePressed(MouseEvent e) {
-                    getController().endTurn();
-                }
-            });
+
         }
     }
 }
