@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -19,10 +19,9 @@
 
 package net.sf.freecol.client.gui.panel;
 
-import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.gui.i18n.Messages;
-import net.sf.freecol.common.Specification;
+import net.sf.freecol.common.model.Specification;
 import net.sf.freecol.common.model.Building;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.ColonyTile;
@@ -305,7 +304,7 @@ public class LabourData {
             if (isSummary()) {
                 return null;
             }
-            return Messages.getName(unitType);
+            return Messages.message(unitType.getNameKey());
         }
 
         public boolean hasDetails() {
@@ -376,7 +375,7 @@ public class LabourData {
     private UnitData scout;
 
     public LabourData(FreeColClient client) {
-        Specification spec = FreeCol.getSpecification();
+        Specification spec = client.getGame().getSpecification();
         for (UnitType unitType : spec.getUnitTypeList()) {
             if (!unitType.isAvailableTo(client.getMyPlayer())) {
                 continue;
@@ -436,7 +435,7 @@ public class LabourData {
                 for (Colony colony : player.getColonies()) {
                     LocationData data = unitData.details.containsKey(colony) ? unitData.getLocationData(colony) : null;
 
-                    int netProduction = colony.getProductionNetOf(expertProduction);
+                    int netProduction = colony.getNetProductionOf(expertProduction);
                     if (data != null) {
                         data.netProduction = netProduction;
                     }
@@ -447,7 +446,7 @@ public class LabourData {
     }
 
     public static List<UnitType> getLabourTypes(Player player) {
-        List<UnitType> unitTypes = FreeCol.getSpecification().getUnitTypeList();
+        List<UnitType> unitTypes = player.getSpecification().getUnitTypeList();
         ArrayList<UnitType> labourTypes = new ArrayList<UnitType>();
         for (UnitType unitType : unitTypes) {
             if (unitType.hasSkill() && unitType.isAvailableTo(player)) {
