@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -36,31 +36,29 @@ import javax.swing.JPanel;
 import javax.swing.JSpinner;
 import javax.swing.JTextPane;
 import javax.swing.SpinnerNumberModel;
-import javax.swing.text.Style;
 import javax.swing.text.StyleConstants;
-import javax.swing.text.StyleContext;
 import javax.swing.text.StyledDocument;
 
+import net.miginfocom.swing.MigLayout;
 import net.sf.freecol.client.gui.Canvas;
 import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.ColonyTradeItem;
 import net.sf.freecol.common.model.DiplomaticTrade;
+import net.sf.freecol.common.model.DiplomaticTrade.TradeStatus;
 import net.sf.freecol.common.model.GoldTradeItem;
 import net.sf.freecol.common.model.Goods;
 import net.sf.freecol.common.model.GoodsTradeItem;
+import net.sf.freecol.common.model.NationSummary;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Player.Stance;
 import net.sf.freecol.common.model.Settlement;
 import net.sf.freecol.common.model.StanceTradeItem;
+import net.sf.freecol.common.model.StringTemplate;
 import net.sf.freecol.common.model.TradeItem;
 import net.sf.freecol.common.model.Unit;
 import net.sf.freecol.common.model.UnitTradeItem;
 
-import org.w3c.dom.Element;
-
-import net.miginfocom.swing.MigLayout;
-
 
 /**
  * The panel that allows negotiations between players.
@@ -129,34 +127,23 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
             this.otherPlayer = sender;
         }
 
-        if (player.getStance(otherPlayer) == Stance.WAR) {
+        if (player.atWarWith(otherPlayer)) {
             if (!hasPeaceOffer()) {
                 Stance stance = Stance.PEACE;
                 this.agreement.add(new StanceTradeItem(getGame(), player, otherPlayer, stance));
             }
         }
 
-        summary = new JTextPane();
+        summary = getDefaultTextPane();
         summary.setOpaque(false);
         summary.setEditable(false);
-
-        StyledDocument document = summary.getStyledDocument();
-        //Initialize some styles.
-        Style def = StyleContext.getDefaultStyleContext().getStyle(StyleContext.DEFAULT_STYLE);
-        
-        Style regular = document.addStyle("regular", def);
-        StyleConstants.setFontFamily(def, "Dialog");
-        StyleConstants.setBold(def, true);
-        StyleConstants.setFontSize(def, 12);
-
-        Style buttonStyle = document.addStyle("button", regular);
-        StyleConstants.setForeground(buttonStyle, LINK_COLOR);
     }
 
     /**
      * Set up the dialog.
      *
      */
+    @Override
     public void initialize() {
         
         sendButton = new JButton(Messages.message("negotiationDialog.send"));
@@ -186,16 +173,8 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
             unitOffer = new UnitTradeItemPanel(this, player);
         */
 
-        int numberOfTradeItems = 4;
-        int extraRows = 2; // headline and buttons
-
         setLayout(new MigLayout("wrap 3", "[200, fill][300, fill][200, fill]", ""));
 
-        int demandColumn = 1;
-        int summaryColumn = 3;
-        int offerColumn = 5;
-
-        int row = 1;
         add(new JLabel(Messages.message("negotiationDialog.demand")), "center");
         add(new JLabel(Messages.message("negotiationDialog.offer")), "skip, center");
 
@@ -243,7 +222,7 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
             loop: while ((end = input.indexOf('%', start + 1)) >= 0) {
                 String var = input.substring(start, end + 1);
                 if (var.equals("%nation%")) {
-                    insertText(Messages.getNationAsString(sender));
+                    insertText(Messages.message(sender.getNationName()));
                     start = end + 1;
                     continue loop;
                 } else if (var.equals("%offers%")) {
@@ -305,19 +284,10 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
         }
     }
     
-    private void updateDemandItems(){
+    private void updateDemandItems() {
         // Update the gold amount that can be demanded
-        int foreignGold = 0;
-        Element report = getController().getForeignAffairsReport();
-        int number = report.getChildNodes().getLength();
-        for (int i = 0; i < number; i++) {
-            Element enemyElement = (Element) report.getChildNodes().item(i);
-            Player enemy = (Player) getGame().getFreeColGameObject(enemyElement.getAttribute("player"));
-            if (enemy == otherPlayer) {
-                foreignGold = Integer.parseInt(enemyElement.getAttribute("gold"));
-                break;
-            }
-        }
+        NationSummary ns = getController().getNationSummary(otherPlayer);
+        int foreignGold = (ns == null) ? 0 : ns.getGold();
         goldDemand.setAvailableGold(foreignGold);
         
         if(unit.isCarrier()){
@@ -378,18 +348,20 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
                 foundItem = true;
                 String description = "";
                 if (item instanceof StanceTradeItem) {
-                    description = Player.getStanceAsString(((StanceTradeItem) item).getStance());
+                    description = Messages.getStanceAsString(((StanceTradeItem) item).getStance());
                 } else if (item instanceof GoldTradeItem) {
-                    String gold = String.valueOf(((GoldTradeItem) item).getGold());
-                    description = Messages.message("tradeItem.gold.long", "%amount%", gold);
+                    int gold = ((GoldTradeItem) item).getGold();
+                    description = Messages.message(StringTemplate.template("tradeItem.gold.long")
+                                                   .addAmount("%amount%", gold));
                 } else if (item instanceof ColonyTradeItem) {
-                    description = Messages.message("tradeItem.colony.long", 
-                                                   "%colony%", ((ColonyTradeItem) item).getColony().getName());
+                    description = Messages.message(StringTemplate.template("tradeItem.colony.long")
+                                                   .addName("%colony%", ((ColonyTradeItem) item).getColonyName()));
                 } else if (item instanceof GoodsTradeItem) {
-                    description = String.valueOf(((GoodsTradeItem) item).getGoods().getAmount()) + " " +
-                        Messages.getName(((GoodsTradeItem) item).getGoods());
+                    description = Messages.message(StringTemplate.template("model.goods.goodsAmount")
+                                                   .addAmount("%amount%", ((GoodsTradeItem) item).getGoods().getAmount())
+                                                   .add("%goods%", ((GoodsTradeItem) item).getGoods().getNameKey()));
                 } else if (item instanceof UnitTradeItem) {
-                    description = Messages.getLabel(((UnitTradeItem) item).getUnit());
+                    description = Messages.message(((UnitTradeItem) item).getUnit().getLabel());
                 }
                 try {
                     JButton button = getLinkButton(description, null, String.valueOf(index));
@@ -475,9 +447,9 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
 
 
     /**
-     * Sets the <code>stance</code> between the players.
+     * Trade a stance change between the players.
      *
-     * @param stance a <code>Stance</code> value
+     * @param stance The <code>Stance</code> to trade.
      */
     public void setStance(Stance stance) {
         agreement.add(new StanceTradeItem(getGame(), otherPlayer, player, stance));
@@ -485,8 +457,7 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
 
 
     /**
-     * Returns the stance being offered, or Integer.MIN_VALUE if none
-     * is being offered.
+     * Returns the stance being offered.
      *
      * @return a <code>Stance</code> value
      */
@@ -501,14 +472,17 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
      * 
      * @param event The incoming action event
      */
+    @Override
     public void actionPerformed(ActionEvent event) {
         String command = event.getActionCommand();
         if (command.equals(CANCEL)) {
-            setResponse(null);
+            agreement.setStatus(TradeStatus.REJECT_TRADE);
+            setResponse(agreement);
         } else if (command.equals(ACCEPT)) {
-            agreement.setAccept(true);
+            agreement.setStatus(TradeStatus.ACCEPT_TRADE);
             setResponse(agreement);
         } else if (command.equals(SEND)) {
+            agreement.setStatus(TradeStatus.PROPOSE_TRADE);
             setResponse(agreement);
         } else {
             int index = Integer.parseInt(command);
@@ -701,24 +675,35 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
 
         class StanceItem {
             private Stance value;
-            StanceItem(Stance value) {
-                this.value = value;
+
+            StanceItem (Stance value) {
+               if (value == null)
+                  throw new NullPointerException();
+               this.value = value;
             }
             
+            @Override
             public String toString() {
-                return Player.getStanceAsString(value);
+                return Messages.getStanceAsString(value);
             }
             
             Stance getValue() {
                 return value;
             }
             
+            @Override
             public boolean equals(Object other) {
                 if (other == null || !(other instanceof StanceItem)) {
                     return false;
                 }
                 return value.equals(((StanceItem) other).value);
             }
+
+            @Override
+            public int hashCode()
+            {
+               return value.hashCode();
+            }
         }
         
         private JComboBox stanceBox;
@@ -732,6 +717,7 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
          *
          * @param parent a <code>NegotiationDialog</code> value
          * @param source a <code>Player</code> value
+         * @param target <code>Player</code>
          */
         public StanceTradeItemPanel(NegotiationDialog parent, Player source, Player target) {
             this.negotiationDialog = parent;
@@ -794,6 +780,7 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
          *
          * @param parent a <code>NegotiationDialog</code> value
          * @param source a <code>Player</code> value
+         * @param gold int ??
          */
         public GoldTradeItemPanel(NegotiationDialog parent, Player source, int gold) {
             this.player = source;
