@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -33,6 +33,7 @@ import java.util.logging.Logger;
 
 import javax.swing.AbstractCellEditor;
 import javax.swing.BorderFactory;
+import javax.swing.ImageIcon;
 import javax.swing.JButton;
 import javax.swing.JComboBox;
 import javax.swing.JLabel;
@@ -46,24 +47,23 @@ import javax.swing.table.TableCellEditor;
 import javax.swing.table.TableCellRenderer;
 import javax.swing.table.TableColumn;
 
-import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.control.PreGameController;
 import net.sf.freecol.client.gui.Canvas;
 import net.sf.freecol.client.gui.ImageLibrary;
-import net.sf.freecol.client.gui.panel.ColopediaPanel.PanelType;
 import net.sf.freecol.client.gui.i18n.Messages;
-import net.sf.freecol.common.Specification;
+import net.sf.freecol.client.gui.panel.ColopediaPanel.PanelType;
 import net.sf.freecol.common.model.EuropeanNationType;
 import net.sf.freecol.common.model.Nation;
 import net.sf.freecol.common.model.NationOptions;
-import net.sf.freecol.common.model.NationOptions.Advantages;
-import net.sf.freecol.common.model.NationOptions.NationState;
 import net.sf.freecol.common.model.NationType;
 import net.sf.freecol.common.model.Player;
+import net.sf.freecol.common.model.NationOptions.NationState;
+import net.sf.freecol.common.resources.ResourceManager;
 
 
 public final class PlayersTable extends JTable {
 
+    @SuppressWarnings("unused")
     private static final Logger logger = Logger.getLogger(PlayersTable.class.getName());
 
     public static final int NATION_COLUMN = 0, AVAILABILITY_COLUMN = 1, ADVANTAGE_COLUMN = 2,
@@ -78,9 +78,6 @@ public final class PlayersTable extends JTable {
     };
 
 
-    private static final EuropeanNationType[] europeans = 
-        FreeCol.getSpecification().getEuropeanNationTypes().toArray(new EuropeanNationType[0]);
-
     private static final NationState[] allStates = new NationState[] {
         NationState.AVAILABLE,
         NationState.AI_ONLY,
@@ -145,14 +142,12 @@ public final class PlayersTable extends JTable {
         
         TableColumn advantagesColumn = getColumnModel().getColumn(ADVANTAGE_COLUMN);
         if (nationOptions.getNationalAdvantages() == NationOptions.Advantages.SELECTABLE) {
-            advantagesColumn.setCellEditor(new AdvantageCellEditor());
+            advantagesColumn.setCellEditor(new AdvantageCellEditor(canvas.getSpecification().getEuropeanNationTypes()));
         }
         advantagesColumn.setCellRenderer(new AdvantageCellRenderer(nationOptions.getNationalAdvantages()));
         advantagesColumn.setHeaderRenderer(renderer);
 
         TableColumn colorsColumn = getColumnModel().getColumn(COLOR_COLUMN);
-        ColorCellEditor colorCellEditor = new ColorCellEditor(canvas);
-        colorsColumn.setCellEditor(colorCellEditor);
         colorsColumn.setCellRenderer(new ColorCellRenderer(true));
 
         TableColumn playerColumn = getColumnModel().getColumn(PLAYER_COLUMN);
@@ -232,8 +227,8 @@ public final class PlayersTable extends JTable {
                                                        boolean hasFocus, int row, int column) {
 
             Nation nation = (Nation) value;
-            setText(Messages.getName(nation));
-            setIcon(library.getScaledImageIcon(library.getCoatOfArmsImageIcon(nation), 0.5f));
+            setText(Messages.message(nation.getNameKey()));
+            setIcon(new ImageIcon(library.getCoatOfArmsImage(nation, 0.5)));
             return this;
         }
     }
@@ -335,9 +330,9 @@ public final class PlayersTable extends JTable {
                     }
                 }
                 Nation nation = (Nation) table.getValueAt(row, NATION_COLUMN);
-                label.setText(nation.getRulerName());
+                label.setText(Messages.message(nation.getRulerNameKey()));
             } else {
-                label.setText(player.getName());
+                label.setText(player.getDisplayName());
             }
             return label;
         }
@@ -393,7 +388,7 @@ public final class PlayersTable extends JTable {
         public PlayersTableModel(PreGameController pgc, NationOptions nationOptions, Player owningPlayer) {
             nations = new ArrayList<Nation>();
             players = new HashMap<Nation, Player>();
-            for (Nation nation : Specification.getSpecification().getNations()) {
+            for (Nation nation : owningPlayer.getSpecification().getNations()) {
                 NationState state = nationOptions.getNations().get(nation);
                 if (state != null) {
                     nations.add(nation);
@@ -487,11 +482,7 @@ public final class PlayersTable extends JTable {
                         return players.get(nation).getNationType();
                     }
                 case COLOR_COLUMN:
-                    if (players.get(nation) == null) {
-                        return nation.getColor();
-                    } else {
-                        return players.get(nation).getColor();
-                    }
+                    return ResourceManager.getColor(nation.getId() + ".color");
                 case PLAYER_COLUMN:
                     return players.get(nation);
                 }
@@ -542,14 +533,10 @@ public final class PlayersTable extends JTable {
                     preGameController.setAvailable(nations.get(row), (NationState) value);
                     update();
                     break;
-                case COLOR_COLUMN:
-                    preGameController.setColor((Color) value);
-                    break;
                 case PLAYER_COLUMN:
                     Nation nation = nations.get(row);
                     if (nationOptions.getNationState(nation) == NationState.AVAILABLE) {
                         preGameController.setNation(nation);
-                        preGameController.setColor(nation.getColor());
                         preGameController.setNationType(nation.getType());
                         update();
                     }
