@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -21,6 +21,7 @@ package net.sf.freecol.client.gui.panel;
 
 import java.awt.Component;
 import java.awt.GridLayout;
+import java.awt.Image;
 import java.awt.datatransfer.DataFlavor;
 import java.awt.datatransfer.Transferable;
 import java.awt.datatransfer.UnsupportedFlavorException;
@@ -36,6 +37,7 @@ import java.util.logging.Logger;
 
 import javax.swing.BorderFactory;
 import javax.swing.DefaultListModel;
+import javax.swing.ImageIcon;
 import javax.swing.JButton;
 import javax.swing.JComboBox;
 import javax.swing.JComponent;
@@ -44,24 +46,26 @@ import javax.swing.JList;
 import javax.swing.JPanel;
 import javax.swing.JScrollPane;
 import javax.swing.JTextField;
-import javax.swing.ListSelectionModel;
+import javax.swing.ListCellRenderer;
 import javax.swing.TransferHandler;
 import javax.swing.event.ListSelectionEvent;
 import javax.swing.event.ListSelectionListener;
+import javax.swing.plaf.PanelUI;
 
-import net.sf.freecol.FreeCol;
+import net.miginfocom.swing.MigLayout;
 import net.sf.freecol.client.gui.Canvas;
 import net.sf.freecol.client.gui.i18n.Messages;
+import net.sf.freecol.client.gui.plaf.FreeColSelectedPanelUI;
 import net.sf.freecol.common.model.Colony;
+import net.sf.freecol.common.model.Europe;
 import net.sf.freecol.common.model.GoodsType;
 import net.sf.freecol.common.model.Location;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Settlement;
+import net.sf.freecol.common.model.StringTemplate;
 import net.sf.freecol.common.model.TradeRoute;
 import net.sf.freecol.common.model.TradeRoute.Stop;
 
-import net.miginfocom.swing.MigLayout;
-
 
 /**
  * Allows the user to edit trade routes.
@@ -70,14 +74,10 @@ public final class TradeRouteInputDialog extends FreeColDialog<Boolean> implemen
 
     private static final Logger logger = Logger.getLogger(TradeRouteInputDialog.class.getName());
 
-    private static final int OK = 0, CANCEL = 1;
+    public static final DataFlavor STOP_FLAVOR = new DataFlavor(Stop.class, "Stop");
 
     private TradeRoute originalRoute;
 
-    private final JButton ok = new JButton(Messages.message("ok"));
-
-    private final JButton cancel = new JButton(Messages.message("cancel"));
-
     private final JButton addStopButton = new JButton(Messages.message("traderouteDialog.addStop"));
 
     private final JButton removeStopButton = new JButton(Messages.message("traderouteDialog.removeStop"));
@@ -109,7 +109,7 @@ public final class TradeRouteInputDialog extends FreeColDialog<Boolean> implemen
 
     /**
      * The constructor that will add the items to this panel.
-     * 
+     *
      * @param parent The parent of this panel.
      */
     public TradeRouteInputDialog(final Canvas parent, TradeRoute newRoute) {
@@ -117,20 +117,13 @@ public final class TradeRouteInputDialog extends FreeColDialog<Boolean> implemen
 
         originalRoute = newRoute;
 
-        ok.setActionCommand(String.valueOf(OK));
-        ok.addActionListener(this);
-        enterPressesWhenFocused(ok);
-
-        cancel.setActionCommand(String.valueOf(CANCEL));
-        cancel.addActionListener(this);
-        enterPressesWhenFocused(cancel);
-        setCancelComponent(cancel);
-
         goodsPanel = new GoodsPanel();
         goodsPanel.setTransferHandler(cargoHandler);
         cargoPanel = new CargoPanel();
         cargoPanel.setTransferHandler(cargoHandler);
 
+        stopList.setCellRenderer(new StopRenderer());
+        stopList.setFixedCellHeight(48);
         stopList.addListSelectionListener(new ListSelectionListener() {
             public void valueChanged(ListSelectionEvent e) {
                 updateButtons();
@@ -142,23 +135,29 @@ public final class TradeRouteInputDialog extends FreeColDialog<Boolean> implemen
             public void actionPerformed(ActionEvent e) {
                 int startIndex = -1;
                 int endIndex = -1;
-                if (destinationSelector.getSelectedIndex() == 0 ) {   // All colonies + Europe
+                if (destinationSelector.getSelectedIndex() == 0 ) {
+                    // All colonies + Europe
                     startIndex = 1;
-                    endIndex = destinationSelector.getItemCount() - 1;  // will use inverted order
+                    endIndex = destinationSelector.getItemCount() - 1;
                 } else {
-                    startIndex = destinationSelector.getSelectedIndex();  // just 1 colony
+                    // just 1 colony
+                    startIndex = destinationSelector.getSelectedIndex();
                     endIndex = startIndex;
                 }
+                List<GoodsType> cargo = new ArrayList<GoodsType>();
+                for (Component comp : cargoPanel.getComponents()) {
+                    CargoLabel label = (CargoLabel) comp;
+                    cargo.add(label.getType());
+                }
+                int maxIndex = stopList.getMaxSelectionIndex();
                 for (int i = startIndex; i <= endIndex; i++) {
-                    Stop stop = originalRoute.new Stop((Location) destinationSelector.getItemAt(i) );
-                    for (Component comp : cargoPanel.getComponents()) {
-                        CargoLabel label = (CargoLabel) comp;
-                        stop.addCargo(label.getType());
-                    }
-                    if (stopList.getSelectedIndex() == -1) {
+                    Stop stop = originalRoute.new Stop((Location) destinationSelector.getItemAt(i));
+                    stop.setCargo(cargo);
+                    if (maxIndex < 0) {
                         listModel.addElement(stop);
                     } else {
-                        listModel.add(stopList.getSelectedIndex() + 1, stop);
+                        maxIndex++;
+                        listModel.add(maxIndex, stop);
                     }
                 }
             }
@@ -167,41 +166,50 @@ public final class TradeRouteInputDialog extends FreeColDialog<Boolean> implemen
         // button for deleting Stop
         removeStopButton.addActionListener(new ActionListener() {
             public void actionPerformed(ActionEvent e) {
-                listModel.removeElement(stopList.getSelectedValue());
+                int count = 0;
+                for (int index : stopList.getSelectedIndices()) {
+                    listModel.remove(index - count);
+                    count++;
+                }
             }
         });
 
-        // TODO: allow reordering of stops
-        stopList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
         stopList.setDragEnabled(true);
+        stopList.setTransferHandler(new StopHandler());
         stopList.addListSelectionListener(new ListSelectionListener() {
             public void valueChanged(ListSelectionEvent e) {
-                cargoPanel.initialize((Stop) stopList.getSelectedValue());
+                if (!e.getValueIsAdjusting()) {
+                    int[] indices = stopList.getSelectedIndices();
+                    if (indices.length > 0) {
+                        cargoPanel.initialize((Stop) listModel.get(indices[0]));
+                    }
+                }
             }
         });
 
-        setLayout(new MigLayout("wrap 3", "[][fill][fill]", ""));
+        setLayout(new MigLayout("wrap 4, fill", "[]20[fill]rel"));
 
         add(getDefaultHeader(Messages.message("traderouteDialog.editRoute")),
             "span, align center");
         add(tradeRouteView, "span 1 5, grow");
         add(nameLabel);
-        add(tradeRouteName);        
+        add(tradeRouteName, "span");
         add(destinationLabel);
-        add(destinationSelector);
-        add(goodsPanel, "span");
-        add(cargoPanel, "span, height 80:");
-        add(addStopButton);
+        add(destinationSelector, "span");
+        add(addStopButton, "skip 2");
         add(removeStopButton);
-        add(ok, "newline 20, span, split 2, tag ok");
-        add(cancel, "tag cancel");
+        add(goodsPanel, "span");
+        add(cargoPanel, "span, height 80:, growy");
+        add(okButton, "newline 20, span, split 2, tag ok");
+        add(cancelButton, "tag cancel");
 
         TradeRoute tradeRoute = newRoute.clone();
 
         Player player = getMyPlayer();
 
         // combo box for selecting destination
-        destinationSelector.addItem(Messages.message("report.allColonies", "%number%", ""));
+        destinationSelector.addItem(Messages.message(StringTemplate.template("report.allColonies")
+                                                     .addName("%number%", "")));
         if (player.getEurope() != null) {
             destinationSelector.addItem(player.getEurope());
         }
@@ -234,7 +242,7 @@ public final class TradeRouteInputDialog extends FreeColDialog<Boolean> implemen
         // set name of trade route
         tradeRouteName.setText(tradeRoute.getName());
 
-        setSize(getPreferredSize());
+        restoreSavedSize(getPreferredSize());
 
     }
 
@@ -243,77 +251,80 @@ public final class TradeRouteInputDialog extends FreeColDialog<Boolean> implemen
      * otherwise.
      */
     public void updateButtons() {
-        if (stopList.getSelectedIndex() == -1) {
+        if (stopList.getSelectedIndices().length == 0) {
             removeStopButton.setEnabled(false);
         } else {
             removeStopButton.setEnabled(true);
         }
     }
 
-    public void requestFocus() {
-        ok.requestFocus();
-    }
-    
-    private boolean verifyNewTradeRoute(){
-    	// Verify that it has at least two stops
-    	if(listModel.getSize() < 2){
-    		 getCanvas().errorMessage("traderouteDialog.notEnoughStops");
-    		 return false;
-    	}
-    	
-    	Player player = getCanvas().getClient().getMyPlayer();
-        for (int index = 0; index < listModel.getSize(); index++) {
-            Stop stop = (Stop) listModel.getElementAt(index);
-            if(!TradeRoute.isStopValid(player, stop)){
-            	return false;
+    /**
+     * Check that the trade route is valid.
+     *
+     * @return True if the trade route is valid.
+     */
+    private boolean verifyNewTradeRoute() {
+        Player player = getCanvas().getClient().getMyPlayer();
+
+        // Check that the name is unique
+        for (TradeRoute route : player.getTradeRoutes()) {
+            if (route.getId().equals(originalRoute.getId())) continue;
+            if (route.getName().equals(tradeRouteName.getText())) {
+                getCanvas().errorMessage("traderouteDialog.duplicateName");
+                return false;
             }
         }
-        
-    	return true;
+
+        // Verify that it has at least two stops
+        if (listModel.getSize() < 2) {
+            getCanvas().errorMessage("traderouteDialog.notEnoughStops");
+            return false;
+        }
+
+        // Check that all stops are valid
+        for (int index = 0; index < listModel.getSize(); index++) {
+            Stop stop = (Stop) listModel.get(index);
+            if (!TradeRoute.isStopValid(player, stop)) {
+                return false;
+            }
+        }
+
+        return true;
     }
 
     /**
      * This function analyses an event and calls the right methods to take care
      * of the user's requests.
-     * 
+     *
      * @param event The incoming ActionEvent.
      */
     public void actionPerformed(ActionEvent event) {
         String command = event.getActionCommand();
-        try {
-            switch (Integer.valueOf(command).intValue()) {
-            case OK:
-            	
-            	if(! verifyNewTradeRoute())
-            		break;
-            	
+        if (OK.equals(command)) {
+            if (verifyNewTradeRoute()) {
                 getCanvas().remove(this);
                 originalRoute.setName(tradeRouteName.getText());
                 ArrayList<Stop> stops = new ArrayList<Stop>();
                 for (int index = 0; index < listModel.getSize(); index++) {
-                    stops.add((Stop) listModel.getElementAt(index));
+                    stops.add((Stop) listModel.get(index));
                 }
                 originalRoute.setStops(stops);
                 // TODO: update trade routes only if they have been modified
                 getController().updateTradeRoute(originalRoute);
-                setResponse(new Boolean(true));
-                break;
-            case CANCEL:
-                getCanvas().remove(this);
-                setResponse(new Boolean(false));
-                break;
-            default:
-                logger.warning("Invalid ActionCommand: invalid number.");
+                setResponse(Boolean.TRUE);
             }
-        } catch (NumberFormatException e) {
-            logger.warning("Invalid Actioncommand: not a number.");
+        } else if (CANCEL.equals(command)) {
+            getCanvas().remove(this);
+            setResponse(Boolean.FALSE);
+        } else {
+            super.actionPerformed(event);
         }
     }
 
 
     /**
      * Special label for goods type.
-     * 
+     *
      * TODO: clean this up for 0.7.0 -- The GoodsLabel needs to be modified so
      * that it can act as a GoodsTypeLabel (like this label), an
      * AbstractGoodsLabel and a CargoLabel (its current function).
@@ -341,8 +352,8 @@ public final class TradeRouteInputDialog extends FreeColDialog<Boolean> implemen
     public class GoodsPanel extends JPanel {
 
         public GoodsPanel() {
-            super(new GridLayout(0, 5, margin, margin));
-            for (GoodsType goodsType : FreeCol.getSpecification().getGoodsTypeList()) {
+            super(new GridLayout(0, 4, margin, margin));
+            for (GoodsType goodsType : getSpecification().getGoodsTypeList()) {
                 if (goodsType.isStorable()) {
                     CargoLabel label = new CargoLabel(goodsType);
                     add(label);
@@ -358,14 +369,12 @@ public final class TradeRouteInputDialog extends FreeColDialog<Boolean> implemen
     /**
      * Panel for the cargo the carrier is supposed to take on board at a certain
      * stop.
-     * 
+     *
      * TODO: create a single cargo panel for this purpose and the use in the
      * ColonyPanel, the EuropePanel and the CaptureGoodsDialog.
      */
     public class CargoPanel extends JPanel {
 
-        // private Stop stop;
-
         public CargoPanel() {
             super();
             setOpaque(false);
@@ -388,7 +397,7 @@ public final class TradeRouteInputDialog extends FreeColDialog<Boolean> implemen
 
     /**
      * TransferHandler for CargoLabels.
-     * 
+     *
      * TODO: check whether this could/should be folded into the
      * DefaultTransferHandler.
      */
@@ -410,11 +419,14 @@ public final class TradeRouteInputDialog extends FreeColDialog<Boolean> implemen
                         CargoLabel newLabel = new CargoLabel(label.getType());
                         cargoPanel.add(newLabel);
                         cargoPanel.revalidate();
-                        Stop stop = (Stop) stopList.getSelectedValue();
-                        if (stop != null) {
+                        int[] indices = stopList.getSelectedIndices();
+                        for (int index : indices) {
+                            Stop stop = (Stop) listModel.get(index);
                             stop.addCargo(label.getType());
                             stop.setModified(true);
                         }
+                        stopList.revalidate();
+                        stopList.repaint();
                     }
                     return true;
                 } catch (UnsupportedFlavorException ufe) {
@@ -432,9 +444,10 @@ public final class TradeRouteInputDialog extends FreeColDialog<Boolean> implemen
                 CargoLabel label = (CargoLabel) data.getTransferData(DefaultTransferHandler.flavor);
                 if (source.getParent() instanceof CargoPanel) {
                     cargoPanel.remove(label);
-                    Stop stop = (Stop) stopList.getSelectedValue();
-                    if (stop != null) {
-                        ArrayList<GoodsType> cargo = stop.getCargo();
+                    int[] indices = stopList.getSelectedIndices();
+                    for (int stopIndex : indices) {
+                        Stop stop = (Stop) listModel.get(stopIndex);
+                        ArrayList<GoodsType> cargo = new ArrayList<GoodsType>(stop.getCargo());
                         for (int index = 0; index < cargo.size(); index++) {
                             if (cargo.get(index) == label.getType()) {
                                 cargo.remove(index);
@@ -444,6 +457,8 @@ public final class TradeRouteInputDialog extends FreeColDialog<Boolean> implemen
                         }
                         stop.setCargo(cargo);
                     }
+                    stopList.revalidate();
+                    stopList.repaint();
                     cargoPanel.revalidate();
                     cargoPanel.repaint();
                 }
@@ -464,23 +479,152 @@ public final class TradeRouteInputDialog extends FreeColDialog<Boolean> implemen
         }
     }
 
+    public class StopTransferable implements Transferable {
+
+        private List<Stop> stops;
+
+        public StopTransferable(List<Stop> stops) {
+            this.stops = stops;
+        }
+
+        public Object getTransferData(DataFlavor flavor) {
+            return stops;
+        }
+
+        public DataFlavor[] getTransferDataFlavors() {
+            return new DataFlavor[] { STOP_FLAVOR };
+        }
+
+        public boolean isDataFlavorSupported(DataFlavor flavor) {
+            return flavor == STOP_FLAVOR;
+        }
+    }
+
     /*
-     * public class StopPanel extends JPanel {
-     * 
-     * private Location location; private ArrayList<Integer> goods = new
-     * ArrayList<Integer>(); private final JComboBox destinationBox; private
-     * final JPanel goodsPanel; private final
-     * 
-     * public void saveSettings() { if (export.isSelected() !=
-     * colony.getExports(goodsType)) {
-     * getController().setExports(colony,
-     * goodsType, export.isSelected()); colony.setExports(goodsType,
-     * export.isSelected()); } colony.getLowLevel()[goodsType] =
-     * ((SpinnerNumberModel) lowLevel.getModel()).getNumber().intValue();
-     * colony.getHighLevel()[goodsType] = ((SpinnerNumberModel)
-     * highLevel.getModel()).getNumber().intValue();
-     * colony.getExportLevel()[goodsType] = ((SpinnerNumberModel)
-     * exportLevel.getModel()).getNumber().intValue();
-     *  } }
+     * TransferHandler for Stops.
      */
+    public class StopHandler extends TransferHandler {
+
+        protected Transferable createTransferable(JComponent c) {
+            JList list = (JList) c;
+            DefaultListModel model = (DefaultListModel) list.getModel();
+            List<Stop> stops = new ArrayList<Stop>();
+            for (int index : list.getSelectedIndices()) {
+                stops.add((Stop) model.get(index));
+            }
+            return new StopTransferable(stops);
+        }
+
+        public int getSourceActions(JComponent c) {
+            return MOVE;
+        }
+
+        public boolean importData(JComponent target, Transferable data) {
+            if (canImport(target, data.getTransferDataFlavors())) {
+                try {
+                    List stops = (List) data.getTransferData(STOP_FLAVOR);
+                    if (target instanceof JList) {
+                        JList list = (JList) target;
+                        DefaultListModel model = (DefaultListModel) list.getModel();
+                        int index = list.getMaxSelectionIndex();
+                        for (Object o : stops) {
+                            Stop stop = originalRoute.new Stop((Stop) o);
+                            if (index < 0) {
+                                model.addElement(stop);
+                            } else {
+                                index++;
+                                model.add(index, stop);
+                            }
+                        }
+                    }
+                    return true;
+                } catch (UnsupportedFlavorException ufe) {
+                    logger.warning(ufe.toString());
+                } catch (IOException ioe) {
+                    logger.warning(ioe.toString());
+                }
+            }
+
+            return false;
+        }
+
+        protected void exportDone(JComponent source, Transferable data, int action) {
+            try {
+                if (source instanceof JList && action == MOVE) {
+                    JList stopList = (JList) source;
+                    DefaultListModel listModel = (DefaultListModel) stopList.getModel();
+                    for (Object o : (List) data.getTransferData(STOP_FLAVOR)) {
+                        listModel.removeElement(o);
+                    }
+                }
+            } catch (Exception e) {
+                logger.warning(e.toString());
+            }
+        }
+
+        public boolean canImport(JComponent c, DataFlavor[] flavors) {
+            for (int i = 0; i < flavors.length; i++) {
+                if (flavors[i].equals(STOP_FLAVOR)) {
+                    return true;
+                }
+            }
+            return false;
+        }
+    }
+
+    private class StopRenderer implements ListCellRenderer {
+
+        private final JPanel SELECTED_COMPONENT = new JPanel();
+        private final JPanel NORMAL_COMPONENT = new JPanel();
+
+        public StopRenderer() {
+            NORMAL_COMPONENT.setLayout(new MigLayout("", "[80, center][]"));
+            NORMAL_COMPONENT.setOpaque(false);
+            SELECTED_COMPONENT.setLayout(new MigLayout("", "[80, center][]"));
+            SELECTED_COMPONENT.setOpaque(false);
+            SELECTED_COMPONENT.setUI((PanelUI) FreeColSelectedPanelUI.createUI(SELECTED_COMPONENT));
+        }
+
+        /**
+         * Returns a <code>ListCellRenderer</code> for the given <code>JList</code>.
+         *
+         * @param list The <code>JList</code>.
+         * @param value The list cell.
+         * @param index The index in the list.
+         * @param isSelected <code>true</code> if the given list cell is selected.
+         * @param hasFocus <code>false</code> if the given list cell has the focus.
+         * @return The <code>ListCellRenderer</code>
+         */
+        public Component getListCellRendererComponent(JList list, Object value, int index,
+                                                      boolean isSelected, boolean hasFocus) {
+
+            JPanel panel = (isSelected ? SELECTED_COMPONENT : NORMAL_COMPONENT);
+            panel.removeAll();
+            panel.setForeground(list.getForeground());
+            panel.setFont(list.getFont());
+            Stop stop = (Stop) value;
+            Location location = stop.getLocation();
+            JLabel icon, name;
+            if (location instanceof Europe) {
+                Europe europe = (Europe) location;
+                Image image = getLibrary().getCoatOfArmsImage(europe.getOwner().getNation(), 0.5);
+                icon = new JLabel(new ImageIcon(image));
+                name = localizedLabel(europe.getNameKey());
+            } else if (location instanceof Colony) {
+                Colony colony = (Colony) location;
+                icon = new JLabel(new ImageIcon(getLibrary().getSettlementImage(colony, 0.5)));
+                name = new JLabel(colony.getName());
+            } else {
+                throw new IllegalStateException("Bogus location: " + location);
+            }
+            panel.add(icon, "spany");
+            panel.add(name, "span, wrap");
+            for (GoodsType cargo : stop.getCargo()) {
+                panel.add(new JLabel(new ImageIcon(getLibrary().getGoodsImage(cargo, 0.5))));
+            }
+            return panel;
+        }
+
+    }
+
 }
