@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -41,16 +41,7 @@ public class FreeColComboBoxUI extends MetalComboBoxUI {
         c.setOpaque(false);
     }
 
-
-    /*protected  JButton createArrowButton() {
-        JButton button = super.createArrowButton();
-
-        // TODO: Make button prettier?
-
-        return button;
-    }*/
-    
     protected ListCellRenderer createRenderer() {
-        return new FreeColComboBoxRenderer.UIResource();
+        return new FreeColComboBoxRenderer();
     }
 }
