@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -19,25 +19,23 @@
 
 package net.sf.freecol.client.gui.plaf;
 
-import java.awt.Image;
 
 import javax.swing.JComponent;
 import javax.swing.plaf.ComponentUI;
 import javax.swing.plaf.basic.BasicPanelUI;
 
-import net.sf.freecol.common.resources.ResourceManager;
+import net.sf.freecol.client.gui.ImageLibrary;
 
 
 /**
-* Draws the image "BackgroundImage" from the defaults table as a tiled
-* background image.
-*/
+ * Draw the "background.FreeColPanel" resource as a tiled background image.
+ */
 public class FreeColPanelUI extends BasicPanelUI {
-    
+
     private static FreeColPanelUI sharedInstance = new FreeColPanelUI();
-    
-    private FreeColPanelUI() {
-    }
+
+
+    private FreeColPanelUI() {}
 
     public static ComponentUI createUI(JComponent c) {
         return sharedInstance;
@@ -45,21 +43,7 @@ public class FreeColPanelUI extends BasicPanelUI {
 
     public void paint(java.awt.Graphics g, javax.swing.JComponent c) {
         if (c.isOpaque()) {
-            int width = c.getWidth();
-            int height = c.getHeight();
-
-            Image tempImage = ResourceManager.getImage("BackgroundImage");
-            
-            if (tempImage != null) {
-                for (int x=0; x<width; x+=tempImage.getWidth(null)) {
-                    for (int y=0; y<height; y+=tempImage.getHeight(null)) {
-                        g.drawImage(tempImage, x, y, null);
-                    }
-                }
-            } else {
-                g.setColor(c.getBackground());
-                g.fillRect(0, 0, width, height);
-            }
+            ImageLibrary.drawTiledImage("background.FreeColPanel", g, c, null);
         }
     }
 }
