@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -23,8 +23,8 @@ package net.sf.freecol.common.model;
 
 import java.util.ArrayList;
 import java.util.Collections;
+import java.util.HashSet;
 import java.util.Iterator;
-import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Set;
 import java.util.logging.Logger;
@@ -34,43 +34,51 @@ import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
-import net.sf.freecol.FreeCol;
 
 import org.w3c.dom.Element;
 
 /**
-* Represents a work location on a tile.
-*/
-public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownable {
+ * Represents a work location on a tile. Each ColonyTile except the
+ * colony center tile provides a work place for a single unit and
+ * produces a single type of goods. The colony center tile generally
+ * produces two different of goods, one food type and one new world
+ * raw material.
+ */
+public class ColonyTile extends FreeColGameObject
+    implements WorkLocation, Ownable {
 
+    @SuppressWarnings("unused")
     private static final Logger logger = Logger.getLogger(ColonyTile.class.getName());
 
     public static final String UNIT_CHANGE = "UNIT_CHANGE";
 
-    private Colony colony;
-    private Tile workTile;
-    private Unit unit;
-    private boolean colonyCenterTile;
+    /** The colony this colony tile belongs to. */
+    protected Colony colony;
+
+    /** The tile to work. */
+    protected Tile workTile;
+
+    /** The unit working the tile. */
+    protected Unit unit;
+
+    /** Is this colony tile at the center of the colony. */
+    protected boolean colonyCenterTile;
+
 
     /**
-    * Creates a new <code>ColonyTile</code>.
-    *
-    * @param game The <code>Game</code> this object belongs to.
-    * @param colony The <code>Colony</code> this object belongs to.
-    * @param workTile The tile in which this <code>ColonyTile</code> represents a
-    *                 <code>WorkLocation</code> for.
-    */
-    public ColonyTile(Game game, Colony colony, Tile workTile) {
+     * Constructor for ServerColonyTile.
+     */
+    protected ColonyTile() {
+        // empty constructor
+    }
+
+    /**
+     * Constructor for ServerColonyTile.
+     *
+     * @param game The <code>Game</code> this object belongs to.
+     */
+    public ColonyTile(Game game) {
         super(game);
-
-        this.colony = colony;
-        this.workTile = workTile;
-
-        if (colony.getTile() == workTile) {
-            colonyCenterTile = true;
-        } else {
-            colonyCenterTile = false;
-        }
     }
 
     /**
@@ -86,7 +94,7 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
 
         readFromXML(in);
     }
-    
+
     /**
      * Initiates a new <code>Building</code> from an
      * XML representation.
@@ -102,7 +110,7 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
     }
 
     /**
-     * Initiates a new <code>ColonyTile</code> 
+     * Initiates a new <code>ColonyTile</code>
      * with the given ID. The object should later be
      * initialized by calling either
      * {@link #readFromXML(XMLStreamReader)} or
@@ -115,6 +123,41 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
         super(game, id);
     }
 
+    /**
+     * Returns the (non-unique) name of this <code>ColonyTile</code>.
+     * @return The name of this ColonyTile.
+     */
+    public StringTemplate getLocationName() {
+        String name = getColony().getName();
+        if (isColonyCenterTile()) {
+            return StringTemplate.name(name);
+        } else {
+            return StringTemplate.template("nearLocation")
+                .addName("%location%", name);
+        }
+    }
+
+    /**
+     * Returns the name of this ColonyTile for a particular player.
+     *
+     * @param player The <code>Player</code> to prepare the name for.
+     * @return The name of this ColonyTile.
+     */
+    public StringTemplate getLocationNameFor(Player player) {
+        return getLocationName();
+    }
+
+    /**
+     * Returns a description of the tile, with the name of the tile
+     * and any improvements made to it (road/plow).
+     *
+     * @return The description label for this tile
+     */
+    public StringTemplate getLabel() {
+        return workTile.getLabel();
+    }
+
+
     /**
     * Gets the owner of this <code>Ownable</code>.
     *
@@ -145,77 +188,71 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
         return colonyCenterTile;
     }
 
-
     /**
-    * Gets the work tile.
-    *
-    * @return The tile in which this <code>ColonyTile</code> represents a
-    *         <code>WorkLocation</code> for.
-    */
+     * Gets the work tile.
+     *
+     * @return The tile in which this <code>ColonyTile</code> represents a
+     *         <code>WorkLocation</code> for.
+     */
     public Tile getWorkTile() {
         return workTile;
     }
 
-
     /**
-    * Gets the tile where the colony is located.
-    * @return The <code>Tile</code>.
-    */
+     * Gets the tile where the colony is located.
+     * @return The <code>Tile</code>.
+     */
     public Tile getTile() {
         return colony.getTile();
     }
 
-    
     public GoodsContainer getGoodsContainer() {
         return null;
     }
-    
+
 
     /**
-    * Gets the <code>Unit</code> currently working on this <code>ColonyTile</code>.
-    *
-    * @return The <code>Unit</code> or <i>null</i> if no unit is present.
-    * @see #setUnit
-    */
-    public Unit getUnit() {
-        return unit;
+     * Gets a pointer to the settlement containing this tile.
+     *
+     * @return This settlement.
+     */
+    public Settlement getSettlement() {
+        return colony;
     }
-    
+
     /**
-    * Gets a pointer to the colony containing this tile.
-    * @return The <code>Colony</code>.
-    */
+     * Gets a pointer to the colony containing this tile.
+     *
+     * @return The <code>Colony</code>.
+     */
     public Colony getColony() {
         return colony;
     }
 
     /**
-    * Sets a <code>Unit</code> to this <code>ColonyTile</code>.
-    *
-    * @param unit The <code>Unit</code>.
-    * @see #getUnit
-    */
-    public void setUnit(Unit unit) {
-        Unit oldUnit = getUnit();
-        this.unit = unit;
-        if (oldUnit != null) {
-            GoodsType workType = oldUnit.getWorkType();
-            firePropertyChange(workType.getId(), getProductionOf(oldUnit, workType), null);
-        }
-        if (unit != null) {
-            GoodsType workType = unit.getWorkType();
-            // SOMEHOW, workType was null in unit tests
-            if (workType != null) {
-                firePropertyChange(workType.getId(), null, getProductionOf(unit, workType));
-            }
-        }
+     * Gets the <code>Unit</code> currently working on this <code>ColonyTile</code>.
+     *
+     * @return The <code>Unit</code> or <i>null</i> if no unit is present.
+     * @see #setUnit
+     */
+    public Unit getUnit() {
+        return unit;
     }
 
+    /**
+     * Sets a <code>Unit</code> to this <code>ColonyTile</code>.
+     *
+     * @param unit The <code>Unit</code>.
+     * @see #getUnit
+     */
+    public void setUnit(Unit unit) {
+        this.unit = unit;
+    }
 
     /**
-    * Gets the amount of Units at this <code>ColonyTile</code>.
-    * @return The amount of Units at this <code>ColonyTile</code>.
-    */
+     * Gets the amount of Units at this <code>ColonyTile</code>.
+     * @return The amount of Units at this <code>ColonyTile</code>.
+     */
     public int getUnitCount() {
         return (getUnit() != null) ? 1 : 0;
     }
@@ -229,7 +266,7 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
         if (getUnit() != null) {
             for (WorkLocation wl : getColony().getWorkLocations()) {
                 if (wl != this && wl.canAdd(getUnit())) {
-                    getUnit().work(wl);
+                    getUnit().setLocation(wl);
                     break;
                 }
             }
@@ -247,37 +284,16 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
         Player player = getOwner();
 
         // Not workable if there is a settlement, hostile occupation,
-        // unable to work water, or lost city rumour to Europeans.
+        // or can not be claimed.
         Tile tile = getWorkTile();
         if (tile.getSettlement() != null
             || tile.getOccupyingUnit() != null
-            || !(tile.isLand() || colony.hasAbility("model.ability.produceInWater"))
-            || (player.isEuropean() && tile.hasLostCityRumour())) {
+            || (tile.getOwningSettlement() != colony
+                && !player.canClaimForSettlement(tile))
+            || (!tile.isLand()
+                && !colony.hasAbility("model.ability.produceInWater"))) {
             return false;
         }
-
-        // Special cases when tile owned by another settlement.
-        Settlement settlement = tile.getOwningSettlement();
-        if (settlement == null) {
-            ; // OK
-        } else if (settlement instanceof Colony) {
-            // Disallow if owned by other Europeans or in active use.
-            Colony otherColony = (Colony) settlement;
-            if (otherColony != colony) {
-                if (otherColony.getOwner() != player
-                    || otherColony.getColonyTile(tile).getUnit() != null) {
-                    return false;
-                }
-            }
-        } else if (settlement instanceof IndianSettlement) {
-            // Disallow if owned and valued by natives.
-            if (player.getLandPrice(tile) > 0) {
-                return false;
-            }
-        } else {
-            throw new IllegalStateException("Bogus settlement");
-        }
-
         return true;
     }
 
@@ -306,54 +322,75 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
 
 
     /**
-    * Add the specified <code>Locatable</code> to this <code>WorkLocation</code>.
-    * @param locatable The <code>Locatable</code> that shall be added to this <code>WorkLocation</code>.
-    */
-    public void add(Locatable locatable) {
-        if (isColonyCenterTile() || unit != null) {
-            throw new IllegalStateException("Other unit present while adding a unit to ColonyTile:" + getId());
+     * Add the specified locatable to this colony tile.
+     *
+     * @param locatable The <code>Locatable</code> to add.
+     */
+    public void add(final Locatable locatable) {
+        if (isColonyCenterTile()) {
+            throw new IllegalStateException("Can not add to colony center: "
+                                            + toString());
+        } else if (unit != null) {
+            throw new IllegalStateException("Can not add to occupied tile: "
+                                            + toString());
+        } else if (getWorkTile().getOwningSettlement() != getColony()) {
+            throw new IllegalStateException("Can not add to tile owned by another colony: "
+                                            + toString());
+        } else if (!canAdd(locatable)) {
+            throw new IllegalStateException("Can not add " + locatable
+                                            + " to " + toString());
         }
 
-        if (!canAdd(locatable)) {
-            if (getWorkTile().getOwningSettlement() != null && getWorkTile().getOwningSettlement() != getColony()) {
-                throw new IllegalArgumentException("Cannot add locatable to this location: another colony claims this land!");
+        final Unit unit = (Unit) locatable;
+        setUnit(unit);
+        unit.setState(Unit.UnitState.IN_COLONY);
+        getColony().invalidateCache();
+
+        // Choose a sensible work type only if none already specified.
+        if (unit.getWorkType() == null) {
+            AbstractGoods goods = workTile.getType().getPrimaryGoods();
+            if (goods == null) {
+                goods = workTile.getType().getSecondaryGoods();
+                if (goods == null
+                    && !workTile.getType().getProduction().isEmpty()) {
+                    goods = workTile.getType().getProduction().get(0);
+                }
             }
-            throw new IllegalArgumentException("Cannot add locatable to this location: there is a unit here already!");
+            if (goods != null) unit.setWorkType(goods.getType());
         }
 
-        Unit u = (Unit) locatable;
-        u.removeAllEquipment(false);
-        u.setState(Unit.UnitState.IN_COLONY);
-
-        // Find a teacher if available.
-        Unit potentialTeacher = getColony().findTeacher(u);
-        if (potentialTeacher != null) {
-            potentialTeacher.setStudent(u);
-            u.setTeacher(potentialTeacher);
+        Unit teacher = unit.getTeacher();
+        if (teacher == null
+            && (teacher = getColony().findTeacher(unit)) != null) {
+            unit.setTeacher(teacher);
+            teacher.setStudent(unit);
         }
-
-        setUnit(u);
     }
-    
 
     /**
-    * Remove the specified <code>Locatable</code> from this <code>WorkLocation</code>.
-    * @param locatable The <code>Locatable</code> that shall be removed from this <code>WorkLocation</code>.
-    */
-    public void remove(Locatable locatable) {
-        if (getUnit() == null) {
+     * Remove the specified locatable from this colony tile.
+     *
+     * @param locatable The <code>Locatable</code> to be removed.
+     */
+    public void remove(final Locatable locatable) {
+        if (getUnit() == null || !getUnit().equals(locatable)) {
             return;
         }
 
-        if (!getUnit().equals(locatable)) {
-            return;
-        }
-
-        Unit oldUnit = getUnit();
-        oldUnit.setMovesLeft(0);
+        final Unit unit = getUnit();
         setUnit(null);
+        unit.setMovesLeft(0);
+        unit.setState(Unit.UnitState.ACTIVE);
+        getColony().invalidateCache();
+
+        Unit teacher = unit.getTeacher();
+        if (teacher != null) {
+            teacher.setStudent(null);
+            unit.setTeacher(null);
+        }
     }
 
+
     public List<Unit> getUnitList() {
         if(getUnit() == null) {
             return new ArrayList<Unit>();
@@ -368,33 +405,33 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
 
 
     /**
-    * Checks if this <code>ColonyTile</code> contains the given <code>Locatable</code>.
-    * 
-    * @param locatable The <code>Locatable</code>.
-    * @return The result.
-    */
+     * Checks if this <code>ColonyTile</code> contains the given <code>Locatable</code>.
+     *
+     * @param locatable The <code>Locatable</code>.
+     * @return The result.
+     */
     public boolean contains(Locatable locatable) {
         return (locatable == unit) ? true:false;
     }
 
 
     /**
-    * Gets the <code>Unit</code> currently working on this <code>ColonyTile</code>.
-    *
-    * @return The <code>Unit</code> or <i>null</i> if no unit is present.
-    * @see #setUnit
-    */
+     * Gets the <code>Unit</code> currently working on this <code>ColonyTile</code>.
+     *
+     * @return The <code>Unit</code> or <i>null</i> if no unit is present.
+     * @see #setUnit
+     */
     public Unit getFirstUnit() {
         return getUnit();
     }
 
 
     /**
-    * Gets the <code>Unit</code> currently working on this <code>ColonyTile</code>.
-    *
-    * @return The <code>Unit</code> or <i>null</i> if no unit is present.
-    * @see #setUnit
-    */
+     * Gets the <code>Unit</code> currently working on this <code>ColonyTile</code>.
+     *
+     * @return The <code>Unit</code> or <i>null</i> if no unit is present.
+     * @see #setUnit
+     */
     public Unit getLastUnit() {
         return getUnit();
     }
@@ -416,39 +453,7 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
     public boolean isOccupied() {
         return workTile.isOccupied();
     }
-    
-    /**
-    * Prepares this <code>ColonyTile</code> for a new turn.
-    */
-    public void newTurn() {
-        if (isColonyCenterTile()) {
-            produceGoodsCenterTile();
-        } else if (getUnit() != null && !isOccupied()) {
-            produceGoods();
-            workTile.expendResource(getUnit().getWorkType(), getUnit().getType(), colony);
-        }
-    }
 
-    private void produceGoods() {
-        int amount = getProductionOf(getUnit().getWorkType());
-
-        if (amount > 0) {
-            colony.addGoods(getUnit().getWorkType(), amount);
-            unit.modifyExperience(amount);
-        }
-    }
-
-    private void produceGoodsCenterTile() {
-        
-        GoodsType goodsFood = workTile.primaryGoods();
-        colony.addGoods(goodsFood, getProductionOf(goodsFood));
-        
-        GoodsType type2 = workTile.secondaryGoods();
-        if (type2 != null)
-                colony.addGoods(type2, getProductionOf(type2));
-
-    }
-   
     /**
      * Returns a worktype for a unit.
      *
@@ -457,9 +462,9 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
      */
     public GoodsType getWorkType(Unit unit) {
         GoodsType workType = unit.getWorkType();
-        int amount = getProductionOf(unit, workType);
+        int amount = (workType == null) ? 0 : getProductionOf(unit, workType);
         if (amount == 0) {
-            List<GoodsType> farmedGoodsTypes = FreeCol.getSpecification().getFarmedGoodsTypeList();
+            List<GoodsType> farmedGoodsTypes = getSpecification().getFarmedGoodsTypeList();
             for(GoodsType farmedGoods : farmedGoodsTypes) {
                 int newAmount = getProductionOf(unit, farmedGoods);
                 if (newAmount > amount) {
@@ -470,7 +475,73 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
         }
         return workType;
     }
-    
+
+    /**
+     * Returns the primary production of a colony center tile. In the
+     * standard rule sets, this is always some kind of food and all
+     * tile improvements contribute to the production.
+     *
+     * @return an <code>AbstractGoods</code> value
+     */
+    private AbstractGoods getPrimaryProduction() {
+        if (workTile.getType().getPrimaryGoods() == null) {
+            return null;
+        } else {
+            AbstractGoods primaryProduction = new AbstractGoods(workTile.getType().getPrimaryGoods());
+            int potential = primaryProduction.getAmount();
+            if (workTile.getTileItemContainer() != null) {
+                potential = workTile.getTileItemContainer()
+                    .getTotalBonusPotential(primaryProduction.getType(), null,
+                                            potential, false);
+            }
+            primaryProduction.setAmount(potential + Math.max(0, getColony().getProductionBonus()));
+            return primaryProduction;
+        }
+    }
+
+    /**
+     * Returns the secondary production of a colony center tile. Only
+     * natural tile improvements, such as rivers, contribute to the
+     * production. Artificial tile improvements, such as plowing, are
+     * ignored.
+     *
+     * @return an <code>int</code> value
+     */
+    private AbstractGoods getSecondaryProduction() {
+        if (workTile.getType().getSecondaryGoods() == null) {
+            return null;
+        } else {
+            AbstractGoods secondaryProduction = new AbstractGoods(workTile.getType().getSecondaryGoods());
+            int potential = secondaryProduction.getAmount();
+            if (workTile.getTileItemContainer() != null) {
+                potential = workTile.getTileItemContainer()
+                    .getTotalBonusPotential(secondaryProduction.getType(), null,
+                                            potential, true);
+            }
+            secondaryProduction.setAmount(potential + Math.max(0, getColony().getProductionBonus()));
+            return secondaryProduction;
+        }
+    }
+
+    public List<AbstractGoods> getProduction() {
+        List<AbstractGoods> result = new ArrayList<AbstractGoods>(2);
+        if (isColonyCenterTile()) {
+            AbstractGoods primaryProduction = getPrimaryProduction();
+            if (primaryProduction != null) {
+                result.add(primaryProduction);
+            }
+            AbstractGoods secondaryProduction = getSecondaryProduction();
+            if (secondaryProduction != null) {
+                result.add(secondaryProduction);
+            }
+        } else if (getUnit() != null) {
+            GoodsType goodsType = getUnit().getWorkType();
+            result.add(new AbstractGoods(goodsType, getProductionOf(getUnit(), goodsType)));
+        }
+        return result;
+    }
+
+
     /**
      * Returns the production of the given type of goods.
      *
@@ -478,16 +549,20 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
      * @return an <code>int</code> value
      */
     public int getProductionOf(GoodsType goodsType) {
-        if (goodsType == null) {
-            throw new IllegalArgumentException("GoodsType must not be 'null'.");
-        } else if (getUnit() == null) {
-            if (isColonyCenterTile() &&
-                (goodsType.isFoodType() || 
-                 goodsType.equals(workTile.secondaryGoods()))) {
-                return workTile.potential(goodsType, null);
+        if (isColonyCenterTile()) {
+            if (workTile.getType().getPrimaryGoods() != null
+                && workTile.getType().getPrimaryGoods().getType() == goodsType) {
+                return getPrimaryProduction().getAmount();
+            } else if (workTile.getType().getSecondaryGoods() != null
+                       && workTile.getType().getSecondaryGoods().getType() == goodsType) {
+                return getSecondaryProduction().getAmount();
             } else {
                 return 0;
             }
+        } else if (goodsType == null) {
+            throw new IllegalArgumentException("GoodsType must not be 'null'.");
+        } else if (getUnit() == null) {
+            return 0;
         } else if (goodsType.equals(getUnit().getWorkType())) {
             return getProductionOf(getUnit(), goodsType);
         } else {
@@ -506,11 +581,11 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
         if (goodsType == null) {
             throw new IllegalArgumentException("GoodsType must not be 'null'.");
         } else {
-            Set<Modifier> result = new LinkedHashSet<Modifier>();
+            Set<Modifier> result = new HashSet<Modifier>();
             if (getUnit() == null) {
                 if (isColonyCenterTile() &&
-                    (goodsType.isFoodType() || 
-                     goodsType.equals(workTile.secondaryGoods()))) {
+                    (workTile.getType().isPrimaryGoodsType(goodsType)
+                     || workTile.getType().isSecondaryGoodsType(goodsType))) {
                     result.addAll(workTile.getProductionBonus(goodsType, null));
                     result.addAll(getColony().getFeatureContainer().getModifierSet(goodsType.getId()));
                 }
@@ -534,40 +609,58 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
         if (unit == null) {
             throw new IllegalArgumentException("Unit must not be 'null'.");
         } else if (workTile.isLand() || colony.hasAbility("model.ability.produceInWater")) {
-            int production = unit.getProductionOf(goodsType, workTile.potential(goodsType, unit.getType()));
-            if (production > 0) {
-                production = Math.max(1, production + colony.getProductionBonus());
+            Set<Modifier> modifiers = workTile.getProductionBonus(goodsType, unit.getType());
+            if (FeatureContainer.applyModifierSet(0f, getGame().getTurn(), modifiers) > 0) {
+                modifiers.addAll(unit.getModifierSet(goodsType.getId()));
+                modifiers.add(colony.getProductionModifier(goodsType));
+                modifiers.addAll(getColony().getModifierSet(goodsType.getId()));
+                List<Modifier> modifierList = new ArrayList<Modifier>(modifiers);
+                Collections.sort(modifierList);
+                return Math.max(1, (int) FeatureContainer.applyModifiers(0, getGame().getTurn(), modifierList));
+            } else {
+                return 0;
             }
-            return production;
         } else {
             return 0;
         }
     }
 
-
-    public void dispose() {
+    /**
+     * Removes all references to this object.
+     *
+     * @return A list of disposed objects.
+     */
+    public List<FreeColGameObject> disposeList() {
+        List<FreeColGameObject> objects = new ArrayList<FreeColGameObject>();
         if (unit != null) {
-            unit.dispose();
+            objects.addAll(unit.disposeList());
         }
+        objects.addAll(super.disposeList());
+        return objects;
+    }
 
-        super.dispose();
+    /**
+     * Dispose of this ColonyTile.
+     */
+    public void dispose() {
+        disposeList();
     }
 
     /**
      * This method writes an XML-representation of this object to
      * the given stream.
-     * 
+     *
      * <br><br>
-     * 
-     * Only attributes visible to the given <code>Player</code> will 
+     *
+     * Only attributes visible to the given <code>Player</code> will
      * be added to that representation if <code>showAll</code> is
      * set to <code>false</code>.
-     *  
+     *
      * @param out The target stream.
-     * @param player The <code>Player</code> this XML-representation 
+     * @param player The <code>Player</code> this XML-representation
      *      should be made for, or <code>null</code> if
      *      <code>showAll == true</code>.
-     * @param showAll Only attributes visible to <code>player</code> 
+     * @param showAll Only attributes visible to <code>player</code>
      *      will be added to the representation if <code>showAll</code>
      *      is set to <i>false</i>.
      * @param toSavedGame If <code>true</code> then information that
@@ -578,8 +671,8 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
     protected void toXMLImpl(XMLStreamWriter out, Player player, boolean showAll, boolean toSavedGame) throws XMLStreamException {
         // Start element:
         out.writeStartElement(getXMLElementTagName());
-        
-        // Add attributes:       
+
+        // Add attributes:
         out.writeAttribute("ID", getId());
         out.writeAttribute("colony", colony.getId());
         out.writeAttribute("workTile", workTile.getId());
@@ -602,21 +695,47 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
         colony = getFreeColGameObject(in, "colony", Colony.class);
         workTile = getFreeColGameObject(in, "workTile", Tile.class);
         colonyCenterTile = (colony.getTile() == workTile);
-        
+
         unit = null;
         while (in.nextTag() != XMLStreamConstants.END_ELEMENT) {
             if (in.getLocalName().equals(Unit.getXMLElementTagName())) {
-                setUnit(updateFreeColGameObject(in, Unit.class));
+                unit = updateFreeColGameObject(in, Unit.class);
             }
         }
 
     }
 
+    /**
+     * Partial writer, so that "remove" messages can be brief.
+     *
+     * @param out The target stream.
+     * @param fields The fields to write.
+     * @throws XMLStreamException If there are problems writing the stream.
+     */
+    @Override
+    protected void toXMLPartialImpl(XMLStreamWriter out, String[] fields)
+        throws XMLStreamException {
+        toXMLPartialByClass(out, getClass(), fields);
+    }
+
+    /**
+     * Partial reader, so that "remove" messages can be brief.
+     *
+     * @param in The input stream with the XML.
+     * @throws XMLStreamException If there are problems reading the stream.
+     */
+    @Override
+    protected void readFromXMLPartialImpl(XMLStreamReader in)
+        throws XMLStreamException {
+        readFromXMLPartialByClass(in, getClass());
+    }
+
     /**
      * Will return the position of the tile and the name of the colony in
      * addition to the FreeColObject.toString().
-     * 
-     * @return A representation of a colony-tile that can be used for debugging.
+     *
+     * @return A representation of a colony-tile that can be used for
+     *    debugging.
      */
     public String toString() {
         return "ColonyTile " + getWorkTile().getPosition().toString()
@@ -624,9 +743,9 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
     }
 
     /**
-    * Gets the tag name of the root element representing this object.
-    * @return "colonyTile".
-    */
+     * Gets the tag name of the root element representing this object.
+     * @return "colonyTile".
+     */
     public static String getXMLElementTagName() {
         return "colonyTile";
     }
