@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -32,19 +32,18 @@ import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
-import net.sf.freecol.FreeCol;
-import net.sf.freecol.client.gui.i18n.Messages;
-import net.sf.freecol.common.Specification;
 import org.w3c.dom.Element;
 
 /**
- * Represents Europe in the game. Each <code>Player</code> has it's own
- * <code>Europe</code>. <p/> <br>
- * <br>
- * <p/> Europe is the place where you can {@link #recruit} and {@link #train}
- * new units. You may also sell/buy goods.
+ * Represents Europe in the game. Each <code>Player</code> has it's
+ * own <code>Europe</code>.
+ *
+ * <br><br>
+ *
+ * <p>In Europe, you can recruit, train and purchase new units. You
+ * can also equip units, as well as sell and buy goods.
  */
-public final class Europe extends FreeColGameObject implements Location, Ownable, Named {
+public class Europe extends FreeColGameObject implements Location, Ownable, Named {
 
     private static final Logger logger = Logger.getLogger(Europe.class.getName());
 
@@ -65,7 +64,15 @@ public final class Europe extends FreeColGameObject implements Location, Ownable
     private UnitType[] recruitables = { null, null, null };
     public static final int RECRUIT_COUNT = 3;
 
-    private java.util.Map<UnitType, Integer> unitPrices = new HashMap<UnitType, Integer>();
+    public static enum MigrationType {
+        NORMAL,     // Unit decided to migrate
+        RECRUIT,    // Player is paying
+        FOUNTAIN    // As a result of a Fountain of Youth discovery
+    }
+
+
+    protected java.util.Map<UnitType, Integer> unitPrices
+        = new HashMap<UnitType, Integer>();
 
     private int recruitPrice;
 
@@ -80,13 +87,20 @@ public final class Europe extends FreeColGameObject implements Location, Ownable
 
 
     /**
-     * Creates a new <code>Europe</code>.
-     * 
+     * Constructor for ServerEurope.
+     */
+    protected Europe() {
+        // empty constructor
+    }
+
+    /**
+     * Constructor for ServerEurope.
+     *
      * @param game The <code>Game</code> in which this object belong.
      * @param owner The <code>Player</code> that will be using this object of
      *            <code>Europe</code>.
      */
-    public Europe(Game game, Player owner) {
+    protected Europe(Game game, Player owner) {
         super(game);
         this.owner = owner;
 
@@ -96,7 +110,7 @@ public final class Europe extends FreeColGameObject implements Location, Ownable
 
     /**
      * Initializes this object from an XML-representation of this object.
-     * 
+     *
      * @param game The <code>Game</code> in which this object belong.
      * @param in The input stream containing the XML.
      * @throws XMLStreamException if an error occurred during parsing.
@@ -109,7 +123,7 @@ public final class Europe extends FreeColGameObject implements Location, Ownable
 
     /**
      * Initializes this object from an XML-representation of this object.
-     * 
+     *
      * @param game The <code>Game</code> in which this object belong.
      * @param e An XML-element that will be used to initialize this object.
      */
@@ -124,7 +138,7 @@ public final class Europe extends FreeColGameObject implements Location, Ownable
      * should later be initialized by calling either
      * {@link #readFromXML(XMLStreamReader)} or
      * {@link #readFromXMLElement(Element)}.
-     * 
+     *
      * @param game The <code>Game</code> in which this object belong.
      * @param id The unique identifier for this object.
      */
@@ -142,8 +156,8 @@ public final class Europe extends FreeColGameObject implements Location, Ownable
     public boolean canBuildEquipment(EquipmentType equipmentType) {
         for (AbstractGoods requiredGoods : equipmentType.getGoodsRequired()) {
             GoodsType goodsType = requiredGoods.getType();
-            if (!(getOwner().canTrade(goodsType) &&
-                  getOwner().getGold() >= getOwner().getMarket().getBidPrice(goodsType, requiredGoods.getAmount()))) {
+            if (!(getOwner().canTrade(goodsType)
+                  && getOwner().checkGold(getOwner().getMarket().getBidPrice(goodsType, requiredGoods.getAmount())))) {
                 return false;
             }
         }
@@ -153,7 +167,7 @@ public final class Europe extends FreeColGameObject implements Location, Ownable
 
     /**
      * Returns true if not all recruitables are of the same type.
-     * 
+     *
      * @return a <code>boolean</code> value
      */
     public boolean recruitablesDiffer() {
@@ -162,7 +176,7 @@ public final class Europe extends FreeColGameObject implements Location, Ownable
 
     /**
      * Gets the type of the recruitable in Europe at the given slot.
-     * 
+     *
      * @param slot The slot of the recruitable whose type needs to be returned.
      *            Should be 0, 1 or 2. NOTE - used to be 1, 2 or 3 and was
      *            called with 1-3 by some classes and 0-2 by others, the method
@@ -181,7 +195,7 @@ public final class Europe extends FreeColGameObject implements Location, Ownable
     /**
      * Sets the type of the recruitable in Europe at the given slot to the given
      * type.
-     * 
+     *
      * @param slot The slot of the recruitable whose type needs to be set.
      *            Should be 0, 1 or 2. NOTE - changed in order to match
      *            getRecruitable above!
@@ -197,47 +211,29 @@ public final class Europe extends FreeColGameObject implements Location, Ownable
         }
     }
 
-    /**
-     * Recruits a unit from Europe.
-     * 
-     * @param slot The slot the recruited unit(type) came from. This is needed
-     *            for setting a new recruitable to this slot.
-     * @param unit The recruited unit.
-     * @param newRecruitable The recruitable that will fill the now empty slot.
-     * @exception IllegalArgumentException if <code>unit == null</code>.
-     * @exception IllegalStateException if the player recruiting the unit cannot
-     *                afford the price.
-     */
-    public void recruit(int slot, Unit unit, UnitType newRecruitable) {
-        if (unit == null) {
-            throw new IllegalArgumentException("Unit must not be 'null'.");
-        } else if (getRecruitPrice() > unit.getOwner().getGold()) {
-            throw new IllegalStateException("Not enough gold to recruit " + unit + ".");
-        }
-
-        unit.getOwner().modifyGold(-getRecruitPrice());
-        incrementRecruitPrice();
-        unit.setLocation(this);
-        firePropertyChange(UNIT_CHANGE, getUnitCount() - 1, getUnitCount());
-        unit.getOwner().updateImmigrationRequired();
-        unit.getOwner().reduceImmigration();
-
-        setRecruitable(slot, newRecruitable);
-    }
-
     /**
      * Returns <i>null</i>.
-     * 
+     *
      * @return <i>null</i>.
      */
     public Tile getTile() {
         return null;
     }
 
+
+    /**
+     * Europe does not belong to a settlement.
+     *
+     * @return Null.
+     */
+    public Settlement getSettlement() {
+        return null;
+    }
+
     /**
      * Europe does not belong to a colony.
-     * 
-     * @return Always returns null.
+     *
+     * @return Null.
      */
     public Colony getColony() {
         return null;
@@ -245,7 +241,7 @@ public final class Europe extends FreeColGameObject implements Location, Ownable
 
     /**
      * Adds a <code>Locatable</code> to this Location.
-     * 
+     *
      * @param locatable The <code>Locatable</code> to add to this Location.
      */
     public void add(Locatable locatable) {
@@ -260,20 +256,18 @@ public final class Europe extends FreeColGameObject implements Location, Ownable
             if (!newUnit.isBetweenEuropeAndNewWorld()) {
                 newUnit.setState(Unit.UnitState.SENTRY);
             }
-            firePropertyChange(UNIT_CHANGE, getUnitCount() - 1, getUnitCount());
         }
     }
 
     /**
      * Removes a <code>Locatable</code> from this Location.
-     * 
+     *
      * @param locatable The <code>Locatable</code> to remove from this
      *            Location.
      */
     public void remove(Locatable locatable) {
         if (locatable instanceof Unit) {
             units.remove(locatable);
-            firePropertyChange(UNIT_CHANGE, getUnitCount() + 1, getUnitCount());
         } else {
             logger.warning("Tried to remove an unrecognized 'Locatable' from a europe.");
         }
@@ -282,7 +276,7 @@ public final class Europe extends FreeColGameObject implements Location, Ownable
     /**
      * Checks if the specified <code>Locatable</code> is at this
      * <code>Location</code>.
-     * 
+     *
      * @param locatable The <code>Locatable</code> to test the presence of.
      * @return The result.
      */
@@ -301,7 +295,7 @@ public final class Europe extends FreeColGameObject implements Location, Ownable
     /**
      * Checks whether or not the specified locatable may be added to this
      * <code>Location</code>.
-     * 
+     *
      * @param locatable The <code>Locatable</code> to test the addabillity of.
      * @return <i>true</i>.
      */
@@ -311,7 +305,7 @@ public final class Europe extends FreeColGameObject implements Location, Ownable
 
     /**
      * Gets the amount of Units at this Location.
-     * 
+     *
      * @return The amount of Units at this Location.
      */
     public int getUnitCount() {
@@ -322,7 +316,7 @@ public final class Europe extends FreeColGameObject implements Location, Ownable
      * Gets a <code>List</code> of every <code>Unit</code> directly located
      * in this <code>Europe</code>. This does not include <code>Unit</code>s
      * on ships.
-     * 
+     *
      * @return The <code>List</code>.
      */
     public List<Unit> getUnitList() {
@@ -330,21 +324,33 @@ public final class Europe extends FreeColGameObject implements Location, Ownable
     }
 
     /**
-     * Dispose of all units in this <code>Europe</code>.
+     * Dispose of this <code>Europe</code>.
+     *
+     * @return A list of disposed objects.
      */
-    public void disposeUnitList() {
-        while (!units.isEmpty()) {
-            Unit unit = units.remove(0);
-            unit.dispose();
+    public List<FreeColGameObject> disposeList() {
+        List<FreeColGameObject> objects = new ArrayList<FreeColGameObject>();
+        while (units.size() > 0) {
+            objects.addAll(units.remove(0).disposeList());
         }
         units = null;
+
+        objects.addAll(super.disposeList());
+        return objects;
+    }
+
+    /**
+     * Removes all references to this object.
+     */
+    public void dispose() {
+        disposeList();
     }
 
     /**
      * Gets an <code>Iterator</code> of every <code>Unit</code> directly
      * located in this <code>Europe</code>. This does not include
      * <code>Unit</code>s on ships.
-     * 
+     *
      * @return The <code>Iterator</code>.
      */
     public Iterator<Unit> getUnitIterator() {
@@ -353,7 +359,7 @@ public final class Europe extends FreeColGameObject implements Location, Ownable
 
     /**
      * Gets the first <code>Unit</code> in this <code>Europe</code>.
-     * 
+     *
      * @return The first <code>Unit</code> in this <code>Europe</code>.
      */
     public Unit getFirstUnit() {
@@ -366,7 +372,7 @@ public final class Europe extends FreeColGameObject implements Location, Ownable
 
     /**
      * Gets the last <code>Unit</code> in this <code>Europe</code>.
-     * 
+     *
      * @return The last <code>Unit</code> in this <code>Europe</code>.
      */
     public Unit getLastUnit() {
@@ -379,10 +385,10 @@ public final class Europe extends FreeColGameObject implements Location, Ownable
 
     /**
      * Returns the price of a unit in Europe.
-     * 
+     *
      * @param unitType The type of unit of which you need the price.
      * @return The price of this unit when trained in Europe.
-     *         'UnitType.UNDEFINED' is returned in case the unit cannot be
+     *         'UNDEFINED' is returned in case the unit cannot be
      *         bought.
      */
     public int getUnitPrice(UnitType unitType) {
@@ -394,59 +400,9 @@ public final class Europe extends FreeColGameObject implements Location, Ownable
         }
     }
 
-    /**
-     * Train or purchase a unit in Europe.
-     * 
-     * @param unit The trained unit.
-     * @exception IllegalArgumentException if <code>unit == null</code>.
-     * @exception IllegalArgumentException if the unit to be trained doesn't
-     *                have price
-     * @exception IllegalStateException if the player recruiting the unit cannot
-     *                afford the price.
-     */
-    public void train(Unit unit) {
-        if (unit == null) {
-            throw new IllegalArgumentException("Unit must not be 'null'.");
-        }
-
-        int price = getUnitPrice(unit.getType());
-        if (price <= 0) {
-            throw new IllegalArgumentException("Unit price must be a positive integer.");
-        } else if (getUnitPrice(unit.getType()) > unit.getOwner().getGold()) {
-            throw new IllegalStateException("Not enough gold to train " + unit + ".");
-        }
-
-        unit.getOwner().modifyGold(-price);
-        increasePrice(unit, price);
-        unit.setLocation(this);
-        firePropertyChange(UNIT_CHANGE, getUnitCount() - 1, getUnitCount());
-    }
-
-    /**
-     * Increases the price for a unit, if needed.  Applicable to both trained
-     * and purchased units.
-     *
-     * @param unit The unit, trained or purchased
-     * @param price The current price of the unit
-     */
-    private void increasePrice(Unit unit, int price) {
-        Specification spec = Specification.getSpecification();
-        String baseOption = "model.option.priceIncreasePerType";
-        String name = unit.getType().getId().substring(unit.getType().getId().lastIndexOf('.'));
-        String option = (spec.getBooleanOption(baseOption).getValue()) 
-            ? "model.option.priceIncrease" + name
-            : "model.option.priceIncrease";
-        int increase = (spec.hasOption(option)) 
-            ? spec.getIntegerOption(option).getValue()
-            : 0;
-        if (increase != 0) {
-            unitPrices.put(unit.getType(), new Integer(price + increase));
-        }
-    }
-
     /**
      * Gets the current price for a recruit.
-     * 
+     *
      * @return The current price of the recruit in this <code>Europe</code>.
      */
     public int getRecruitPrice() {
@@ -456,10 +412,14 @@ public final class Europe extends FreeColGameObject implements Location, Ownable
         return Math.max((recruitPrice * difference) / required, recruitLowerCap);
     }
 
-    private void incrementRecruitPrice() {
-        recruitPrice += Specification.getSpecification().getIntegerOption("model.option.recruitPriceIncrease").getValue();
-        recruitLowerCap += Specification.getSpecification().getIntegerOption("model.option.lowerCapIncrease")
-                .getValue();
+    /**
+     * Increases the base price and lower cap for recruits.
+     * Only called from the server side.
+     */
+    public void increaseRecruitmentDifficulty() {
+        Specification spec = getSpecification();
+        recruitPrice += spec.getIntegerOption("model.option.recruitPriceIncrease").getValue();
+        recruitLowerCap += spec.getIntegerOption("model.option.lowerCapIncrease").getValue();
     }
 
     /**
@@ -471,7 +431,7 @@ public final class Europe extends FreeColGameObject implements Location, Ownable
 
     /**
      * Sets the owner of this <code>Ownable</code>.
-     * 
+     *
      * @param p The <code>Player</code> that should take ownership of this
      *            {@link Ownable}.
      * @exception UnsupportedOperationException is always thrown by this method.
@@ -481,29 +441,31 @@ public final class Europe extends FreeColGameObject implements Location, Ownable
     }
 
     /**
-     * Prepares this object for a new turn. TODO: give Europe a shipyard and
-     * remove this
+     * Returns the name of this location.
+     *
+     * @return The name of this location.
      */
-    public void newTurn() {
-        // Repair any damaged ships:
-        for (Unit unit : getUnitList()) {
-            if (unit.isNaval() && unit.isUnderRepair()) {
-                unit.setHitpoints(unit.getHitpoints() + 1);
-                if (!unit.isUnderRepair()) {
-                    addModelMessage(this, ModelMessage.MessageType.DEFAULT, unit, "model.unit.shipRepaired", "%unit%",
-                            Messages.getLabel(unit), "%repairLocation%", getName());
-                }
-            }
-        }
+    public StringTemplate getLocationName() {
+        return StringTemplate.key(getNameKey());
+    }
+
+    /**
+     * Returns the name of this location for a particular player.
+     *
+     * @param player The <code>Player</code> to prepare the name for.
+     * @return The name of this location.
+     */
+    public StringTemplate getLocationNameFor(Player player) {
+        return getLocationName();
     }
 
     /**
      * Returns the name of the owner's home port.
-     * 
+     *
      * @return The name of this location.
      */
-    public String getName() {
-        return getOwner().getEuropeName();
+    public String getNameKey() {
+        return getOwner().getEuropeNameKey();
     }
 
     /**
@@ -527,14 +489,14 @@ public final class Europe extends FreeColGameObject implements Location, Ownable
     /**
      * This method writes an XML-representation of this object to the given
      * stream.
-     * 
+     *
      * <br>
      * <br>
-     * 
+     *
      * Only attributes visible to the given <code>Player</code> will be added
      * to that representation if <code>showAll</code> is set to
      * <code>false</code>.
-     * 
+     *
      * @param out The target stream.
      * @param player The <code>Player</code> this XML-representation should be
      *            made for, or <code>null</code> if
@@ -576,14 +538,14 @@ public final class Europe extends FreeColGameObject implements Location, Ownable
 
     /**
      * Initialize this object from an XML-representation of this object.
-     * 
+     *
      * @param in The input stream with the XML.
      * @throws XMLStreamException if a problem was encountered during parsing.
      */
     protected void readFromXMLImpl(XMLStreamReader in) throws XMLStreamException {
         setId(in.getAttributeValue(null, ID_ATTRIBUTE));
 
-        Specification spec = FreeCol.getSpecification();
+        Specification spec = getSpecification();
         for (int index = 0; index < recruitables.length; index++) {
             String unitTypeId = in.getAttributeValue(null, "recruit" + index);
             if (unitTypeId != null) {
@@ -615,9 +577,34 @@ public final class Europe extends FreeColGameObject implements Location, Ownable
         }
     }
 
+    /**
+     * Partial writer, so that "remove" messages can be brief.
+     *
+     * @param out The target stream.
+     * @param fields The fields to write.
+     * @throws XMLStreamException If there are problems writing the stream.
+     */
+    @Override
+    protected void toXMLPartialImpl(XMLStreamWriter out, String[] fields)
+        throws XMLStreamException {
+        toXMLPartialByClass(out, getClass(), fields);
+    }
+
+    /**
+     * Partial reader, so that "remove" messages can be brief.
+     *
+     * @param in The input stream with the XML.
+     * @throws XMLStreamException If there are problems reading the stream.
+     */
+    @Override
+    protected void readFromXMLPartialImpl(XMLStreamReader in)
+        throws XMLStreamException {
+        readFromXMLPartialByClass(in, getClass());
+    }
+
     /**
      * Gets the tag name of the root element representing this object.
-     * 
+     *
      * @return "europe".
      */
     public static String getXMLElementTagName() {
