@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -21,17 +21,26 @@ package net.sf.freecol.common.model;
 
 import java.util.HashMap;
 import java.util.HashSet;
+import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
 import java.util.Set;
-
-import net.sf.freecol.common.Specification;
+import java.util.logging.Logger;
 
 public class FeatureContainer {
 
+    private static final Logger logger = Logger.getLogger(FeatureContainer.class.getName());
+
     private Map<String, Set<Ability>> abilities = new HashMap<String, Set<Ability>>();
     private Map<String, Set<Modifier>> modifiers = new HashMap<String, Set<Modifier>>();
 
+    private Specification specification;
+
+
+    public FeatureContainer(Specification specification) {
+        this.specification = specification;
+    }
+
     public Set<Ability> getAbilities() {
         Set<Ability> result = new HashSet<Ability>();
         for (Set<Ability> abilitySet : abilities.values()) {
@@ -80,7 +89,7 @@ public class FeatureContainer {
      * @return a <code>Set<Feature></code> value
      */
     public Set<Ability> getAbilitySet(String id, FreeColGameObjectType objectType, Turn turn) {
-        if (Specification.getSpecification().getAbilities(id) == null) {
+        if (specification.getAbilities(id) == null) {
             throw new IllegalArgumentException("Unknown ability key: " + id);
         }
         Set<Ability> abilitySet = abilities.get(id);
@@ -127,8 +136,9 @@ public class FeatureContainer {
      * @return a <code>boolean</code> value
      */
     public boolean hasAbility(String id, FreeColGameObjectType objectType, Turn turn) {
-        if (Specification.getSpecification().getAbilities(id) == null) {
-            throw new IllegalArgumentException("Unknown ability key: " + id);
+        if (specification.getAbilities(id) == null) {
+            logger.warning("Unknown ability key: " + id + ". Possible spelling error?");
+            return false;
         }
         Set<Ability> abilitySet = abilities.get(id);
         if (abilitySet == null) {
@@ -199,17 +209,18 @@ public class FeatureContainer {
      * @return a <code>Set<Feature></code> value
      */
     public Set<Modifier> getModifierSet(String id, FreeColGameObjectType objectType, Turn turn) {
-        if (Specification.getSpecification().getModifiers(id) == null &&
-            Specification.getSpecification().getType(id) == null) {
-            throw new IllegalArgumentException("Unknown modifier key: " + id);
+        HashSet<Modifier> result = new HashSet<Modifier>();
+
+        if (specification.getModifiers(id) == null) {
+            logger.warning("Unknown modifier key: " + id + ". Possible spelling error?");
+            return result;
         }
         Set<Modifier> modifierSet = modifiers.get(id);
         if (modifierSet == null) {
-            return new HashSet<Modifier>();
+            return result;
         } else if (objectType == null) {
             return modifierSet;
         } else {
-            Set<Modifier> result = new HashSet<Modifier>();
             for (Modifier modifier : modifierSet) {
                 if (modifier.appliesTo(objectType, turn)) {
                     result.add(modifier);
@@ -255,6 +266,52 @@ public class FeatureContainer {
         return applyModifierSet(number, turn, getModifierSet(id, objectType, turn));
     }
 
+    /**
+     * Applies a given Set of Modifiers to the given float value.
+     *
+     * @param number a <code>float</code> value
+     * @param turn a <code>Turn</code> value
+     * @return a <code>float</code> value
+     */
+    public static float applyModifiers(float number, Turn turn, List<Modifier> modifierSet) {
+        if (modifierSet == null) {
+            return number;
+        }
+        float result = number;
+        for (Modifier modifier : modifierSet) {
+            float value = modifier.getValue();
+            if (value == Modifier.UNKNOWN) {
+                return Modifier.UNKNOWN;
+            }
+            if (modifier.hasIncrement() && turn != null) {
+                int diff = turn.getNumber() - modifier.getFirstTurn().getNumber();
+                switch(modifier.getIncrementType()) {
+                case ADDITIVE:
+                    value += modifier.getIncrement() * diff;
+                    break;
+                case MULTIPLICATIVE:
+                    value *= modifier.getIncrement() * diff;
+                    break;
+                case PERCENTAGE:
+                    value += (value * modifier.getIncrement() * diff) / 100;
+                    break;
+                }
+            }
+            switch(modifier.getType()) {
+            case ADDITIVE:
+                result += value;
+                break;
+            case MULTIPLICATIVE:
+                result *= value;
+                break;
+            case PERCENTAGE:
+                result += (result * value) / 100;
+                break;
+            }
+        }
+        return result;
+    }
+
     /**
      * Applies a given Set of Modifiers to the given float value.
      *
@@ -305,7 +362,6 @@ public class FeatureContainer {
         return result;
     }
 
-
     /**
      * Add the given Ability to the set of Abilities present. If the
      * Ability given can not be combined with a Ability with the same
@@ -367,6 +423,15 @@ public class FeatureContainer {
         }
     }
 
+    /**
+     * Describe <code>removeAbilities</code> method here.
+     *
+     * @param id a <code>String</code> value
+     */
+    public void removeAbilities(String id) {
+        abilities.remove(id);
+    }
+
     /**
      * Removes and returns a Modifier from this modifier set.
      *
@@ -388,6 +453,15 @@ public class FeatureContainer {
         }
     }
 
+    /**
+     * Describe <code>removeModifiers</code> method here.
+     *
+     * @param id a <code>String</code> value
+     */
+    public void removeModifiers(String id) {
+        modifiers.remove(id);
+    }
+
     /**
      * Describe <code>add</code> method here.
      *
@@ -440,4 +514,46 @@ public class FeatureContainer {
         return modifiers.containsKey(key);
     }
 
+    /**
+     * Replaces the source field. This is necessary because objects
+     * may inherit Features from other, abstract objects.
+     *
+     * @param oldSource a <code>FreeColGameObjectType</code> value
+     * @param newSource a <code>FreeColGameObjectType</code> value
+     */
+    public void replaceSource(FreeColGameObjectType oldSource, FreeColGameObjectType newSource) {
+        for (Ability ability : getAbilities()) {
+            if (ability.getSource() == oldSource) {
+                removeAbility(ability);
+                Ability newAbility = new Ability(ability);
+                newAbility.setSource(newSource);
+                addAbility(newAbility);
+            }
+        }
+        for (Modifier modifier : getModifiers()) {
+            if (modifier.getSource() == oldSource) {
+                removeModifier(modifier);
+                Modifier newModifier = new Modifier(modifier);
+                newModifier.setSource(newSource);
+                addModifier(newModifier);
+            }
+        }
+    }
+
+    /**
+     * Debug helper.
+     */
+    public String toString() {
+        StringBuilder result = new StringBuilder();
+        result.append("[FeatureContainer [abilities");
+        for (Ability ability : getAbilities()) {
+            result.append(" " + ability.toString());
+        }
+        result.append("][modifiers");
+        for (Modifier modifier : getModifiers()) {
+            result.append(" " + modifier.toString());
+        }
+        result.append("]]");
+        return result.toString();
+    }
 }
