@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -21,6 +21,8 @@ package net.sf.freecol.common.model;
 
 import java.lang.reflect.Constructor;
 import java.lang.reflect.Method;
+import java.util.ArrayList;
+import java.util.List;
 import java.util.logging.Logger;
 
 import javax.xml.stream.XMLStreamConstants;
@@ -28,11 +30,10 @@ import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
+import net.sf.freecol.common.util.Introspector;
 import net.sf.freecol.common.util.Utils;
 import net.sf.freecol.server.model.ServerGame;
-import net.sf.freecol.common.util.Introspector;
 
-import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 
 /**
@@ -46,14 +47,14 @@ abstract public class FreeColGameObject extends FreeColObject {
     private boolean disposed = false;
     private boolean uninitialized;
 
-    protected FreeColGameObject() {    
+    protected FreeColGameObject() {
         logger.info("FreeColGameObject without ID created.");
         uninitialized = false;
     }
-    
+
 
     /**
-     * Creates a new <code>FreeColGameObject</code> with an automatically assigned 
+     * Creates a new <code>FreeColGameObject</code> with an automatically assigned
      * ID and registers this object at the specified <code>Game</code>.
      *
      * @param game The <code>Game</code> in which this object belong.
@@ -62,17 +63,18 @@ abstract public class FreeColGameObject extends FreeColObject {
         this.game = game;
 
         if (game != null && game instanceof ServerGame) {
-            setId(getRealXMLElementTagName() + ":" + ((ServerGame)game).getNextID());
+            setDefaultId(game);
         } else if (this instanceof Game) {
             setId("0");
         } else {
-            logger.warning("Created 'FreeColGameObject' with 'game == null': " + this);
+            logger.warning("Created 'FreeColGameObject' with 'game == null':"
+                           + this);
         }
-        
+
         uninitialized = false;
     }
-    
-    
+
+
     /**
      * Initiates a new <code>FreeColGameObject</code> from an <code>Element</code>.
      *
@@ -109,7 +111,7 @@ abstract public class FreeColGameObject extends FreeColObject {
     }
 
     /**
-     * Initiates a new <code>FreeColGameObject</code> 
+     * Initiates a new <code>FreeColGameObject</code>
      * with the given ID. The object should later be
      * initialized by calling either
      * {@link #readFromXML(XMLStreamReader)} or
@@ -126,11 +128,21 @@ abstract public class FreeColGameObject extends FreeColObject {
         }
 
         setId(id);
-        
+
         uninitialized = true;
     }
-    
-    
+
+    /**
+     * Sets the Id from the real type and the next Id in the server.
+     * Split out only to help out a backward compatibility reader.
+     *
+     * @param game The <code>Game</code> this object is in.
+     */
+    protected void setDefaultId(Game game) {
+        setId(getRealXMLElementTagName() + ":"
+              + ((ServerGame)game).getNextID());
+    }
+
     /**
      * Gets the game object this <code>FreeColGameObject</code> belongs to.
      * @return The <code>game</code>.
@@ -141,34 +153,55 @@ abstract public class FreeColGameObject extends FreeColObject {
 
 
     /**
-     * Gets the <code>GameOptions</code> that is associated with the 
-     * {@link Game} owning this <code>FreeColGameObject</code>.
-     * 
-     * @return The same <code>GameOptions</code>-object as returned
-     *       by <code>getGame().getGameOptions()</code>.
+     * Describe <code>getSpecification</code> method here.
+     *
+     * @return a <code>Specification</code> value
      */
-    public GameOptions getGameOptions() {
-        return game.getGameOptions();
+    @Override
+    public Specification getSpecification() {
+        if (game == null) {
+            return null;
+        } else {
+            return game.getSpecification();
+        }
     }
 
-    
     /**
      * Sets the game object this <code>FreeColGameObject</code> belongs to.
      * @param game The <code>game</code>.
      */
     public void setGame(Game game) {
         this.game = game;
-    }    
+    }
 
 
+    /**
+     * Low level base dispose.
+     */
+    public void fundamentalDispose() {
+        disposed = true;
+        getGame().removeFreeColGameObject(getId());
+    }
+
+    /**
+     * Removes all references to this object.
+     *
+     * @return A list of disposed objects.
+     */
+    public List<FreeColGameObject> disposeList() {
+        fundamentalDispose();
+
+        List<FreeColGameObject> objects = new ArrayList<FreeColGameObject>();
+        objects.add(this);
+        return objects;
+    }
+
     /**
      * Removes all references to this object.
      */
     public void dispose() {
-        disposed = true;
-        getGame().removeFreeColGameObject(getId());
+        disposeList();
     }
-    
 
     /**
      * Checks if this object has been disposed.
@@ -180,25 +213,17 @@ abstract public class FreeColGameObject extends FreeColObject {
     }
 
     /**
-     * Checks if this <code>FreeColGameObject</code> 
+     * Checks if this <code>FreeColGameObject</code>
      * is uninitialized. That is: it has been referenced
      * by another object, but has not yet been updated with
      * {@link #readFromXML}.
-     * 
+     *
      * @return <code>true</code> if this object is not initialized.
      */
     public boolean isUninitialized() {
         return uninitialized;
     }
 
-    /**
-     * Updates the id. This method should be overwritten
-     * by server model objects.
-     */
-    public void updateID() {
-        
-    }
-    
     /**
      * This method writes an XML-representation of this object to
      * the given stream for the purpose of storing this object
@@ -215,7 +240,7 @@ abstract public class FreeColGameObject extends FreeColObject {
 
     /**
      * Makes an XML-representation of this object.
-     * 
+     *
      * @param out The output stream.
      * @throws XMLStreamException if there are any problems writing to the
      *             stream.
@@ -223,55 +248,56 @@ abstract public class FreeColGameObject extends FreeColObject {
     protected void toXMLImpl(XMLStreamWriter out) throws XMLStreamException {
         toXMLImpl(out, null, false, false);
     }
-            
+
     /**
      * This method writes an XML-representation of this object to
      * the given stream.
-     * 
+     *
      * <br><br>
-     * 
-     * Only attributes visible to the given <code>Player</code> will 
+     *
+     * Only attributes visible to the given <code>Player</code> will
      * be added to that representation if <code>showAll</code> is
      * set to <code>false</code>.
-     *  
+     *
      * @param out The target stream.
-     * @param player The <code>Player</code> this XML-representation 
+     * @param player The <code>Player</code> this XML-representation
      *      should be made for, or <code>null</code> if
      *      <code>showAll == true</code>.
-     * @param showAll Only attributes visible to <code>player</code> 
+     * @param showAll Only attributes visible to <code>player</code>
      *      will be added to the representation if <code>showAll</code>
      *      is set to <i>false</i>.
      * @param toSavedGame If <code>true</code> then information that
      *      is only needed when saving a game is added.
      * @throws XMLStreamException if there are any problems writing
      *      to the stream.
-     */    
-    abstract protected void toXMLImpl(XMLStreamWriter out, Player player, boolean showAll, 
+     */
+    abstract protected void toXMLImpl(XMLStreamWriter out, Player player, boolean showAll,
                                       boolean toSavedGame) throws XMLStreamException;
 
 
     /**
      * This method writes an XML-representation of this object to
      * the given stream.
-     * 
+     *
      * <br><br>
-     * 
-     * Only attributes visible to the given <code>Player</code> will 
+     *
+     * Only attributes visible to the given <code>Player</code> will
      * be added to that representation if <code>showAll</code> is
      * set to <code>false</code>.
-     *  
+     *
      * @param out The target stream.
-     * @param player The <code>Player</code> this XML-representation 
+     * @param player The <code>Player</code> this XML-representation
      *      should be made for, or <code>null</code> if
      *      <code>showAll == true</code>.
-     * @param showAll Only attributes visible to <code>player</code> 
+     * @param showAll Only attributes visible to <code>player</code>
      *      will be added to the representation if <code>showAll</code>
      *      is set to <i>false</i>.
      * @param toSavedGame If <code>true</code> then information that
      *      is only needed when saving a game is added.
      * @throws XMLStreamException if there are any problems writing
      *      to the stream.
-     */    
+     */
+    @Override
     public final void toXML(XMLStreamWriter out, Player player, boolean showAll,
                             boolean toSavedGame) throws XMLStreamException {
         if (toSavedGame && !showAll) {
@@ -279,13 +305,14 @@ abstract public class FreeColGameObject extends FreeColObject {
         }
         toXMLImpl(out, player, showAll, toSavedGame);
     }
-    
+
     /**
      * Initialize this object from an XML-representation of this object.
      * @param in The input stream containing the XML.
      * @throws XMLStreamException if a problem was encountered
      *      during parsing.
      */
+    @Override
     public final void readFromXML(XMLStreamReader in) throws XMLStreamException {
         uninitialized = false;
         super.readFromXML(in);
@@ -298,6 +325,7 @@ abstract public class FreeColGameObject extends FreeColObject {
      *
      * @return The unique ID of this object.
      */
+    // TODO: remove compatibility code: use established instead
     public Integer getIntegerID() {
         String stringPart = getRealXMLElementTagName() + ":";
         return new Integer(getId().substring(stringPart.length()));
@@ -311,6 +339,7 @@ abstract public class FreeColGameObject extends FreeColObject {
         } catch (Exception e) {}
         return tagName;
     }
+    // end TODO
 
     /**
      * Sets the unique ID of this object. When setting a new ID to this object,
@@ -319,6 +348,7 @@ abstract public class FreeColGameObject extends FreeColObject {
      *
      * @param newID the unique ID of this object,
      */
+    @Override
     public final void setId(String newID) {
         if (game != null && !(this instanceof Game)) {
             if (!newID.equals(getId())) {
@@ -334,112 +364,7 @@ abstract public class FreeColGameObject extends FreeColObject {
         }
     }
 
-    
-    /**
-     * Sets the ID of this object for temporary use with
-     * <code>toXMLElement</code>. This method does not
-     * register the object.
-     *
-     * @param newID the unique ID of this object,
-     */
-    public void setFakeID(String newID) {
-        super.setId(newID);
-    }
 
-    
-    /**
-     * Creates a <code>ModelMessage</code> and uses
-     * <code>Player.addModelMessage(modelMessage)</code> to register
-     * it.
-     *
-     * <br><br><br>
-     *
-     * Example:<br><br>
-     *
-     * Using <code>addModelMessage(this, someType, "messageID",
-     * "%test1%", "ok1", "%test2%", "ok2")</code> with the entry
-     * "messageID=This is %test1% and %test2%" in {@link
-     * net.sf.freecol.client.gui.i18n.Messages Messages}, would give
-     * the following message: "This is ok1 and ok2".
-     *
-     * @param source The source of the message. This is what the
-     *               message should be associated with. In addition,
-     *               the owner of the source is the player getting the
-     *               message.
-     * @param type The type of message. This is used for filtering
-     * purposes.
-     * @param messageID The ID of the message to display. See: {@link
-     * net.sf.freecol.client.gui.i18n.Messages Messages}.
-     * @param data Contains the data to be displayed in the message.
-     * @see net.sf.freecol.client.gui.Canvas Canvas
-     * @see Player#addModelMessage(ModelMessage)
-     * @see ModelMessage
-     */    
-    protected void addModelMessage(FreeColGameObject source, ModelMessage.MessageType type,
-                                   String messageID, String... data) {
-        addModelMessage(source, type, null, messageID, data);
-    }
-
-    /**
-     * Creates a <code>ModelMessage</code> and uses
-     * <code>Player.addModelMessage(modelMessage)</code> to register
-     * it.
-     *
-     * <br><br><br>
-     *
-     * Example:<br><br>
-     *
-     * Using <code>addModelMessage(this, someType, someDisplay,
-     * "messageID", "%test1%", "ok1", "%test2%", "ok2")</code> with
-     * the entry "messageID=This is %test1% and %test2%" in {@link
-     * net.sf.freecol.client.gui.i18n.Messages Messages}, would give
-     * the following message: "This is ok1 and ok2".
-     *
-     * @param source The source of the message. This is what the
-     *               message should be associated with. In addition,
-     *               the owner of the source is the player getting the
-     *               message.
-     * @param type The type of message. This is used for filtering
-     * purposes.
-     * @param display The Object to display if that is not the
-     * source. A message about <code>Goods</code> in a
-     * <code>Colony</code> might wish to display the goods instead of
-     * the colony, for example.
-     * @param messageID The ID of the message to display. See: {@link
-     * net.sf.freecol.client.gui.i18n.Messages Messages}.
-     * @param data Contains the data to be displayed in the message.
-     * @see net.sf.freecol.client.gui.Canvas Canvas
-     * @see Player#addModelMessage(ModelMessage)
-     * @see ModelMessage
-     */    
-    protected void addModelMessage(FreeColGameObject source, ModelMessage.MessageType type, 
-                                   FreeColObject display, String messageID, String... data) {
-        ModelMessage message = new ModelMessage(source, type, display, messageID, data);
-        
-        addModelMessage(message);
-    }
-
-    /**
-     * Receives a <code>ModelMessage</code> and uses
-     * <code>Player.addModelMessage(modelMessage)</code> to register
-     * it.
-     *     
-     * @param message The message to send. In addition, the owner of the source
-     *                is the player getting the message.
-     *               
-     * @see net.sf.freecol.client.gui.Canvas Canvas
-     * @see Player#addModelMessage(ModelMessage)
-     * @see ModelMessage
-     */
-    protected void addModelMessage(ModelMessage message){
-    	if (message.getSource() == null) {
-    		logger.warning("ModelMessage with ID " + message.getId() + " has null source.");
-    	} else if (message.getOwner() == null) {
-    		logger.warning("ModelMessage with ID " + message.getId() + " has null owner.");
-    	}
-    	message.getOwner().addModelMessage(message);
-    }
-    
     /**
      * Checks if this object has the specified ID.
      *
@@ -464,26 +389,29 @@ abstract public class FreeColGameObject extends FreeColObject {
             return false;
         }
     }
-    
+
     /**
      * Checks if the given <code>FreeColGameObject</code> equals this object.
      *
      * @param o The <code>FreeColGameObject</code> to compare against this object.
      * @return <i>true</i> if the two <code>FreeColGameObject</code> are equal and <i>false</i> otherwise.
      */
+    @Override
     public boolean equals(Object o) {
         return (o instanceof FreeColGameObject) ? equals((FreeColGameObject) o) : false;
     }
-        
+
+    @Override
     public int hashCode() {
         return getId().hashCode();
     }
 
-    
+
     /**
      * Returns a string representation of the object.
      * @return The <code>String</code>
      */
+    @Override
     public String toString() {
         return getClass().getName() + ": " + getId() + " (super's hash code: " +
             Integer.toHexString(super.hashCode()) + ")";
@@ -520,24 +448,23 @@ abstract public class FreeColGameObject extends FreeColObject {
     }
 
     public <T extends FreeColGameObject> T updateFreeColGameObject(XMLStreamReader in, Class<T> returnClass) {
-        final String attributeString = in.getAttributeValue(null, ID_ATTRIBUTE);
-        if (attributeString == null) {
-            return null;
-        } else {
-            T returnValue = returnClass.cast(getGame().getFreeColGameObject(attributeString));
-            try {
-                if (returnValue == null) {
-                    Constructor<T> c = returnClass.getConstructor(Game.class, XMLStreamReader.class);
-                    returnValue = returnClass.cast(c.newInstance(getGame(), in));
-                } else {
-                    returnValue.readFromXML(in);
-                }
-                return returnValue;
-            } catch(Exception e) {
-                logger.warning("Failed to update FreeColGameObject with ID " + attributeString);
-                e.printStackTrace();
-                return null;
+        final String idString = in.getAttributeValue(null, ID_ATTRIBUTE);
+        if (idString == null) return null;
+        FreeColGameObject fcgo = getGame().getFreeColGameObject(idString);
+        T returnValue = (fcgo == null) ? null : returnClass.cast(fcgo);
+        try {
+            if (returnValue == null) {
+                Constructor<T> c = returnClass.getConstructor(Game.class, XMLStreamReader.class);
+                returnValue = returnClass.cast(c.newInstance(getGame(), in));
+            } else {
+                returnValue.readFromXML(in);
             }
+            return returnValue;
+        } catch (Exception e) {
+            logger.warning("Failed to update FreeColGameObject with ID "
+                           + idString);
+            e.printStackTrace();
+            return null;
         }
     }
 
@@ -624,18 +551,6 @@ abstract public class FreeColGameObject extends FreeColObject {
         while (in.nextTag() != XMLStreamConstants.END_ELEMENT);
     }
 
-    /**
-     * Convenience function to add this object to an element intended to
-     * signal removal of the object.
-     *
-     * @param removeElement The remove element.
-     */
-    public void addToRemoveElement(Element removeElement) {
-        Document doc = removeElement.getOwnerDocument();
-        removeElement.appendChild(this.toXMLElementPartial(doc));
-    }
-
-
     /**
      * Gets the tag name of the root element representing this object.
      * This method should be overwritten by any sub-class, preferably
