@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -22,15 +22,19 @@ package net.sf.freecol.common.model;
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
 import java.beans.PropertyChangeSupport;
+import java.io.File;
+import java.io.FileNotFoundException;
+import java.io.FileOutputStream;
 import java.io.IOException;
+import java.io.OutputStream;
 import java.io.PrintWriter;
 import java.io.StringReader;
 import java.io.StringWriter;
-import java.lang.reflect.Array;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.InvocationTargetException;
 import java.util.ArrayList;
 import java.util.List;
+import java.util.logging.Level;
 import java.util.logging.Logger;
 
 import javax.xml.parsers.DocumentBuilder;
@@ -57,11 +61,19 @@ public abstract class FreeColObject {
 
     protected static Logger logger = Logger.getLogger(FreeColObject.class.getName());
 
+    public static final int INFINITY = Integer.MAX_VALUE;
+    public static final int UNDEFINED = Integer.MIN_VALUE;
+
+    protected static final String ID_ATTRIBUTE = "ID";
+
+    /**
+     * XML tag name for value attribute.
+     */
+    protected static final String VALUE_TAG = "value";
+
     /**
      * XML tag name for ID attribute.
      */
-    protected static final String ID_ATTRIBUTE = "ID";
-
     // this is what we use for the specification
     // TODO: standardize on this spelling
     public static final String ID_ATTRIBUTE_TAG = "id";
@@ -81,7 +93,7 @@ public abstract class FreeColObject {
      */
     private String id;
 
-    private PropertyChangeSupport pcs = new PropertyChangeSupport(this);
+    private PropertyChangeSupport pcs = null;
 
     /**
      * Get the <code>Id</code> value.
@@ -101,6 +113,15 @@ public abstract class FreeColObject {
         this.id = newId;
     }
 
+    /**
+     * Describe <code>getSpecification</code> method here.
+     *
+     * @return a <code>Specification</code> value
+     */
+    public Specification getSpecification() {
+        return null;
+    }
+
     /**
      * Describe <code>hasAbility</code> method here.
      *
@@ -111,6 +132,50 @@ public abstract class FreeColObject {
         return false;
     }
 
+    /**
+     * Debugging tool, dump object XML to System.err.
+     */
+    public void dumpObject() {
+        save(System.err);
+    }
+
+    /**
+     * Writes the object to the given file.
+     *
+     * @param file the save file
+     * @throws FileNotFoundException
+     */
+    public void save(File file) throws FileNotFoundException {
+        save(new FileOutputStream(file));
+    }
+
+    /**
+     * Writes the object to the given output stream
+     *
+     * @param out the OutputStream
+     */
+    public void save(OutputStream out) {
+        XMLOutputFactory xof = XMLOutputFactory.newInstance();
+        XMLStreamWriter xsw = null;
+        try {
+            xsw = xof.createXMLStreamWriter(out, "UTF-8");
+            xsw.writeStartDocument("UTF-8", "1.0");
+            this.toXML(xsw, null, true, true);
+            xsw.writeEndDocument();
+            xsw.flush();
+        } catch (Exception e) {
+            logger.log(Level.WARNING, "Exception while writing object.", e);
+        } finally {
+            try {
+                if (xsw != null) {
+                    xsw.close();
+                }
+            } catch (Exception e) {
+                logger.log(Level.WARNING, "Exception while closing save stream.", e);
+            }
+        }
+    }
+
     /**
      * This method writes an XML-representation of this object to
      * the given stream.
@@ -226,21 +291,21 @@ public abstract class FreeColObject {
                 StringWriter swe = new StringWriter();
                 pce.printStackTrace(new PrintWriter(swe));
                 logger.warning(swe.toString());
-                throw new IllegalStateException("ParserConfigurationException");
+                throw new IllegalStateException("ParserConfigurationException: " + pce.getMessage());
             } catch (SAXException se) {
                 StringWriter swe = new StringWriter();
                 se.printStackTrace(new PrintWriter(swe));
                 logger.warning(swe.toString());
-                throw new IllegalStateException("SAXException");
+                throw new IllegalStateException("SAXException: " + se.getMessage());
             } catch (IOException ie) {
                 StringWriter swe = new StringWriter();
                 ie.printStackTrace(new PrintWriter(swe));
                 logger.warning(swe.toString());
-                throw new IllegalStateException("IOException");
+                throw new IllegalStateException("IOException: " + ie.getMessage());
             }
         } catch (XMLStreamException e) {
             logger.warning(e.toString());
-            throw new IllegalStateException("XMLStreamException");
+            throw new IllegalStateException("XMLStreamException: " + e.getMessage());
         }
     }
 
@@ -301,6 +366,13 @@ public abstract class FreeColObject {
         toXML(out, null, true, false);
     }
 
+    public void toXML(XMLStreamWriter out, String tag) throws XMLStreamException {
+        out.writeStartElement(tag);
+        writeAttributes(out);
+        writeChildren(out);
+        out.writeEndElement();
+    }
+
     /**
      * Initialize this object from an XML-representation of this object.
      * @param element An XML-element that will be used to initialize
@@ -374,121 +446,6 @@ public abstract class FreeColObject {
         return array;
     }
 
-    /**
-     * Reads an XML-representation of an array.
-     *
-     * @param tagName The tagname for the <code>Element</code>
-     *       representing the array.
-     * @param in The input stream with the XML.
-     * @param type The type of the elements to be read. This type
-     *      needs to have a constructor accepting a single
-     *      <code>String</code>.
-     * @return The array.
-     * @throws XMLStreamException if a problem was encountered
-     *      during parsing.
-     */
-    @SuppressWarnings("unchecked")
-        protected <T> T[] readFromArrayElement(String tagName, XMLStreamReader in, Class<T> type)
-        throws XMLStreamException {
-        if (!in.getLocalName().equals(tagName)) {
-            in.nextTag();
-        }
-
-        final int size = Integer.parseInt(in.getAttributeValue(null, ARRAY_SIZE));
-        T[] array = (T[]) Array.newInstance(type, size);
-
-        for (int x=0; x<array.length; x++) {
-            try {
-                final String value = in.getAttributeValue(null, "x" + Integer.toString(x));
-                final T object;
-                if (value != null) {
-                    Constructor<T> c = type.getConstructor(type);
-                    object = c.newInstance(new Object[] {value});
-                } else {
-                    object = null;
-                }
-                array[x] = object;
-            } catch (InvocationTargetException e) {
-                throw new RuntimeException(e);
-            } catch (NoSuchMethodException e) {
-                throw new RuntimeException(e);
-            } catch (InstantiationException e) {
-                throw new RuntimeException(e);
-            } catch (IllegalAccessException e) {
-                throw new RuntimeException(e);
-            }
-        }
-
-        in.nextTag();
-        return array;
-    }
-
-    /**
-     * Creates an XML-representation of an array.
-     *
-     * @param tagName The tagname for the <code>Element</code>
-     *       representing the array.
-     * @param array The array to represent.
-     * @param out The target stream.
-     * @throws XMLStreamException if there are any problems writing
-     *      to the stream.
-     */
-    protected <T> void toArrayElement(String tagName, T[] array, XMLStreamWriter out)
-        throws XMLStreamException {
-        out.writeStartElement(tagName);
-
-        out.writeAttribute(ARRAY_SIZE, Integer.toString(array.length));
-        for (int x=0; x < array.length; x++) {
-            out.writeAttribute("x" + Integer.toString(x), array[x].toString());
-        }
-
-        out.writeEndElement();
-    }
-
-    /**
-     * Creates an XML-representation of an array.
-     *
-     * @param tagName The tagname for the <code>Element</code>
-     *       representing the array.
-     * @param array The array to represent.
-     * @param out The target stream.
-     * @throws XMLStreamException if there are any problems writing
-     *      to the stream.
-     */
-    protected void toArrayElement(String tagName, int[] array, XMLStreamWriter out)
-        throws XMLStreamException {
-        out.writeStartElement(tagName);
-
-        out.writeAttribute(ARRAY_SIZE, Integer.toString(array.length));
-        for (int x=0; x < array.length; x++) {
-            out.writeAttribute("x" + Integer.toString(x), Integer.toString(array[x]));
-        }
-
-        out.writeEndElement();
-    }
-
-    /**
-     * Creates an XML-representation of a list.
-     *
-     * @param tagName The tagname for the <code>Element</code>
-     *       representing the array.
-     * @param array The array to represent.
-     * @param out The target stream.
-     * @throws XMLStreamException if there are any problems writing
-     *      to the stream.
-     */
-    protected <T> void toListElement(String tagName, List<T> array, XMLStreamWriter out)
-        throws XMLStreamException {
-        out.writeStartElement(tagName);
-
-        out.writeAttribute(ARRAY_SIZE, Integer.toString(array.size()));
-        for (int x=0; x < array.size(); x++) {
-            out.writeAttribute("x" + Integer.toString(x), array.get(x).toString());
-        }
-
-        out.writeEndElement();
-    }
-
     /**
      * Reads an XML-representation of a list.
      *
@@ -505,11 +462,11 @@ public abstract class FreeColObject {
     protected <T> List<T> readFromListElement(String tagName, XMLStreamReader in, Class<T> type)
         throws XMLStreamException {
         if (!in.getLocalName().equals(tagName)) {
-            in.nextTag();
+            throw new XMLStreamException(tagName + " expected, not:" + in.getLocalName());
         }
         final int length = Integer.parseInt(in.getAttributeValue(null, ARRAY_SIZE));
         List<T> list = new ArrayList<T>(length);
-        for (int x=0; x<length; x++) {
+        for (int x = 0; x < length; x++) {
             try {
                 final String value = in.getAttributeValue(null, "x" + Integer.toString(x));
                 final T object;
@@ -530,8 +487,9 @@ public abstract class FreeColObject {
                 throw new RuntimeException(e);
             }
         }
-
-        in.nextTag();
+        if (in.nextTag() != XMLStreamConstants.END_ELEMENT) {
+            throw new XMLStreamException(tagName + " end expected, not: " + in.getLocalName());
+        }
         return list;
     }
 
@@ -614,7 +572,7 @@ public abstract class FreeColObject {
      * @param defaultValue a <code>boolean</code> value
      * @return an <code>boolean</code> value
      */
-    public boolean getAttribute(XMLStreamReader in, String attributeName, boolean defaultValue) {
+    public static boolean getAttribute(XMLStreamReader in, String attributeName, boolean defaultValue) {
         final String attributeString = in.getAttributeValue(null, attributeName);
         if (attributeString != null) {
             return Boolean.parseBoolean(attributeString);
@@ -671,20 +629,45 @@ public abstract class FreeColObject {
      *      during parsing.
      */
     protected void readFromXMLImpl(XMLStreamReader in) throws XMLStreamException {
-        setId(in.getAttributeValue(null, ID_ATTRIBUTE_TAG));
-        // TODO: get rid of compatibility code
-        if (getId() == null) {
-            setId(in.getAttributeValue(null, ID_ATTRIBUTE));
-        }
         readAttributes(in);
         readChildren(in);
     }
 
+    /**
+     * Initialize this object from an XML-representation of this object.
+     *
+     * @param in The input stream with the XML.
+     * @param specification a <code>Specification</code> value
+     * @exception XMLStreamException if a problem was encountered
+     *      during parsing.
+     */
+    protected void readFromXMLImpl(XMLStreamReader in, Specification specification)
+        throws XMLStreamException {
+        readAttributes(in, specification);
+        readChildren(in, specification);
+    }
+
     protected void readAttributes(XMLStreamReader in) throws XMLStreamException {
-        // do nothing
+        readAttributes(in, null);
+    }
+
+    protected void readAttributes(XMLStreamReader in, Specification specification)
+        throws XMLStreamException {
+        String newId = in.getAttributeValue(null, ID_ATTRIBUTE_TAG);
+        // TODO: get rid of compatibility code
+        if (newId == null) {
+            setId(in.getAttributeValue(null, ID_ATTRIBUTE));
+        } else {
+            setId(newId);
+        }
     }
 
     protected void readChildren(XMLStreamReader in) throws XMLStreamException {
+        readChildren(in, null);
+    }
+
+    protected void readChildren(XMLStreamReader in, Specification specification)
+        throws XMLStreamException {
         while (in.nextTag() != XMLStreamConstants.END_ELEMENT) {
             // do nothing
         }
@@ -731,67 +714,111 @@ public abstract class FreeColObject {
         throw new UnsupportedOperationException("Partial update of unsupported type.");
     }
 
-    // TODO: make these abstract
-    protected void writeAttributes(XMLStreamWriter out) throws XMLStreamException {}
-    protected void writeChildren(XMLStreamWriter out) throws XMLStreamException {}
+    protected void writeAttributes(XMLStreamWriter out) throws XMLStreamException {
+        if (getId() == null) {
+            logger.warning("FreeColObject with null id: " + toString());
+        } else {
+            out.writeAttribute(ID_ATTRIBUTE_TAG, getId());
+        }
+    }
+
+    protected void writeChildren(XMLStreamWriter out) throws XMLStreamException {
+        // do nothing
+    }
 
 
     //  ---------- PROPERTY CHANGE SUPPORT DELEGATES ----------
 
     public void addPropertyChangeListener(PropertyChangeListener listener) {
+        if (pcs == null) {
+            pcs = new PropertyChangeSupport(this);
+        }
         pcs.addPropertyChangeListener(listener);
     }
 
     public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener) {
+        if (pcs == null) {
+            pcs = new PropertyChangeSupport(this);
+        }
         pcs.addPropertyChangeListener(propertyName, listener);
     }
 
     public void fireIndexedPropertyChange(String propertyName, int index, boolean oldValue, boolean newValue) {
-        pcs.fireIndexedPropertyChange(propertyName, index, oldValue, newValue);
+        if (pcs != null) {
+            pcs.fireIndexedPropertyChange(propertyName, index, oldValue, newValue);
+        }
     }
 
     public void fireIndexedPropertyChange(String propertyName, int index, int oldValue, int newValue) {
-        pcs.fireIndexedPropertyChange(propertyName, index, oldValue, newValue);
+        if (pcs != null) {
+            pcs.fireIndexedPropertyChange(propertyName, index, oldValue, newValue);
+        }
     }
 
     public void fireIndexedPropertyChange(String propertyName, int index, Object oldValue, Object newValue) {
-        pcs.fireIndexedPropertyChange(propertyName, index, oldValue, newValue);
+        if (pcs != null) {
+            pcs.fireIndexedPropertyChange(propertyName, index, oldValue, newValue);
+        }
     }
 
     public void firePropertyChange(PropertyChangeEvent event) {
-        pcs.firePropertyChange(event);
+        if (pcs != null) {
+            pcs.firePropertyChange(event);
+        }
     }
 
     public void firePropertyChange(String propertyName, boolean oldValue, boolean newValue) {
-        pcs.firePropertyChange(propertyName, oldValue, newValue);
+        if (pcs != null) {
+            pcs.firePropertyChange(propertyName, oldValue, newValue);
+        }
     }
 
     public void firePropertyChange(String propertyName, int oldValue, int newValue) {
-        pcs.firePropertyChange(propertyName, oldValue, newValue);
+        if (pcs != null) {
+            pcs.firePropertyChange(propertyName, oldValue, newValue);
+        }
     }
 
     public void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
-        pcs.firePropertyChange(propertyName, oldValue, newValue);
+        if (pcs != null) {
+            pcs.firePropertyChange(propertyName, oldValue, newValue);
+        }
     }
 
     public PropertyChangeListener[] getPropertyChangeListeners() {
-        return pcs.getPropertyChangeListeners();
+        if (pcs == null) {
+            return new PropertyChangeListener[0];
+        } else {
+            return pcs.getPropertyChangeListeners();
+        }
     }
 
     public PropertyChangeListener[] getPropertyChangeListeners(String propertyName) {
-        return pcs.getPropertyChangeListeners(propertyName);
+        if (pcs == null) {
+            return new PropertyChangeListener[0];
+        } else {
+            return pcs.getPropertyChangeListeners(propertyName);
+        }
     }
 
     public boolean hasListeners(String propertyName) {
-        return pcs.hasListeners(propertyName);
+        if (pcs == null) {
+            return false;
+        } else {
+            return pcs.hasListeners(propertyName);
+        }
     }
 
     public void removePropertyChangeListener(PropertyChangeListener listener) {
-        pcs.removePropertyChangeListener(listener);
+        if (pcs != null) {
+            pcs.removePropertyChangeListener(listener);
+        }
     }
 
     public void removePropertyChangeListener(String propertyName, PropertyChangeListener listener) {
-        pcs.removePropertyChangeListener(propertyName, listener);
+        if (pcs != null) {
+            pcs.removePropertyChangeListener(propertyName, listener);
+        }
     }
 
 
