@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -27,13 +27,13 @@ import java.util.Map;
 import java.util.Map.Entry;
 import java.util.logging.Logger;
 
+import net.sf.freecol.common.model.Ownable;
+
 import javax.xml.stream.XMLStreamConstants;
 import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
-import net.sf.freecol.client.gui.i18n.Messages;
-import net.sf.freecol.common.Specification;
 
 import org.w3c.dom.Element;
 
@@ -41,7 +41,7 @@ import org.w3c.dom.Element;
  * Contains goods and can be used by a {@link Location} to make certain
  * tasks easier.
  */
-public class GoodsContainer extends FreeColGameObject {
+public class GoodsContainer extends FreeColGameObject implements Ownable {
 
     @SuppressWarnings("unused")
     private static final Logger logger = Logger.getLogger(Location.class.getName());
@@ -118,6 +118,49 @@ public class GoodsContainer extends FreeColGameObject {
         readFromXMLElement(e);
     }
 
+    /**
+     * Gets the owner of this <code>GoodsContainer</code>.
+     *
+     * @return The <code>Player</code> controlling this
+     *         {@link Ownable}.
+     */
+    public Player getOwner() {
+        return (parent instanceof Ownable) ? ((Ownable) parent).getOwner()
+            : null;
+    }
+
+    /**
+     * Sets the owner of this <code>Ownable</code>.
+     *
+     * @param p The <code>Player</code> that should take ownership
+     *      of this {@link Ownable}.
+     * @exception UnsupportedOperationException if not implemented.
+     */
+    public void setOwner(Player p) {
+        throw new UnsupportedOperationException("Can not set GoodsContainer owner");
+    }
+
+    /**
+     * Removes all references to this object.
+     *
+     * @return A list of disposed objects.
+     */
+    public List<FreeColGameObject> disposeList() {
+        storedGoods.clear();
+
+        List<FreeColGameObject> objects = new ArrayList<FreeColGameObject>();
+        objects.addAll(super.disposeList());
+        return objects;
+    }
+
+    /**
+     * Dispose of this GoodsContainer.
+     */
+    public void dispose() {
+        disposeList();
+    }
+
+
     /**
      * Adds a <code>Goods</code> to this containter.
      * @param g The Goods to add to this container.
@@ -136,16 +179,15 @@ public class GoodsContainer extends FreeColGameObject {
         int newAmount = oldAmount + amount;
 
         if (newAmount < 0) {
-            throw new IllegalStateException("Operation would leave " + (newAmount) + " goods of type " 
-                                            + type + " in Location " + parent);
+            throw new IllegalStateException("Operation would leave " + (newAmount) + " goods of type "
+                                            + type.getNameKey() + " in Location " + parent);
         } else if (newAmount == 0) {
             storedGoods.remove(type);
         } else {
             storedGoods.put(type, newAmount);
         }
-        firePropertyChange(type.getId(), oldAmount, newAmount);
     }
-    
+
     /**
      * Removes Goods from this containter.
      * @param g The Goods to remove from this container.
@@ -155,7 +197,7 @@ public class GoodsContainer extends FreeColGameObject {
     }
 
     public Goods removeGoods(GoodsType type) {
-        return removeGoods(type, Integer.MAX_VALUE);
+        return removeGoods(type, INFINITY);
     }
 
     /**
@@ -176,56 +218,59 @@ public class GoodsContainer extends FreeColGameObject {
             removedGoods = new Goods(getGame(), parent, type, oldAmount);
             storedGoods.remove(type);
         }
-        firePropertyChange(type.getId(), oldAmount, newAmount);
         return removedGoods;
     }
 
     /**
      * Removes all goods above given amount, provided that the goods
      * are storable and do not ignore warehouse limits.
-     * 
+     *
      * @param newAmount The treshold.
      */
     public void removeAbove(int newAmount) {
         for (GoodsType goodsType : storedGoods.keySet()) {
-            if (goodsType.isStorable() && !goodsType.limitIgnored() && 
+            if (goodsType.isStorable() && !goodsType.limitIgnored() &&
                 storedGoods.get(goodsType) > newAmount) {
                 setAmount(goodsType, newAmount);
             }
         }
     }
 
-    private void setAmount(GoodsType goodsType, int newAmount) {
-        int oldAmount = getGoodsCount(goodsType);
+    /**
+     * Set the amount of goods in this container.
+     *
+     * @param goodsType The <code>GoodsType</code> to set the amount of.
+     * @param newAmount The new amount.
+     */
+    public void setAmount(GoodsType goodsType, int newAmount) {
         if (newAmount == 0) {
             storedGoods.remove(goodsType);
         } else {
             storedGoods.put(goodsType, newAmount);
         }
-        firePropertyChange(goodsType.getId(), oldAmount, newAmount);
-    }        
+    }
 
     /**
      * Removes all goods.
-     * 
+     *
      */
     public void removeAll() {
         storedGoods.clear();
     }
 
-    
+
     /**
      * Checks if any storable type of goods has reached the given
      * amount.
-     * 
+     *
      * @param amount The amount.
-     * @return <code>true</code> if any type of goods, 
-     * except for <code>Goods.FOOD</code>, has reached 
+     * @return <code>true</code> if any type of goods,
+     * except for <code>Goods.FOOD</code>, has reached
      * the given amount.
      */
     public boolean hasReachedCapacity(int amount) {
         for (GoodsType goodsType : storedGoods.keySet()) {
-            if (goodsType.isStorable() && !goodsType.limitIgnored() && 
+            if (goodsType.isStorable() && !goodsType.limitIgnored() &&
                 storedGoods.get(goodsType) > amount) {
                 return true;
             }
@@ -259,7 +304,7 @@ public class GoodsContainer extends FreeColGameObject {
     /**
      * Returns the amount of one type of Goods at the beginning of the turn.
      * @param type The type of Goods being looked for in this container.
-     * @return The amount of this type of Goods in this container. at the beginning of the turn 
+     * @return The amount of this type of Goods in this container. at the beginning of the turn
      */
     public int getOldGoodsCount(GoodsType type) {
         if (oldStoredGoods.containsKey(type)) {
@@ -268,7 +313,7 @@ public class GoodsContainer extends FreeColGameObject {
             return 0;
         }
     }
-    
+
     public Goods getGoods(GoodsType goodsType) {
         return new Goods(getGame(), parent, goodsType, getGoodsCount(goodsType));
     }
@@ -325,7 +370,7 @@ public class GoodsContainer extends FreeColGameObject {
         return totalGoods;
     }
 
-    
+
     /**
      * Gets an <code>Iterator</code> of every <code>Goods</code> in this
      * <code>GoodsContainer</code>. There is only one <code>Goods</code>
@@ -375,76 +420,50 @@ public class GoodsContainer extends FreeColGameObject {
     }
 
     /**
-     * Removes goods exceeding limit and reports on goods exceeding levels.
+     * Has this goods containers contents changed from what was recorded
+     * last time the state was saved?
      *
+     * @return True if the contents have changed.
      */
-    public void cleanAndReport() {
-        if (!(parent instanceof Colony)) {
-            return;
+    public boolean hasChanged() {
+        for (GoodsType type : getSpecification().getGoodsTypeList()) {
+            int oldCount = getOldGoodsCount(type);
+            int newCount = getGoodsCount(type);
+            if (oldCount != newCount) return true;
         }
-        Colony colony = (Colony) parent;
-        int limit = colony.getWarehouseCapacity();
-        int adjustment = limit / CARGO_SIZE;
+        return false;
+    }
 
-        for (GoodsType goodsType : storedGoods.keySet()) {
-        	boolean isFood = goodsType.isFoodType();
-        	boolean ignoreLimits = !isFood && goodsType.limitIgnored(); 
-            if (ignoreLimits || !goodsType.isStorable()) {
-                continue;
-            }
-            ExportData exportData = colony.getExportData(goodsType);
-            int low = exportData.getLowLevel() * adjustment;
-            int high = exportData.getHighLevel() * adjustment;
-            int amount = storedGoods.get(goodsType).intValue();
-            int oldAmount = getOldGoodsCount(goodsType);
-            // Food does not obey the upper limits, only the lower ones
-            if (!isFood && amount > limit) {
-                // limit has been exceeded
-                int waste = amount - limit;
-                setAmount(goodsType, limit);
-                addModelMessage(colony, ModelMessage.MessageType.WAREHOUSE_CAPACITY, goodsType,
-                                "model.building.warehouseWaste",
-                                "%goods%", Messages.getName(goodsType),
-                                "%waste%", String.valueOf(waste),
-                                "%colony%", colony.getName());
-            } else if (!isFood && amount == limit && oldAmount < limit) {
-                // limit has been reached during this turn
-                addModelMessage(colony, ModelMessage.MessageType.WAREHOUSE_CAPACITY, goodsType,
-                                "model.building.warehouseOverfull",
-                                "%goods%", Messages.getName(goodsType),
-                                "%colony%", colony.getName());
-            } else if (!isFood && amount > high && oldAmount <= high) {
-                addModelMessage(colony, ModelMessage.MessageType.WAREHOUSE_CAPACITY, goodsType,
-                                "model.building.warehouseFull",
-                                "%goods%", Messages.getName(goodsType),
-                                "%level%", String.valueOf(high),
-                                "%colony%", colony.getName());
-            } else if (amount < low && oldAmount >= low) {
-                addModelMessage(colony, ModelMessage.MessageType.WAREHOUSE_CAPACITY, goodsType,
-                                "model.building.warehouseEmpty",
-                                "%goods%", Messages.getName(goodsType),
-                                "%level%", String.valueOf(low),
-                                "%colony%", colony.getName());
+    /**
+     * Fire property changes for all goods that have seen level changes
+     * since the last saveState().
+     */
+    public void fireChanges() {
+        for (GoodsType type : getSpecification().getGoodsTypeList()) {
+            int oldCount = getOldGoodsCount(type);
+            int newCount = getGoodsCount(type);
+            if (oldCount != newCount) {
+                firePropertyChange(type.getId(), oldCount, newCount);
             }
         }
-
+        oldStoredGoods.clear();
     }
 
     /**
      * This method writes an XML-representation of this object to
      * the given stream.
-     * 
+     *
      * <br><br>
-     * 
-     * Only attributes visible to the given <code>Player</code> will 
+     *
+     * Only attributes visible to the given <code>Player</code> will
      * be added to that representation if <code>showAll</code> is
      * set to <code>false</code>.
-     *  
+     *
      * @param out The target stream.
-     * @param player The <code>Player</code> this XML-representation 
+     * @param player The <code>Player</code> this XML-representation
      *      should be made for, or <code>null</code> if
      *      <code>showAll == true</code>.
-     * @param showAll Only attributes visible to <code>player</code> 
+     * @param showAll Only attributes visible to <code>player</code>
      *      will be added to the representation if <code>showAll</code>
      *      is set to <i>false</i>.
      * @param toSavedGame If <code>true</code> then information that
@@ -458,29 +477,27 @@ public class GoodsContainer extends FreeColGameObject {
         out.writeStartElement(getXMLElementTagName());
 
         out.writeAttribute("ID", getId());
-        if (!storedGoods.isEmpty()) {
-            out.writeStartElement(STORED_GOODS_TAG);
-            for (Map.Entry<GoodsType, Integer> entry : storedGoods.entrySet()) {
-                out.writeStartElement(Goods.getXMLElementTagName());
-                out.writeAttribute("type", entry.getKey().getId());
-                out.writeAttribute("amount", entry.getValue().toString());
-                out.writeEndElement();
-            }
-            out.writeEndElement();
-        }
-        if (!oldStoredGoods.isEmpty()) {
-            out.writeStartElement(OLD_STORED_GOODS_TAG);
-            for (Map.Entry<GoodsType, Integer> entry : oldStoredGoods.entrySet()) {
-                out.writeStartElement(Goods.getXMLElementTagName());
-                out.writeAttribute("type", entry.getKey().getId());
-                out.writeAttribute("amount", entry.getValue().toString());
-                out.writeEndElement();
-            }
-            out.writeEndElement();
+        if (showAll || toSavedGame || player == getOwner()) {
+            writeStorage(out, STORED_GOODS_TAG, storedGoods);
+            writeStorage(out, OLD_STORED_GOODS_TAG, oldStoredGoods);
         }
         out.writeEndElement();
     }
 
+    private void writeStorage(XMLStreamWriter out, String tag, Map<GoodsType, Integer> storage)
+        throws XMLStreamException {
+        if (!storage.isEmpty()) {
+            out.writeStartElement(tag);
+            for (Map.Entry<GoodsType, Integer> entry : storage.entrySet()) {
+                out.writeStartElement(Goods.getXMLElementTagName());
+                out.writeAttribute("type", entry.getKey().getId());
+                out.writeAttribute("amount", entry.getValue().toString());
+                out.writeEndElement();
+            }
+            out.writeEndElement();
+        }
+    }
+
     /**
      * Initialize this object from an XML-representation of this object.
      * @param in The input stream with the XML.
@@ -489,43 +506,57 @@ public class GoodsContainer extends FreeColGameObject {
         setId(in.getAttributeValue(null, "ID"));
         storedGoods.clear();
         oldStoredGoods.clear();
+        Map<GoodsType, Integer> storage;
 
         while (in.nextTag() != XMLStreamConstants.END_ELEMENT) {
             if (in.getLocalName().equals(STORED_GOODS_TAG)) {
-                while (in.nextTag() != XMLStreamConstants.END_ELEMENT) {
-                    if (in.getLocalName().equals(Goods.getXMLElementTagName())) {
-                        GoodsType goodsType = Specification.getSpecification().getGoodsType(in.getAttributeValue(null, "type"));
-                        Integer amount = new Integer(in.getAttributeValue(null, "amount"));
-                        storedGoods.put(goodsType, amount);
-                    }
-                    in.nextTag();
-                }
+                storage = storedGoods;
             } else if (in.getLocalName().equals(OLD_STORED_GOODS_TAG)) {
-                while (in.nextTag() != XMLStreamConstants.END_ELEMENT) {
-                    if (in.getLocalName().equals(Goods.getXMLElementTagName())) {
-                        GoodsType goodsType = Specification.getSpecification().getGoodsType(in.getAttributeValue(null, "type"));
-                        Integer amount = new Integer(in.getAttributeValue(null, "amount"));
-                        oldStoredGoods.put(goodsType, amount);
-                    }
-                    in.nextTag();
+                storage = oldStoredGoods;
+            } else {
+                continue;
+            }
+            while (in.nextTag() != XMLStreamConstants.END_ELEMENT) {
+                if (in.getLocalName().equals(Goods.getXMLElementTagName())) {
+                    GoodsType goodsType = getGame().getSpecification()
+                        .getGoodsType(in.getAttributeValue(null, "type"));
+                    Integer amount = new Integer(in.getAttributeValue(null, "amount"));
+                    storage.put(goodsType, amount);
                 }
+                in.nextTag();
             }
         }
     }
 
 
     /**
-     * Gets the tag name of the root element representing this object.
-     * @return "goodsContainer".
+     * Partial writer, so that "remove" messages can be brief.
+     *
+     * @param out The target stream.
+     * @param fields The fields to write.
+     * @throws XMLStreamException If there are problems writing the stream.
      */
-    public static String getXMLElementTagName() {
-        return "goodsContainer";
+    @Override
+    protected void toXMLPartialImpl(XMLStreamWriter out, String[] fields)
+        throws XMLStreamException {
+        toXMLPartialByClass(out, getClass(), fields);
     }
-    
-    
+
+    /**
+     * Partial reader, so that "remove" messages can be brief.
+     *
+     * @param in The input stream with the XML.
+     * @throws XMLStreamException If there are problems reading the stream.
+     */
+    @Override
+    protected void readFromXMLPartialImpl(XMLStreamReader in)
+        throws XMLStreamException {
+        readFromXMLPartialByClass(in, getClass());
+    }
+
     /**
      * Creates a <code>String</code> representation of this
-     * <code>GoodsContainer</code>.    
+     * <code>GoodsContainer</code>.
      */
     public String toString() {
         StringBuffer sb = new StringBuffer(200);
@@ -536,4 +567,11 @@ public class GoodsContainer extends FreeColGameObject {
         return sb.toString();
     }
 
+    /**
+     * Gets the tag name of the root element representing this object.
+     * @return "goodsContainer".
+     */
+    public static String getXMLElementTagName() {
+        return "goodsContainer";
+    }
 }
