@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -17,69 +17,70 @@
  *  along with FreeCol.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-
 package net.sf.freecol.common.model;
 
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Map;
+import java.util.Random;
 
 import javax.xml.stream.XMLStreamConstants;
 import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
-import net.sf.freecol.FreeCol;
-import net.sf.freecol.client.gui.i18n.Messages;
+import net.sf.freecol.common.util.Utils;
 
 import org.w3c.dom.Element;
 
+
 /**
- * This class implements a simple economic model whereby a market holds all
- * goods that have been sold and the price of a particular type of good is
+ * This class implements a simple economic model where a market holds
+ * all goods to be sold and the price of a particular goods type is
  * determined solely by its availability in that market.
  */
 public final class Market extends FreeColGameObject implements Ownable {
 
-    public static final int MINIMUM_PRICE = 1, MAXIMUM_PRICE = 19;
+    /**
+     * European markets are bottomless.  Goods present never decrease
+     * below this threshold.
+     */
+    public static final int MINIMUM_AMOUNT = 100;
 
-    private Player owner;
-    
     /**
      * Constant for specifying the access to this <code>Market</code>
-     * when {@link #buy(GoodsType, int, Player) buying} and
-     * {@link #sell(GoodsType, int, Player, int) selling} goods.
+     * when selling goods.
      */
-    public static final int EUROPE = 0, CUSTOM_HOUSE = 1;
-    
+    public static enum Access {
+        EUROPE,
+        CUSTOM_HOUSE,
+    }
+
     private final Map<GoodsType, MarketData> marketData
         = new HashMap<GoodsType, MarketData>();
 
+    private Player owner;
+
     private ArrayList<TransactionListener> transactionListeners
         = new ArrayList<TransactionListener>();
-    
-    // ----------------------------------------------------------- constructors
 
+
+    /**
+     * Main constructor for creating a market for a new player.
+     */
     public Market(Game game, Player player) {
         super(game);
         this.owner = player;
-        
+
         /*
-         * Create the objects to hold the market data for each type of
-         * goods and seed these objects with the initial amount of
-         * each type of goods.
+         * Create the market data containers for each type of goods
+         * and seed these objects with the initial amount of each type
+         * of goods.
          */
-        for (GoodsType goodsType : FreeCol.getSpecification().getGoodsTypeList()) {
-            MarketData data = new MarketData(goodsType);
+        for (GoodsType goodsType : getSpecification().getGoodsTypeList()) {
             if (goodsType.isStorable()) {
-                data.setAmountInMarket(goodsType.getInitialAmount());
-                data.setPaidForSale(goodsType.getInitialSellPrice());
-                data.setCostToBuy(goodsType.getInitialBuyPrice());
-                data.setInitialPrice(goodsType.getInitialSellPrice());
-                data.setOldPrice(goodsType.getInitialBuyPrice());
-                priceGoods(goodsType);
+                marketData.put(goodsType, new MarketData(game, goodsType));
             }
-            marketData.put(goodsType, data);
         }
     }
 
@@ -124,51 +125,53 @@ public final class Market extends FreeColGameObject implements Ownable {
     }
     
     /**
-     * Adjust the price for a particular type of goods.
+     * Describe <code>putMarketData</code> method here.
      *
      * @param goodsType a <code>GoodsType</code> value
-     * @return True if the price of the goods in the market has changed.
+     * @param data a <code>MarketData</code> value
      */
-    private boolean priceGoods(GoodsType goodsType) {
-        boolean changed = false;
-        MarketData data = marketData.get(goodsType);
-        if (data != null) {
-            int oldPrice = data.getCostToBuy();
-            int newSalePrice = data.getInitialPrice();
-            newSalePrice = Math.round(newSalePrice * goodsType.getInitialAmount()
-                                      / (float) data.getAmountInMarket());
-            int newPrice = newSalePrice + goodsType.getPriceDifference();
+    private void putMarketData(GoodsType goodsType, MarketData data) {
+        marketData.put(goodsType, data);
+    }
 
-            // dirty work-around to limit prices of new world goods
-            // and related manufactured goods
-            if ((goodsType.isNewWorldGoodsType()
-                 || (goodsType.getRawMaterial() != null
-                     && goodsType.getRawMaterial().isNewWorldGoodsType()))
-                && newSalePrice > data.getInitialPrice() + 2) {
-                newSalePrice = data.getInitialPrice() + 2;
-                newPrice = newSalePrice + goodsType.getPriceDifference();
-            }
-
-            if (newPrice > MAXIMUM_PRICE) {
-                newPrice = MAXIMUM_PRICE;
-                newSalePrice = newPrice - goodsType.getPriceDifference();
-            } else if (newSalePrice < MINIMUM_PRICE) {
-                newSalePrice = MINIMUM_PRICE;
-                newPrice = newSalePrice + goodsType.getPriceDifference();
-            }
-
-            data.setOldPrice(oldPrice);
-            data.setCostToBuy(newPrice);
-            data.setPaidForSale(newSalePrice);
-            changed = newPrice != oldPrice;
+    /**
+     * Gets the market data for a specified goods type, creating it
+     * if it does not exist yet.
+     *
+     * @param goodsType The <code>GoodsType</code> to query.
+     * @return The <code>MarketData</code> required.
+     */
+    private MarketData requireMarketData(GoodsType goodsType) {
+        MarketData data = getMarketData(goodsType);
+        if (data == null) {
+            data = new MarketData(getGame(), goodsType);
+            putMarketData(goodsType, data);
         }
-        return changed;
+        return data;
     }
 
     // ------------------------------------------------------------ API methods
 
     /**
-     * Return the market data for a type of goods.
+     * Makes a slight randomization to all the new world and luxury goods
+     * types.  Used at the start of game by the PreGameController.
+     *
+     * @param random A pseudo-random number source.
+     */
+    public void randomizeInitialPrice(Random random) {
+        for (GoodsType type : getGame().getSpecification().getGoodsTypeList()) {
+            if (type.isNewWorldGoodsType() || type.isNewWorldLuxuryType()) {
+                int add = Utils.randomInt(null, null, random, 3);
+                if (add > 0) {
+                    setInitialPrice(type, add + getInitialPrice(type));
+                }
+            }
+        }
+    }
+
+    /**
+     * Return the market data for a type of goods.  This one is public
+     * so the server can send individual MarketData updates.
      *
      * @param goodsType a <code>GoodsType</code> value
      * @return a <code>MarketData</code> value
@@ -177,16 +180,6 @@ public final class Market extends FreeColGameObject implements Ownable {
         return marketData.get(goodsType);
     }
 
-    /**
-     * Describe <code>putMarketData</code> method here.
-     *
-     * @param goodsType a <code>GoodsType</code> value
-     * @param data a <code>MarketData</code> value
-     */
-    public void putMarketData(GoodsType goodsType, MarketData data) {
-        marketData.put(goodsType, data);
-    }
-
     /**
      * Gets the owner of this <code>Market</code>.
      *
@@ -199,20 +192,31 @@ public final class Market extends FreeColGameObject implements Ownable {
     /**
      * Sets the owner of this <code>Market</code>.
      *
-     * @param owner The <code>Player</code> that shall own this <code>Market</code>.
+     * @param owner The <code>Player</code> to own this <code>Market</code>.
      */
     public void setOwner(Player owner) {
         this.owner = owner;
     }
 
+    /**
+     * Has a type of goods been traded in this market?
+     *
+     * @param type The type of goods to consider.
+     * @return True if the goods type has been traded.
+     */
+    public boolean hasBeenTraded(GoodsType type) {
+        MarketData data = getMarketData(type);
+        return data != null && data.getTraded();
+    }
+
     /**
      * Determines the cost to buy a single unit of a particular type of good.
      *
      * @param type A <code>GoodsType</code> value.
      * @return The cost to buy a single unit of the given type of goods.
      */
-    public int costToBuy(GoodsType type) {
-        MarketData data = marketData.get(type);
+    public int getCostToBuy(GoodsType type) {
+        MarketData data = getMarketData(type);
         return (data == null) ? 0 : data.getCostToBuy();
     }
 
@@ -224,137 +228,47 @@ public final class Market extends FreeColGameObject implements Ownable {
      * @return The price for a single unit of the given type of goods
      *         if it is for sale.
      */
-    public int paidForSale(GoodsType type) {
-        MarketData data = marketData.get(type);
+    public int getPaidForSale(GoodsType type) {
+        MarketData data = getMarketData(type);
         return (data == null) ? 0 : data.getPaidForSale();
     }
 
-
-    /**
-     * Sells an amount of a particular type of good with the proceeds
-     * of the sale being paid to a supplied player. The goods is sold
-     * using {@link #EUROPE} as the accesspoint for this market.
-     * Note that post-independence this no longer refers to a specific port.
-     *
-     * @param goods The <code>Goods</code> object being sold.
-     * @param player The <code>Player</code> selling the goods.
-     */
-    public void sell(Goods goods, Player player) {
-        GoodsType type = goods.getType();
-        int amount = goods.getAmount();
-
-        goods.setLocation(null);
-        sell(type, amount, player, Market.EUROPE);
-    }
-
-    /**
-     * Sells an amount of a particular type of good with the proceeds
-     * of the sale being paid to a supplied player. The goods is
-     * sold using {@link #EUROPE} as the accesspoint for this market.
-     *
-     * @param type The type of goods to be sold.
-     * @param amount The amount of goods to be sold.
-     * @param player The <code>Player</code> selling the goods.
-     */
-    public void sell(GoodsType type, int amount, Player player) {
-        sell(type, amount, player, Market.EUROPE);
-    }
-
-    /**
-     * Sells an amount of a particular type of good with the proceeds
-     * of the sale being paid to a supplied player.
-     *
-     * @param type   The type of goods to be sold.
-     * @param amount The amount of goods to be sold.
-     * @param player The <code>Player</code> selling the goods.
-     * @param marketAccess The place where goods are traded.
-     */
-    public void sell(GoodsType type, int amount, Player player, int marketAccess) {
-        if (player.canTrade(type, marketAccess)) {
-            int tax = player.getTax();
-            int incomeBeforeTaxes = getSalePrice(type, amount);
-            int incomeAfterTaxes = ((100 - tax) * incomeBeforeTaxes) / 100;
-            player.modifyGold(incomeAfterTaxes);
-            player.modifySales(type, amount);
-            player.modifyIncomeBeforeTaxes(type, incomeBeforeTaxes);
-            player.modifyIncomeAfterTaxes(type, incomeAfterTaxes);
-            
-            int unitPrice = paidForSale(type);
-            for(TransactionListener listener : transactionListeners) {
-                listener.logSale(type, amount, unitPrice, tax);
-            }
-
-            amount = (int) player.getFeatureContainer()
-                .applyModifier(amount, "model.modifier.tradeBonus",
-                               type, getGame().getTurn());
-            if (addGoodsToMarket(type, amount)) {
-                player.addModelMessage(makePriceMessage(type));
-            }
-        } else {
-            addModelMessage(this, ModelMessage.MessageType.WARNING,
-                            "model.europe.market", "%goods%", Messages.getName(type));
-        }
-    }
-
-    /**
-     * Buys an amount of a particular type of good with the cost being
-     * met by a supplied player.
-     *
-     * @param goodsType The type of the good that is being bought.
-     * @param amount The amount of goods that are being bought.
-     * @param player The <code>Player</code> buying the goods.
-     * @throws IllegalStateException If the <code>player</code> cannot afford
-     *                               to buy the goods.
-     */
-    public void buy(GoodsType goodsType, int amount, Player player) {
-        int price = getBidPrice(goodsType, amount);
-        if (price > player.getGold()) {
-            throw new IllegalStateException("Player " + player.getName()
-                                            + " tried to buy " + Integer.toString(amount)
-                                            + " " + goodsType.toString()
-                                            + " for " + Integer.toString(price)
-                                            + " but has " + Integer.toString(player.getGold())
-                                            + " gold.");
-        }
-        player.modifyGold(-price);
-        player.modifySales(goodsType, -amount);
-        player.modifyIncomeBeforeTaxes(goodsType, -price);
-        player.modifyIncomeAfterTaxes(goodsType, -price);
-
-        int unitPrice = costToBuy(goodsType);
-        for(TransactionListener listener : transactionListeners) {
-            listener.logPurchase(goodsType, amount, unitPrice);
-        }
-
-        amount = (int) player.getFeatureContainer()
-            .applyModifier(amount, "model.modifier.tradeBonus",
-                           goodsType, getGame().getTurn());
-        if (addGoodsToMarket(goodsType, -amount)) {
-            player.addModelMessage(makePriceMessage(goodsType));
-        }
-    }
-
     /**
      * Add (or remove) some goods to this market.
      * 
      * @param goodsType The <code>GoodsType</code> to add.
      * @param amount The amount of goods.
-     * @return True if the goods changes price due to this change.
      */
-    public boolean addGoodsToMarket(GoodsType goodsType, int amount) {
-        MarketData data = getMarketData(goodsType);
-        if (data == null) {
-            data = new MarketData(goodsType);
-            marketData.put(goodsType, data);
-        }
+    public void addGoodsToMarket(GoodsType goodsType, int amount) {
+        MarketData data = requireMarketData(goodsType);
 
-        /*
-         * Markets are bottomless, amount can not go below the
-         * threshold of 100.
-         */
-        data.setAmountInMarket(Math.max(100, data.getAmountInMarket() + amount));
+        // Markets are bottomless, amount can not go below the threshold
+        data.setAmountInMarket(Math.max(MINIMUM_AMOUNT,
+                                        data.getAmountInMarket() + amount));
         data.setTraded(true);
-        return priceGoods(goodsType);
+        data.price();
+    }
+
+    /**
+     * Gets the initial price of a given goods type.
+     *
+     * @param goodsType The <code>GoodsType</code> to get the initial price of.
+     * @return The initial price.
+     */
+    public int getInitialPrice(GoodsType goodsType) {
+        MarketData data = requireMarketData(goodsType);
+        return data.getInitialPrice();
+    }
+
+    /**
+     * Sets the initial price of a given goods type.
+     *
+     * @param goodsType The <code>GoodsType</code> to set the initial price of.
+     * @param amount The new initial price.
+     */
+    public void setInitialPrice(GoodsType goodsType, int amount) {
+        MarketData data = requireMarketData(goodsType);
+        data.setInitialPrice(amount);
     }
 
     /**
@@ -365,7 +279,7 @@ public final class Market extends FreeColGameObject implements Ownable {
      * @return The bid price of the given goods.
      */
     public int getBidPrice(GoodsType type, int amount) {
-        MarketData data = marketData.get(type);
+        MarketData data = getMarketData(type);
         return (data == null) ? 0 : amount * data.getCostToBuy();
     }
 
@@ -377,7 +291,7 @@ public final class Market extends FreeColGameObject implements Ownable {
      * @return The sale price of the given goods.
      */
     public int getSalePrice(GoodsType type, int amount) {
-        MarketData data = marketData.get(type);
+        MarketData data = getMarketData(type);
         return (data == null) ? 0 : amount * data.getPaidForSale();
     }
 
@@ -391,6 +305,154 @@ public final class Market extends FreeColGameObject implements Ownable {
         return getSalePrice(goods.getType(), goods.getAmount());
     }
 
+    /**
+     * Gets the arrears for of a given goods type.
+     *
+     * @param goodsType The <code>GoodsType</code> to get arrears for.
+     * @return The arrears.
+     */
+    public int getArrears(GoodsType goodsType) {
+        MarketData data = getMarketData(goodsType);
+        return (data == null) ? 0 : data.getArrears();
+    }
+
+    /**
+     * Sets the arrears associated with a type of goods.
+     *
+     * @param goodsType The <code>GoodsType</code> to set the arrears for.
+     * @param value The amount of arrears to set.
+     */
+    public void setArrears(GoodsType goodsType, int value) {
+        MarketData data = requireMarketData(goodsType);
+        data.setArrears(value);
+    }
+
+    /**
+     * Gets the sales of a type of goods.
+     *
+     * @param goodsType The <code>GoodsType</code> to get the sales for.
+     * @return The current sales amount.
+     */
+    public int getSales(GoodsType goodsType) {
+        MarketData data = getMarketData(goodsType);
+        return (data == null) ? 0 : data.getSales();
+    }
+
+    /**
+     * Modifies the sales of a type of goods.
+     *
+     * @param goodsType The <code>GoodsType</code> to set the sales for.
+     * @param amount The amount of sales to add to the current amount.
+     */
+    public void modifySales(GoodsType goodsType, int amount) {
+        if (amount != 0) {
+            MarketData data = requireMarketData(goodsType);
+            data.setSales(data.getSales() + amount);
+            data.setTraded(true);
+        }
+    }
+
+    /**
+     * Gets the income before taxes for a type of goods.
+     *
+     * @param goodsType The <code>GoodsType</code> to get the income for.
+     * @return The current income before taxes.
+     */
+    public int getIncomeBeforeTaxes(GoodsType goodsType) {
+        MarketData data = getMarketData(goodsType);
+        return (data == null) ? 0 : data.getIncomeBeforeTaxes();
+    }
+
+    /**
+     * Modifies the income before taxes on sales of a type of goods.
+     *
+     * @param goodsType The <code>GoodsType</code> to set the income for.
+     * @param amount The amount of tax income to add to the current amount.
+     */
+    public void modifyIncomeBeforeTaxes(GoodsType goodsType, int amount) {
+        MarketData data = requireMarketData(goodsType);
+        data.setIncomeBeforeTaxes(data.getIncomeBeforeTaxes() + amount);
+    }
+
+    /**
+     * Gets the income after taxes for a type of goods.
+     *
+     * @param goodsType The <code>GoodsType</code> to get the income for.
+     * @return The current income after taxes.
+     */
+    public int getIncomeAfterTaxes(GoodsType goodsType) {
+        MarketData data = getMarketData(goodsType);
+        return (data == null) ? 0 : data.getIncomeAfterTaxes();
+    }
+
+    /**
+     * Modifies the income after taxes on sales of a type of goods.
+     *
+     * @param goodsType The <code>GoodsType</code> to set the income for.
+     * @param amount The amount of tax income to add to the current amount.
+     */
+    public void modifyIncomeAfterTaxes(GoodsType goodsType, int amount) {
+        MarketData data = requireMarketData(goodsType);
+        data.setIncomeAfterTaxes(data.getIncomeAfterTaxes() + amount);
+    }
+
+    /**
+     * Gets the amount of a goods type in the market.
+     *
+     * @param goodsType The <code>GoodsType</code> to get the amount of.
+     * @return The current amount of the goods in the market.
+     */
+    public int getAmountInMarket(GoodsType goodsType) {
+        MarketData data = getMarketData(goodsType);
+        return (data == null) ? 0 : data.getAmountInMarket();
+    }
+
+    /**
+     * Has the price of a type of goods changed in this market?
+     *
+     * @param goodsType The type of goods to consider.
+     * @return True if the price has changed.
+     */
+    public boolean hasPriceChanged(GoodsType goodsType) {
+        MarketData data = getMarketData(goodsType);
+        return data != null && data.getOldPrice() != 0
+            && data.getOldPrice() != data.getCostToBuy();
+    }
+
+    /**
+     * Clear any price changes for a type of goods.
+     *
+     * @param goodsType The type of goods to consider.
+     */
+    public void flushPriceChange(GoodsType goodsType) {
+        MarketData data = getMarketData(goodsType);
+        if (data != null) {
+            data.setOldPrice(data.getCostToBuy());
+        }
+    }
+
+    /**
+     * Make up a <code>ModelMessage</code> describing the change in this
+     * <code>Market</code> for a specified type of goods.
+     *
+     * @param goodsType The <code>GoodsType</code> that has changed price.
+     * @return A message describing the change.
+     */
+    public ModelMessage makePriceChangeMessage(GoodsType goodsType) {
+        MarketData data = getMarketData(goodsType);
+        int oldPrice = data.getOldPrice();
+        int newPrice = data.getCostToBuy();
+        return (oldPrice == newPrice) ? null
+            : new ModelMessage(ModelMessage.MessageType.MARKET_PRICES,
+                               ((newPrice > oldPrice)
+                                ? "model.market.priceIncrease"
+                                : "model.market.priceDecrease"),
+                               this, goodsType)
+            .addStringTemplate("%market%", owner.getMarketName())
+            .add("%goods%", goodsType.getNameKey())
+            .addAmount("%buy%", newPrice)
+            .addAmount("%sell%", data.getPaidForSale());
+    }
 
     /**
      * Adds a transaction listener for notification of any transaction
@@ -421,40 +483,10 @@ public final class Market extends FreeColGameObject implements Ownable {
     }
 
 
-    // -------------------------------------------------------- support methods
-
-    /**
-     * Make up a <code>ModelMessage</code> describing the change in this
-     * <code>Market</code> for a specified type of goods.
-     *
-     * @param goodsType The <code>GoodsType</code> that has changed price.
-     * @return A message describing the change.
-     */
-    public ModelMessage makePriceMessage(GoodsType goodsType) {
-        MarketData data = marketData.get(goodsType);
-        int oldPrice = data.getOldPrice();
-        int newPrice = data.getCostToBuy();
-        int newSalePrice = data.getPaidForSale();
-
-        return (oldPrice == newPrice) ? null
-            : new ModelMessage(this,
-                               ModelMessage.MessageType.MARKET_PRICES,
-                               goodsType,
-                               ((newPrice > oldPrice)
-                                ? "model.market.priceIncrease"
-                                : "model.market.priceDecrease"),
-                               "%market%", owner.getMarketName(),
-                               "%goods%", Messages.getName(goodsType),
-                               "%buy%", String.valueOf(newPrice),
-                               "%sell%", String.valueOf(newSalePrice));
-    }
-
     /**
      * This method writes an XML-representation of this object to
      * the given stream.
      * 
-     * <br><br>
-     * 
      * Only attributes visible to the given <code>Player</code> will 
      * be added to that representation if <code>showAll</code> is
      * set to <code>false</code>.
@@ -480,8 +512,10 @@ public final class Market extends FreeColGameObject implements Ownable {
         out.writeAttribute("ID", getId());
         out.writeAttribute("owner", owner.getId());
         
-        for (MarketData data : marketData.values()) {
-            data.toXML(out, player, showAll, toSavedGame);
+        if (player == owner || showAll || toSavedGame) {
+            for (MarketData data : marketData.values()) {
+                data.toXML(out, player, showAll, toSavedGame);
+            }
         }
 
         out.writeEndElement();
@@ -492,24 +526,25 @@ public final class Market extends FreeColGameObject implements Ownable {
      *
      * @param in The input stream with the XML.
      * @throws XMLStreamException on problems with the stream.
-     * TODO: Get rid of the for-priceGoods() when the server sends all
+     * TODO: Get rid of the price() when the server sends all
      * price changes.
      */
-    protected void readFromXMLImpl(XMLStreamReader in) throws XMLStreamException {
-        setId(in.getAttributeValue(null, "ID"));
+    protected void readFromXMLImpl(XMLStreamReader in)
+        throws XMLStreamException {
+        Game game = getGame();
+        setId(in.getAttributeValue(null, FreeColObject.ID_ATTRIBUTE));
         owner = getFreeColGameObject(in, "owner", Player.class);
 
         while (in.nextTag() != XMLStreamConstants.END_ELEMENT) {
             if (in.getLocalName().equals(MarketData.getXMLElementTagName())) {
-                MarketData data = new MarketData();
-                data.readFromXML(in);
-                marketData.put(data.getGoodsType(), data);
-            }
-        }
-        
-        for (GoodsType goodsType : FreeCol.getSpecification().getGoodsTypeList()) {
-            if (goodsType.isStorable()) {
-                priceGoods(goodsType);
+                String id = in.getAttributeValue(null, FreeColObject.ID_ATTRIBUTE);
+                MarketData data = (MarketData) game.getFreeColGameObject(id);
+                if (data == null) {
+                    data = new MarketData(game, in);
+                } else {
+                    data.readFromXML(in);
+                }
+                putMarketData(data.getGoodsType(), data);
             }
         }
     }
