@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -23,9 +23,15 @@ package net.sf.freecol.common.model;
 /**
  * An object with a name that can be changed.
  */
-public interface Nameable extends Named {
+public interface Nameable {
 
 
+    /**
+    * Gets the name of this <code>Named</code>.
+    * @return The name of the <code>Named</code>.
+    */
+    public String getName();
+
     /**
     * Sets the name for this <code>Nameable</code>.
     * @param newName The new name for the <code>Nameable</code>.
