@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -21,19 +21,14 @@
 package net.sf.freecol.common.model;
 
 /**
- * An object that has a proper name, such as a Settlement, Unit or
- * Player. Do not use this for objects that derive their name from
- * their type, such as Building, or Tile, or from other fields. Do not
- * use this to localize names, either. Use Messages.getName() and
- * similar methods instead.
+ * An object that has a name.
  */
 public interface Named {
 
-    /**
-     * Gets the name of this <code>Named</code> object.
-     *
-     * @return The name of the <code>Named</code> object.
-     */
-    public String getName();
 
+    /**
+    * Gets the name of this <code>Named</code>.
+    * @return The name of the <code>Named</code>.
+    */
+    public String getNameKey();
 }
