@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -22,13 +22,16 @@ package net.sf.freecol.common.model;
 import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Random;
+import java.util.Set;
 import java.util.logging.Logger;
 
 import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
+import javax.xml.stream.XMLStreamWriter;
 
-import net.sf.freecol.FreeCol;
-import net.sf.freecol.common.model.Map.Direction;
+import net.sf.freecol.common.model.Unit.UnitState;
+import net.sf.freecol.common.model.UnitTypeChange.ChangeType;
 
 import org.w3c.dom.Element;
 
@@ -38,18 +41,9 @@ import org.w3c.dom.Element;
  */
 abstract public class Settlement extends FreeColGameObject implements Location, Named, Ownable {
 
-    private static final Logger logger = Logger.getLogger(Settlement.class.getName()); 
-    
-    public static enum SettlementType {
-        SMALL, MEDIUM, LARGE, 
-            SMALL_STOCKADE,
-            MEDIUM_STOCKADE, MEDIUM_FORT,
-            LARGE_STOCKADE, LARGE_FORT, LARGE_FORTRESS, 
-            UNDEAD, 
-            INDIAN_CAMP, INDIAN_VILLAGE, AZTEC_CITY, INCA_CITY }
-    
-    public static final int RADIUS = 1;
-    public static final int FOOD_CONSUMPTION = 2;
+    private static final Logger logger = Logger.getLogger(Settlement.class.getName());
+
+    public static final int FOOD_PER_COLONIST = 200;
 
     /** The <code>Player</code> owning this <code>Settlement</code>. */
     protected Player owner;
@@ -59,19 +53,26 @@ abstract public class Settlement extends FreeColGameObject implements Location,
 
     /** The <code>Tile</code> where this <code>Settlement</code> is located. */
     protected Tile tile;
-    
+
     protected GoodsContainer goodsContainer;
-    
-    /**
-     * Whether this is the capital of the nation.
-     */
-    private boolean isCapital = false;
 
     /**
      * Contains the abilities and modifiers of this Colony.
      */
-    protected FeatureContainer featureContainer = new FeatureContainer();
+    private FeatureContainer featureContainer;
 
+    /**
+     * Describe type here.
+     */
+    private SettlementType type;
+
+
+    /**
+     * Empty constructor needed for Colony -> ServerColony.
+     */
+    protected Settlement() {
+        // empty constructor
+    }
 
     /**
      * Creates a new <code>Settlement</code>.
@@ -79,7 +80,7 @@ abstract public class Settlement extends FreeColGameObject implements Location,
      * @param game The <code>Game</code> in which this object belong.
      * @param owner The owner of this <code>Settlement</code>.
      * @param name The name for this <code>Settlement</code>.
-     * @param tile The location of the <code>Settlement</code>.    
+     * @param tile The location of the <code>Settlement</code>.
      */
     public Settlement(Game game, Player owner, String name, Tile tile) {
         super(game);
@@ -87,23 +88,13 @@ abstract public class Settlement extends FreeColGameObject implements Location,
         this.name = name;
         this.tile = tile;
 
-        // Relocate any worker already on the Tile (from another Settlement):
-        if (tile.getOwningSettlement() != null) {
-            if (tile.getOwningSettlement() instanceof Colony) {
-                Colony oc = (Colony) tile.getOwningSettlement();
-                ColonyTile ct = oc.getColonyTile(tile);
-                ct.relocateWorkers();
-            } else if (tile.getOwningSettlement() instanceof IndianSettlement) {
-                logger.warning("An indian settlement is already owning the tile.");
-            } else {
-                logger.warning("An unknown type of settlement is already owning the tile.");
-            }
-        }
+        featureContainer = new FeatureContainer(game.getSpecification());
+        setType(owner.getNationType().getSettlementType(false));
 
         owner.addSettlement(this);
     }
 
-    
+
 
     /**
      * Initiates a new <code>Settlement</code> from an <code>Element</code>.
@@ -129,7 +120,7 @@ abstract public class Settlement extends FreeColGameObject implements Location,
     }
 
     /**
-     * Initiates a new <code>Settlement</code> 
+     * Initiates a new <code>Settlement</code>
      * with the given ID. The object should later be
      * initialized by calling either
      * {@link #readFromXML(XMLStreamReader)} or
@@ -141,7 +132,39 @@ abstract public class Settlement extends FreeColGameObject implements Location,
     public Settlement(Game game, String id) {
         super(game, id);
     }
-    
+
+    /**
+     * Get the <code>Type</code> value.
+     *
+     * @return a <code>SettlementType</code> value
+     */
+    public final SettlementType getType() {
+        return type;
+    }
+
+    /**
+     * Set the <code>Type</code> value.
+     *
+     * @param newType The new Type value.
+     */
+    public final void setType(final SettlementType newType) {
+        if (type != null) {
+            featureContainer.remove(type.getFeatureContainer());
+        }
+        this.type = newType;
+        if (newType != null) {
+            featureContainer.add(newType.getFeatureContainer());
+        }
+    }
+
+    public Set<Modifier> getModifierSet(String key) {
+        return featureContainer.getModifierSet(key);
+    }
+
+    // TODO: remove this again
+    public String getNameKey() {
+        return getName();
+    }
 
     /**
      * Gets the name of this <code>Settlement</code>.
@@ -152,6 +175,14 @@ abstract public class Settlement extends FreeColGameObject implements Location,
         return name;
     }
 
+    /**
+     * Gets the name of this <code>Settlement</code> for a particular player.
+     *
+     * @param player A <code>Player</code> to return the name for.
+     * @return The name as a <code>String</code>.
+     */
+    abstract public String getNameFor(Player player);
+
     /**
      * Sets the name of this <code>Settlement</code>.
      *
@@ -162,21 +193,28 @@ abstract public class Settlement extends FreeColGameObject implements Location,
     }
 
     /**
+     * Gets an image key for this settlement.
+     */
+    abstract public String getImageKey();
+
+     /**
      * Returns <code>true</code> if this is the Nation's capital.
      *
      * @return <code>true</code> if this is the Nation's capital.
      */
     public boolean isCapital() {
-        return isCapital;
+        return getType().isCapital();
     }
 
-    /**
+     /**
      * Sets the capital value.
      *
      * @param isCapital a <code>boolean</code> value
      */
     public void setCapital(boolean isCapital) {
-        this.isCapital = isCapital;
+        if (isCapital() != isCapital) {
+            setType(owner.getNationType().getSettlementType(isCapital));
+        }
     }
 
 
@@ -189,6 +227,15 @@ abstract public class Settlement extends FreeColGameObject implements Location,
         return featureContainer;
     }
 
+    /**
+     * Describe <code>setFeatureContainer</code> method here.
+     *
+     * @param container a <code>FeatureContainer</code> value
+     */
+    protected void setFeatureContainer(FeatureContainer container) {
+        featureContainer = container;
+    }
+
     /**
      * Gets this colony's line of sight.
      * @return The line of sight offered by this
@@ -196,18 +243,42 @@ abstract public class Settlement extends FreeColGameObject implements Location,
      * @see Player#canSee(Tile)
      */
     public int getLineOfSight() {
-        return 2;
+        return getType().getVisibleRadius();
     }
-    
+
 
     /**
-     * Gets the <code>Unit</code> that is currently defending this <code>Settlement</code>.
-     * @param attacker The target that would be attacking this <code>Settlement</code>.
-     * @return The <code>Unit</code> that has been chosen to defend this <code>Settlement</code>.
+     * Gets the <code>Unit</code> that is currently defending this
+     * <code>Settlement</code>.
+     *
+     * @param attacker The unit be attacking this <code>Settlement</code>.
+     * @return The <code>Unit</code> that has been chosen to defend
+     * this <code>Settlement</code>.
      */
     abstract public Unit getDefendingUnit(Unit attacker);
 
-    
+    /**
+     * Gets the range of gold plunderable when this settlement is captured.
+     *
+     * @param attacker The <code>Unit</code> that takes the settlement.
+     * @return A <code>RandomRange</code> encapsulating the range of plunder
+     *     available.
+     */
+    abstract public RandomRange getPlunderRange(Unit attacker);
+
+    /**
+     * Gets an amount of plunder when this settlement is taken.
+     *
+     * @param attacker The <code>Unit</code> that takes the settlement.
+     * @param random A pseudo-random number source.
+     * @return An amount of gold plundered.
+     */
+    public int getPlunder(Unit attacker, Random random) {
+        RandomRange range = getPlunderRange(attacker);
+        return (range == null) ? 0
+            : range.getAmount("Plunder " + getName(), random, false);
+    }
+
     /**
      * Gets the <code>Tile</code> where this <code>Settlement</code> is located.
      * @return The <code>Tile</code> where this <code>Settlement</code> is located.
@@ -215,83 +286,20 @@ abstract public class Settlement extends FreeColGameObject implements Location,
     public Tile getTile() {
         return tile;
     }
-    
-    /**
-     * Gets a <code>Tile</code> from the neighbourhood of this 
-     * <code>Colony</code>.
-     * 
-     * @param x The x-coordinate of the <code>Tile</code>.
-     * @param y The y-coordinate of the <code>Tile</code>. 
-     * @return The <code>Tile</code>.
-     */
-    public Tile getTile(int x, int y) {
-        if (x==0 && y==0) {
-            return getGame().getMap().getNeighbourOrNull(Direction.N, tile);
-        } else if (x==0 && y== 1) {
-            return getGame().getMap().getNeighbourOrNull(Direction.NE, tile);
-        } else if (x==0 && y== 2) {
-            return getGame().getMap().getNeighbourOrNull(Direction.E, tile);
-        } else if (x==1 && y== 0) {
-            return getGame().getMap().getNeighbourOrNull(Direction.NW, tile);
-        } else if (x==1 && y== 1) {
-            return tile;
-        } else if (x==1 && y== 2) {
-            return getGame().getMap().getNeighbourOrNull(Direction.SE, tile);
-        } else if (x==2 && y== 0) {
-            return getGame().getMap().getNeighbourOrNull(Direction.W, tile);
-        } else if (x==2 && y== 1) {
-            return getGame().getMap().getNeighbourOrNull(Direction.SW, tile);
-        } else if (x==2 && y== 2) {
-            return getGame().getMap().getNeighbourOrNull(Direction.S, tile);
-        } else {
-            return null;
-        }
-    }
-
-    /**
-     * Claim and explore the surrounding tiles.
-     * Try to be as greedy as possible.
-     */
-    private void claimTiles() {
-        Map map = getGame().getMap();
-        Tile settlementTile = getTile();
-
-        settlementTile.setOwningSettlement(this);
-        settlementTile.setOwner(owner);
-        owner.setExplored(settlementTile);
-        settlementTile.updatePlayerExploredTiles();
-        for (Tile tile : map.getSurroundingTiles(settlementTile, getRadius())) {
-            claimTile(tile);
-        }
-        for (Tile tile : map.getSurroundingTiles(settlementTile, getLineOfSight())) {
-            owner.setExplored(tile);
-        }
-    }
-
-    /**
-     * Claim a single <code>Tile</code> for this Settlement.
-     *
-     * @param tile a <code>Tile</code> value
-     */
-    public void claimTile(Tile tile) {
-        // Indian players do not own water tiles per Col1 rules
-        if (tile.isLand() || owner.isEuropean()) {
-            if (tile.getOwner() == null
-                || tile.getOwningSettlement() == null
-                || tile.getOwningSettlement() == this) {
-                tile.setOwningSettlement(this);
-                tile.setOwner(owner);
-                tile.updatePlayerExploredTiles();
-            }
-        }
-    }
 
     /**
      * Put a prepared settlement onto the map.
      */
     public void placeSettlement() {
-        claimTiles();
+        List<Tile> tiles = getGame().getMap().getClaimableTiles(owner, tile,
+                                                                getRadius());
         tile.setSettlement(this);
+        for (Tile t : tiles) {
+            t.changeOwnership(owner, this);
+        }
+        for (Tile t : tile.getSurroundingTiles(getLineOfSight())) {
+            owner.setExplored(t);
+        }
         owner.invalidateCanSeeTiles();
     }
 
@@ -308,33 +316,71 @@ abstract public class Settlement extends FreeColGameObject implements Location,
     /**
      * Sets the owner of this <code>Settlement</code>.
      *
-     * @param owner The <code>Player</code> that shall own this <code>Settlement</code>.
+     * @param player The new owner of this <code>Settlement</code>.
+     */
+    public void setOwner(Player player) {
+        owner = player;
+    }
+
+    /**
+     * Change the owner of this <code>Settlement</code>.
+     *
+     * @param newOwner The <code>Player</code> that shall own this
+     *            <code>Settlement</code>.
      * @see #getOwner
      */
-    public void setOwner(Player owner) {
-        Player oldOwner = this.owner;        
-        this.owner = owner;
-        
+    public void changeOwner(Player newOwner) {
+        Player oldOwner = this.owner;
+        setOwner(newOwner);
+
         if (oldOwner.hasSettlement(this)) {
             oldOwner.removeSettlement(this);
         }
-        if (!owner.hasSettlement(this)) {
-            owner.addSettlement(this);
+        if (!newOwner.hasSettlement(this)) {
+            newOwner.addSettlement(this);
+        }
+
+        List<Unit> units = getUnitList();
+        units.addAll(getTile().getUnitList());
+        for (Unit u : units) {
+            u.setState(UnitState.ACTIVE);
+            UnitType type = u.getTypeChange((newOwner.isUndead())
+                                            ? ChangeType.UNDEAD
+                                            : ChangeType.CAPTURE, newOwner);
+            if (type != null) u.setType(type);
+            u.setOwner(newOwner);
+        }
+
+        for (Tile t : getOwnedTiles()) {
+            t.changeOwnership(newOwner, this);
         }
-        
-        claimTiles();
         oldOwner.invalidateCanSeeTiles();
-        owner.invalidateCanSeeTiles();
+        newOwner.invalidateCanSeeTiles();
 
         if (getGame().getFreeColGameObjectListener() != null) {
-            getGame().getFreeColGameObjectListener().ownerChanged(this, oldOwner, owner);
+            getGame().getFreeColGameObjectListener()
+                .ownerChanged(this, oldOwner, newOwner);
         }
     }
 
+    /**
+     * Gets the goods container for this settlement.
+     *
+     * @return The settlements goods container.
+     */
     public GoodsContainer getGoodsContainer() {
         return goodsContainer;
     }
 
+    /**
+     * Sets the goods container for this settlement.
+     *
+     * @param goodsContainer The new <code>GoodsContainer</code> to use.
+     */
+    public void setGoodsContainer(GoodsContainer goodsContainer) {
+        this.goodsContainer = goodsContainer;
+    }
+
     /**
      * Gets an <code>Iterator</code> of every <code>Goods</code> in this
      * <code>GoodsContainer</code>. Each <code>Goods</code> have a maximum
@@ -350,67 +396,71 @@ abstract public class Settlement extends FreeColGameObject implements Location,
      * Gets an <code>List</code> with every <code>Goods</code> in this
      * <code>Colony</code>. There is only one <code>Goods</code> for each
      * type of goods.
-     * 
+     *
      * @return The <code>Iterator</code>.
      */
     public List<Goods> getCompactGoods() {
         return goodsContainer.getCompactGoods();
     }
 
+    /**
+     * Get the tiles this settlement owns.
+     *
+     * @return A list of tiles.
+     */
+    public List<Tile> getOwnedTiles() {
+        Tile settlementTile = getTile();
+        ArrayList<Tile> tiles = new ArrayList<Tile>();
+        for (Tile t : settlementTile.getSurroundingTiles(getRadius())) {
+            if (t.getOwningSettlement() == this) tiles.add(t);
+        }
+        tiles.add(settlementTile);
+        return tiles;
+    }
 
     /**
-     * Adds a <code>Locatable</code> to this Location.
+     * Dispose of this settlement.
      *
-     * @param locatable The <code>Locatable</code> to add to this Location.
+     * @return A list of disposed objects.
      */
-    public abstract void add(Locatable locatable);
+    public List<FreeColGameObject> disposeList() {
+        List<FreeColGameObject> objects = new ArrayList<FreeColGameObject>();
+        if (owner != null && goodsContainer != null) {
+            objects.addAll(goodsContainer.disposeList());
+            goodsContainer = null;
+        }
 
-    /**
-     * Removes a <code>Locatable</code> from this Location.
-     *
-     * @param locatable The <code>Locatable</code> to remove from this Location.
-     */
-    public abstract void remove(Locatable locatable);
+        if (owner != null && getTile() != null
+            && getTile().getSettlement() != null) {
+            // Defensive tests to handle transition from calling dispose()
+            // on both sides to when it is only called on server-side.
 
-    public abstract boolean canAdd(Locatable locatable);
+            // Get off the map
+            Tile settlementTile = getTile();
+            List<Tile> lostTiles = getOwnedTiles();
+            for (Tile tile : lostTiles) {
+                tile.changeOwnership(null, null);
+            }
+            settlementTile.setSettlement(null);
 
-    /**
-     * Returns the number of units in this settlement.
-     *
-     * @return The number of units in this settlement.
-     */
-    public abstract int getUnitCount();
-
-    public abstract boolean contains(Locatable locatable);
+            // The owner forgets about the settlement.
+            Player oldOwner = owner;
+            setOwner(null);
+            oldOwner.removeSettlement(this);
+            oldOwner.invalidateCanSeeTiles();
+        }
 
+        objects.addAll(super.disposeList());
+        return objects;
+    }
 
     /**
      * Dispose of this <code>Settlement</code>.
      */
     public void dispose() {
-        Player oldOwner = owner;
-        Tile settlementTile = getTile();
-        Map map = getGame().getMap();
-
-        for (Tile tile : map.getSurroundingTiles(settlementTile, getRadius())) {
-            if (tile.getOwningSettlement() == this
-                || (tile.getOwningSettlement() == null && tile.getOwner() == owner)) {
-                tile.setOwningSettlement(null);
-                tile.setOwner(null);
-                tile.updatePlayerExploredTiles();
-            }
-        }
-        settlementTile.setSettlement(null);
-        settlementTile.setOwner(null);
-        settlementTile.updatePlayerExploredTiles();
-
-        owner = null;
-        oldOwner.removeSettlement(this);
-        oldOwner.invalidateCanSeeTiles();
-        goodsContainer.dispose();
-        super.dispose();
+        disposeList();
     }
-    
+
     /**
      * Gets the radius of what the <code>Settlement</code> considers
      * as it's own land.
@@ -418,18 +468,29 @@ abstract public class Settlement extends FreeColGameObject implements Location,
      * @return Settlement radius
      */
     public int getRadius() {
-        if (isCapital) {
-            return owner.getNationType().getCapitalRadius();
-        } else {
-            return owner.getNationType().getSettlementRadius();
-        }
+        return getType().getClaimableRadius();
     }
 
-    public abstract void newTurn();
+    /**
+     * Gets the current Sons of Liberty in this settlement.
+     */
+    public abstract int getSoL();
+
+    /**
+     * Propagates a global change in tension down to a settlement.
+     * Only apply the change if the settlement is aware of the player
+     * causing alarm.
+     *
+     * @param player The <code>Player</code> towards whom the alarm is felt.
+     * @param addToAlarm The amount to add to the current alarm level.
+     * @return True if the settlement alarm level changes as a result
+     *     of this change.
+     */
+    public abstract boolean propagateAlarm(Player player, int addToAlarm);
 
     /**
      * Removes a specified amount of a type of Goods from this Settlement.
-     * 
+     *
      * @param type The type of Goods to remove from this settlement.
      * @param amount The amount of Goods to remove from this settlement.
      */
@@ -462,7 +523,7 @@ abstract public class Settlement extends FreeColGameObject implements Location,
      * @param amount an <code>int</code> value
      */
     public void addGoods(GoodsType type, int amount) {
-        goodsContainer.addGoods(type.getStoredAs(), amount);
+        goodsContainer.addGoods(type, amount);
     }
 
     public void addGoods(AbstractGoods goods) {
@@ -471,105 +532,63 @@ abstract public class Settlement extends FreeColGameObject implements Location,
 
     /**
      * Gets the amount of one type of Goods at this Settlement.
-     * 
+     *
      * @param type The type of goods to look for.
      * @return The amount of this type of Goods at this Location.
      */
     public int getGoodsCount(GoodsType type) {
-        if (type != null && !type.isStoredAs()) {
-            return goodsContainer.getGoodsCount(type);
-        } else {
-            return 0;
-        }
+        return goodsContainer.getGoodsCount(type);
     }
-    
+
     /**
      * Returns the production of the given type of goods.
-     * 
+     *
      * @param goodsType The type of goods to get the production for.
-     * @return The production of the given type of goods the current turn by the 
+     * @return The production of the given type of goods the current turn by the
      * <code>Settlement</code>
      */
     public abstract int getProductionOf(GoodsType goodsType);
 
+    /**
+     * Returns the number of goods of a given type used by the settlement
+     * each turn.
+     *
+     * @param goodsType a <code>GoodsType</code> value
+     * @return an <code>int</code> value
+     */
+    public int getConsumptionOf(GoodsType goodsType) {
+        int result = 0;
+        for (Unit unit : getUnitList()) {
+            result += unit.getType().getConsumptionOf(goodsType);
+        }
+        return Math.max(0, result);
+    }
+
+    /**
+     * Returns the number of goods of all given types used by the
+     * settlement each turn.
+     *
+     * @param goodsTypes <code>GoodsType</code> values
+     * @return an <code>int</code> value
+     */
+    public int getConsumptionOf(List<GoodsType> goodsTypes) {
+        int result = 0;
+        if (goodsTypes != null) {
+            for (GoodsType goodsType : goodsTypes) {
+                result += getConsumptionOf(goodsType);
+            }
+        }
+        return result;
+    }
+
+
     /**
      * Gives the food needed to keep all units alive in this Settlement.
-     * 
+     *
      * @return The amount of food eaten in this colony each this turn.
      */
     public int getFoodConsumption() {
-        return FOOD_CONSUMPTION * getUnitCount();
-    }
-    
-    /**
-     * Gets food consumption by type
-     */
-    public int getFoodConsumptionByType(GoodsType type) {
-    	// Since for now only model.goods.food are needed for other
-    	// purposes we will hard code the preference of consumption
-    	// for other types of food If later other requirements appear,
-    	// an allocation algorithm needs to be implemented
-    	
-    	if(!type.isFoodType()){
-            logger.warning("Good type given isnt food type");
-            return 0;
-    	}
-    	
-    	int required = getFoodConsumption();
-    	int consumed = 0;
-    	GoodsType corn = FreeCol.getSpecification().getGoodsType("model.goods.food");
-    	
-    	for (GoodsType foodType : FreeCol.getSpecification().getGoodsFood()) {
-            if(foodType == corn){
-                // consumption of corn calculated last
-                continue;
-            }
-    		
-            consumed = Math.min(getProductionOf(foodType),required);
-            if(type == foodType){
-                return consumed;
-            }
-            required -= consumed;
-        }
-    	
-    	// type asked is corn, calculate consumption and return
-    	consumed = Math.min(getProductionOf(corn),required);
-    	
-    	return consumed;
-    }
-
-    protected void removeFood(final int amount) {
-        int rest = amount;
-        List<AbstractGoods> backlog = new ArrayList<AbstractGoods>();
-        for (GoodsType foodType : FreeCol.getSpecification().getGoodsFood()) {
-            int available = getGoodsCount(foodType);
-            if (available >= rest) {
-                removeGoods(foodType, rest);
-                for (AbstractGoods food : backlog) {
-                    removeGoods(food.getType(), food.getAmount());
-                }
-                rest = 0;
-            } else {
-                backlog.add(new AbstractGoods(foodType, available));
-                rest -= available;
-            }
-        }
-        if (rest > 0) {
-            throw new IllegalStateException("Attempted to remove more food than was present.");
-        }
-    }
-            
-    /**
-     * Returns the total amount of food present. 
-     *
-     * @return an <code>int</code> value
-     */
-    public int getFoodCount() {
-        int result = 0;
-        for (GoodsType foodType : FreeCol.getSpecification().getGoodsFood()) {
-            result += getGoodsCount(foodType);
-        }
-        return result;
+        return getConsumptionOf(getSpecification().getFoodGoodsTypeList());
     }
 
     /**
@@ -588,4 +607,34 @@ abstract public class Settlement extends FreeColGameObject implements Location,
         return true;
     }
 
+    protected void readAttributes(XMLStreamReader in) throws XMLStreamException {
+        super.readAttributes(in);
+        setName(in.getAttributeValue(null, "name"));
+        owner = getFreeColGameObject(in, "owner", Player.class);
+        tile = getFreeColGameObject(in, "tile", Tile.class);
+        featureContainer = new FeatureContainer(getGame().getSpecification());
+
+        // TODO: remove 0.9.x compatibility code
+        String typeStr = in.getAttributeValue(null, "settlementType");
+        SettlementType settlementType;
+        if (typeStr == null) {
+            // must be old style
+            String capital = in.getAttributeValue(null, "isCapital");
+            settlementType = owner.getNationType()
+                .getSettlementType("true".equals(capital));
+        } else {
+            settlementType = owner.getNationType().getSettlementType(typeStr);
+        }
+        // end compatibility code
+        setType(settlementType);
+    }
+
+    protected void writeAttributes(XMLStreamWriter out)
+        throws XMLStreamException {
+        out.writeAttribute(ID_ATTRIBUTE, getId());
+        out.writeAttribute("name", getName());
+        out.writeAttribute("tile", tile.getId());
+        out.writeAttribute("settlementType", getType().getId());
+        // Not owner, it is subject to PlayerExploredTile handling.
+    }
 }
