@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -24,7 +24,6 @@ import java.util.Collections;
 import java.util.Comparator;
 import java.util.Iterator;
 import java.util.HashSet;
-import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Set;
 import java.util.logging.Logger;
@@ -34,8 +33,7 @@ import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
-import net.sf.freecol.FreeCol;
-//import net.sf.freecol.common.model.PlayerExploredTile;
+import net.sf.freecol.common.model.PlayerExploredTile;
 
 import org.w3c.dom.Element;
 
@@ -45,6 +43,7 @@ import org.w3c.dom.Element;
  */
 public class TileItemContainer extends FreeColGameObject {
 
+    @SuppressWarnings("unused")
     private static final Logger logger = Logger.getLogger(TileItemContainer.class.getName());
 
     /**
@@ -85,25 +84,6 @@ public class TileItemContainer extends FreeColGameObject {
 
 
 
-    public TileItemContainer(Game game, Tile tile, PlayerExploredTile pet) {
-        super(game);
-
-        if (tile == null) {
-            throw new IllegalArgumentException("Tile must not be 'null'.");
-        }
-
-        this.tile = tile;
-
-        tileItems.addAll(pet.getImprovements());
-        if (pet.getResource() != null) {
-            tileItems.add(pet.getResource());
-        }
-        if (pet.getLostCityRumour() != null) {
-            tileItems.add(pet.getLostCityRumour());
-        }
-        Collections.sort(tileItems, tileItemComparator);
-    }
-
     /**
      * Initiates a new <code>TileItemContainer</code> from an <code>Element</code>.
      *
@@ -147,23 +127,24 @@ public class TileItemContainer extends FreeColGameObject {
         readFromXMLElement(e);
     }
 
-    /**
-     * Clone functions for making a clone of this TileItemContainer
-     */
-    public TileItemContainer clone() {
-        return clone(true, false);
-    }
-    public TileItemContainer clone(boolean importBonuses) {
-        return clone(importBonuses, false);
-    }
-    public TileItemContainer clone(boolean importBonuses, boolean copyOnlyNatural) {
-        TileItemContainer ticClone = new TileItemContainer(getGame(), getTile());
-        ticClone.copyFrom(this, importBonuses, copyOnlyNatural);
-        return ticClone;
-    }
-
     // ------------------------------------------------------------ checking/retrieval functions
 
+    /**
+     * Invalidate the production cache of the owning colony, if there
+     * is one, but only if the tile is actually being used.
+     */
+    private void invalidateCache() {
+        Colony colony = tile.getColony();
+        if (colony != null && colony.isTileInUse(tile)) {
+            colony.invalidateCache();
+        }
+    }
+
+    /**
+     * Return the <code>Tile</code> this TileItemContainer belongs to.
+     *
+     * @return a <code>Tile</code> value
+     */
     public Tile getTile() {
         return tile;
     }
@@ -184,8 +165,14 @@ public class TileItemContainer extends FreeColGameObject {
      */
     public final void setTileItems(final List<TileItem> newTileItems) {
         this.tileItems = newTileItems;
+        invalidateCache();
     }
 
+    /**
+     * Returns the <code>Resource</code> item or null.
+     *
+     * @return a <code>Resource</code> value
+     */
     public Resource getResource() {
         for (TileItem item : tileItems) {
             if (item instanceof Resource) {
@@ -195,6 +182,11 @@ public class TileItemContainer extends FreeColGameObject {
         return null;
     }
 
+    /**
+     * Returns the road improvement or null.
+     *
+     * @return a <code>TileImprovement</code> value
+     */
     public TileImprovement getRoad() {
         for (TileItem item : tileItems) {
             if (item instanceof TileImprovement && ((TileImprovement) item).isRoad()) {
@@ -204,6 +196,11 @@ public class TileItemContainer extends FreeColGameObject {
         return null;
     }
 
+    /**
+     * Returns the river improvement or null.
+     *
+     * @return a <code>TileImprovement</code> value
+     */
     public TileImprovement getRiver() {
         for (TileItem item : tileItems) {
             if (item instanceof TileImprovement && ((TileImprovement) item).isRiver()) {
@@ -227,13 +224,6 @@ public class TileItemContainer extends FreeColGameObject {
         return null;
     }
 
-    public void clear() {
-        for (TileItem item : tileItems) {
-            item.dispose();
-        }
-        tileItems.clear();
-    }
-
     /**
      * Remove improvements incompatible with the given TileType. This
      * method is called whenever the type of the container's tile
@@ -242,16 +232,18 @@ public class TileItemContainer extends FreeColGameObject {
     public void removeIncompatibleImprovements() {
         TileType tileType = tile.getType();
         Iterator<TileItem> iterator = tileItems.iterator();
+        boolean removed = false;
         while (iterator.hasNext()) {
             TileItem item = iterator.next();
-            if ((item instanceof TileImprovement
-                 && !((TileImprovement) item).getType().isTileTypeAllowed(tileType))
-                || (item instanceof Resource
-                    && !tileType.canHaveResourceType(((Resource) item).getType()))) {
+            if (!item.isTileTypeAllowed(tileType)) {
                 iterator.remove();
                 item.dispose();
+                removed = true;
             }
         }
+        if (removed) {
+            invalidateCache();
+        }
     }
 
     /**
@@ -263,7 +255,7 @@ public class TileItemContainer extends FreeColGameObject {
     public List<TileImprovement> getImprovements() {
         return getImprovements(false);
     }
-        
+
     /**
      * Returns a <code>List</code> of the completed
      * <code>TileImprovement</code>s in this
@@ -293,18 +285,27 @@ public class TileItemContainer extends FreeColGameObject {
     }
 
     /**
-     * Determine the total bonus for a GoodsType. Checks Resource and all Improvements.
+     * Determine the total bonus for a GoodsType. Checks Resource and
+     * all Improvements, unless onlyNatural is <code>true</code>, in
+     * which case only natural Improvements will be considered. This
+     * is necessary in order to calculate secondary production, which
+     * does not profit from artificial Improvements, such as plowing.
+     *
      * @param g a <code>GoodsType</code> value
      * @param unitType an <code>UnitType</code> value
      * @param tilePotential an <code>int</code> value
+     * @param onlyNatural a <code>boolean</code> value
      * @return The total bonus
      */
-    public int getTotalBonusPotential(GoodsType g, UnitType unitType, int tilePotential) {
+    public int getTotalBonusPotential(GoodsType g, UnitType unitType, int tilePotential, boolean onlyNatural) {
         int potential = tilePotential;
         int improvementBonus = 0;
         for (TileItem item : tileItems) {
             if (item instanceof TileImprovement) {
-                improvementBonus += ((TileImprovement) item).getBonus(g);
+                TileImprovement improvement = (TileImprovement) item;
+                if (improvement.getType().isNatural() || !onlyNatural) {
+                    improvementBonus += improvement.getBonus(g);
+                }
             } else if (item instanceof Resource) {
                 potential = ((Resource) item).getBonus(g, unitType, potential);
             }
@@ -323,19 +324,17 @@ public class TileItemContainer extends FreeColGameObject {
      * @return a <code>Modifier</code> value
      */
     public Set<Modifier> getProductionBonus(GoodsType goodsType, UnitType unitType) {
-        Set<Modifier> result = new LinkedHashSet<Modifier>();
-        Set<Modifier> itemModifiers = new HashSet<Modifier>();
+        Set<Modifier> result = new HashSet<Modifier>();
         for (TileItem item : tileItems) {
             if (item instanceof Resource) {
                 result.addAll(((Resource) item).getType().getProductionModifier(goodsType, unitType));
             } else if (item instanceof TileImprovement) {
                 Modifier modifier = ((TileImprovement) item).getProductionModifier(goodsType);
                 if (modifier != null) {
-                    itemModifiers.add(modifier);
+                    result.add(modifier);
                 }
             }
         }
-        result.addAll(itemModifiers);
         return result;
     }
 
@@ -357,6 +356,24 @@ public class TileItemContainer extends FreeColGameObject {
         return moveCost;
     }
 
+    /**
+     * Returns a description of the tile, with the name of the tile
+     * and any improvements made to it (road/plow)
+     *
+     * @return The description label for this tile
+     */
+    public StringTemplate getLabel() {
+        StringTemplate label = StringTemplate.label("/");
+        for (TileItem item : tileItems) {
+            if (item instanceof Resource) {
+                label.add(((Resource) item).getType().getNameKey());
+            } else if (item instanceof TileImprovement
+                       && ((TileImprovement) item).isComplete()) {
+                label.add(((TileImprovement) item).getType().getNameKey());
+            }
+        }
+        return label;
+    }
 
     // ------------------------------------------------------------ add/remove from container
 
@@ -379,6 +396,7 @@ public class TileItemContainer extends FreeColGameObject {
                     if (((TileImprovement) oldItem).getMagnitude() < ((TileImprovement) item).getMagnitude()) {
                         tileItems.set(index, item);
                         oldItem.dispose();
+                        invalidateCache();
                         return item;
                     } else {
                         // Found it, but not replacing.
@@ -386,10 +404,12 @@ public class TileItemContainer extends FreeColGameObject {
                     }
                 } else if (oldItem.getZIndex() > item.getZIndex()) {
                     tileItems.add(index, item);
+                    invalidateCache();
                     return item;
                 }
             }
             tileItems.add(item);
+            invalidateCache();
             return item;
         }
     }
@@ -401,10 +421,16 @@ public class TileItemContainer extends FreeColGameObject {
      * @return The TileItem that has been removed from this container (if any).
      */
     public TileItem removeTileItem(TileItem item) {
-        return (tileItems.remove(item) ? item : null);
+        boolean removed = tileItems.remove(item);
+        if (removed) {
+            invalidateCache();
+            return item;
+        } else {
+            return null;
+        }
     }
 
-    public void removeAll(Class c) {
+    public <T extends TileItem> void removeAll(Class<T> c) {
         Iterator<TileItem> iterator = tileItems.iterator();
         while (iterator.hasNext()) {
             if (c.isInstance(iterator.next())) {
@@ -412,21 +438,22 @@ public class TileItemContainer extends FreeColGameObject {
             }
         }
     }
-    
+
     public void copyFrom(TileItemContainer tic) {
         copyFrom(tic, true, false);
     }
+
     public void copyFrom(TileItemContainer tic, boolean importResources) {
         copyFrom(tic, importResources, false);
     }
+
     public void copyFrom(TileItemContainer tic, boolean importResources, boolean copyOnlyNatural) {
         tileItems.clear();
         for (TileItem item : tic.getTileItems()) {
             if (item instanceof Resource) {
                 if (importResources) {
                     Resource ticR = (Resource) item;
-                    Resource r = new Resource(getGame(), tile, ticR.getType());
-                    r.setQuantity(ticR.getQuantity());
+                    Resource r = new Resource(getGame(), tile, ticR.getType(), ticR.getQuantity());
                     tileItems.add(r);
                 }
             } else if (item instanceof LostCityRumour && !copyOnlyNatural) {
@@ -446,13 +473,6 @@ public class TileItemContainer extends FreeColGameObject {
         }
     }
 
-    /**
-     * Removes all TileItems.
-     */
-    public void removeAll() {
-        clear();
-    }
-
     /**
      * Checks if the specified <code>TileItem</code> is in this container.
      *
@@ -477,14 +497,14 @@ public class TileItemContainer extends FreeColGameObject {
         }
         return null;
     }
-    
+
     /**
      * Will check whether this tile has a completed improvement of the given
      * type.
-     * 
+     *
      * Useful for checking whether the tile for instance has a road or is
      * plowed.
-     * 
+     *
      * @param type
      *            The type to check for.
      * @return Whether the tile has the improvement and the improvement is
@@ -499,7 +519,7 @@ public class TileItemContainer extends FreeColGameObject {
      * Removes all references to this object.
      */
     public void dispose() {
-        clear();
+        tileItems.clear();
         super.dispose();
     }
 
@@ -516,11 +536,12 @@ public class TileItemContainer extends FreeColGameObject {
         if (magnitude == TileImprovement.NO_RIVER) {
             return null;
         }
-        TileImprovement river = new TileImprovement(getGame(), tile, FreeCol.getSpecification()
-                                                    .getTileImprovementType("model.improvement.River"));
+        TileImprovement river = new TileImprovement(getGame(), tile, getSpecification()
+                                                    .getTileImprovementType("model.improvement.river"));
         river = (TileImprovement) addTileItem(river);
         river.setMagnitude(magnitude);
         river.setStyle(style);
+        invalidateCache();
         return river;
     }
 
@@ -534,6 +555,7 @@ public class TileItemContainer extends FreeColGameObject {
             TileItem item = iterator.next();
             if (item instanceof TileImprovement && ((TileImprovement) item).isRiver()) {
                 iterator.remove();
+                invalidateCache();
                 return (TileImprovement) item;
             }
         }
@@ -545,18 +567,18 @@ public class TileItemContainer extends FreeColGameObject {
     /**
      * This method writes an XML-representation of this object to
      * the given stream.
-     * 
+     *
      * <br><br>
-     * 
-     * Only attributes visible to the given <code>Player</code> will 
+     *
+     * Only attributes visible to the given <code>Player</code> will
      * be added to that representation if <code>showAll</code> is
      * set to <code>false</code>.
-     *  
+     *
      * @param out The target stream.
-     * @param player The <code>Player</code> this XML-representation 
+     * @param player The <code>Player</code> this XML-representation
      *      should be made for, or <code>null</code> if
      *      <code>showAll == true</code>.
-     * @param showAll Only attributes visible to <code>player</code> 
+     * @param showAll Only attributes visible to <code>player</code>
      *      will be added to the representation if <code>showAll</code>
      *      is set to <i>false</i>.
      * @param toSavedGame If <code>true</code> then information that
@@ -564,16 +586,28 @@ public class TileItemContainer extends FreeColGameObject {
      * @throws XMLStreamException if there are any problems writing
      *      to the stream.
      */
-    protected void toXMLImpl(XMLStreamWriter out, Player player, boolean showAll, boolean toSavedGame) 
+    protected void toXMLImpl(XMLStreamWriter out, Player player, boolean showAll, boolean toSavedGame)
         throws XMLStreamException {
+        PlayerExploredTile pet = (showAll) ? null
+            : tile.getPlayerExploredTile(player);
+
         // Start element:
         out.writeStartElement(getXMLElementTagName());
 
         out.writeAttribute("ID", getId());
         out.writeAttribute("tile", tile.getId());
 
-        for (TileItem item : tileItems) {
-            item.toXML(out, player, showAll, toSavedGame);
+        if (getGame().isClientTrusted() || showAll || toSavedGame
+            || player.canSee(tile)) {
+            for (TileItem item : tileItems) {
+                item.toXML(out, player, showAll, toSavedGame);
+            }
+        } else if (pet != null) {
+            List<TileItem> petItems = pet.getAllTileItems();
+            Collections.sort(petItems, tileItemComparator);
+            for (TileItem item : petItems) {
+                item.toXML(out, player, showAll, toSavedGame);
+            }
         }
 
         out.writeEndElement();
@@ -621,11 +655,11 @@ public class TileItemContainer extends FreeColGameObject {
     public static String getXMLElementTagName() {
         return "tileitemcontainer";
     }
-    
-    
+
+
     /**
      * Creates a <code>String</code> representation of this
-     * <code>TileItemContainer</code>.    
+     * <code>TileItemContainer</code>.
      */
     public String toString() {
         StringBuffer sb = new StringBuffer(60);
