@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -20,71 +20,70 @@
 package net.sf.freecol.common.model;
 
 import java.util.ArrayList;
+import java.util.Iterator;
 import java.util.List;
 import java.util.Map.Entry;
 
-import javax.xml.stream.XMLStreamConstants;
 import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
+import javax.xml.stream.XMLStreamWriter;
 
-import net.sf.freecol.common.Specification;
 import net.sf.freecol.common.model.UnitTypeChange.ChangeType;
 
-public final class UnitType extends BuildableType {
+public final class UnitType extends BuildableType implements Comparable<UnitType>, Consumer {
 
     public static final int DEFAULT_OFFENCE = 0;
     public static final int DEFAULT_DEFENCE = 1;
-    public static final int FOOD_CONSUMPTION = 2;
 
     /**
      * Describe offence here.
      */
-    private int offence;
+    private int offence = DEFAULT_OFFENCE;
 
     /**
      * Describe defence here.
      */
-    private int defence;
+    private int defence = DEFAULT_DEFENCE;
 
     /**
      * Describe space here.
      */
-    private int space;
+    private int space = 0;
 
     /**
      * Describe hitPoints here.
      */
-    private int hitPoints;
+    private int hitPoints = 0;
 
     /**
      * Describe spaceTaken here.
      */
-    private int spaceTaken;
+    private int spaceTaken = 1;
 
     /**
      * Describe skill here.
      */
-    private int skill;
+    private int skill = UNDEFINED;
 
     /**
      * Describe price here.
      */
-    private int price;
+    private int price = UNDEFINED;
 
     /**
      * Describe movement here.
      */
-    private int movement;
-    
+    private int movement = 3;
+
     /**
      * Describe lineOfSight here.
      */
-    private int lineOfSight;
+    private int lineOfSight = 1;
 
     /**
      * Describe recruitProbability here.
      */
-    private int recruitProbability;
+    private int recruitProbability = 0;
 
     /**
      * Describe expertProduction here.
@@ -94,22 +93,22 @@ public final class UnitType extends BuildableType {
     /**
      * How much a Unit of this type contributes to the Player's score.
      */
-    private int scoreValue;
+    private int scoreValue = 0;
 
     /**
-     * Describe pathImage here.
+     * The maximum experience a unit of this type can accumulate.
      */
-    private String pathImage;
+    private int maximumExperience = 0;
 
     /**
      * Describe maximumAttrition here.
      */
-    private int maximumAttrition;
+    private int maximumAttrition = INFINITY;
 
     /**
-     * The ID of the skill this UnitType teaches, mostly its own.
+     * The skill this UnitType teaches, mostly its own.
      */
-    private String skillTaught;
+    private UnitType skillTaught;
 
     /**
      * Describe defaultEquipment here.
@@ -117,27 +116,27 @@ public final class UnitType extends BuildableType {
     private EquipmentType defaultEquipment;
 
     /**
-     * Describe foodConsumed here.
+     * The goods consumed per turn when in a settlement.
      */
-    private int foodConsumed = FOOD_CONSUMPTION;
+    private TypeCountMap<GoodsType> consumption = new TypeCountMap<GoodsType>();
 
     /**
      * The possible type changes for this unit type.
      */
     private List<UnitTypeChange> typeChanges = new ArrayList<UnitTypeChange>();
-    
 
-    public UnitType() {
-        // empty constructor
-    }
-    
+
     /**
      * Creates a new <code>UnitType</code> instance.
      *
-     * @param index an <code>int</code> value
      */
-    public UnitType(int index) {
-        setIndex(index);
+    public UnitType(String id, Specification specification) {
+        super(id, specification);
+        setModifierIndex(Modifier.EXPERT_PRODUCTION_INDEX);
+    }
+
+    public final String getWorkingAsKey() {
+        return getId() + ".workingAs";
     }
 
     /**
@@ -333,10 +332,10 @@ public final class UnitType extends BuildableType {
      * Get the <code>Price</code> value.
      *
      * @return an <code>int</code> value
-     * 
+     *
      * This returns the base price of the <code>UnitType</code>
-     * 
-     * For the actual price of the unit, use {@link Europe#getUnitPrice(UnitType)} 
+     *
+     * For the actual price of the unit, use {@link Europe#getUnitPrice(UnitType)}
      */
     public int getPrice() {
         return price;
@@ -369,6 +368,24 @@ public final class UnitType extends BuildableType {
         this.movement = newMovement;
     }
 
+    /**
+     * Get the <code>MaximumExperience</code> value.
+     *
+     * @return an <code>int</code> value
+     */
+    public final int getMaximumExperience() {
+        return maximumExperience;
+    }
+
+    /**
+     * Set the <code>MaximumExperience</code> value.
+     *
+     * @param newMaximumExperience The new MaximumExperience value.
+     */
+    public final void setMaximumExperience(final int newMaximumExperience) {
+        this.maximumExperience = newMaximumExperience;
+    }
+
     /**
      * Get the <code>MaximumAttrition</code> value.
      *
@@ -440,30 +457,12 @@ public final class UnitType extends BuildableType {
         return typeChanges;
     }
 
-    /**
-     * Get the <code>PathImage</code> value.
-     *
-     * @return a <code>String</code> value
-     */
-    public String getPathImage() {
-        return pathImage;
-    }
-
-    /**
-     * Set the <code>PathImage</code> value.
-     *
-     * @param newPathImage The new PathImage value.
-     */
-    public void setPathImage(final String newPathImage) {
-        this.pathImage = newPathImage;
-    }
-
     /**
      * Get the <code>SkillTaught</code> value.
      *
-     * @return an <code>String</code> value
+     * @return an <code>UnitType</code> value
      */
-    public String getSkillTaught() {
+    public UnitType getSkillTaught() {
         return skillTaught;
     }
 
@@ -472,29 +471,10 @@ public final class UnitType extends BuildableType {
      *
      * @param newSkillTaught The new SkillTaught value.
      */
-    public void setSkillTaught(final String newSkillTaught) {
+    public void setSkillTaught(final UnitType newSkillTaught) {
         this.skillTaught = newSkillTaught;
     }
 
-    /**
-     * Returns the amount of food this unit type consumes per turn. At
-     * the moment, this is 2 for all unit types.
-     *
-     * @return an <code>int</code> value
-     */
-    public int getFoodConsumed() {
-        return foodConsumed;
-    }
-
-    /**
-     * Set the <code>FoodConsumed</code> value.
-     *
-     * @param newFoodConsumed The new FoodConsumed value.
-     */
-    public void setFoodConsumed(final int newFoodConsumed) {
-        this.foodConsumed = newFoodConsumed;
-    }
-
     /**
      * Returns true if the UnitType is available to the given
      * Player.
@@ -519,19 +499,47 @@ public final class UnitType extends BuildableType {
      * @param player a <code>Player</code> value
      * @return an <code>UnitType</code> value
      */
-    public UnitType getUnitTypeChange(ChangeType changeType, Player player) {
+    public UnitType getTargetType(ChangeType changeType, Player player) {
+        UnitTypeChange change = getUnitTypeChange(changeType, player);
+        return (change == null) ? null : change.getNewUnitType();
+    }
+
+    /**
+     * Describe <code>getUnitTypeChange</code> method here.
+     *
+     * @param changeType an <code>UnitTypeChange.Type</code> value
+     * @param player a <code>Player</code> value
+     * @return an <code>UnitType</code> value
+     */
+    public UnitTypeChange getUnitTypeChange(ChangeType changeType, Player player) {
         for (UnitTypeChange change : typeChanges) {
             if (change.asResultOf(changeType) && change.appliesTo(player)) {
                 UnitType result = change.getNewUnitType();
                 if (result.isAvailableTo(player)) {
-                    return result;
+                    return change;
                 }
             }
         }
         return null;
     }
 
-    
+
+    /**
+     * Returns the <code>UnitTypeChange</code> associated with the
+     * given <code>UnitType</code>, or <code>null</code> if there is
+     * none.
+     *
+     * @param newType the target UnitType
+     * @return the type change
+     */
+    public UnitTypeChange getUnitTypeChange(UnitType newType) {
+        for (UnitTypeChange change : typeChanges) {
+            if (change.getNewUnitType() == newType) {
+                return change;
+            }
+        }
+        return null;
+    }
 
     /**
      * Return true if this UnitType can be upgraded to the given
@@ -539,17 +547,17 @@ public final class UnitType extends BuildableType {
      * is null, return true if the UnitType can be upgraded to any
      * other UnitType by the given means of education.
      *
-     * @param newType the UnitType to learn
+     * @param newType The UnitType to learn (may be null in the case
+     *     of attempting to move to a native settlement when the skill
+     *     taught there is still unknown).
      * @param changeType an <code>ChangeType</code> value
      * @return <code>true</code> if can learn the given UnitType
      */
     public boolean canBeUpgraded(UnitType newType, ChangeType changeType) {
         for (UnitTypeChange change : typeChanges) {
-            if (change.asResultOf(changeType)) {
-                if (newType == null
-                    || newType == change.getNewUnitType()) {
-                    return true;
-                }
+            if ((newType == null || newType == change.getNewUnitType())
+                && change.getProbability(changeType) > 0) {
+                return true;
             }
         }
         return false;
@@ -606,56 +614,164 @@ public final class UnitType extends BuildableType {
         return UNDEFINED;
     }
 
-    public void readAttributes(XMLStreamReader in, Specification specification)
-            throws XMLStreamException {
-        offence = getAttribute(in, "offence", DEFAULT_OFFENCE);
-        defence = getAttribute(in, "defence", DEFAULT_DEFENCE);
-        movement = Integer.parseInt(in.getAttributeValue(null, "movement"));
-        lineOfSight = getAttribute(in, "lineOfSight", 1);
-        scoreValue = getAttribute(in, "scoreValue", 0);
-        space = getAttribute(in, "space", 0);
-        hitPoints = getAttribute(in, "hitPoints", 0);
-        spaceTaken = getAttribute(in, "spaceTaken", 1);
-        maximumAttrition = getAttribute(in, "maximumAttrition", Integer.MAX_VALUE);
-        skillTaught = getAttribute(in, "skillTaught", getId());
-
-        pathImage = in.getAttributeValue(null, "pathImage");
-
-        recruitProbability = getAttribute(in, "recruitProbability", 0);
-        skill = getAttribute(in, "skill", UNDEFINED);
-
-        setPopulationRequired(getAttribute(in, "population-required", 1));
-
-        price = getAttribute(in, "price", UNDEFINED);
-
-        expertProduction = specification.getType(in, "expert-production", GoodsType.class, null);
-
+    /**
+     * Is this a naval unit type?
+     *
+     * @return True if this is a naval unit type.
+     */
+    public boolean isNaval() {
+        return hasAbility("model.ability.navalUnit");
     }
 
-    public void readChildren(XMLStreamReader in, Specification specification) throws XMLStreamException {
-        while (in.nextTag() != XMLStreamConstants.END_ELEMENT) {
-            String nodeName = in.getLocalName();
-            if ("upgrade".equals(nodeName)) {
-                typeChanges.add(new UnitTypeChange(in, specification));
-            } else if ("downgrade".equals(nodeName)) {
-                UnitTypeChange change = new UnitTypeChange(in, specification);
-                if (change.getChangeTypes().isEmpty()) {
-                    // add default downgrade type
-                    change.getChangeTypes().add(ChangeType.CLEAR_SKILL);
-                }
-                typeChanges.add(change);
-            } else if ("default-equipment".equals(nodeName)) {
-                String equipmentString = in.getAttributeValue(null, "id");
-                if (equipmentString != null) {
-                    defaultEquipment = specification.getEquipmentType(equipmentString);
-                }
-                in.nextTag(); // close this element
-            } else {
-                super.readChild(in, specification);
+    public void readAttributes(XMLStreamReader in) throws XMLStreamException {
+        super.readAttributes(in);
+        String extendString = in.getAttributeValue(null, "extends");
+        UnitType parent = (extendString == null) ? this :
+            getSpecification().getUnitType(extendString);
+        offence = getAttribute(in, "offence", parent.offence);
+        defence = getAttribute(in, "defence", parent.defence);
+        movement = getAttribute(in, "movement", parent.movement);
+        lineOfSight = getAttribute(in, "lineOfSight", parent.lineOfSight);
+        scoreValue = getAttribute(in, "scoreValue", parent.scoreValue);
+        space = getAttribute(in, "space", parent.space);
+        hitPoints = getAttribute(in, "hitPoints", parent.hitPoints);
+        spaceTaken = getAttribute(in, "spaceTaken", parent.spaceTaken);
+        maximumExperience = getAttribute(in, "maximumExperience", parent.maximumExperience);
+        maximumAttrition = getAttribute(in, "maximumAttrition", parent.maximumAttrition);
+        String skillString = in.getAttributeValue(null, "skillTaught");
+        skillTaught = (skillString == null) ? this : getSpecification().getUnitType(skillString);
+
+        recruitProbability = getAttribute(in, "recruitProbability", parent.recruitProbability);
+        skill = getAttribute(in, "skill", parent.skill);
+
+        setPopulationRequired(getAttribute(in, "population-required", parent.getPopulationRequired()));
+
+        price = getAttribute(in, "price", parent.price);
+
+        expertProduction = getSpecification().getType(in, "expert-production", GoodsType.class,
+                                                 parent.expertProduction);
+
+        if (parent != this) {
+            typeChanges.addAll(parent.typeChanges);
+            defaultEquipment = parent.defaultEquipment;
+            consumption.putAll(parent.consumption);
+            getFeatureContainer().add(parent.getFeatureContainer());
+            if (parent.isAbstractType()) {
+                getFeatureContainer().replaceSource(parent, this);
             }
         }
     }
 
+    public void readChild(XMLStreamReader in) throws XMLStreamException {
+        String nodeName = in.getLocalName();
+        if ("downgrade".equals(nodeName)
+            || "upgrade".equals(nodeName)) {
+            if (getAttribute(in, "delete", false)) {
+                String unitId = in.getAttributeValue(null, "unit");
+                Iterator<UnitTypeChange> iterator = typeChanges.iterator();
+                while (iterator.hasNext()) {
+                    if (unitId.equals(iterator.next().getNewUnitType().getId())) {
+                        iterator.remove();
+                        break;
+                    }
+                }
+                in.nextTag();
+            } else {
+                UnitTypeChange change = new UnitTypeChange(in, getSpecification());
+                if ("downgrade".equals(nodeName)
+                    && change.getChangeTypes().isEmpty()) {
+                    // add default downgrade type
+                    change.getChangeTypes().put(ChangeType.CLEAR_SKILL, 100);
+                }
+                typeChanges.add(change);
+            }
+        } else if ("default-equipment".equals(nodeName)) {
+            String equipmentString = in.getAttributeValue(null, ID_ATTRIBUTE_TAG);
+            if (equipmentString != null) {
+                defaultEquipment = getSpecification().getEquipmentType(equipmentString);
+            }
+            in.nextTag(); // close this element
+        } else if ("consumes".equals(nodeName)) {
+            String typeString = in.getAttributeValue(null, ID_ATTRIBUTE_TAG);
+            String valueString = in.getAttributeValue(null, VALUE_TAG);
+            if (typeString != null && valueString != null) {
+                try {
+                    GoodsType type = getSpecification().getGoodsType(typeString);
+                    int amount = Integer.parseInt(valueString);
+                    consumption.incrementCount(type, amount);
+                } catch(Exception e) {
+                    logger.warning("Failed to parse integer " + valueString);
+                }
+            }
+            in.nextTag(); // close this element
+        } else {
+            super.readChild(in);
+        }
+    }
+
+
+    /**
+     * Makes an XML-representation of this object.
+     *
+     * @param out The output stream.
+     * @throws XMLStreamException if there are any problems writing to the
+     *             stream.
+     */
+    public void toXMLImpl(XMLStreamWriter out) throws XMLStreamException {
+        super.toXML(out, getXMLElementTagName());
+    }
+
+    public void writeAttributes(XMLStreamWriter out) throws XMLStreamException {
+        super.writeAttributes(out);
+        out.writeAttribute("offence", Integer.toString(offence));
+        out.writeAttribute("defence", Integer.toString(defence));
+        out.writeAttribute("movement", Integer.toString(movement));
+        out.writeAttribute("lineOfSight", Integer.toString(lineOfSight));
+        out.writeAttribute("scoreValue", Integer.toString(scoreValue));
+        out.writeAttribute("space", Integer.toString(space));
+        out.writeAttribute("spaceTaken", Integer.toString(spaceTaken));
+        out.writeAttribute("hitPoints", Integer.toString(hitPoints));
+        out.writeAttribute("maximumExperience", Integer.toString(maximumExperience));
+        if (maximumAttrition < INFINITY) {
+            out.writeAttribute("maximumAttrition", Integer.toString(maximumAttrition));
+        }
+        out.writeAttribute("recruitProbability", Integer.toString(recruitProbability));
+        if (skill != UNDEFINED) {
+            out.writeAttribute("skill", Integer.toString(skill));
+        }
+        if (price != UNDEFINED) {
+            out.writeAttribute("price", Integer.toString(price));
+        }
+        out.writeAttribute("skillTaught", skillTaught.getId());
+        if (expertProduction != null) {
+            out.writeAttribute("expert-production", expertProduction.getId());
+        }
+    }
+
+    protected void writeChildren(XMLStreamWriter out) throws XMLStreamException {
+        super.writeChildren(out);
+        for (UnitTypeChange change : typeChanges) {
+            change.toXMLImpl(out);
+        }
+        if (defaultEquipment != null) {
+            out.writeStartElement("default-equipment");
+            out.writeAttribute(ID_ATTRIBUTE_TAG, defaultEquipment.getId());
+            out.writeEndElement();
+        }
+        if (!consumption.isEmpty()) {
+            for (GoodsType goodsType : consumption.keySet()) {
+                out.writeStartElement("consumes");
+                out.writeAttribute(ID_ATTRIBUTE_TAG, goodsType.getId());
+                out.writeAttribute(VALUE_TAG, Integer.toString(consumption.getCount(goodsType)));
+                out.writeEndElement();
+            }
+        }
+    }
+
+    public static String getXMLElementTagName() {
+        return "unit-type";
+    }
+
 
     /**
      * Returns true if this UnitType has a skill.
@@ -691,11 +807,56 @@ public final class UnitType extends BuildableType {
         if (base == 0) {
             return 0;
         }
-        
-        base = (int) featureContainer.applyModifier(base, goodsType.getId());
+
+        base = (int) getFeatureContainer().applyModifier(base, goodsType.getId());
         return Math.max(base, 1);
     }
 
+     /**
+     * Returns the number of units of the given GoodsType this
+     * UnitType consumes per turn (when in a settlement).
+     *
+     * @return units consumed
+     */
+    public int getConsumptionOf(GoodsType goodsType) {
+        return consumption.getCount(goodsType);
+    }
+
+    // Interface Comparable
+
+    /**
+     * {@inheritDoc}
+     */
+    public int compareTo(UnitType other) {
+        return getIndex() - other.getIndex();
+    }
+
+    // Interface Consumer
+
+    /**
+     * Returns a list of GoodsTypes this Consumer consumes.
+     *
+     * @return a <code>List</code> value
+     */
+    public List<AbstractGoods> getConsumedGoods() {
+        List<AbstractGoods> result = new ArrayList<AbstractGoods>();
+        for (GoodsType goodsType : consumption.keySet()) {
+            result.add(new AbstractGoods(goodsType, consumption.getCount(goodsType)));
+        }
+        return result;
+    }
+
+    /**
+     * The priority of this Consumer. The higher the priority, the
+     * earlier will the Consumer be allowed to consume the goods it
+     * requires.
+     *
+     * @return an <code>int</code> value
+     */
+    public int getPriority() {
+        // TODO: make this configurable
+        return UNIT_PRIORITY;
+    }
+
 
-    
 }
