@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -22,15 +22,15 @@ package net.sf.freecol.common.networking;
 
 
 /**
-* Contains the network constants. These constants
-* are used by the controllers and input handlers when
-* they are communicating.
-*/
+ * Contains the network constants. These constants are used by the
+ * controllers and input handlers when they are communicating.
+ */
 public interface NetworkConstants {
 
+    // Claiming land by stealing it
+    public static final int STEAL_LAND = -1;
 
-    // Trade:
+    // Trade
     public static final int NO_TRADE = -1,
                             NO_NEED_FOR_THE_GOODS = 0;
-
 }
