@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -21,12 +21,12 @@
  *
  * This class is specialized to control the "Colonial" player type,
  * using it for any other type will fail.
- * 
+ *
  * It is currently not used in any standard game setting,
- * but can be activated by starting a game using the "--experimentalAI" arg. 
- * 
+ * but can be activated by starting a game using the "--experimentalAI" arg.
+ *
  ******************************************************************************/
-   
+
 package net.sf.freecol.server.ai;
 
 import java.io.PrintWriter;
@@ -37,14 +37,12 @@ import java.util.Comparator;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
-import java.util.Vector;
 import java.util.logging.Logger;
 
 import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
-import net.sf.freecol.FreeCol;
 import net.sf.freecol.common.model.Building;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.ColonyTile;
@@ -63,20 +61,19 @@ import net.sf.freecol.common.model.Map;
 import net.sf.freecol.common.model.Ownable;
 import net.sf.freecol.common.model.PathNode;
 import net.sf.freecol.common.model.Player;
+import net.sf.freecol.common.model.Player.PlayerType;
+import net.sf.freecol.common.model.Player.Stance;
 import net.sf.freecol.common.model.Settlement;
+import net.sf.freecol.common.model.Specification;
 import net.sf.freecol.common.model.StanceTradeItem;
 import net.sf.freecol.common.model.Tension;
 import net.sf.freecol.common.model.Tile;
-import net.sf.freecol.common.model.TileImprovement;
 import net.sf.freecol.common.model.TradeItem;
 import net.sf.freecol.common.model.Unit;
+import net.sf.freecol.common.model.Unit.Role;
 import net.sf.freecol.common.model.UnitTradeItem;
 import net.sf.freecol.common.model.UnitType;
-import net.sf.freecol.common.model.Player.PlayerType;
-import net.sf.freecol.common.model.Player.Stance;
-import net.sf.freecol.common.model.Unit.Role;
 import net.sf.freecol.common.model.pathfinding.GoalDecider;
-import net.sf.freecol.common.networking.Message;
 import net.sf.freecol.common.networking.NetworkConstants;
 import net.sf.freecol.server.ai.goal.ManageMissionariesGoal;
 import net.sf.freecol.server.ai.mission.BuildColonyMission;
@@ -98,13 +95,13 @@ import net.sf.freecol.server.model.ServerPlayer;
 import org.w3c.dom.Element;
 
 /**
- * 
+ *
  * Objects of this class contains AI-information for a single {@link Player} and
  * is used for controlling this player.
- * 
+ *
  * <br />
  * <br />
- * 
+ *
  * The method {@link #startWorking} gets called by the
  * {@link AIInGameInputHandler} when it is this player's turn.
  */
@@ -121,12 +118,12 @@ public class ColonialAIPlayer extends AIPlayer {
 
     /**
      * Goal to manage missionary units of this player.
-     */         
+     */
     private ManageMissionariesGoal mGoal = new ManageMissionariesGoal((AIPlayer)this, null, 1.0f);
 
     /**
      * Creates a new <code>AIPlayer</code>.
-     * 
+     *
      * @param aiMain The main AI-class.
      * @param player The player that should be associated with this
      *            <code>AIPlayer</code>.
@@ -139,7 +136,7 @@ public class ColonialAIPlayer extends AIPlayer {
     /**
      * Creates a new <code>AIPlayer</code> and reads the information from the
      * given <code>Element</code>.
-     * 
+     *
      * @param aiMain The main AI-class.
      * @param element The XML-element containing information.
      */
@@ -151,7 +148,7 @@ public class ColonialAIPlayer extends AIPlayer {
 
     /**
      * Creates a new <code>AIPlayer</code>.
-     * 
+     *
      * @param aiMain The main AI-object.
      * @param in The input stream containing the XML.
      * @throws XMLStreamException if a problem was encountered during parsing.
@@ -172,7 +169,8 @@ public class ColonialAIPlayer extends AIPlayer {
      * returns.
      */
     public void startWorking() {
-        logger.fine("Entering AI code for: " + getPlayer().getNation());
+        logger.fine("Entering AI code for: " + getPlayer().toString()
+                    + ", year " + getGame().getTurn());
 
         //this AI assumes that player is _not_ REF, _not_ Indian, but European!
         if (getPlayer().isREF() ||
@@ -180,11 +178,11 @@ public class ColonialAIPlayer extends AIPlayer {
             !getPlayer().isEuropean()) {
             throw new IllegalStateException("Player type not supported by this AI!");
         }
-        
+
         // TODO: find some intelligent solution to set a proper strategy!
         this.strategy = AIStrategy.TRADE;
 
-        //reset unit iterator to all units without current goal         
+        //reset unit iterator to all units without current goal
         clearAIUnits();
 
         //add goals for missionaries (this will remove them from the iterator)
@@ -194,9 +192,9 @@ public class ColonialAIPlayer extends AIPlayer {
         boolean furtherPlanning = true;
         while (furtherPlanning) {
             mGoal.doPlanning();
-            furtherPlanning = mGoal.needsPlanning();            
+            furtherPlanning = mGoal.needsPlanning();
         }
-        
+
         //manage the remaining units as before
         cheat();
         determineStances();
@@ -216,16 +214,16 @@ public class ColonialAIPlayer extends AIPlayer {
         rearrangeWorkersInColonies();
         abortInvalidMissions();
         ensureCorrectMissions();
-        
+
         clearAIUnits();
     }
 
     /**
      * Returns an <code>Iterator</code> over all the
      * <code>TileImprovement</code>s needed by all of this player's colonies.
-     * 
+     *
      * @return The <code>Iterator</code>.
-     * @see TileImprovement
+     * @see net.sf.freecol.common.model.TileImprovement
      */
     public Iterator<TileImprovementPlan> getTileImprovementPlanIterator() {
         ArrayList<TileImprovementPlan> tileImprovements = new ArrayList<TileImprovementPlan>();
@@ -239,7 +237,7 @@ public class ColonialAIPlayer extends AIPlayer {
         }
         return tileImprovements.iterator();
     }
-    
+
     /**
      * Remove a <code>TileImprovementPlan</code> from the list
      */
@@ -259,11 +257,10 @@ public class ColonialAIPlayer extends AIPlayer {
      * players into building more colonies. The method will be removed after the
      * proper code for deciding whether a colony should be built or not has been
      * implemented.
-     * 
+     *
      * @return <code>true</code> if the AI should build more colonies.
      */
-    public boolean hasFewColonies() {        
-        logger.finest("Entering method hasFewColonies");
+    public boolean hasFewColonies() {
         if (!getPlayer().canBuildColonies()) {
             return false;
         }
@@ -273,17 +270,19 @@ public class ColonialAIPlayer extends AIPlayer {
             numberOfColonies++;
             numberOfWorkers += colony.getUnitCount();
         }
-        
-        logger.finest("Leaving method hasFewColonies");
-        return numberOfColonies <= 2 || numberOfColonies >= 3
-                && numberOfWorkers / numberOfColonies > numberOfColonies - 2;
+
+        boolean result = numberOfColonies <= 2
+            || (numberOfColonies >= 3
+                && numberOfWorkers / numberOfColonies > numberOfColonies - 2);
+        logger.finest("hasFewColonies = " + result);
+        return result;
     }
 
     /**
      * Returns an <code>Iterator</code> for all the wishes. The items are
      * sorted by the {@link Wish#getValue value}, with the item having the
      * highest value appearing first in the <code>Iterator</code>.
-     * 
+     *
      * @return The <code>Iterator</code>.
      * @see Wish
      */
@@ -304,7 +303,7 @@ public class ColonialAIPlayer extends AIPlayer {
 
     /**
      * Selects the most useful founding father offered.
-     * 
+     *
      * @param foundingFathers The founding fathers on offer.
      * @return The founding father selected.
      */
@@ -326,7 +325,7 @@ public class ColonialAIPlayer extends AIPlayer {
 
     /**
      * Decides whether to accept the monarch's tax raise or not.
-     * 
+     *
      * @param tax The new tax rate to be considered.
      * @return <code>true</code> if the tax raise should be accepted.
      */
@@ -335,12 +334,12 @@ public class ColonialAIPlayer extends AIPlayer {
         if (toBeDestroyed == null) {
             return false;
         }
-        
+
         GoodsType goodsType = toBeDestroyed.getType();
         if (goodsType.isFoodType() || goodsType.isBreedable()) {
             // we should be able to produce food and horses ourselves
             return false;
-        } else if (goodsType.isMilitaryGoods() || 
+        } else if (goodsType.isMilitaryGoods() ||
                    goodsType.isTradeGoods() ||
                    goodsType.isBuildingMaterial()) {
             if (getGame().getTurn().getAge() == 3) {
@@ -353,7 +352,7 @@ public class ColonialAIPlayer extends AIPlayer {
         } else {
             int averageIncome = 0;
             int numberOfGoods = 0;
-            List<GoodsType> goodsTypes = FreeCol.getSpecification().getGoodsTypeList();
+            List<GoodsType> goodsTypes = getGame().getSpecification().getGoodsTypeList();
             for (GoodsType type : goodsTypes) {
                 if (type.isStorable()) {
                     averageIncome += getPlayer().getIncomeAfterTaxes(type);
@@ -372,7 +371,7 @@ public class ColonialAIPlayer extends AIPlayer {
 
     /**
      * Decides whether to accept an Indian demand, or not.
-     * 
+     *
      * @param unit The unit making demands.
      * @param colony The colony where demands are being made.
      * @param goods The goods demanded.
@@ -391,7 +390,7 @@ public class ColonialAIPlayer extends AIPlayer {
 
     /**
      * Decides whether to accept a mercenary offer, or not.
-     * 
+     *
      * @return <code>true</code> if this <code>AIPlayer</code> accepts the
      *         offer and <code>false</code> otherwise.
      */
@@ -476,10 +475,10 @@ public class ColonialAIPlayer extends AIPlayer {
         return accept;
     }
 
-    
+
     /**
      * Called after another <code>Player</code> sends a <code>trade</code> message
-     * 
+     *
      * @param goods The goods which we are going to offer
      */
     public void registerSellGoods(Goods goods) {
@@ -489,23 +488,25 @@ public class ColonialAIPlayer extends AIPlayer {
 
     /**
      * Called when another <code>Player</code> proposes a trade.
-     * 
-     * 
+     *
+     *
      * @param unit The foreign <code>Unit</code> trying to trade.
+     * @param settlement The <code>Settlement</code> this player owns and
+     *            which the given <code>Unit</code> is trying to trade.
      * @param goods The goods the given <code>Unit</code> is trying to sell.
      * @param gold The suggested price.
      * @return The price this <code>AIPlayer</code> suggests or
      *         {@link NetworkConstants#NO_TRADE}.
      */
-    public int buyProposition(Unit unit, Goods goods, int gold) {
+    public int buyProposition(Unit unit, Settlement settlement, Goods goods, int gold) {
         //method currently only called for owners of indian settlements
         logger.warning("buyProposition() not implemented for ColonialAIPlayer!");
         return NetworkConstants.NO_TRADE;
     }
 
     /**
-     * Called when another <code>Player</code> proposes a trade.
-     * 
+     * Called when another <code>Player</code> proposes a sale.
+     *
      * @param unit The foreign <code>Unit</code> trying to trade.
      * @param settlement The <code>Settlement</code> this player owns and
      *            which the given <code>Unit</code> if trying to sell goods.
@@ -514,13 +515,13 @@ public class ColonialAIPlayer extends AIPlayer {
      * @return The price this <code>AIPlayer</code> suggests or
      *         {@link NetworkConstants#NO_TRADE}.
      */
-    public int tradeProposition(Unit unit, Settlement settlement, Goods goods, int gold) {
-        logger.finest("Entering method tradeProposition");
+    public int sellProposition(Unit unit, Settlement settlement, Goods goods, int gold) {
+        logger.finest("Entering method sellProposition");
         if (settlement instanceof Colony) {
             Colony colony = (Colony) settlement;
             Player otherPlayer = unit.getOwner();
             // the client should have prevented this
-            if (getPlayer().getStance(otherPlayer) == Stance.WAR) {
+            if (getPlayer().atWarWith(otherPlayer)) {
                 return NetworkConstants.NO_TRADE;
             }
             // don't pay for more than fits in the warehouse
@@ -541,7 +542,7 @@ public class ColonialAIPlayer extends AIPlayer {
 
     /**
      * Writes this object to an XML stream.
-     * 
+     *
      * @param out The target stream.
      * @throws XMLStreamException if there are any problems writing to the
      *             stream.
@@ -555,7 +556,7 @@ public class ColonialAIPlayer extends AIPlayer {
 
     /**
      * Reads information for this object from an XML stream.
-     * 
+     *
      * @param in The input stream with the XML.
      * @throws XMLStreamException if there are any problems reading from the
      *             stream.
@@ -568,7 +569,7 @@ public class ColonialAIPlayer extends AIPlayer {
 
     /**
      * Returns the tag name of the root element representing this object.
-     * 
+     *
      * @return the tag name.
      */
     public static String getXMLElementTagName() {
@@ -585,17 +586,17 @@ public class ColonialAIPlayer extends AIPlayer {
     private void cheat() {
         logger.finest("Entering method cheat");
         // TODO-AI-CHEATING: REMOVE WHEN THE AI IS GOOD ENOUGH:
-        for (GoodsType goodsType : FreeCol.getSpecification().getGoodsTypeList()) {
-            getPlayer().resetArrears(goodsType);
+        for (GoodsType goodsType : getGame().getSpecification().getGoodsTypeList()) {
+            getPlayer().getMarket().setArrears(goodsType, 0);
         }
-        
+
         //TODO: This seems to buy units the AIPlayer can't possibly use (see BR#2566180)
         if (getAIMain().getFreeColServer().isSingleplayer() && getPlayer().isAI()
                 && getPlayer().getPlayerType() == PlayerType.COLONIAL) {
             Europe europe = getPlayer().getEurope();
-            List<UnitType> unitTypes = FreeCol.getSpecification().getUnitTypeList();
-            
-            if (getRandom().nextInt(10) == 1) {
+            List<UnitType> unitTypes = getGame().getSpecification().getUnitTypeList();
+
+            if (getAIRandom().nextInt(10) == 1) {
                 int price = 0;
                 UnitType unitToTrain = null;
                 for (UnitType unitType : unitTypes) {
@@ -614,25 +615,18 @@ public class ColonialAIPlayer extends AIPlayer {
                 }
                 if (unit != null && unit.isColonist()) {
                     // no need to equip artillery units with muskets or horses
-                    getPlayer().modifyGold(getPlayer().getMarket().getBidPrice(Goods.MUSKETS, 50));
-                    getPlayer().modifyGold(getPlayer().getMarket().getBidPrice(Goods.HORSES, 50));
-                    
-                    Element clearSpecialityElement = Message.createNewRootElement("clearSpeciality");
-                    clearSpecialityElement.setAttribute("unit", unit.getId());
-                    sendAndWaitSafely(clearSpecialityElement);
-                    Element equipMusketsElement = Message.createNewRootElement("equipUnit");
-                    equipMusketsElement.setAttribute("unit", unit.getId());
-                    equipMusketsElement.setAttribute("type", "model.equipment.muskets");
-                    equipMusketsElement.setAttribute("amount", Integer.toString(50));
-                    sendAndWaitSafely(equipMusketsElement);
-                    Element equipHorsesElement = Message.createNewRootElement("equipUnit");
-                    equipHorsesElement.setAttribute("unit", unit.getId());
-                    equipHorsesElement.setAttribute("type", "model.equipment.horses");
-                    equipHorsesElement.setAttribute("amount", Integer.toString(50));
-                    sendAndWaitSafely(equipHorsesElement);
+                    Specification spec = getGame().getSpecification();
+                    GoodsType muskets = spec.getGoodsType("model.goods.muskets");
+                    GoodsType horses = spec.getGoodsType("model.goods.horses");
+                    getPlayer().modifyGold(getPlayer().getMarket().getBidPrice(muskets, 50));
+                    getPlayer().modifyGold(getPlayer().getMarket().getBidPrice(horses, 50));
+
+                    AIMessage.askClearSpeciality(getAIUnit(unit));
+                    AIMessage.askEquipUnit(getAIUnit(unit), spec.getEquipmentType("model.equipment.horses"), 1);
+                    AIMessage.askEquipUnit(getAIUnit(unit), spec.getEquipmentType("model.equipment.muskets"), 1);
                 }
             }
-            if (getRandom().nextInt(40) == 21) {
+            if (getAIRandom().nextInt(40) == 21) {
                 int total = 0;
                 ArrayList<UnitType> navalUnits = new ArrayList<UnitType>();
                 for (UnitType unitType : unitTypes) {
@@ -641,9 +635,9 @@ public class ColonialAIPlayer extends AIPlayer {
                         total += europe.getUnitPrice(unitType);
                     }
                 }
-                
+
                 UnitType unitToPurchase = null;
-                int random = getRandom().nextInt(total);
+                int random = getAIRandom().nextInt(total);
                 total = 0;
                 for (UnitType unitType : navalUnits) {
                     total += unitType.getPrice();
@@ -669,7 +663,7 @@ public class ColonialAIPlayer extends AIPlayer {
             AIUnit au = it.next();
             if (!au.hasMission()
                     && (au.getUnit().getLocation() instanceof ColonyTile || au.getUnit().getLocation() instanceof Building)) {
-                AIColony ac = (AIColony) getAIMain().getAIObject(au.getUnit().getColony());
+                AIColony ac = getAIColony(au.getUnit().getColony());
                 au.setMission(new WorkInsideColonyMission(getAIMain(), au, ac));
             }
         }
@@ -677,53 +671,14 @@ public class ColonialAIPlayer extends AIPlayer {
 
     /**
      * Determines the stances towards each player.
-     * 
+     *
      * That is: should we declare war?
      */
     private void determineStances() {
         logger.finest("Entering method determineStances");
         Player player = getPlayer();
         for (Player p : getGame().getPlayers()) {
-            if(p == player){
-                continue;
-            }
-            
-            Stance stance = player.getStance(p);
-            Tension tension = player.getTension(p);
-            // not contacted yet
-            if (stance == Stance.UNCONTACTED || tension == null) {
-                continue;
-            }
-
-            // REF is always at war with rebel player
-            if (p.getREFPlayer() == player && p.getPlayerType() == PlayerType.REBEL) {
-                tension.modify(1000);
-                if(stance != Stance.WAR){
-                    player.changeRelationWithPlayer(p, Stance.WAR);
-                }
-                continue;
-            }
-            
-            // Ready to go to war
-            if (stance != Stance.WAR 
-                    && tension.getLevel() == Tension.Level.HATEFUL) {
-                player.changeRelationWithPlayer(p, Stance.WAR);
-                continue;
-            }
-            
-            // Ready to back down
-            if (stance == Stance.WAR
-                    && tension.getLevel().compareTo(Tension.Level.CONTENT) <= 0) {
-                player.changeRelationWithPlayer(p, Stance.CEASE_FIRE);
-                continue;
-            }
-            
-            // all friends now
-            if (stance == Stance.CEASE_FIRE
-                    && tension.getLevel().compareTo(Tension.Level.HAPPY) <= 0) {
-                player.changeRelationWithPlayer(p, Stance.PEACE);
-                continue;
-            }
+            if (p != player && !p.isDead()) determineStance(p);
         }
     }
 
@@ -805,16 +760,6 @@ public class ColonialAIPlayer extends AIPlayer {
             }
 
             c.rearrangeWorkers(getConnection());
-            
-            ArrayList<Tile> tilesToUpdate = new ArrayList<Tile>();
-            for (ColonyTile colonyTile : c.getColony().getColonyTiles()) {
-                boolean isOccupied = colonyTile.getUnit() != null;
-                boolean wasOccupied = oldWorkTiles.remove(colonyTile.getWorkTile());
-                if (isOccupied != wasOccupied) {
-                    tilesToUpdate.add(colonyTile.getWorkTile());
-                }
-            }
-            sendUpdatedTilesToAll(tilesToUpdate);
         }
     }
 
@@ -823,13 +768,9 @@ public class ColonialAIPlayer extends AIPlayer {
      */
     private void giveNormalMissions() {
         logger.finest("Entering method giveNormalMissions");
-        
-        int numberOfUnits = FreeCol.getSpecification().numberOfUnitTypes();
+
         // Create a datastructure for the worker wishes:
-        Vector<ArrayList<Wish>> workerWishes = new Vector<ArrayList<Wish>>(numberOfUnits);
-        for (int i = 0; i < numberOfUnits; i++) {
-            workerWishes.add(new ArrayList<Wish>());
-        }
+        java.util.Map<UnitType, ArrayList<Wish>> workerWishes = new HashMap<UnitType, ArrayList<Wish>>();
 
         Iterator<AIColony> aIterator = getAIColonyIterator();
         while (aIterator.hasNext()) {
@@ -837,29 +778,35 @@ public class ColonialAIPlayer extends AIPlayer {
             while (wIterator.hasNext()) {
                 Wish w = wIterator.next();
                 if (w instanceof WorkerWish && w.getTransportable() == null) {
-                    workerWishes.get(((WorkerWish) w).getUnitType().getIndex()).add(w);
+                    UnitType unitType = ((WorkerWish) w).getUnitType();
+                    ArrayList<Wish> wishes = workerWishes.get(unitType);
+                    if (wishes == null) {
+                        wishes = new ArrayList<Wish>();
+                        workerWishes.put(unitType, wishes);
+                    }
+                    wishes.add(w);
                 }
             }
         }
 
-        
+
         final boolean fewColonies = hasFewColonies();
         boolean isPioneerReq = PioneeringMission.getPlayerPioneers(this).size() == 0;
         Iterator<AIUnit> aiUnitsIterator = getAIUnitIterator();
         while (aiUnitsIterator.hasNext()) {
             AIUnit aiUnit = aiUnitsIterator.next();
-            
+
             if (aiUnit.hasMission()) {
                 continue;
             }
-            
+
             Unit unit = aiUnit.getUnit();
-            
+
             if (unit.isUninitialized()) {
                 logger.warning("Trying to assign a mission to an uninitialized object: " + unit.getId());
                 continue;
             }
-            
+
             // Setup as a pioneer if unit is:
             //      - already with tools, or
             //      - an expert pioneer, or
@@ -872,23 +819,23 @@ public class ColonialAIPlayer extends AIPlayer {
                 isPioneerReq = false;
                 continue;
             }
-            
+
             if (unit.canCarryTreasure()) {
                 aiUnit.setMission(new CashInTreasureTrainMission(getAIMain(), aiUnit));
             } else if (unit.hasAbility("model.ability.scoutIndianSettlement") &&
                        ScoutingMission.isValid(aiUnit)) {
                 aiUnit.setMission(new ScoutingMission(getAIMain(), aiUnit));
             } else if ((unit.isOffensiveUnit() || unit.isDefensiveUnit())
-                       && (!unit.isColonist() || unit.hasAbility("model.ability.expertSoldier") || 
+                       && (!unit.isColonist() || unit.hasAbility("model.ability.expertSoldier") ||
                         getGame().getTurn().getNumber() > 5)) {
                 giveMilitaryMission(aiUnit);
-            } else if (unit.isColonist()) {                
+            } else if (unit.isColonist()) {
                 /*
                  * Motivated by (speed) performance: This map stores the
                  * distance between the unit and the destination of a Wish:
                  */
                 HashMap<Location, Integer> distances = new HashMap<Location, Integer>(121);
-                for (ArrayList<Wish> al : workerWishes) {
+                for (ArrayList<Wish> al : workerWishes.values()) {
                     for (Wish w : al) {
                         if (!distances.containsKey(w.getDestination())) {
                             distances.put(w.getDestination(), unit.getTurnsToReach(w.getDestination()));
@@ -898,7 +845,7 @@ public class ColonialAIPlayer extends AIPlayer {
 
                 // Check if this unit is needed as an expert (using:
                 // "WorkerWish"):
-                ArrayList<Wish> wishList = workerWishes.get(unit.getType().getIndex());
+                ArrayList<Wish> wishList = workerWishes.get(unit.getType());
                 WorkerWish bestWish = null;
                 int bestTurns = Integer.MAX_VALUE;
                 for (int i = 0; i < wishList.size(); i++) {
@@ -937,7 +884,7 @@ public class ColonialAIPlayer extends AIPlayer {
                 if (colonyTile != null) {
                     bestTurns = unit.getTurnsToReach(colonyTile);
                 }
-                
+
                 // Check if we can find a better site to work than a new colony:
                 if (!fewColonies || colonyTile == null || bestTurns > 10) {
                     for (int i = 0; i < workerWishes.size(); i++) {
@@ -976,21 +923,20 @@ public class ColonialAIPlayer extends AIPlayer {
                 }
                 // Choose to build a new colony:
                 if (colonyTile != null) {
-                	Mission mission = new BuildColonyMission(getAIMain(), 
-                							aiUnit, 
-                							colonyTile, 
+                	Mission mission = new BuildColonyMission(getAIMain(),
+                							aiUnit,
+                							colonyTile,
                 							getPlayer().getColonyValue(colonyTile));
                     aiUnit.setMission(mission);
-                    
-                    boolean isUnitOnCarrier = aiUnit.getUnit().isOnCarrier(); 
+
+                    boolean isUnitOnCarrier = aiUnit.getUnit().isOnCarrier();
                     if (isUnitOnCarrier) {
-                        AIUnit carrier = (AIUnit) getAIMain().getAIObject(
-                                (FreeColGameObject) aiUnit.getUnit().getLocation());
-                        
+                        AIUnit carrier = getAIUnit((Unit) aiUnit.getUnit().getLocation());
+
                         //make verification of carrier mission
                         Mission carrierMission = carrier.getMission();
-                        
-                        boolean isCarrierMissionToTransport = carrierMission instanceof TransportMission; 
+
+                        boolean isCarrierMissionToTransport = carrierMission instanceof TransportMission;
                         if(!isCarrierMissionToTransport){
                         	throw new IllegalStateException("Carrier carrying unit not on a transport mission");
                         }
@@ -1026,9 +972,9 @@ public class ColonialAIPlayer extends AIPlayer {
     }
 
     /**
-     * 
+     *
      * Makes every unit perform their mission.
-     * 
+     *
      */
     private void doMissions() {
         logger.finest("Entering method doMissions");
@@ -1050,11 +996,11 @@ public class ColonialAIPlayer extends AIPlayer {
 
     int getDefendColonyMissionValue(Unit u, Colony colony, int turns) {
         logger.finest("Entering method getDefendColonyMissionValue");
-        
+
         // Sanitation
         if(colony == null)
         	return Integer.MIN_VALUE;
-        
+
         // Temporary helper method for: giveMilitaryMission
         int value = 10025 - turns;
         int numberOfDefendingUnits = 0;
@@ -1065,7 +1011,7 @@ public class ColonialAIPlayer extends AIPlayer {
             if (m != null && m instanceof DefendSettlementMission) {
                 if (((DefendSettlementMission) m).getSettlement() == colony) {
                 	//TODO: this decrease seems too little
-                	value -= 6; 
+                	value -= 6;
                     numberOfDefendingUnits++;
                 }
             }
@@ -1087,16 +1033,16 @@ public class ColonialAIPlayer extends AIPlayer {
 
     int getUnitSeekAndDestroyMissionValue(Unit unit, Tile newTile, int turns) {
         logger.finest("Entering method getUnitSeekAndDestroyMissionValue");
-        
+
         Unit defender = newTile.getDefendingUnit(unit);
-        
+
         if(!isTargetValidForSeekAndDestroy(unit, defender)){
         	return Integer.MIN_VALUE;
         }
-        
+
         int value = 10020;
         CombatModel combatModel = unit.getGame().getCombatModel();
-        
+
         if (getBestTreasureTrain(newTile) != null) {
         	value += Math.min(getBestTreasureTrain(newTile).getTreasureAmount() / 10, 50);
         }
@@ -1104,11 +1050,11 @@ public class ColonialAIPlayer extends AIPlayer {
         		newTile.getSettlement() == null) {
         	value += 200 - combatModel.getDefencePower(unit, defender) * 2 - turns * 50;
         }
-            
+
         value += combatModel.getOffencePower(defender, unit) -
               combatModel.getDefencePower(defender, unit);
         value -= turns * 10;
- 
+
         if (!defender.isNaval()) {
         	if (defender.hasAbility("model.ability.expertSoldier")
                     && !defender.isArmed()) {
@@ -1128,23 +1074,23 @@ public class ColonialAIPlayer extends AIPlayer {
                     }
                 }
             }
-        }   
+        }
         return Math.max(0, value);
     }
-    
+
     /**
      * TODO: Package for access by a test only - necessary?
-     */     
+     */
     boolean isTargetValidForSeekAndDestroy(Unit attacker, Unit defender) {
         if (attacker.getOwner()!=getPlayer()) {
             logger.warning("isTargetValidForSeekAndDestroy() called for other players unit!");
             return false;
         }
-    
+
         if (defender == null) { // Sanitation
             return false;
         }
-    	
+
         // A naval unit cannot attack a land unit and vice-versa
         if (attacker.isNaval() != defender.isNaval()) {
             return false;
@@ -1163,34 +1109,29 @@ public class ColonialAIPlayer extends AIPlayer {
             return false;
         }
 
-        boolean atWar = attackerPlayer.getStance(defenderPlayer) == Stance.WAR;
-
         // If european, do not attack if not at war
-        if (!atWar) {
-            return false;
-        }
-        return true;
+        return attackerPlayer.atWarWith(defenderPlayer);
     }
-        
+
     /**
      * Gives a military <code>Mission</code> to the given unit. <br>
      * <br>
      * <b>This method should only be used on units owned by european players.</b>
      *
      * TODO: Package for access by a test only - necessary?
-     * 
+     *
      * @param aiUnit The unit.
      */
     void giveMilitaryMission(AIUnit aiUnit) {
         logger.finest("Entering method giveMilitaryMission");
         /*
-         * 
+         *
          * Temporary method for giving a military mission.
-         * 
+         *
          * This method will be removed when "MilitaryStrategy" and
-         * 
+         *
          * the "Tactic"-classes has been implemented.
-         * 
+         *
          */
         final Unit unit = aiUnit.getUnit();
         Unit carrier = (unit.isOnCarrier()) ? (Unit) unit.getLocation() : null;
@@ -1201,18 +1142,20 @@ public class ColonialAIPlayer extends AIPlayer {
         // Determine starting tile:
         Tile startTile = unit.getTile();
         if (startTile == null) {
-            if (unit.isOnCarrier()) {
-                startTile = (Tile) ((Unit) unit.getLocation()).getEntryLocation();
-            } else {
-                startTile = (Tile) unit.getOwner().getEntryLocation();
+            startTile = ((unit.isOnCarrier()) ? ((Unit) unit.getLocation())
+                         : unit).getFullEntryLocation();
+            if (startTile == null) {
+                logger.warning("Unable to determine entry location for: "
+                               + unit.toString());
+                return;
             }
         }
         /*
-         * 
+         *
          * Checks if we are currently located on a Tile with a Settlement
-         * 
+         *
          * which requires defenders:
-         * 
+         *
          */
         if (unit.getColony() != null) {
             bestTarget = unit.getColony();
@@ -1384,7 +1327,7 @@ public class ColonialAIPlayer extends AIPlayer {
         if(transportables.isEmpty()){
             return;
         }
-        
+
         // Update the priority
         for (Transportable t : transportables){
             t.increaseTransportPriority();
@@ -1400,13 +1343,13 @@ public class ColonialAIPlayer extends AIPlayer {
                 vacantTransports.add(au.getMission());
             }
         }
-        
+
         // save further processing
         // we must only do this verification after the priority update
         if(vacantTransports.isEmpty()){
             return;
         }
-        
+
         // order the list by priority
         Collections.sort(transportables, new Comparator<Transportable>() {
             public int compare(Transportable o1, Transportable o2) {
@@ -1431,7 +1374,7 @@ public class ColonialAIPlayer extends AIPlayer {
             Location transportableLoc = t.getTransportLocatable().getLocation();
             boolean isTransportableAlreadyOnCarrier = transportableLoc instanceof Unit;
             if (isTransportableAlreadyOnCarrier) {
-                AIUnit carrierAI = (AIUnit) getAIMain().getAIObject((Unit) transportableLoc);
+                AIUnit carrierAI = getAIUnit((Unit) transportableLoc);
                 Mission m = carrierAI.getMission();
                 if (m instanceof TransportMission) {
                     ((TransportMission) m).addToTransportList(t);
@@ -1491,7 +1434,7 @@ public class ColonialAIPlayer extends AIPlayer {
     /**
      * Returns the treasure train carrying the largest treasure
      * located on the given <code>Tile</code>.
-     * 
+     *
      * @param tile a <code>Tile</code> value
      * @return The best treasure train or <code>null</code> if no treasure
      *         train is located on this <code>Tile</code>.
@@ -1500,7 +1443,7 @@ public class ColonialAIPlayer extends AIPlayer {
         Unit bestTreasureTrain = null;
         for (Unit unit : tile.getUnitList()) {
             if (unit.canCarryTreasure() &&
-                (bestTreasureTrain == null || 
+                (bestTreasureTrain == null ||
                  bestTreasureTrain.getTreasureAmount() < unit.getTreasureAmount())) {
                 bestTreasureTrain = unit;
             }
@@ -1514,12 +1457,12 @@ public class ColonialAIPlayer extends AIPlayer {
         while (it.hasNext()) {
             AIUnit au = it.next();
             Unit u = au.getUnit();
-            
+
             if (u.getRole()==Role.MISSIONARY) {
                 logger.info("Found missionary unit:"+u.getId());
                 mGoal.addUnit(au);
                 //NOTE: Removes units being assigned a goal from the units list
-                it.remove(); 
+                it.remove();
             }
         }
     }
@@ -1527,10 +1470,10 @@ public class ColonialAIPlayer extends AIPlayer {
     /**
      * Returns an iterator over all <code>AIUnit</code>s owned by this
      * player, which currently do _not_ have a goal.
-     * 
+     *
      * This is a temporary override to allow the old AI code to keep working
      * with some units, while exempting those already managed by new code.
-     * 
+     *
      * @return The <code>Iterator</code>.
      */
     protected Iterator<AIUnit> getAIUnitIterator() {
@@ -1540,13 +1483,13 @@ public class ColonialAIPlayer extends AIPlayer {
             Iterator<Unit> unitsIterator = getPlayer().getUnitIterator();
             while (unitsIterator.hasNext()) {
                 Unit theUnit = unitsIterator.next();
-                AIUnit a = (AIUnit) getAIMain().getAIObject(theUnit.getId());
-                if (a != null && a.getGoal()==null) {
-                    //NOTE: Only adds units that currently don't have a goal.
-                    au.add(a);
+                AIUnit a = getAIUnit(theUnit);
+                if (a != null) {
+                    if (a.getGoal() == null) au.add(a);
                 } else {
-                    logger.warning("Could not find the AIUnit for: " + theUnit + " (" + theUnit.getId() + ") - "
-                            + (getGame().getFreeColGameObject(theUnit.getId()) != null));
+                    logger.warning("Could not find the AIUnit for: "
+                                   + theUnit + " (" + theUnit.getId() + ") - "
+                                   + (getGame().getFreeColGameObject(theUnit.getId()) != null));
                 }
             }
             myAIUnits = au;
@@ -1556,11 +1499,11 @@ public class ColonialAIPlayer extends AIPlayer {
 
     /**
      * Helper method to let implementing subclasses clear aiUnits.
-     * Override, see above!     
-     */         
+     * Override, see above!
+     */
     protected void clearAIUnits() {
         logger.info("Override: clearAIUnits()!");
-        myAIUnits.clear();    
+        myAIUnits.clear();
     }
 
 }
