@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -28,7 +28,6 @@ import java.util.List;
 import java.util.Set;
 import java.util.logging.Logger;
 
-import net.sf.freecol.common.Specification;
 import net.sf.freecol.common.model.AbstractGoods;
 import net.sf.freecol.common.model.BuildableType;
 import net.sf.freecol.common.model.Building;
@@ -36,13 +35,11 @@ import net.sf.freecol.common.model.BuildingType;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.ColonyTile;
 import net.sf.freecol.common.model.Game;
-import net.sf.freecol.common.model.Goods;
 import net.sf.freecol.common.model.GoodsType;
 import net.sf.freecol.common.model.Market;
 import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.Unit;
 import net.sf.freecol.common.model.UnitType;
-
 import net.sf.freecol.server.ai.ColonyProfile.ProfileType;
 
 import org.w3c.dom.Document;
@@ -50,13 +47,13 @@ import org.w3c.dom.Element;
 
 /**
  * Objects of this class describes the plan the AI has for a <code>Colony</code>.
- * 
+ *
  * <br>
  * <br>
- * 
+ *
  * A <code>ColonyPlan</code> contains {@link WorkLocationPlan}s which defines
  * the production of each {@link Building} and {@link ColonyTile}.
- * 
+ *
  * @see Colony
  */
 public class ColonyPlan {
@@ -80,11 +77,6 @@ public class ColonyPlan {
     // per building?
     private static final int MAX_LEVEL = 3;
     private static final int MIN_RAW_GOODS_THRESHOLD = 20;
-    
-    private static final GoodsType hammersType = Specification.getSpecification().getGoodsType("model.goods.hammers");
-    private static final GoodsType toolsType = Specification.getSpecification().getGoodsType("model.goods.tools");
-    private static final GoodsType lumberType = Specification.getSpecification().getGoodsType("model.goods.lumber");
-    private static final GoodsType oreType = Specification.getSpecification().getGoodsType("model.goods.ore");
 
     /**
      * The FreeColGameObject this AIObject contains AI-information for.
@@ -96,19 +88,19 @@ public class ColonyPlan {
     private ArrayList<WorkLocationPlan> workLocationPlans = new ArrayList<WorkLocationPlan>();
 
     private GoodsType primaryRawMaterial = null;
-    
+
     private GoodsType secondaryRawMaterial = null;
 
     /**
      * Describe profile here.
      */
-    private ColonyProfile profile = new ColonyProfile();
+    private ColonyProfile profile;
+
+
 
-    
-    
     /**
      * Creates a new <code>ColonyPlan</code>.
-     * 
+     *
      * @param aiMain The main AI-object.
      * @param colony The colony to make a <code>ColonyPlan</code> for.
      */
@@ -123,7 +115,7 @@ public class ColonyPlan {
 
     /**
      * Creates a new <code>ColonyPlan</code>.
-     * 
+     *
      * @param aiMain The main AI-object.
      * @param element An <code>Element</code> containing an XML-representation
      *            of this object.
@@ -136,7 +128,7 @@ public class ColonyPlan {
     /**
      * Returns the <code>WorkLocationPlan</code>s associated with this
      * <code>ColonyPlan</code>.
-     * 
+     *
      * @return The list of <code>WorkLocationPlan</code>s .
      */
     public List<WorkLocationPlan> getWorkLocationPlans() {
@@ -146,7 +138,7 @@ public class ColonyPlan {
     /**
      * Returns the <code>WorkLocationPlan</code>s associated with this
      * <code>ColonyPlan</code> sorted by production in a decreasing order.
-     * 
+     *
      * @return The list of <code>WorkLocationPlan</code>s .
      */
     public List<WorkLocationPlan> getSortedWorkLocationPlans() {
@@ -192,7 +184,7 @@ public class ColonyPlan {
     /**
      * Gets an <code>Iterator</code> for everything to be built in the
      * <code>Colony</code>.
-     * 
+     *
      * @return An iterator containing all the <code>Buildable</code> sorted by
      *         priority (highest priority first).
      */
@@ -205,7 +197,7 @@ public class ColonyPlan {
         }
 
         List<Buildable> buildables = new ArrayList<Buildable>();
-        
+
         List<BuildingType> docks = new ArrayList<BuildingType>();
         List<BuildingType> customs = new ArrayList<BuildingType>();
         List<BuildingType> builders = new ArrayList<BuildingType>();
@@ -215,13 +207,13 @@ public class ColonyPlan {
         List<BuildingType> churches = new ArrayList<BuildingType>();
         List<BuildingType> townHalls = new ArrayList<BuildingType>();
 
-        for (BuildingType type : Specification.getSpecification().getBuildingTypeList()) {
+        for (BuildingType type : colony.getSpecification().getBuildingTypeList()) {
             if (type.hasAbility("model.ability.produceInWater")) {
                 docks.add(type);
-            } 
+            }
             if (type.hasAbility("model.ability.export")) {
                 customs.add(type);
-            } 
+            }
             if (type.hasAbility("model.ability.teach")) {
                 schools.add(type);
             }
@@ -247,7 +239,7 @@ public class ColonyPlan {
 
         List<UnitType> buildableDefenders = new ArrayList<UnitType>();
         UnitType bestWagon = null;
-        for (UnitType unitType : Specification.getSpecification().getUnitTypeList()) {
+        for (UnitType unitType : colony.getSpecification().getUnitTypeList()) {
             if (unitType.getDefence() > UnitType.DEFAULT_DEFENCE
                 && !unitType.hasAbility("model.ability.navalUnit")
                 && !unitType.getGoodsRequired().isEmpty()) {
@@ -270,6 +262,8 @@ public class ColonyPlan {
 
         if (colony.isLandLocked()) {
             // landlocked colonies need transportation
+            /* Disable this until the AI actually does something with
+               its wagon trains
             int landLockedColonies = 0;
             for (Colony otherColony : colony.getOwner().getColonies()) {
                 if (otherColony.isLandLocked()) {
@@ -280,6 +274,7 @@ public class ColonyPlan {
                 buildables.add(new Buildable(bestWagon, WAGON_TRAIN_PRIORITY
                                              * (landLockedColonies - wagonTrains)));
             }
+            */
         } else if (!colony.hasAbility("model.ability.produceInWater")) {
             // coastal colonies need docks
             int potential = 0;
@@ -317,7 +312,7 @@ public class ColonyPlan {
                 // of other buildings (printing press, newspaper)
                 GoodsType outputType = b.getGoodsOutputType();
                 if (outputType != null) {
-                    for (BuildingType otherType : Specification.getSpecification()
+                    for (BuildingType otherType : colony.getSpecification()
                              .getBuildingTypeList()) {
                         if (!otherType.getModifierSet(outputType.getId()).isEmpty()
                             && colony.canBuild(otherType)) {
@@ -381,7 +376,7 @@ public class ColonyPlan {
         for (BuildingType buildingType : military) {
             if (colony.canBuild(buildingType)) {
                 if (colony.getBuilding(buildingType) == null
-                    && (buildingType.getConsumedGoodsType() == null 
+                    && (buildingType.getConsumedGoodsType() == null
                         || buildingType.getConsumedGoodsType().isFarmed())) {
                     buildables.add(new Buildable(buildingType, UPGRADE_PRIORITY));
                 } else {
@@ -392,7 +387,7 @@ public class ColonyPlan {
 
         // add artillery if necessary
         // TODO: at some point, we will have to add ships
-        if (((AIColony) aiMain.getAIObject(colony)).isBadlyDefended()) {
+        if (aiMain.getAIColony(colony).isBadlyDefended()) {
             for (UnitType unitType : buildableDefenders) {
                 if (colony.canBuild(unitType)) {
                     int priority = (profile.getType() == ProfileType.LARGE
@@ -402,7 +397,7 @@ public class ColonyPlan {
                     break;
                 }
             }
-        }        
+        }
 
         // improve education
         if (profile.getType() != ProfileType.SMALL) {
@@ -458,7 +453,7 @@ public class ColonyPlan {
 
     /**
      * Gets the main AI-object.
-     * 
+     *
      * @return The main AI-object.
      */
     public AIMain getAIMain() {
@@ -467,7 +462,7 @@ public class ColonyPlan {
 
     /**
      * Get the <code>Game</code> this object is associated to.
-     * 
+     *
      * @return The <code>Game</code>.
      */
     public Game getGame() {
@@ -480,38 +475,56 @@ public class ColonyPlan {
      * manufactured.
      */
     public void create() {
-        
+
         workLocationPlans.clear();
         if (profile.getType() == ProfileType.OUTPOST) {
-            GoodsType goodsType = profile.getPreferredProduction().get(0);
-            workLocationPlans.add(new WorkLocationPlan(getAIMain(),
-                                                       getBestTileToProduce(goodsType),
-                                                       goodsType));
+            GoodsType goodsType;
+            ColonyTile productionTile;
+            if ((goodsType = profile.getPreferredProduction().get(0)) != null
+                && (productionTile = getBestTileToProduce(goodsType)) != null) {
+                workLocationPlans.add(new WorkLocationPlan(getAIMain(),
+                                                           productionTile,
+                                                           goodsType));
+            }
             return;
         }
-                
-        Building townHall = colony.getBuildingForProducing(Goods.BELLS);
-        
+
+        GoodsType bells = colony.getSpecification().getGoodsType("model.goods.bells");
+        GoodsType food = colony.getSpecification().getGoodsType("model.goods.grain");
+        GoodsType lumber = colony.getSpecification().getGoodsType("model.goods.lumber");
+        GoodsType muskets = colony.getSpecification().getGoodsType("model.goods.muskets");
+        GoodsType ore = colony.getSpecification().getGoodsType("model.goods.ore");
+        GoodsType silver = colony.getSpecification().getGoodsType("model.goods.silver");
+
+        Building townHall = colony.getBuildingForProducing(bells);
+
         // Choose the best production for each tile:
         for (ColonyTile ct : colony.getColonyTiles()) {
-
-            if (ct.getWorkTile().getOwningSettlement() != null &&
-                ct.getWorkTile().getOwningSettlement() != colony || ct.isColonyCenterTile()) {
+            if (ct.isColonyCenterTile()
+                || (ct.getWorkTile().getOwningSettlement() != null
+                    && ct.getWorkTile().getOwningSettlement() != colony)) {
                 continue;
             }
 
             GoodsType goodsType = getBestGoodsToProduce(ct.getWorkTile());
-            WorkLocationPlan wlp = new WorkLocationPlan(getAIMain(), ct, goodsType);
-            workLocationPlans.add(wlp);
+            if (goodsType != null) {
+                WorkLocationPlan wlp = new WorkLocationPlan(getAIMain(),
+                                                            ct, goodsType);
+                workLocationPlans.add(wlp);
+            }
         }
-        
-        // We need to find what, if any, is still required for what we are building
+
+        // We need to find what, if any, is still required for what we
+        // are building
         GoodsType buildingReq = null;
         GoodsType buildingRawMat = null;
         Building buildingReqProducer = null;
-        
+        final GoodsType hammersType = colony.getSpecification().getGoodsType("model.goods.hammers");
+        final GoodsType lumberType = colony.getSpecification().getGoodsType("model.goods.lumber");
+        final GoodsType oreType = colony.getSpecification().getGoodsType("model.goods.ore");
+
         buildingReq = getBuildingReqGoods();
-        
+
         if(buildingReq != null){
             if(buildingReq == hammersType){
                 buildingRawMat = lumberType;
@@ -524,10 +537,10 @@ public class ColonyPlan {
 
         // Try to ensure that we produce the raw material necessary for
         //what we are building
-        boolean buildingRawMatReq = buildingRawMat != null 
+        boolean buildingRawMatReq = buildingRawMat != null
                                     && colony.getGoodsCount(buildingRawMat) < MIN_RAW_GOODS_THRESHOLD
-                                    && getProductionOf(buildingRawMat) <= 0; 
-        
+                                    && getProductionOf(buildingRawMat) <= 0;
+
         if(buildingRawMatReq) {
             WorkLocationPlan bestChoice = null;
             int highestPotential = 0;
@@ -555,12 +568,13 @@ public class ColonyPlan {
         secondaryRawMaterial = null;
         int primaryRawMaterialProduction = 0;
         int secondaryRawMaterialProduction = 0;
-        List<GoodsType> goodsTypeList = Specification.getSpecification().getGoodsTypeList();
+        List<GoodsType> goodsTypeList = colony.getSpecification().getGoodsTypeList();
         for (GoodsType goodsType : goodsTypeList) {
             // only consider goods that can be transformed
             // do not consider hammers as a valid transformation
-            if (goodsType.getProducedMaterial() == null 
-                    || goodsType.getProducedMaterial() == hammersType) {
+            if ((goodsType.getProducedMaterial() == null
+                 && !goodsType.isFoodType())
+                || goodsType.getProducedMaterial() == hammersType) {
                 continue;
             }
             if (getProductionOf(goodsType) > primaryRawMaterialProduction) {
@@ -584,12 +598,12 @@ public class ColonyPlan {
                 continue;
             }
             if (wlp.getGoodsType() == primaryRawMaterial || wlp.getGoodsType() == secondaryRawMaterial
-                    || wlp.getGoodsType() == Goods.LUMBER || wlp.getGoodsType() == Goods.ORE
-                    || wlp.getGoodsType() == Goods.SILVER) {
+                    || wlp.getGoodsType() == lumber || wlp.getGoodsType() == ore
+                    || wlp.getGoodsType() == silver) {
                 continue;
             }
             // TODO: find out about unit working here, if any (?)
-            if (((ColonyTile) wlp.getWorkLocation()).getWorkTile().potential(Goods.FOOD, null) <= 2) {
+            if (((ColonyTile) wlp.getWorkLocation()).getWorkTile().potential(food, null) <= 2) {
                 if (wlp.getGoodsType() == null) {
                     // on arctic tiles nothing can be produced
                     wlpIterator.remove();
@@ -600,14 +614,14 @@ public class ColonyPlan {
                 continue;
             }
 
-            wlp.setGoodsType(Goods.FOOD);
+            wlp.setGoodsType(food);
         }
 
         // Produce the goods required for what is being built, if:
         //     - anything is being built, and
         //     - there is either production or stock of the raw material
-        if(buildingReq != null && 
-            (getProductionOf(buildingRawMat) > 0 
+        if(buildingReq != null &&
+            (getProductionOf(buildingRawMat) > 0
               || colony.getGoodsCount(buildingRawMat) > 0)){
             WorkLocationPlan wlp = new WorkLocationPlan(getAIMain(),
                     colony.getBuildingForProducing(buildingReq), buildingReq);
@@ -615,22 +629,22 @@ public class ColonyPlan {
         }
 
         // Place a statesman:
-        WorkLocationPlan townHallWlp = new WorkLocationPlan(getAIMain(), townHall, Goods.BELLS);
+        WorkLocationPlan townHallWlp = new WorkLocationPlan(getAIMain(), townHall, bells);
         workLocationPlans.add(townHallWlp);
 
         // Place a colonist to manufacture the primary goods:
         if (primaryRawMaterial != null) {
             GoodsType producedGoods = primaryRawMaterial.getProducedMaterial();
             Building b = colony.getBuildingForProducing(producedGoods);
-            if (b != null) {
+            if (producedGoods != null && b != null) {
                 WorkLocationPlan wlp = new WorkLocationPlan(getAIMain(), b, producedGoods);
                 workLocationPlans.add(wlp);
             }
         }
 
         // Remove the secondary goods if we need food:
-        if (secondaryRawMaterial != null 
-                && getFoodProduction() < workLocationPlans.size() * Colony.FOOD_CONSUMPTION 
+        if (secondaryRawMaterial != null
+                && needsFood()
                 && secondaryRawMaterial.isNewWorldGoodsType()) {
             Iterator<WorkLocationPlan> wlpIterator2 = workLocationPlans.iterator();
             while (wlpIterator2.hasNext()) {
@@ -638,8 +652,8 @@ public class ColonyPlan {
                 if (wlp.getWorkLocation() instanceof ColonyTile && wlp.getGoodsType() == secondaryRawMaterial) {
                     Tile t = ((ColonyTile) wlp.getWorkLocation()).getWorkTile();
                     // TODO: find out about unit working here, if any (?)
-                    if (t.getMaximumPotential(Goods.FOOD, null) > 2) {
-                        wlp.setGoodsType(Goods.FOOD);
+                    if (t.getMaximumPotential(food, null) > 2) {
+                        wlp.setGoodsType(food);
                     } else {
                         wlpIterator2.remove();
                     }
@@ -648,15 +662,15 @@ public class ColonyPlan {
         }
 
         // Remove the workers on the primary goods one-by-one if we need food:
-        if (getFoodProduction() < workLocationPlans.size() * Colony.FOOD_CONSUMPTION) {
+        if (needsFood()) {
             Iterator<WorkLocationPlan> wlpIterator2 = workLocationPlans.iterator();
-            while (wlpIterator2.hasNext() && getFoodProduction() < workLocationPlans.size() * Colony.FOOD_CONSUMPTION) {
+            while (wlpIterator2.hasNext() && needsFood()) {
                 WorkLocationPlan wlp = wlpIterator2.next();
                 if (wlp.getWorkLocation() instanceof ColonyTile && wlp.getGoodsType() == primaryRawMaterial) {
                     Tile t = ((ColonyTile) wlp.getWorkLocation()).getWorkTile();
                     // TODO: find out about unit working here, if any (?)
-                    if (t.getMaximumPotential(Goods.FOOD, null) > 2) {
-                        wlp.setGoodsType(Goods.FOOD);
+                    if (t.getMaximumPotential(food, null) > 2) {
+                        wlp.setGoodsType(food);
                     } else {
                         wlpIterator2.remove();
                     }
@@ -665,9 +679,9 @@ public class ColonyPlan {
         }
 
         // Remove the manufacturer if we still lack food:
-        if (getFoodProduction() < workLocationPlans.size() * Colony.FOOD_CONSUMPTION) {
+        if (needsFood()) {
             Iterator<WorkLocationPlan> wlpIterator2 = workLocationPlans.iterator();
-            while (wlpIterator2.hasNext() && getFoodProduction() < workLocationPlans.size() * Colony.FOOD_CONSUMPTION) {
+            while (wlpIterator2.hasNext() && needsFood()) {
                 WorkLocationPlan wlp = wlpIterator2.next();
                 if (wlp.getWorkLocation() instanceof Building) {
                     Building b = (Building) wlp.getWorkLocation();
@@ -686,33 +700,33 @@ public class ColonyPlan {
         if(colony.getGoodsCount(buildingRawMat) > 0){
             buildMatToGo = buildingRawMat;
         }
-        if (getFoodProduction() < workLocationPlans.size() * Colony.FOOD_CONSUMPTION) {
+        if (needsFood()) {
             Iterator<WorkLocationPlan> wlpIterator2 = workLocationPlans.iterator();
-            while (wlpIterator2.hasNext() && getFoodProduction() < workLocationPlans.size() * Colony.FOOD_CONSUMPTION) {
+            while (wlpIterator2.hasNext() && needsFood()) {
                 WorkLocationPlan wlp = wlpIterator2.next();
                 if (wlp.getWorkLocation() instanceof ColonyTile && wlp.getGoodsType() == buildMatToGo) {
                     wlpIterator2.remove();
                 }
             }
             // still lacking food, removing the rest
-            if (getFoodProduction() < workLocationPlans.size() * Colony.FOOD_CONSUMPTION) {
+            if (needsFood()) {
                 buildMatToGo = (buildMatToGo == buildingRawMat)? buildingReq : buildingRawMat;
-                
+
                 wlpIterator2 = workLocationPlans.iterator();
-                while (wlpIterator2.hasNext() && getFoodProduction() < workLocationPlans.size() * Colony.FOOD_CONSUMPTION) {
+                while (wlpIterator2.hasNext() && needsFood()) {
                     WorkLocationPlan wlp = wlpIterator2.next();
                     if (wlp.getWorkLocation() instanceof ColonyTile && wlp.getGoodsType() == buildMatToGo) {
                         wlpIterator2.remove();
                     }
                 }
             }
-        }        
-        
+        }
+
         // Primary allocations done
         // Beginning secondary allocations
-        
+
         // Not enough food for more allocations, save work and stop here
-        if(getFoodProduction() < workLocationPlans.size() * Colony.FOOD_CONSUMPTION + 2){
+        if (getFoodProduction() < getNextFoodConsumption()) {
             return;
         }
 
@@ -726,46 +740,48 @@ public class ColonyPlan {
             boolean blacksmithAdded = false;
 
             // Add a manufacturer for the secondary type of goods:
-            if (getFoodProduction() >= workLocationPlans.size() * Colony.FOOD_CONSUMPTION + 2 &&
+            if (getFoodProduction() >= getNextFoodConsumption() &&
                 secondaryRawMaterial != null &&
                 12 * secondaryWorkers + 6 <= getProductionOf(secondaryRawMaterial) &&
                 secondaryWorkers <= MAX_LEVEL) {
                 GoodsType producedGoods = secondaryRawMaterial.getProducedMaterial();
                 Building b = colony.getBuildingForProducing(producedGoods);
-                if (b != null) {
+                if (producedGoods != null && b != null) {
                     WorkLocationPlan wlp = new WorkLocationPlan(getAIMain(), b, producedGoods);
                     workLocationPlans.add(wlp);
                     colonistAdded = true;
                     secondaryWorkers++;
-                    if (secondaryRawMaterial == Goods.ORE) {
+                    if (secondaryRawMaterial == ore) {
                         blacksmithAdded = true;
                     }
                 }
             }
 
             // Add a manufacturer for the primary type of goods:
-            if (getFoodProduction() >= workLocationPlans.size() * Colony.FOOD_CONSUMPTION + 2 && primaryRawMaterial != null
-                    && 12 * primaryWorkers + 6 <= getProductionOf(primaryRawMaterial)
-                    && primaryWorkers <= MAX_LEVEL) {
+            if (getFoodProduction() >= getNextFoodConsumption()
+                && primaryRawMaterial != null
+                && 12 * primaryWorkers + 6 <= getProductionOf(primaryRawMaterial)
+                && primaryWorkers <= MAX_LEVEL) {
                 GoodsType producedGoods = primaryRawMaterial.getProducedMaterial();
                 Building b = colony.getBuildingForProducing(producedGoods);
-                if (b != null) {
+                if (producedGoods != null && b != null) {
                     WorkLocationPlan wlp = new WorkLocationPlan(getAIMain(), b, producedGoods);
                     workLocationPlans.add(wlp);
                     colonistAdded = true;
                     primaryWorkers++;
-                    if (primaryRawMaterial == Goods.ORE) {
+                    if (primaryRawMaterial == ore) {
                         blacksmithAdded = true;
                     }
                 }
             }
 
             // Add a gunsmith:
-            if (blacksmithAdded && getFoodProduction() >= workLocationPlans.size() * Colony.FOOD_CONSUMPTION + 2
-                    && gunsmiths < MAX_LEVEL) {
-                Building b = colony.getBuildingForProducing(Goods.MUSKETS);
+            if (blacksmithAdded
+                && getFoodProduction() >= getNextFoodConsumption()
+                && gunsmiths < MAX_LEVEL) {
+                Building b = colony.getBuildingForProducing(muskets);
                 if (b != null) {
-                    WorkLocationPlan wlp = new WorkLocationPlan(getAIMain(), b, Goods.MUSKETS);
+                    WorkLocationPlan wlp = new WorkLocationPlan(getAIMain(), b, muskets);
                     workLocationPlans.add(wlp);
                     colonistAdded = true;
                     gunsmiths++;
@@ -773,10 +789,11 @@ public class ColonyPlan {
             }
 
             // Add builders
-            if (getFoodProduction() >= workLocationPlans.size() * Colony.FOOD_CONSUMPTION + 2
-                    && buildingReqProducer != null 
-                    && buildingReqProducer.getProduction() * builders <= getProductionOf(buildingRawMat) 
-                    && buildingReqProducer.getMaxUnits() < builders) {
+            if (getFoodProduction() >= getNextFoodConsumption()
+                && buildingReq != null
+                && buildingReqProducer != null
+                && buildingReqProducer.getProduction() * builders <= getProductionOf(buildingRawMat)
+                && buildingReqProducer.getMaxUnits() < builders) {
                 WorkLocationPlan wlp = new WorkLocationPlan(getAIMain(), buildingReqProducer, buildingReq);
                 workLocationPlans.add(wlp);
                 colonistAdded = true;
@@ -793,9 +810,21 @@ public class ColonyPlan {
         // TODO: Add preacher
     }
 
+    private int getFoodConsumption() {
+        return workLocationPlans.size() * colony.getOwner().getMaximumFoodConsumption();
+    }
+
+    private int getNextFoodConsumption() {
+        return (workLocationPlans.size() + 1) * colony.getOwner().getMaximumFoodConsumption();
+    }
+
+    private boolean needsFood() {
+        return (getFoodProduction() < getFoodConsumption());
+    }
+
     /**
      * Returns the production of the given type of goods according to this plan.
-     * 
+     *
      * @param goodsType The type of goods to check the production for.
      * @return The maximum possible production of the given type of goods
      *         according to this <code>ColonyPlan</code>.
@@ -810,8 +839,8 @@ public class ColonyPlan {
         }
 
         // Add values for the center tile:
-        if (goodsType == colony.getTile().primaryGoods() ||
-            goodsType == colony.getTile().secondaryGoods()) {
+        if (colony.getTile().getType().isPrimaryGoodsType(goodsType)
+            || colony.getTile().getType().isSecondaryGoodsType(goodsType)) {
             amount += colony.getTile().getMaximumPotential(goodsType, null);
         }
 
@@ -820,13 +849,13 @@ public class ColonyPlan {
 
     /**
      * Returns the production of food according to this plan.
-     * 
+     *
      * @return The maximum possible food production
      *         according to this <code>ColonyPlan</code>.
      */
     public int getFoodProduction() {
         int amount = 0;
-        for (GoodsType foodType : Specification.getSpecification().getGoodsFood()) {
+        for (GoodsType foodType : colony.getSpecification().getFoodGoodsTypeList()) {
             amount += getProductionOf(foodType);
         }
 
@@ -836,7 +865,7 @@ public class ColonyPlan {
     /**
      * Determines the best goods to produce on a given <code>Tile</code>
      * within this colony.
-     * 
+     *
      * @param t The <code>Tile</code>.
      * @return The type of goods.
      */
@@ -894,7 +923,7 @@ public class ColonyPlan {
             if ((tile.getOwningSettlement() == null
                  || tile.getOwningSettlement() == colony)
                 && !ct.isColonyCenterTile()) {
-                for (GoodsType goodsType : Specification.getSpecification().getFarmedGoodsTypeList()) {
+                for (GoodsType goodsType : colony.getSpecification().getFarmedGoodsTypeList()) {
                     int production = market.getSalePrice(goodsType, tile.potential(goodsType, unitType));
                     if (bestProduction == null || value < production) {
                         value = production;
@@ -906,29 +935,33 @@ public class ColonyPlan {
         return bestProduction;
     }
 
-    
+
     public void adjustProductionAndManufacture(){
-        List<GoodsType> rawMatList = new ArrayList<GoodsType>(); 
-    
+        List<GoodsType> rawMatList = new ArrayList<GoodsType>();
+
+        final GoodsType hammersType = colony.getSpecification().getGoodsType("model.goods.hammers");
+        final GoodsType lumberType = colony.getSpecification().getGoodsType("model.goods.lumber");
+        final GoodsType oreType = colony.getSpecification().getGoodsType("model.goods.ore");
+
         if(getBuildingReqGoods() == hammersType){
             rawMatList.add(lumberType);
         }
         rawMatList.add(oreType);
-        
+
         if (primaryRawMaterial != null
                 && primaryRawMaterial != lumberType
                 && primaryRawMaterial != oreType
                 && !primaryRawMaterial.isFoodType()) {
             rawMatList.add(primaryRawMaterial);
         }
-        
+
         if (secondaryRawMaterial != null
                 && secondaryRawMaterial != lumberType
                 && secondaryRawMaterial != oreType
                 && !secondaryRawMaterial.isFoodType()) {
             rawMatList.add(secondaryRawMaterial);
         }
-        
+
         for(GoodsType rawMat : rawMatList){
             GoodsType producedGoods = rawMat.getProducedMaterial();
             if(producedGoods == null){
@@ -937,16 +970,16 @@ public class ColonyPlan {
             adjustProductionAndManufactureFor(rawMat,producedGoods);
         }
     }
-    
+
     public void adjustProductionAndManufactureFor(GoodsType rawMat, GoodsType producedGoods){
         Building factory = colony.getBuildingForProducing(producedGoods);
         if(factory == null){
             return;
         }
-        
+
         List<Unit> producers = new ArrayList<Unit>();
         int stockRawMat = colony.getGoodsCount(rawMat);
-       
+
         for(ColonyTile t : colony.getColonyTiles()){
             if(t.isColonyCenterTile()){
                 continue;
@@ -958,9 +991,9 @@ public class ColonyPlan {
             if(u.getWorkType() != rawMat){
                 continue;
             }
-            producers.add(u); 
+            producers.add(u);
         }
-        
+
         if(producers.size() == 0){
             return;
         }
@@ -971,7 +1004,7 @@ public class ColonyPlan {
                     GoodsType goodsType = u1.getWorkType();
                     int prodU1 = ((ColonyTile) u1.getLocation()).getProductionOf(u1, goodsType);
                     int prodU2 = ((ColonyTile) u2.getLocation()).getProductionOf(u2, goodsType);
-                    
+
                     if(prodU1 > prodU2){
                         return 1;
                     }
@@ -982,7 +1015,7 @@ public class ColonyPlan {
                 }
         };
         Collections.sort(producers, comp);
-        
+
         // shift units gathering raw materials to production of manufactured goods
         Iterator<Unit> iter = new ArrayList<Unit>(producers).iterator();
         while(iter.hasNext()){
@@ -990,7 +1023,7 @@ public class ColonyPlan {
             if(stockRawMat < 50 && producers.size() < 2){
                 return;
             }
-            
+
             if(factory.getUnitCount() == factory.getMaxUnits()){
                 return;
             }
@@ -1000,33 +1033,36 @@ public class ColonyPlan {
                 continue;
             }
 
-            // get  the production values if the unit is shifted 
-            int rawProd = colony.getProductionNextTurn(rawMat) - ((ColonyTile)u.getWorkTile()).getProductionOf(u, rawMat);
-            int mfnProd = colony.getProductionNextTurn(producedGoods) + factory.getAdditionalProductionNextTurn(u);
+            // get  the production values if the unit is shifted
+            int rawProd = colony.getNetProductionOf(rawMat)
+                - ((ColonyTile)u.getWorkTile()).getProductionOf(u, rawMat);
+            int mfnProd = colony.getNetProductionOf(producedGoods)
+                + factory.getAdditionalProductionNextTurn(u);
             if(stockRawMat < 50 && rawProd < mfnProd){
                 return;
             }
-            
-            u.work(factory);
+
+            u.setLocation(factory);
             u.setWorkType(producedGoods);
             producers.remove(u);
         }
     }
-    
+
     public GoodsType getBuildingReqGoods(){
         BuildableType currBuild = colony.getCurrentlyBuilding();
         if(currBuild == null){
             return null;
         }
-        
+        final GoodsType hammersType = colony.getSpecification().getGoodsType("model.goods.hammers");
+        final GoodsType toolsType = colony.getSpecification().getGoodsType("model.goods.tools");
+
         if(colony.getGoodsCount(hammersType) < currBuild.getAmountRequiredOf(hammersType)){
             return hammersType;
-        }
-        else{
+        } else{
             return toolsType;
         }
     }
-    
+
     public GoodsType getPrimaryRawMaterial(){
         return primaryRawMaterial;
     }
@@ -1034,10 +1070,10 @@ public class ColonyPlan {
     public GoodsType getSecondaryRawMaterial(){
         return secondaryRawMaterial;
     }
-    
+
     /**
      * Gets the <code>Colony</code> this <code>ColonyPlan</code> controls.
-     * 
+     *
      * @return The <code>Colony</code>.
      */
     public Colony getColony() {
@@ -1045,19 +1081,22 @@ public class ColonyPlan {
     }
 
     private void selectProfile() {
+        List<GoodsType> preferredProduction = new ArrayList<GoodsType>();
+        preferredProduction.addAll(getGame().getSpecification().getFoodGoodsTypeList());
+        preferredProduction.addAll(getGame().getSpecification().getLibertyGoodsTypeList());
+        ProfileType type = ProfileType.SMALL;
         int size = colony.getUnitCount();
-        if (size < 4) {
-            profile.setType(ProfileType.SMALL);
-        } else if (size > 8) {
-            profile.setType(ProfileType.LARGE);
+        if (size > 8) {
+            type = ProfileType.LARGE;
         } else if (size > 12) {
-            profile.setType(ProfileType.CAPITAL);
+            type = ProfileType.CAPITAL;
         }
+        profile = new ColonyProfile(type, preferredProduction);
     }
 
     /**
      * Creates an XML-representation of this object.
-     * 
+     *
      * @param document The <code>Document</code> in which the
      *            XML-representation should be created.
      * @return The XML-representation.
@@ -1073,7 +1112,7 @@ public class ColonyPlan {
     /**
      * Updates this object from an XML-representation of a
      * <code>ColonyPlan</code>.
-     * 
+     *
      * @param element The XML-representation.
      */
     public void readFromXMLElement(Element element) {
@@ -1084,13 +1123,13 @@ public class ColonyPlan {
 
     /**
      * Returns the tag name of the root element representing this object.
-     * 
+     *
      * @return "colonyPlan"
      */
     public static String getXMLElementTagName() {
         return "colonyPlan";
     }
-    
+
     /**
      * Creates a <code>String</code> representation of this plan.
      */
@@ -1101,18 +1140,18 @@ public class ColonyPlan {
         sb.append(profile.getType().toString());
         sb.append("\n");
         for (GoodsType goodsType : profile.getPreferredProduction()) {
-            sb.append(goodsType);
+            sb.append(goodsType.getNameKey());
             sb.append("\n");
         }
         sb.append("\n\nWORK LOCATIONS:\n");
         for (WorkLocationPlan p : getSortedWorkLocationPlans()) {
-            sb.append(p.getGoodsType() + " (" + p.getWorkLocation() + ")\n");
+            sb.append(p.getGoodsType().getNameKey() + " (" + p.getWorkLocation() + ")\n");
         }
         sb.append("\n\nBUILD QUEUE:\n");
         final Iterator<BuildableType> it = getBuildable();
         while (it.hasNext()) {
             final BuildableType b = it.next();
-            sb.append(b);
+            sb.append(b.getNameKey());
             sb.append('\n');
         }
         return sb.toString();
