@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -19,7 +19,6 @@
 
 package net.sf.freecol.server.ai;
 
-import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Comparator;
@@ -29,25 +28,23 @@ import java.util.logging.Logger;
 import net.sf.freecol.common.model.Building;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.ColonyTile;
+import net.sf.freecol.common.model.EquipmentType;
 import net.sf.freecol.common.model.Europe;
 import net.sf.freecol.common.model.FreeColGameObject;
 import net.sf.freecol.common.model.Goods;
+import net.sf.freecol.common.model.GoodsType;
 import net.sf.freecol.common.model.Location;
 import net.sf.freecol.common.model.Map;
 import net.sf.freecol.common.model.Ownable;
 import net.sf.freecol.common.model.PathNode;
 import net.sf.freecol.common.model.Player;
-import net.sf.freecol.common.model.Player.Stance;
 import net.sf.freecol.common.model.Settlement;
 import net.sf.freecol.common.model.Tension;
 import net.sf.freecol.common.model.Tile;
-import net.sf.freecol.common.model.TileImprovement;
 import net.sf.freecol.common.model.Unit;
 import net.sf.freecol.common.model.Unit.UnitState;
-import net.sf.freecol.common.model.pathfinding.GoalDecider;
 import net.sf.freecol.common.model.UnitType;
-import net.sf.freecol.common.model.Map.Position;
-import net.sf.freecol.common.networking.Message;
+import net.sf.freecol.common.model.pathfinding.GoalDecider;
 import net.sf.freecol.common.networking.NetworkConstants;
 import net.sf.freecol.server.ai.mission.DefendSettlementMission;
 import net.sf.freecol.server.ai.mission.Mission;
@@ -56,8 +53,6 @@ import net.sf.freecol.server.ai.mission.UnitSeekAndDestroyMission;
 import net.sf.freecol.server.ai.mission.UnitWanderHostileMission;
 import net.sf.freecol.server.ai.mission.WorkInsideColonyMission;
 
-import org.w3c.dom.Element;
-
 /**
  *
  * Objects of this class contains AI-information for a single {@link Player} and
@@ -73,50 +68,23 @@ public abstract class EuropeanAIPlayer extends NewAIPlayer {
 
     private static final Logger logger = Logger.getLogger(EuropeanAIPlayer.class.getName());
 
-    /**
-     * Checks if this player has a single Man-of-War.
-     * @return <code>true</code> if this player owns
-     *      a single Man-of-War.
-     */
-    public boolean hasManOfWar() {
-        Iterator<Unit> it = getPlayer().getUnitIterator();
-        while (it.hasNext()) {
-            Unit unit = it.next();
-            if ("model.unit.manOWar".equals(unit.getType().getId())) {
-                return true;
-            }
-        }
-        return false;
-    }
-
     /**
      * Ask the server to train a unit in Europe on behalf of the AIPlayer
      */
     public AIUnit trainAIUnitInEurope(UnitType unitType) {
-
         if (unitType==null) {
             throw new IllegalArgumentException("Invalid UnitType.");
         }
 
-        AIUnit unit = null;
-        try {
-            Element trainUnitInEuropeElement = Message.createNewRootElement("trainUnitInEurope");
-            trainUnitInEuropeElement.setAttribute("unitType", unitType.getId());
-            Element reply = this.getConnection().ask(trainUnitInEuropeElement);
-            if (reply!=null && reply.getTagName().equals("trainUnitInEuropeConfirmed")) {
-                Element unitElement = (Element) reply.getChildNodes().item(0);
-                String unitID = unitElement.getAttribute("ID");
-                unit = (AIUnit) getAIMain().getAIObject(unitID);
-                if (unit==null) {
-                    logger.warning("Could not train the specified AI unit "+unitType.getId()+" in europe.");
-                }
-            } else {
-                logger.warning("Could not train the specified AI unit "+unitType.getId()+" in europe.");
-            }
-        } catch (IOException e) {
-            logger.warning("Could not send \"trainUnitInEurope\"-message to the server.");
+        AIUnit aiUnit = null;
+        Europe europe = getPlayer().getEurope();
+        int n = europe.getUnitCount();
+
+        if (AIMessage.askTrainUnitInEurope(getConnection(), unitType)
+            && europe.getUnitCount() == n+1) {
+            aiUnit = getAIUnit(europe.getUnitList().get(n));
         }
-        return unit;
+        return aiUnit;
     }
 
 
@@ -124,27 +92,15 @@ public abstract class EuropeanAIPlayer extends NewAIPlayer {
      * Ask the server to recruit a unit in Europe on behalf of the AIPlayer
      */
     public AIUnit recruitAIUnitInEurope(int slot) {
+        AIUnit aiUnit = null;
+        Europe europe = getPlayer().getEurope();
+        int n = europe.getUnitCount();
 
-        AIUnit unit = null;
-        Element recruitUnitInEuropeElement = Message.createNewRootElement("recruitUnitInEurope");
-        recruitUnitInEuropeElement.setAttribute("slot", Integer.toString(slot));
-        try {
-            Element reply = this.getConnection().ask(recruitUnitInEuropeElement);
-            if (reply!=null && reply.getTagName().equals("recruitUnitInEuropeConfirmed")) {
-                Element unitElement = (Element) reply.getChildNodes().item(0);
-                String unitID = unitElement.getAttribute("ID");
-                unit = (AIUnit) getAIMain().getAIObject(unitID);
-                if (unit==null) {
-                    logger.warning("Could not recruit the specified AI unit in europe");
-                }
-                return unit;
-            } else {
-                logger.warning("Could not recruit the specified AI unit in europe.");
-            }
-        } catch (IOException e) {
-            logger.warning("Could not send \"recruitUnitInEurope\"-message to the server.");
+        if (AIMessage.askEmigrate(getConnection(), slot)
+            && europe.getUnitCount() == n+1) {
+            aiUnit = getAIUnit(europe.getUnitList().get(n));
         }
-        return unit;
+        return aiUnit;
     }
 
     /**
@@ -158,7 +114,7 @@ public abstract class EuropeanAIPlayer extends NewAIPlayer {
             AIUnit au = it.next();
             if (!au.hasMission()
                 && (au.getUnit().getLocation() instanceof ColonyTile || au.getUnit().getLocation() instanceof Building)) {
-                AIColony ac = (AIColony) getAIMain().getAIObject(au.getUnit().getColony());
+                AIColony ac = getAIMain().getAIColony(au.getUnit().getColony());
                 au.setMission(new WorkInsideColonyMission(getAIMain(), au, ac));
             }
         }
@@ -203,16 +159,6 @@ public abstract class EuropeanAIPlayer extends NewAIPlayer {
             }
 
             c.rearrangeWorkers(getConnection());
-
-            ArrayList<Tile> tilesToUpdate = new ArrayList<Tile>();
-            for (ColonyTile colonyTile : c.getColony().getColonyTiles()) {
-                boolean isOccupied = colonyTile.getUnit() != null;
-                boolean wasOccupied = oldWorkTiles.remove(colonyTile.getWorkTile());
-                if (isOccupied != wasOccupied) {
-                    tilesToUpdate.add(colonyTile.getWorkTile());
-                }
-            }
-            sendUpdatedTilesToAll(tilesToUpdate);
         }
     }
 
@@ -236,7 +182,6 @@ public abstract class EuropeanAIPlayer extends NewAIPlayer {
      * Takes the necessary actions to secure a european colony
      */
     protected void secureColony(Colony colony) {
-        Map map = getPlayer().getGame().getMap();
         int olddefenders = 0;
         int defenders = 0;
         int threat = 0;
@@ -248,9 +193,8 @@ public abstract class EuropeanAIPlayer extends NewAIPlayer {
                 defenders++;
             }
         }
-        Iterator<Position> positionIterator = map.getCircleIterator(colony.getTile().getPosition(), true, 5);
-        while (positionIterator.hasNext()) {
-            Tile t = map.getTile(positionIterator.next());
+
+        for (Tile t: colony.getTile().getSurroundingTiles(5)) {
             if (t.getFirstUnit() != null) {
                 if (t.getFirstUnit().getOwner() == getPlayer()) {
                     Iterator<Unit> uit = t.getUnitIterator();
@@ -328,6 +272,8 @@ public abstract class EuropeanAIPlayer extends NewAIPlayer {
                 recruitCount = inColonyCount - 1;
             }
             // Actually go through and arm our people.
+            EquipmentType muskets = getGame().getSpecification().getEquipmentType("model.equipment.muskets");
+            EquipmentType horses = getGame().getSpecification().getEquipmentType("model.equipment.horses");
             boolean needMuskets = false;
             boolean needHorses = false;
             ui = recruits.iterator();
@@ -335,30 +281,15 @@ public abstract class EuropeanAIPlayer extends NewAIPlayer {
                 Unit u = (ui.next());
                 if (!u.isArmed() && u.canBeEquippedWith(muskets)) {
                     recruitCount--;
-                    Element equipUnitElement = Message.createNewRootElement("equipUnit");
-                    equipUnitElement.setAttribute("unit", u.getId());
-                    equipUnitElement.setAttribute("type", muskets.getId());
-                    equipUnitElement.setAttribute("amount", "1");
-                    u.equipWith(muskets);
-                    sendAndWaitSafely(equipUnitElement);
-                    Element putOutsideColonyElement = Message.createNewRootElement("putOutsideColony");
-                    putOutsideColonyElement.setAttribute("unit", u.getId());
-                    u.putOutsideColony();
-                    sendAndWaitSafely(putOutsideColonyElement);
+                    AIMessage.askEquipUnit(getAIUnit(u), muskets, 1);
+                    AIMessage.askPutOutsideColony(getAIUnit(u));
                     // Check if the unit can fortify before sending the order
                     if (u.checkSetState(UnitState.FORTIFYING)) {
-                        Element changeStateElement = Message.createNewRootElement("changeState");
-                        changeStateElement.setAttribute("unit", u.getId());
-                        changeStateElement.setAttribute("state", UnitState.FORTIFYING.toString());
-                        sendAndWaitSafely(changeStateElement);
+                        AIMessage.askChangeState(getAIUnit(u), UnitState.FORTIFYING);
                     }
                     olddefenders++;
                     if (!u.isMounted() && u.canBeEquippedWith(horses)) {
-                        equipUnitElement = Message.createNewRootElement("equipUnit");
-                        equipUnitElement.setAttribute("unit", u.getId());
-                        equipUnitElement.setAttribute("type", horses.getId());
-                        equipUnitElement.setAttribute("amount", "1");
-                        sendAndWaitSafely(equipUnitElement);
+                        AIMessage.askEquipUnit(getAIUnit(u), horses, 1);
                     } else {
                         needHorses = true;
                     }
@@ -378,6 +309,8 @@ public abstract class EuropeanAIPlayer extends NewAIPlayer {
                     }
                 }
             }
+            GoodsType musketType = getAIMain().getGame().getSpecification().getGoodsType("model.goods.muskets");
+            GoodsType horsesType = getAIMain().getGame().getSpecification().getGoodsType("model.goods.horses");
             if (needMuskets && ac != null) {
                 // Check and see if we have already made a GoodsWish for
                 // here.
@@ -389,15 +322,14 @@ public abstract class EuropeanAIPlayer extends NewAIPlayer {
                         continue;
                     }
                     GoodsWish gw = (GoodsWish) w;
-                    if (gw.getGoodsType() == Goods.MUSKETS) {
+                    if (gw.getGoodsType() == musketType) {
                         made = true;
                     }
                 }
                 if (made == false) {
                     // Add a new GoodsWish onto the stack.
-                    ac
-                        .addGoodsWish(new GoodsWish(getAIMain(), colony, (threat - olddefenders) * 50,
-                                                    Goods.MUSKETS));
+                    ac.addGoodsWish(new GoodsWish(getAIMain(), colony, (threat - olddefenders) * 50,
+                                                  musketType));
                 }
             }
             if (needHorses && ac != null) {
@@ -411,13 +343,13 @@ public abstract class EuropeanAIPlayer extends NewAIPlayer {
                         continue;
                     }
                     GoodsWish gw = (GoodsWish) w;
-                    if (gw.getGoodsType() == Goods.HORSES) {
+                    if (gw.getGoodsType() == horsesType) {
                         made = true;
                     }
                 }
                 if (made == false) {
                     // Add a new GoodsWish onto the stack.
-                    ac.addGoodsWish(new GoodsWish(getAIMain(), colony, (threat - defenders) * 50, Goods.HORSES));
+                    ac.addGoodsWish(new GoodsWish(getAIMain(), colony, (threat - defenders) * 50, horsesType));
                 }
             }
             defenders = olddefenders;
@@ -534,10 +466,12 @@ public abstract class EuropeanAIPlayer extends NewAIPlayer {
         // Determine starting tile:
         Tile startTile = unit.getTile();
         if (startTile == null) {
-            if (unit.isOnCarrier()) {
-                startTile = (Tile) ((Unit) unit.getLocation()).getEntryLocation();
-            } else {
-                startTile = (Tile) unit.getOwner().getEntryLocation();
+            startTile = ((unit.isOnCarrier()) ? ((Unit) unit.getLocation())
+                         : unit).getFullEntryLocation();
+            if (startTile == null) {
+                logger.warning("Unable to determine entry location for: "
+                               + unit.toString());
+                return;
             }
         }
         /*
@@ -683,7 +617,7 @@ public abstract class EuropeanAIPlayer extends NewAIPlayer {
      * <code>TileImprovement</code>s needed by all of this player's colonies.
      *
      * @return The <code>Iterator</code>.
-     * @see TileImprovement
+     * @see net.sf.freecol.common.model.TileImprovement
      */
     public Iterator<TileImprovementPlan> getTileImprovementPlanIterator() {
         ArrayList<TileImprovementPlan> tileImprovements = new ArrayList<TileImprovementPlan>();
@@ -707,7 +641,6 @@ public abstract class EuropeanAIPlayer extends NewAIPlayer {
      * @return <code>true</code> if the AI should build more colonies.
      */
     public boolean hasFewColonies() {
-        logger.finest("Entering method hasFewColonies");
         if (!getPlayer().canBuildColonies()) {
             return false;
         }
@@ -718,9 +651,11 @@ public abstract class EuropeanAIPlayer extends NewAIPlayer {
             numberOfWorkers += colony.getUnitCount();
         }
 
-        logger.finest("Leaving method hasFewColonies");
-        return numberOfColonies <= 2 || numberOfColonies >= 3
-            && numberOfWorkers / numberOfColonies > numberOfColonies - 2;
+        boolean result = numberOfColonies <= 2
+            || (numberOfColonies >= 3
+                && numberOfWorkers / numberOfColonies > numberOfColonies - 2);
+        logger.finest("hasFewColonies = " + result);
+        return result;
     }
 
     /**
@@ -788,8 +723,7 @@ public abstract class EuropeanAIPlayer extends NewAIPlayer {
             Transportable t = ti.next();
             t.increaseTransportPriority();
             if (t.getTransportLocatable().getLocation() instanceof Unit) {
-                Mission m = ((AIUnit) getAIMain().getAIObject(
-                                                              (FreeColGameObject) t.getTransportLocatable().getLocation())).getMission();
+                Mission m = getAIMain().getAIUnit((Unit) t.getTransportLocatable().getLocation()).getMission();
                 if (m instanceof TransportMission) {
                     ((TransportMission) m).addToTransportList(t);
                 }
@@ -846,7 +780,7 @@ public abstract class EuropeanAIPlayer extends NewAIPlayer {
     }
 
     /**
-     * Called when another <code>Player</code> proposes a trade.
+     * Called when another <code>Player</code> proposes a sale.
      *
      *
      * @param unit The foreign <code>Unit</code> trying to trade.
@@ -857,12 +791,12 @@ public abstract class EuropeanAIPlayer extends NewAIPlayer {
      * @return The price this <code>AIPlayer</code> suggests or
      *         {@link NetworkConstants#NO_TRADE}.
      */
-    public int tradeProposition(Unit unit, Settlement settlement, Goods goods, int gold) {
-        logger.finest("Entering method tradeProposition");
+    public int sellProposition(Unit unit, Settlement settlement, Goods goods, int gold) {
+        logger.finest("Entering method sellProposition");
         Colony colony = (Colony) settlement;
         Player otherPlayer = unit.getOwner();
         // the client should have prevented this
-        if (getPlayer().getStance(otherPlayer) == Stance.WAR) {
+        if (getPlayer().atWarWith(otherPlayer)) {
             return NetworkConstants.NO_TRADE;
         }
         // don't pay for more than fits in the warehouse
@@ -898,7 +832,7 @@ public abstract class EuropeanAIPlayer extends NewAIPlayer {
     public Iterator<AIColony> getAIColonyIterator() {
         ArrayList<AIColony> ac = new ArrayList<AIColony>();
         for (Colony colony : getPlayer().getColonies()) {
-            AIColony a = (AIColony) getAIMain().getAIObject(colony.getId());
+            AIColony a = getAIColony(colony);
             if (a != null) {
                 ac.add(a);
             } else {
@@ -934,4 +868,4 @@ public abstract class EuropeanAIPlayer extends NewAIPlayer {
 
 
 
-}
\ No newline at end of file
+}
