@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -19,32 +19,30 @@
 
 package net.sf.freecol.server.ai;
 
-import java.io.IOException;
 import java.io.PrintWriter;
 import java.io.StringWriter;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Iterator;
-import java.util.logging.Level;
+import java.util.List;
 import java.util.logging.Logger;
 
 import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
-import net.sf.freecol.FreeCol;
+import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.CombatModel;
-import net.sf.freecol.common.model.EquipmentType;
 import net.sf.freecol.common.model.Goods;
 import net.sf.freecol.common.model.IndianSettlement;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Player.PlayerType;
 import net.sf.freecol.common.model.Player.Stance;
+import net.sf.freecol.common.model.Settlement;
 import net.sf.freecol.common.model.Tension;
 import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.Unit;
 import net.sf.freecol.common.networking.Connection;
-import net.sf.freecol.common.networking.Message;
 import net.sf.freecol.common.networking.NetworkConstants;
 import net.sf.freecol.server.ai.mission.UnitWanderHostileMission;
 import net.sf.freecol.server.ai.mission.UnitWanderMission;
@@ -68,10 +66,6 @@ public abstract class NewAIPlayer extends AIObject {
 
     private static final Logger logger = Logger.getLogger(AIPlayer.class.getName());
 
-    protected static EquipmentType muskets = FreeCol.getSpecification().getEquipmentType("model.equipment.muskets");
-    protected static EquipmentType horses = FreeCol.getSpecification().getEquipmentType("model.equipment.horses");
-    protected static EquipmentType toolsType = FreeCol.getSpecification().getEquipmentType("model.equipment.tools");
-
     /*
      * Stores temporary information for sessions (trading with another player
      * etc).
@@ -130,6 +124,118 @@ public abstract class NewAIPlayer extends AIObject {
         readFromXML(in);
     }
 
+    /**
+     * Returns the <code>Player</code> this <code>AIPlayer</code> is
+     * controlling.
+     *
+     * @return The <code>Player</code>.
+     */
+    public Player getPlayer() {
+        return player;
+    }
+
+    /**
+     * Returns the ID for this <code>AIPlayer</code>. This is the same as the
+     * ID for the {@link Player} this <code>AIPlayer</code> controls.
+     *
+     * @return The ID.
+     */
+    @Override
+    public String getId() {
+        return player.getId();
+    }
+
+    /**
+     * Gets the connection to the server.
+     *
+     * @return The connection that can be used when communication with the
+     *         server.
+     */
+    public Connection getConnection() {
+        if (debuggingConnection != null) {
+            return debuggingConnection;
+        } else {
+            return ((DummyConnection) player.getConnection()).getOtherConnection();
+        }
+    }
+
+    /**
+     * Sets the <code>Connection</code> to be used while communicating with
+     * the server.
+     *
+     * This method is only used for debugging.
+     *
+     * @param debuggingConnection The connection to be used for debugging.
+     */
+    public void setDebuggingConnection(Connection debuggingConnection) {
+        this.debuggingConnection = debuggingConnection;
+    }
+
+    /**
+     * Clears the cache of AI units.
+     */
+    protected void clearAIUnits() {
+        aiUnits.clear();
+    }
+
+    /**
+     * Build the cache of AI units.
+     */
+    private void createAIUnits() {
+        clearAIUnits();
+        for (Unit u : player.getUnits()) {
+            AIUnit a = getAIUnit(u);
+            if (a != null) {
+                aiUnits.add(a);
+            } else {
+                logger.warning("Could not find the AIUnit for: "
+                               + u + " (" + u.getId() + ")");
+            }
+        }
+    }
+
+    /**
+     * Gets a list of all the AIUnits owned by this player.
+     *
+     * @return A list of all the AIUnits owned by this player.
+     */
+    protected List<AIUnit> getAIUnits() {
+        if (aiUnits.size() == 0) createAIUnits();
+        return new ArrayList<AIUnit>(aiUnits);
+    }
+
+    /**
+     * Returns an iterator over all the <code>AIUnit</code>s owned by this
+     * player.
+     *
+     * @return The <code>Iterator</code>.
+     */
+    protected Iterator<AIUnit> getAIUnitIterator() {
+        if (aiUnits.size() == 0) createAIUnits();
+        return aiUnits.iterator();
+    }
+
+    /**
+     * Gets the AI unit corresponding to a given unit, if any.
+     *
+     * @param unit The <code>Unit</code> to look up.
+     * @return The corresponding AI unit or null if not found.
+     */
+    protected AIUnit getAIUnit(Unit unit) {
+        return getAIMain().getAIUnit(unit);
+    }
+
+    /**
+     * Gets the AI colony corresponding to a given colony, if any.
+     *
+     * @param colony The <code>Colony</code> to look up.
+     * @return The corresponding AI colony or null if not found.
+     */
+    protected AIColony getAIColony(Colony colony) {
+        return getAIMain().getAIColony(colony);
+    }
+
+
     /**
      *
      * Tells this <code>AIPlayer</code> to make decisions. The
@@ -140,30 +246,22 @@ public abstract class NewAIPlayer extends AIObject {
 
     /**
      * Determines the stances towards each player.
-     *
-     * That is: should we declare war?
      */
     protected void determineStances() {
         logger.finest("Entering method determineStances");
         Player player = getPlayer();
         for (Player p : getGame().getPlayers()) {
-            if (p != player) {
-                Stance stance = getPlayer().getStance(p);
-                Tension tension = getPlayer().getTension(p);
-                if (stance != Stance.UNCONTACTED && tension != null) {
-                    if (p.getREFPlayer() == getPlayer() && p.getPlayerType() == PlayerType.REBEL) {
-                        tension.modify(1000);
-                    }
-                    if (stance != Stance.WAR &&
-                        tension.getLevel() == Tension.Level.HATEFUL) {
-                        getPlayer().changeRelationWithPlayer(p, Stance.WAR);
-                    } else if (stance == Stance.WAR
-                               && tension.getLevel().compareTo(Tension.Level.CONTENT) <= 0) {
-                        getPlayer().changeRelationWithPlayer(p, Stance.CEASE_FIRE);
-                    } else if (stance == Stance.CEASE_FIRE
-                               && tension.getLevel().compareTo(Tension.Level.HAPPY) <= 0) {
-                        getPlayer().changeRelationWithPlayer(p, Stance.PEACE);
-                    }
+            if (p != player && !p.isDead()) {
+                Stance newStance;
+                if (p.getREFPlayer() == player
+                    && p.getPlayerType() == PlayerType.REBEL) {
+                    newStance = Stance.WAR;
+                } else {
+                    newStance = player.getStance(p).getStanceFromTension(player.getTension(p));
+                }
+                if (newStance != player.getStance(p)) {
+                    getAIMain().getFreeColServer().getInGameController()
+                        .changeStance(player, newStance, p, true);
                 }
             }
         }
@@ -207,58 +305,6 @@ public abstract class NewAIPlayer extends AIObject {
         }
     }
 
-    /**
-     * Send an element and ignore IO exceptions. This was used all over the
-     * place, no better use a single method.
-     *
-     * @param element The element.
-     */
-    protected void sendAndWaitSafely(Element element) {
-        logger.finest("Entering method sendAndWaitSafely");
-        try {
-            if (logger.isLoggable(Level.FINER)) {
-                logger.finer("AI player (" + this + ") sending " + element.getTagName() + "...");
-            }
-            getConnection().sendAndWait(element);
-            if (logger.isLoggable(Level.FINER)) {
-                logger.finer("Sent and waited, returning.");
-            }
-        } catch (IOException e) {
-            logger.log(Level.WARNING, "Couldn't send AI element " + element.getTagName() + "!", e);
-        }
-    }
-
-    /**
-     * Send some tiles to update to all players which can see them
-     *
-     * @param tiles The tiles to update.
-     */
-    protected void sendUpdatedTilesToAll(ArrayList<Tile> tiles) {
-        Iterator<Player> enemyPlayerIterator = getGame().getPlayerIterator();
-        while (enemyPlayerIterator.hasNext()) {
-            ServerPlayer enemyPlayer = (ServerPlayer) enemyPlayerIterator.next();
-            if (equals(enemyPlayer) || enemyPlayer.getConnection() == null) {
-                continue;
-            }
-            try {
-                Element updateElement = Message.createNewRootElement("update");
-                boolean send = false;
-                for(Tile tile : tiles) {
-                    if (enemyPlayer.canSee(tile)) {
-                        updateElement.appendChild(tile.toXMLElement(enemyPlayer, updateElement.getOwnerDocument()));
-                        send = true;
-                    }
-                }
-                if (send) {
-                    enemyPlayer.getConnection().send(updateElement);
-                }
-            } catch (IOException e) {
-                logger.warning("Could not send message to: " + enemyPlayer.getName() + " with connection "
-                               + enemyPlayer.getConnection());
-            }
-        }
-    }
-
     /**
      *
      * Makes every unit perform their mission.
@@ -383,7 +429,7 @@ public abstract class NewAIPlayer extends AIObject {
             return false;
         }
 
-        boolean notAtWar = attackerPlayer.getStance(defenderPlayer) != Stance.WAR;
+        boolean notAtWar = !attackerPlayer.atWarWith(defenderPlayer);
         // if european, can only attack units whose owners are at war
         if(attackerPlayer.isEuropean() && notAtWar){
             return false;
@@ -400,79 +446,6 @@ public abstract class NewAIPlayer extends AIObject {
         return true;
     }
 
-    /**
-     * Returns an iterator over all the <code>AIUnit</code>s owned by this
-     * player.
-     *
-     * @return The <code>Iterator</code>.
-     */
-    public Iterator<AIUnit> getAIUnitIterator() {
-        if (aiUnits.size() == 0) {
-            ArrayList<AIUnit> au = new ArrayList<AIUnit>();
-            Iterator<Unit> unitsIterator = player.getUnitIterator();
-            while (unitsIterator.hasNext()) {
-                Unit theUnit = unitsIterator.next();
-                AIUnit a = (AIUnit) getAIMain().getAIObject(theUnit.getId());
-                if (a != null) {
-                    au.add(a);
-                } else {
-                    logger.warning("Could not find the AIUnit for: " + theUnit + " (" + theUnit.getId() + ") - "
-                                   + (getGame().getFreeColGameObject(theUnit.getId()) != null));
-                }
-            }
-            aiUnits = au;
-        }
-        return aiUnits.iterator();
-    }
-
-    /**
-     * Returns the <code>Player</code> this <code>AIPlayer</code> is
-     * controlling.
-     *
-     * @return The <code>Player</code>.
-     */
-    public Player getPlayer() {
-        return player;
-    }
-
-    /**
-     * Gets the connection to the server.
-     *
-     * @return The connection that can be used when communication with the
-     *         server.
-     */
-    public Connection getConnection() {
-        if (debuggingConnection != null) {
-            return debuggingConnection;
-        } else {
-            return ((DummyConnection) player.getConnection()).getOtherConnection();
-        }
-    }
-
-    /**
-     *
-     * Sets the <code>Connection</code> to be used while communicating with
-     * the server.
-     *
-     * This method is only used for debugging.
-     *
-     * @param debuggingConnection The connection to be used for debugging.
-     */
-    public void setDebuggingConnection(Connection debuggingConnection) {
-        this.debuggingConnection = debuggingConnection;
-    }
-
-    /**
-     * Returns the ID for this <code>AIPlayer</code>. This is the same as the
-     * ID for the {@link Player} this <code>AIPlayer</code> controls.
-     *
-     * @return The ID.
-     */
-    @Override
-    public String getId() {
-        return player.getId();
-    }
-
     /**
      * Writes this object to an XML stream.
      *
@@ -517,31 +490,32 @@ public abstract class NewAIPlayer extends AIObject {
      * @param goods The goods which we are going to offer
      */
     public void registerSellGoods(Goods goods) {
-        String goldKey = "tradeGold#" + goods.getType().getIndex() + "#" + goods.getAmount()
+        String goldKey = "tradeGold#" + goods.getType().getId() + "#" + goods.getAmount()
             + "#" + goods.getLocation().getId();
         sessionRegister.put(goldKey, null);
     }
 
     /**
-     * Called when another <code>Player</code> proposes a trade.
+     * Called when another <code>Player</code> proposes to buy.
      *
      *
      * @param unit The foreign <code>Unit</code> trying to trade.
+     * @param settlement The <code>Settlement</code> this player owns and
+     *            which the given <code>Unit</code> is trading.
      * @param goods The goods the given <code>Unit</code> is trying to sell.
      * @param gold The suggested price.
      * @return The price this <code>AIPlayer</code> suggests or
      *         {@link NetworkConstants#NO_TRADE}.
      */
-    public int buyProposition(Unit unit, Goods goods, int gold) {
-        logger.finest("Entering method tradeProposition");
-        IndianSettlement settlement = (IndianSettlement) goods.getLocation();
-        String goldKey = "tradeGold#" + goods.getType().getIndex() + "#" + goods.getAmount()
+    public int buyProposition(Unit unit, Settlement settlement, Goods goods, int gold) {
+        logger.finest("Entering method buyProposition");
+        String goldKey = "tradeGold#" + goods.getType().getId() + "#" + goods.getAmount()
             + "#" + settlement.getId();
         String hagglingKey = "tradeHaggling#" + unit.getId();
 
         Integer registered = sessionRegister.get(goldKey);
         if (registered == null) {
-            int price = settlement.getPriceToSell(goods)
+            int price = ((IndianSettlement) settlement).getPriceToSell(goods)
                 + player.getTension(unit.getOwner()).getValue();
             sessionRegister.put(goldKey, new Integer(price));
             return price;
@@ -558,7 +532,7 @@ public abstract class NewAIPlayer extends AIObject {
                 if (sessionRegister.containsKey(hagglingKey)) {
                     haggling = sessionRegister.get(hagglingKey).intValue();
                 }
-                if (getRandom().nextInt(3 + haggling) <= 3) {
+                if (getAIRandom().nextInt(3 + haggling) <= 3) {
                     sessionRegister.put(goldKey, new Integer(gold));
                     sessionRegister.put(hagglingKey, new Integer(haggling + 1));
                     return gold;
@@ -569,8 +543,4 @@ public abstract class NewAIPlayer extends AIObject {
             }
         }
     }
-
-    protected void clearAIUnits() {
-        aiUnits.clear();
-    }
 }
