@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -20,7 +20,6 @@
 
 package net.sf.freecol.server.ai.mission;
 
-import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.List;
@@ -30,22 +29,20 @@ import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
-import net.sf.freecol.FreeCol;
 import net.sf.freecol.common.model.AbstractGoods;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.EquipmentType;
-import net.sf.freecol.common.model.PathNode;
-import net.sf.freecol.common.model.Tile;
-import net.sf.freecol.common.model.TileImprovement;
-import net.sf.freecol.common.model.TileItemContainer;
-import net.sf.freecol.common.model.Unit;
 import net.sf.freecol.common.model.Map.Direction;
+import net.sf.freecol.common.model.PathNode;
+import net.sf.freecol.common.model.Player;
+import net.sf.freecol.common.model.Tile;
+import net.sf.freecol.common.model.Unit;
 import net.sf.freecol.common.model.Unit.UnitState;
 import net.sf.freecol.common.networking.Connection;
-import net.sf.freecol.common.networking.Message;
+import net.sf.freecol.common.networking.NetworkConstants;
 import net.sf.freecol.server.ai.AIColony;
 import net.sf.freecol.server.ai.AIMain;
-import net.sf.freecol.server.ai.AIObject;
+import net.sf.freecol.server.ai.AIMessage;
 import net.sf.freecol.server.ai.AIPlayer;
 import net.sf.freecol.server.ai.AIUnit;
 import net.sf.freecol.server.ai.TileImprovementPlan;
@@ -55,39 +52,37 @@ import org.w3c.dom.Element;
 
 /**
  * Mission for controlling a pioneer.
- * 
+ *
  * @see net.sf.freecol.common.model.Unit.Role#PIONEER
  */
 public class PioneeringMission extends Mission {
-    /* 
+    /*
      * TODO-LATER: "updateTileImprovementPlan" should be called
      *             only once (in the beginning of the turn).
      */
-    
+
     private static final Logger logger = Logger.getLogger(PioneeringMission.class.getName());
 
-    private static final EquipmentType toolsType = FreeCol.getSpecification().getEquipmentType("model.equipment.tools");
-
     private static enum PioneeringMissionState {GET_TOOLS,IMPROVING};
-    
+
     private PioneeringMissionState state = PioneeringMissionState.GET_TOOLS;
-    
+
     private TileImprovementPlan tileImprovementPlan = null;
-    
+
     private Colony colonyWithTools = null;
 
     private boolean invalidateMission = false;
 
     /**
      * Creates a mission for the given <code>AIUnit</code>.
-     * 
+     *
      * @param aiMain The main AI-object.
      * @param aiUnit The <code>AIUnit</code> this mission
      *        is created for.
      */
     public PioneeringMission(AIMain aiMain, AIUnit aiUnit) {
         super(aiMain, aiUnit);
-        
+
         boolean hasTools = getUnit().hasAbility("model.ability.improveTerrain");
         if(hasTools){
             state = PioneeringMissionState.IMPROVING;
@@ -100,7 +95,7 @@ public class PioneeringMission extends Mission {
 
     /**
      * Loads a mission from the given element.
-     * 
+     *
      * @param aiMain The main AI-object.
      * @param element An <code>Element</code> containing an
      *      XML-representation of this object.
@@ -112,19 +107,19 @@ public class PioneeringMission extends Mission {
 
     /**
      * Creates a new <code>PioneeringMission</code> and reads the given element.
-     * 
+     *
      * @param aiMain The main AI-object.
      * @param in The input stream containing the XML.
      * @throws XMLStreamException if a problem was encountered
      *      during parsing.
-     * @see AIObject#readFromXML
+     * @see net.sf.freecol.server.ai.AIObject#readFromXML
      */
     public PioneeringMission(AIMain aiMain, XMLStreamReader in) throws XMLStreamException {
         super(aiMain);
         readFromXML(in);
     }
-    
-    
+
+
     /**
      * Disposes this <code>Mission</code>.
      */
@@ -139,7 +134,7 @@ public class PioneeringMission extends Mission {
     /**
      * Sets the <code>TileImprovementPlan</code> which should
      * be the next target.
-     * 
+     *
      * @param tileImprovementPlan The <code>TileImprovementPlan</code>.
      */
     public void setTileImprovementPlan(TileImprovementPlan tileImprovementPlan) {
@@ -147,9 +142,9 @@ public class PioneeringMission extends Mission {
     }
 
     private void updateTileImprovementPlan() {
-        final AIPlayer aiPlayer = (AIPlayer) getAIMain().getAIObject(getUnit().getOwner().getId());
+        final AIPlayer aiPlayer = getAIMain().getAIPlayer(getUnit().getOwner());
         final Unit carrier = (getUnit().isOnCarrier()) ? (Unit) getUnit().getLocation() : null;
-        
+
         Tile improvementTarget = (tileImprovementPlan != null)? tileImprovementPlan.getTarget():null;
         // invalid tileImprovementPlan, remove and get a new valid one
         if (tileImprovementPlan != null && improvementTarget == null) {
@@ -158,7 +153,7 @@ public class PioneeringMission extends Mission {
             tileImprovementPlan.dispose();
             tileImprovementPlan = null;
         }
-        
+
         // Verify if the improvement has been applied already
         // If it has, remove this improvement
         if (tileImprovementPlan != null &&
@@ -168,26 +163,29 @@ public class PioneeringMission extends Mission {
             tileImprovementPlan.dispose();
             tileImprovementPlan = null;
         }
-        
+
         // mission still valid, no update needed
         if (tileImprovementPlan != null && improvementTarget != null) {
             return;
         }
-        
+
         final Tile startTile;
         if (getUnit().getTile() == null) {
-            if (getUnit().isOnCarrier()) {
-                startTile = (Tile) ((Unit) getUnit().getLocation()).getEntryLocation();
-            } else {
-                startTile = (Tile) getUnit().getOwner().getEntryLocation();
+            startTile = ((getUnit().isOnCarrier())
+                         ? ((Unit) getUnit().getLocation())
+                         : getUnit()).getFullEntryLocation();
+            if (startTile == null) {
+                logger.warning("Unable to determine entry location for: "
+                               + getUnit().toString());
+                return;
             }
         } else {
             startTile = getUnit().getTile();
         }
-                
+
         TileImprovementPlan bestChoice = null;
         int bestValue = 0;
-        Iterator<TileImprovementPlan> tiIterator = aiPlayer.getTileImprovementPlanIterator();            
+        Iterator<TileImprovementPlan> tiIterator = aiPlayer.getTileImprovementPlanIterator();
         while (tiIterator.hasNext()) {
             TileImprovementPlan ti = tiIterator.next();
             if (ti.getPioneer() == null) {
@@ -198,16 +196,16 @@ public class PioneeringMission extends Mission {
                     ti.dispose();
                     continue;
                 }
-                
+
                 PathNode path = null;
                 int value;
                 if (startTile != ti.getTarget()) {
                     path = getGame().getMap().findPath(getUnit(), startTile, ti.getTarget(), carrier);
                     if (path != null) {
                         value = ti.getValue() + 10000 - (path.getTotalTurns()*5);
-                        
+
                         /*
-                         * Avoid picking a TileImprovementPlan with a path being blocked 
+                         * Avoid picking a TileImprovementPlan with a path being blocked
                          * by an enemy unit (apply a penalty to the value):
                          */
                         PathNode pn = path;
@@ -223,34 +221,34 @@ public class PioneeringMission extends Mission {
                     }
                 } else {
                     value = ti.getValue() + 10000;
-                }                
+                }
                 if (value > bestValue) {
                     bestChoice = ti;
                     bestValue = value;
                 }
             }
         }
-        
+
         if (bestChoice != null) {
             tileImprovementPlan = bestChoice;
             bestChoice.setPioneer(getAIUnit());
         }
-        
+
         if(tileImprovementPlan == null){
             invalidateMission = true;
         }
     }
-    
-    
+
+
     /**
      * Performs this mission.
      * @param connection The <code>Connection</code> to the server.
      */
     public void doMission(Connection connection) {
         logger.finest("Entering PioneeringMission.doMission()");
-        
+
         Unit unit = getUnit();
-        
+
         boolean hasTools = getUnit().hasAbility("model.ability.improveTerrain");
         if(unit.getState() == UnitState.IMPROVING || hasTools){
             state = PioneeringMissionState.IMPROVING;
@@ -258,19 +256,19 @@ public class PioneeringMission extends Mission {
         else{
             state = PioneeringMissionState.GET_TOOLS;
         }
-        
+
         while(isValid() && unit.getMovesLeft() > 0){
             switch(state){
-                case GET_TOOLS:
-                    getTools(connection);
-                    break;
-                case IMPROVING:
-                    processImprovementPlan(connection);
-                    break;
-                default:
-                    logger.warning("Unknown state");
-                    invalidateMission = true;
-            }        
+            case GET_TOOLS:
+                getTools(connection);
+                break;
+            case IMPROVING:
+                processImprovementPlan(connection);
+                break;
+            default:
+                logger.warning("Unknown state");
+                invalidateMission = true;
+            }
         }
     }
 
@@ -290,21 +288,20 @@ public class PioneeringMission extends Mission {
             invalidateMission = true;
             return;
         }
-                
+
         // move toward the target tile
         if (getUnit().getTile() != tileImprovementPlan.getTarget()) {
             PathNode pathToTarget = getUnit().findPath(tileImprovementPlan.getTarget());
             if (pathToTarget == null) {
                 invalidateMission = true;
-                return; 
+                return;
             }
 
-            Direction direction = moveTowards(connection, pathToTarget);
-            if (direction != null
-                    && unit.getMoveType(direction).isProgress()) {
-                move(connection, direction);
+            Direction direction = moveTowards(pathToTarget);
+            if (direction != null) {
+                if (!moveButDontAttack(direction)) return;
             }
-            
+
             if(unit.getTile() != tileImprovementPlan.getTarget()){
                 unit.setMovesLeft(0);
             }
@@ -319,63 +316,51 @@ public class PioneeringMission extends Mission {
             logger.warning(errMsg);
             invalidateMission = true;
             return;
-        }            
+        }
+
+        Tile target = tileImprovementPlan.getTarget();
+        Player player = getUnit().getOwner();
+        if (target.getOwner() != player) {
+            // Take control of land before proceeding with mission.
+            // Decide whether to pay or steal.
+            // Currently always pay if we can, steal if we can not.
+            int price = player.getLandPrice(target);
+            if (price < 0) {
+                ; // fail
+            } else {
+                if (price > 0 && !player.checkGold(price)) {
+                    price = NetworkConstants.STEAL_LAND;
+                }
+                AIMessage.askClaimLand(connection, target, null, price);
+            }
+        }
+        if (target.getOwner() != player) {
+            // Failed to take ownership
+            invalidateMission = true;
+            return;
+        }
+
         makeImprovement(connection);
     }
 
     private void makeImprovement(Connection connection) {
-        if(getUnit().getState() == UnitState.IMPROVING){
-            getUnit().setMovesLeft(0);
+        Unit unit = getUnit();
+
+        if (unit.getState() == UnitState.IMPROVING){
+            unit.setMovesLeft(0);
             return;
         }
-        
-        if (getUnit().checkSetState(UnitState.IMPROVING)) {
+
+        if (unit.checkSetState(UnitState.IMPROVING)) {
             // start improving now
-            int price = getUnit().getOwner().getLandPrice(getUnit().getTile());
+            int price = unit.getOwner().getLandPrice(unit.getTile());
             // Buy the land from the Indians first?
             if (price > 0) {
                 // TODO: the AI should buy the land, to avoid indian wars
             }
             // ask to create the TileImprovement
-            Element changeWorkTypeElement = Message.createNewRootElement("workImprovement");
-            changeWorkTypeElement.setAttribute("unit", getUnit().getId());
-            changeWorkTypeElement.setAttribute("improvementType", tileImprovementPlan.getType().getId());
-            Element reply = null;
-            try {
-                reply = connection.ask(changeWorkTypeElement);
-            } catch (IOException e) {
-                logger.warning("Could not send message!");
-            }
-            if (reply==null || !reply.getTagName().equals("workImprovementConfirmed")) {
-                throw new IllegalStateException("Failed to make improvement");
-            }
-            
-            // get the TileItemContainer
-            Element containerElement = (Element)reply.getElementsByTagName(TileItemContainer.getXMLElementTagName()).item(0);
-            if (containerElement != null) {
-                TileItemContainer container = (TileItemContainer) getGame().getFreeColGameObject(containerElement.getAttribute("ID"));
-                if (container == null) {
-                    container = new TileItemContainer(getGame(), getUnit().getTile(), containerElement);
-                    getUnit().getTile().setTileItemContainer(container);
-                } else {
-                    container.readFromXMLElement(containerElement);
-                }
-            }
-            
-            // get the TileImprovement
-            Element improvementElement = (Element)reply.getElementsByTagName(TileImprovement.getXMLElementTagName()).item(0);
-            if (improvementElement==null) {
-                throw new IllegalStateException("Failed to make improvement");
-            }
- 
-            TileImprovement improvement = (TileImprovement) getGame().getFreeColGameObject(improvementElement.getAttribute("ID"));
-            if (improvement == null) {
-                improvement = new TileImprovement(getGame(), improvementElement);
-                getUnit().getTile().add(improvement);
-            } else {
-                improvement.readFromXMLElement(improvementElement);
-            }
-            getUnit().work(improvement);
+            AIMessage.askChangeWorkImprovementType(getAIUnit(),
+                tileImprovementPlan.getType());
         }
     }
 
@@ -384,12 +369,12 @@ public class PioneeringMission extends Mission {
         if(invalidateMission){
             return;
         }
-          
+
         Unit unit = getUnit();
-        
+
         // Not there yet
         if(unit.getTile() != colonyWithTools.getTile()){
-            PathNode path = getGame().getMap().findPath(unit, unit.getTile(), colonyWithTools.getTile());         
+            PathNode path = unit.findPath(colonyWithTools.getTile());
 
             if(path == null){
                 invalidateMission = true;
@@ -397,10 +382,10 @@ public class PioneeringMission extends Mission {
                 return;
             }
 
-            Direction direction = moveTowards(connection, path);
-            moveButDontAttack(connection, direction);
+            Direction direction = moveTowards(path);
+            if (direction == null || !moveButDontAttack(direction)) return;
 
-            // not there yet, remove any moves left 
+            // not there yet, remove any moves left
             if(unit.getTile() != colonyWithTools.getTile()){
                 unit.setMovesLeft(0);
                 return;
@@ -414,7 +399,8 @@ public class PioneeringMission extends Mission {
     private void equipUnitWithTools(Connection connection) {
         Unit unit = getUnit();
         logger.finest("About to equip " + unit + " in " + colonyWithTools.getName());
-        AIColony ac = (AIColony) getAIMain().getAIObject(colonyWithTools);
+        AIColony ac = getAIMain().getAIColony(colonyWithTools);
+        EquipmentType toolsType = getAIMain().getGame().getSpecification().getEquipmentType("model.equipment.tools");
         int amount = toolsType.getMaximumCount();
         for (AbstractGoods materials : toolsType.getGoodsRequired()) {
             int availableAmount = ac.getAvailableGoods(materials.getType());
@@ -422,21 +408,13 @@ public class PioneeringMission extends Mission {
             if (availableAmount < requiredAmount) {
                 invalidateMission = true;
                 return;
-            } 
+            }
             amount = Math.min(amount, availableAmount / requiredAmount);
         }
 
         logger.finest("Equipping " + unit + " at=" + colonyWithTools.getName() + " amount=" + amount);
-        Element equipUnitElement = Message.createNewRootElement("equipUnit");
-        equipUnitElement.setAttribute("unit", unit.getId());
-        equipUnitElement.setAttribute("type", toolsType.getId());
-        equipUnitElement.setAttribute("amount", Integer.toString(amount));
-        try {
-            connection.sendAndWait(equipUnitElement);
-        } catch (Exception e) {
-            logger.warning("Could not send equip message.");
-        }
-        
+        AIMessage.askEquipUnit(getAIUnit(), toolsType, amount);
+
         // Unit is now equipped, get to work
         if(unit.getEquipmentCount(toolsType) > 0){
             state = PioneeringMissionState.IMPROVING;
@@ -445,10 +423,11 @@ public class PioneeringMission extends Mission {
 
 
     private boolean validateColonyWithTools() {
+        EquipmentType toolsType = getAIMain().getGame().getSpecification().getEquipmentType("model.equipment.tools");
         if(colonyWithTools != null){
-            if(colonyWithTools.isDisposed() 
-                    || colonyWithTools.getOwner() != getUnit().getOwner()
-                    || !colonyWithTools.canBuildEquipment(toolsType)){
+            if(colonyWithTools.isDisposed()
+               || colonyWithTools.getOwner() != getUnit().getOwner()
+               || !colonyWithTools.canBuildEquipment(toolsType)){
                 colonyWithTools = null;
             }
         }
@@ -460,7 +439,7 @@ public class PioneeringMission extends Mission {
                 invalidateMission = true;
                 return false;
             }
-            logger.finest("Colony found=" + colonyWithTools.getName());            
+            logger.finest("Colony found=" + colonyWithTools.getName());
         }
         return true;
     }
@@ -474,7 +453,7 @@ public class PioneeringMission extends Mission {
      * {@link TransportMission} in the latter case.
      *
      * @return The destination for this <code>Transportable</code>.
-     */    
+     */
     public Tile getTransportDestination() {
         updateTileImprovementPlan();
         if (tileImprovementPlan == null) {
@@ -505,48 +484,26 @@ public class PioneeringMission extends Mission {
         }
     }
 
-    /**
-     * Checks if this mission is still valid to perform.
-     *
-     * @return <code>true</code> if this mission is still valid to perform
-     *         and <code>false</code> otherwise.
-     */
-    public boolean isValid() {
-        if(getUnit().getTile() == null){
-            return false;
-        }
-        
-        switch(state){
-            case GET_TOOLS:
-                validateColonyWithTools();
-                break;
-            case IMPROVING:
-                updateTileImprovementPlan();
-                break;
-        }        
-        return !invalidateMission;
-    }
-
     /**
      * Checks if this mission is valid for the given unit.
      *
      * @param aiUnit The unit.
      * @return <code>true</code> if this mission is still valid to perform
      *         and <code>false</code> otherwise.
-     */    
+     */
     public static boolean isValid(AIUnit aiUnit) {
         if(!aiUnit.getUnit().isColonist()){
             return false;
         }
-        
+
         if(aiUnit.getUnit().getTile() == null){
             return false;
         }
-        
-        AIPlayer aiPlayer = (AIPlayer) aiUnit.getAIMain().getAIObject(aiUnit.getUnit().getOwner().getId());
-        Iterator<TileImprovementPlan> tiIterator = aiPlayer.getTileImprovementPlanIterator();            
-        
-        
+
+        AIPlayer aiPlayer = aiUnit.getAIMain().getAIPlayer(aiUnit.getUnit().getOwner());
+        Iterator<TileImprovementPlan> tiIterator = aiPlayer.getTileImprovementPlanIterator();
+
+
         boolean foundImprovementPlan = false;
         while (tiIterator.hasNext()) {
             TileImprovementPlan ti = tiIterator.next();
@@ -558,50 +515,54 @@ public class PioneeringMission extends Mission {
             logger.finest("No Improvement plan found, PioneeringMission not valid");
             return false;
         }
-        
+
+        EquipmentType toolsType = aiUnit.getAIMain().getGame().getSpecification()
+            .getEquipmentType("model.equipment.tools");
         boolean unitHasToolsAvail = aiUnit.getUnit().getEquipmentCount(toolsType) > 0;
         if(unitHasToolsAvail){
             logger.finest("Tools equipped, PioneeringMission valid");
-           return true; 
+            return true;
         }
-        
+
         // Search colony with tools to equip the unit with
         Colony colonyWithTools = findColonyWithTools(aiUnit);
         if(colonyWithTools != null){
             logger.finest("Tools found, PioneeringMission valid");
             return true;
         }
-        
+
         logger.finest("Tools not found, PioneeringMission not valid");
         return false;
     }
-    
+
     public static Colony findColonyWithTools(AIUnit aiu) {
         final int MAX_TURN_DISTANCE = 10;
         Colony best = null;
         int bestValue = Integer.MIN_VALUE;
-        
+
         Unit unit = aiu.getUnit();
         // Sanitation
         if(unit == null){
             return null;
         }
-        
+
+        EquipmentType toolsType = aiu.getAIMain().getGame().getSpecification()
+            .getEquipmentType("model.equipment.tools");
         for(Colony colony : unit.getOwner().getColonies()){
             if(!colony.canBuildEquipment(toolsType)) {
                 continue;
             }
 
-            AIColony ac = (AIColony) aiu.getAIMain().getAIObject(colony);
+            AIColony ac = aiu.getAIMain().getAIColony(colony);
             // Sanitation
             if(ac == null){
                 continue;
             }
-            
+
             // check if it possible for the unit to reach the colony
             PathNode pathNode = null;
             if(unit.getTile() != colony.getTile()){
-                pathNode = aiu.getGame().getMap().findPath(unit, unit.getTile(), colony.getTile());
+                pathNode = unit.findPath(colony.getTile());
                 // no path found
                 if(pathNode == null){
                     continue;
@@ -611,34 +572,32 @@ public class PioneeringMission extends Mission {
                     continue;
                 }
             }
-            
+
             int value = 100;
             // Prefer units with plenty of tools
             for(AbstractGoods goods : toolsType.getGoodsRequired()){
                 value += colony.getGoodsCount(goods.getType());
             }
-            
+
             if(pathNode != null){
                 value -= pathNode.getTotalTurns() * 10;
             }
-                
+
             if(best == null || value > bestValue){
                 best = colony;
                 bestValue = value;
             }
-        }        
+        }
         return best;
     }
-    
+
     public static List<AIUnit>getPlayerPioneers(AIPlayer aiPlayer){
         List<AIUnit> list = new ArrayList<AIUnit>();
-        
+
         AIMain aiMain = aiPlayer.getAIMain();
-        for(Unit u : aiPlayer.getPlayer().getUnits()){
-            AIUnit aiu =  (AIUnit) aiMain.getAIObject(u);
-            if(aiu == null){
-                continue;
-            }
+        for (Unit u : aiPlayer.getPlayer().getUnits()) {
+            AIUnit aiu = aiMain.getAIUnit(u);
+            if (aiu == null) continue;
             if(aiu.getMission() instanceof PioneeringMission){
                 list.add(aiu);
             }
@@ -647,7 +606,41 @@ public class PioneeringMission extends Mission {
     }
 
     /**
-     * Writes all of the <code>AIObject</code>s and other AI-related 
+     * Checks if this mission is still valid to perform.
+     *
+     * @return True if this mission is still valid to perform.
+     */
+    public boolean isValid() {
+        if (!super.isValid()
+            || getUnit().getTile() == null
+            || invalidateMission) return false;
+
+        switch (state) {
+        case GET_TOOLS:
+            EquipmentType toolsType = getAIMain().getGame()
+                .getSpecification().getEquipmentType("model.equipment.tools");
+            if (colonyWithTools == null
+                || colonyWithTools.isDisposed()
+                || colonyWithTools.getOwner() != getUnit().getOwner()
+                || !colonyWithTools.canBuildEquipment(toolsType)) {
+                return findColonyWithTools(getAIUnit()) != null;
+            }
+            break;
+        case IMPROVING:
+            Tile target = (tileImprovementPlan == null) ? null
+                : tileImprovementPlan.getTarget();
+            if (tileImprovementPlan == null
+                || target == null
+                || target.hasImprovement(tileImprovementPlan.getType())) {
+                return false;
+            }
+            break;
+        }
+        return true;
+    }
+
+    /**
+     * Writes all of the <code>AIObject</code>s and other AI-related
      * information to an XML-stream.
      *
      * @param out The target stream.
@@ -656,7 +649,7 @@ public class PioneeringMission extends Mission {
      */
     protected void toXMLImpl(XMLStreamWriter out) throws XMLStreamException {
         out.writeStartElement(getXMLElementTagName());
-        
+
         out.writeAttribute("unit", getUnit().getId());
         if (tileImprovementPlan != null) {
             out.writeAttribute("tileImprovementPlan", tileImprovementPlan.getId());
@@ -672,7 +665,7 @@ public class PioneeringMission extends Mission {
      */
     protected void readFromXMLImpl(XMLStreamReader in) throws XMLStreamException {
         setAIUnit((AIUnit) getAIMain().getAIObject(in.getAttributeValue(null, "unit")));
-        
+
         final String tileImprovementPlanStr = in.getAttributeValue(null, "tileImprovementPlan");
         if (tileImprovementPlanStr != null) {
             tileImprovementPlan = (TileImprovementPlan) getAIMain().getAIObject(tileImprovementPlanStr);
@@ -682,7 +675,7 @@ public class PioneeringMission extends Mission {
         } else {
             tileImprovementPlan = null;
         }
-        
+
         in.nextTag();
     }
 
@@ -693,13 +686,13 @@ public class PioneeringMission extends Mission {
     public static String getXMLElementTagName() {
         return "tileImprovementPlanMission";
     }
-    
+
     /**
      * Gets debugging information about this mission.
      * This string is a short representation of this
      * object's state.
-     * 
-     * @return The <code>String</code>: 
+     *
+     * @return The <code>String</code>:
      *      <ul>
      *          <li>"(x, y) P" (for plowing)</li>
      *          <li>"(x, y) R" (for building road)</li>
@@ -708,20 +701,20 @@ public class PioneeringMission extends Mission {
      */
     public String getDebuggingInfo() {
         switch(state){
-            case IMPROVING:
-                if(tileImprovementPlan == null){
-                    return "No target";
-                }
-                final String action = tileImprovementPlan.getType().toString();
-                return tileImprovementPlan.getTarget().getPosition().toString() + " " + action;
-            case GET_TOOLS:
-                if (colonyWithTools == null) {
-                    return "No target";
-                }
-                return "Getting tools from " + colonyWithTools.getName();
-            default:
-                logger.warning("Unknown state");
-                return "";
+        case IMPROVING:
+            if(tileImprovementPlan == null){
+                return "No target";
+            }
+            final String action = tileImprovementPlan.getType().getNameKey();
+            return tileImprovementPlan.getTarget().getPosition().toString() + " " + action;
+        case GET_TOOLS:
+            if (colonyWithTools == null) {
+                return "No target";
+            }
+            return "Getting tools from " + colonyWithTools.getName();
+        default:
+            logger.warning("Unknown state");
+            return "";
         }
     }
 }
