@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2008  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -25,9 +25,10 @@ import junit.framework.TestSuite;
 public class AllTests {
 
     public static Test suite() {
-        TestSuite suite = new TestSuite("Test for net.sf.freecol.server");
+        TestSuite suite = new TestSuite("Test for net.sf.freecol.client");
         //$JUnit-BEGIN$
         suite.addTest(net.sf.freecol.client.control.AllTests.suite());
+        suite.addTest(net.sf.freecol.client.gui.AllTests.suite());
         //$JUnit-END$
         return suite;
     }
