@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -26,7 +26,7 @@ public class AbstractUnitTest extends FreeColTestCase {
     public void testEquipment() {
 
         AbstractUnit newUnit = new AbstractUnit("newUnit", Unit.Role.PIONEER, 1);
-        EquipmentType[] equipment = newUnit.getEquipment();
+        EquipmentType[] equipment = newUnit.getEquipment(spec());
         assertEquals(5, equipment.length);
         assertEquals("model.equipment.tools", equipment[0].getId());
         assertEquals("model.equipment.tools", equipment[1].getId());
