@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -23,47 +23,45 @@ import java.io.StringWriter;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Set;
+
 import javax.xml.stream.XMLOutputFactory;
 import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamWriter;
 
-import net.sf.freecol.FreeCol;
+import net.sf.freecol.common.model.Modifier;
+import net.sf.freecol.server.model.ServerBuilding;
 import net.sf.freecol.util.test.FreeColTestCase;
 
+
 public class BuildingTest extends FreeColTestCase {
-    
-    BuildingType printingPressType = FreeCol.getSpecification().getBuildingType("model.building.PrintingPress");
-    BuildingType newspaperType = FreeCol.getSpecification().getBuildingType("model.building.Newspaper");
-    
+
+    BuildingType printingPressType = spec().getBuildingType("model.building.printingPress");
+    BuildingType newspaperType = spec().getBuildingType("model.building.newspaper");
+
     public void testCanBuildNext() {
-    	Game game = getGame();
-    	game.setMap(getTestMap(plainsType,true));
-    	
-    	Colony colony = getStandardColony();
+        Game game = getGame();
+        game.setMap(getTestMap(true));
+
+        Colony colony = getStandardColony();
 
         // First check with a building that can be fully built with a
         // normal colony
-        BuildingType warehouseType = FreeCol.getSpecification().getBuildingType("model.building.Depot");
-        Building warehouse = new Building(getGame(), colony, warehouseType);
+        BuildingType warehouseType = spec().getBuildingType("model.building.depot");
+        Building warehouse = new ServerBuilding(getGame(), colony, warehouseType);
         colony.addBuilding(warehouse);
         assertTrue(warehouse.canBuildNext());
         warehouse.upgrade();
         assertTrue(warehouse.canBuildNext());
         warehouse.upgrade();
         assertFalse(warehouse.canBuildNext());
-        
-        try {
-        	warehouse.upgrade();
-        	fail();
-        } catch (IllegalStateException e){
-        	// Should throw exception
-        }
+
+        assertFalse(warehouse.upgrade());
         assertFalse(warehouse.canBuildNext());
 
         // Check whether population restrictions work
 
         // Colony smallColony = getStandardColony(1);
-        // 
+        //
         // Colony largeColony = getStandardColony(6);
         // ...
 
@@ -73,16 +71,16 @@ public class BuildingTest extends FreeColTestCase {
 
     public void testInitialColony() {
     	Game game = getGame();
-    	game.setMap(getTestMap(plainsType,true));
+    	game.setMap(getTestMap(true));
 
         Colony colony = getStandardColony();
 
-        BuildingType warehouseType = FreeCol.getSpecification().getBuildingType("model.building.Warehouse");
+        BuildingType warehouseType = spec().getBuildingType("model.building.warehouse");
         Building warehouse = colony.getBuilding(warehouseType);
 
         // Is build as depot...
         assertTrue(warehouse != null);
-        
+
         assertTrue(warehouse.canBuildNext());
 
         // Check other building...
@@ -93,10 +91,10 @@ public class BuildingTest extends FreeColTestCase {
 
     public void testChurch() {
     	Game game = getGame();
-    	game.setMap(getTestMap(plainsType,true));
-    	
+    	game.setMap(getTestMap(true));
+
         Colony colony = getStandardColony(6);
-        BuildingType churchType = FreeCol.getSpecification().getBuildingType("model.building.Chapel");
+        BuildingType churchType = spec().getBuildingType("model.building.chapel");
         assertFalse(churchType.hasAbility("model.ability.dressMissionary"));
 
         Building church = colony.getBuilding(churchType);
@@ -106,26 +104,26 @@ public class BuildingTest extends FreeColTestCase {
         church.upgrade();
         assertTrue(church.getType().hasAbility("model.ability.dressMissionary"));
         assertTrue(colony.hasAbility("model.ability.dressMissionary"));
-        
+
     }
 
     public void testCanAddToBuilding() {
     	Game game = getGame();
-    	game.setMap(getTestMap(plainsType,true));
-    	
+    	game.setMap(getTestMap(true));
+
         Colony colony = getStandardColony(6);
         List<Unit> units = colony.getUnitList();
 
         for (Building building : colony.getBuildings()) {
-            
+
             // schoolhouse is special, see testCanAddToSchool
-            if (building.getType().hasAbility("model.ability.teach")) 
+            if (building.getType().hasAbility("model.ability.teach"))
             	continue;
-            
+
             int maxUnits = building.getMaxUnits();
-            
+
             assertEquals(0, building.getUnitCount());
-            
+
             for (int index = 0; index < maxUnits; index++) {
                 assertTrue("unable to add unit " + index + " to building type " +
                            building.getType(), building.canAdd(units.get(index)));
@@ -139,13 +137,13 @@ public class BuildingTest extends FreeColTestCase {
             }
         }
     }
-    
-    
+
+
     /**
      * WARNING! This test makes implicit assumptions about the
      * schoolhouse that could be invalidated by the
-     * specification. 
-     * 
+     * specification.
+     *
      * TODO: make this more generic.
      */
     public void testCanAddToSchool(){
@@ -159,43 +157,43 @@ public class BuildingTest extends FreeColTestCase {
         UnitType indianConvertType = spec().getUnitType("model.unit.indianConvert");
 
     	Game game = getGame();
-    	game.setMap(getTestMap(plainsType,true));
-        
+    	game.setMap(getTestMap(true));
+
         Colony colony = getStandardColony(10);
-        
+
         Iterator<Unit> units = colony.getUnitIterator();
-        
+
         Unit farmer = units.next();
         farmer.setType(expertFarmerType);
-        
+
         Unit colonist = units.next();
         colonist.setType(freeColonistType);
-        
+
         Unit criminal = units.next();
         criminal.setType(pettyCriminalType);
-        
+
         Unit servant = units.next();
         servant.setType(indenturedServantType);
-        
+
         Unit indian = units.next();
         indian.setType(indianConvertType);
-        
+
         Unit distiller = units.next();
         distiller.setType(masterDistillerType);
-        
+
         Unit elder = units.next();
         elder.setType(elderStatesmanType);
 
         Unit carpenter = units.next();
         carpenter.setType(masterCarpenterType);
-        
+
         // Check school
-        BuildingType schoolType = FreeCol.getSpecification().getBuildingType("model.building.Schoolhouse");
+        BuildingType schoolType = spec().getBuildingType("model.building.schoolhouse");
         Building school = colony.getBuilding(schoolType);
         assertTrue(school == null);
 
         // build school
-        colony.addBuilding(new Building(getGame(), colony, schoolType));
+        colony.addBuilding(new ServerBuilding(getGame(), colony, schoolType));
         school = colony.getBuilding(schoolType);
         assertTrue(school != null);
 
@@ -208,7 +206,7 @@ public class BuildingTest extends FreeColTestCase {
                     school.canAdd(servant));
         assertFalse("able to add indian convert to Schoolhouse",
                     school.canAdd(indian));
-        
+
         assertFalse("able to add elder statesman to Schoolhouse",
                     school.canAdd(elder));
         assertFalse("able to add master distiller to Schoolhouse",
@@ -230,7 +228,7 @@ public class BuildingTest extends FreeColTestCase {
                     school.canAdd(servant));
         assertFalse("able to add indian convert to College",
                     school.canAdd(indian));
-        
+
         assertFalse("able to add elder statesman to College",
                     school.canAdd(elder));
         assertTrue("unable to add master distiller to College",
@@ -245,9 +243,9 @@ public class BuildingTest extends FreeColTestCase {
         school.remove(farmer);
 
         school.upgrade();
-        
-        assertEquals(school.getType().toString(), school.getType(), spec().getBuildingType("model.building.University"));
-        
+
+        assertEquals(school.getType().toString(), school.getType(), spec().getBuildingType("model.building.university"));
+
         // these can never teach
         assertFalse("able to add free colonist to University",
                     school.canAdd(colonist));
@@ -257,7 +255,7 @@ public class BuildingTest extends FreeColTestCase {
                     school.canAdd(servant));
         assertFalse("able to add indian convert to University",
                     school.canAdd(indian));
-        
+
         assertTrue("unable to add elder statesman to University",
                    school.canAdd(elder));
         school.add(elder);
@@ -276,12 +274,10 @@ public class BuildingTest extends FreeColTestCase {
     }
 
     public void testSerialize() {
-    	Game game = getGame();
-    	game.setMap(getTestMap(plainsType,true));
-    	
-        Colony colony = getStandardColony(6);
-        List<Unit> units = colony.getUnitList();
+        Game game = getGame();
+        game.setMap(getTestMap(true));
 
+        Colony colony = getStandardColony(6);
         for (Building building : colony.getBuildings()) {
             try {
                 StringWriter sw = new StringWriter();
@@ -296,30 +292,51 @@ public class BuildingTest extends FreeColTestCase {
     }
 
     public void testStockade() {
-
+        Game game = getGame();
+        game.setMap(getTestMap(true));
         Set<Modifier> modifierSet;
-        
-        BuildingType stockade = spec().getBuildingType("model.building.Stockade");
+
+        Colony colony = getStandardColony(2);
+        modifierSet = colony.getModifierSet("model.modifier.defence");
+        assertEquals(1, modifierSet.size());
+        Modifier modifier = modifierSet.iterator().next();
+        assertEquals(50f, modifier.getValue());
+        assertEquals(Modifier.Type.PERCENTAGE, modifier.getType());
+
+        BuildingType stockade = spec()
+            .getBuildingType("model.building.stockade");
         modifierSet = stockade.getModifierSet("model.modifier.defence");
         assertEquals(1, modifierSet.size());
-        assertEquals(100f, modifierSet.iterator().next().getValue());
+        modifier = modifierSet.iterator().next();
+        assertEquals(100f, modifier.getValue());
+        assertEquals(Modifier.Type.PERCENTAGE, modifier.getType());
+        assertEquals(0f, stockade.getFeatureContainer()
+                     .applyModifier(0, "model.modifier.minimumColonySize"));
 
-        BuildingType fort = spec().getBuildingType("model.building.Fort");
+        BuildingType fort = spec().getBuildingType("model.building.fort");
         modifierSet = fort.getModifierSet("model.modifier.defence");
         assertEquals(1, modifierSet.size());
-        assertEquals(150f, modifierSet.iterator().next().getValue());
-        
-        BuildingType fortress = spec().getBuildingType("model.building.Fortress");
+        modifier = modifierSet.iterator().next();
+        assertEquals(150f, modifier.getValue());
+        assertEquals(Modifier.Type.PERCENTAGE, modifier.getType());
+        assertEquals(0f, stockade.getFeatureContainer()
+                     .applyModifier(0, "model.modifier.minimumColonySize"));
+
+        BuildingType fortress = spec()
+            .getBuildingType("model.building.fortress");
         modifierSet = fortress.getModifierSet("model.modifier.defence");
         assertEquals(1, modifierSet.size());
-        assertEquals(200f, modifierSet.iterator().next().getValue());
-
+        modifier = modifierSet.iterator().next();
+        assertEquals(200f, modifier.getValue());
+        assertEquals(Modifier.Type.PERCENTAGE, modifier.getType());
+        assertEquals(0f, stockade.getFeatureContainer()
+                     .applyModifier(0, "model.modifier.minimumColonySize"));
     }
 
     public void testCottonClothProduction() {
-    	Game game = getGame();
-    	game.setMap(getTestMap(plainsType,true));
-    	
+        Game game = getGame();
+        game.setMap(getTestMap(true));
+
         Colony colony = getStandardColony(2);
         List<Unit> units = colony.getUnitList();
         Unit colonist = units.get(0);
@@ -329,7 +346,7 @@ public class BuildingTest extends FreeColTestCase {
         GoodsType cottonType = spec().getGoodsType("model.goods.cotton");
         GoodsType clothType = spec().getGoodsType("model.goods.cloth");
 
-        Building weaver = colony.getBuilding(spec().getBuildingType("model.building.WeaverHouse"));
+        Building weaver = colony.getBuilding(spec().getBuildingType("model.building.weaverHouse"));
         assertEquals(cottonType, weaver.getGoodsInputType());
         assertEquals(clothType, weaver.getGoodsOutputType());
 
@@ -339,72 +356,92 @@ public class BuildingTest extends FreeColTestCase {
         weaver.add(worker);
         assertEquals(worker, weaver.getUnitList().get(0));
 
-        assertEquals(2, colony.getProductionOf(cottonType));
         colony.addGoods(cottonType, 2);
+        assertEquals(2, colony.getProductionOf(cottonType));
         assertEquals(2, weaver.getProductionOf(clothType));
         assertEquals(2, colony.getProductionOf(clothType));
-        assertEquals(-1, colony.getProductionNetOf(cottonType));
-        assertEquals(3, colony.getProductionNetOf(clothType));
+        assertEquals(-1, colony.getNetProductionOf(cottonType));
+        assertEquals(3, colony.getNetProductionOf(clothType));
 
         colonist.setWorkType(cottonType);
 
         assertEquals(4, colony.getProductionOf(cottonType));
         colony.addGoods(cottonType, 4);
         assertEquals(3, colony.getProductionOf(clothType));
-        assertEquals(1, colony.getProductionNetOf(cottonType));
-        assertEquals(3, colony.getProductionNetOf(clothType));
+        assertEquals(1, colony.getNetProductionOf(cottonType));
+        assertEquals(3, colony.getNetProductionOf(clothType));
 
     }
 
     public void testAutoProduction() {
     	Game game = getGame();
-    	game.setMap(getTestMap(plainsType,true));
-    	
+    	game.setMap(getTestMap(true));
+
         Colony colony = getStandardColony(1);
-        List<Unit> units = colony.getUnitList();
-        GoodsType foodType = spec().getGoodsType("model.goods.food");
+        GoodsType foodType = spec().getPrimaryFoodType();
+        GoodsType grainType = spec().getGoodsType("model.goods.grain");
         GoodsType horsesType = spec().getGoodsType("model.goods.horses");
 
-
-        Building pasture = colony.getBuilding(spec().getBuildingType("model.building.Country"));
-        assertEquals(foodType, pasture.getGoodsInputType());
+        BuildingType country = spec().getBuildingType("model.building.country");
+        Building pasture = colony.getBuilding(country);
+        assertEquals(grainType, pasture.getGoodsInputType());
         assertEquals(horsesType, pasture.getGoodsOutputType());
 
         // no horses yet
-        assertEquals(10, colony.getProductionOf(foodType));
+        assertEquals(8, colony.getNetProductionOf(foodType));
         assertEquals(0, pasture.getProductionOf(horsesType));
-        assertEquals(2, colony.getFoodConsumption());
-        assertEquals(0, pasture.getGoodsInputNextTurn());
-        assertEquals(8, colony.getProductionNetOf(foodType));
-        assertEquals(0, colony.getProductionNetOf(horsesType));
+        assertEquals(0, colony.getNetProductionOf(horsesType));
         assertEquals(0, pasture.getMaximumProduction());
 
-        colony.addGoods(horsesType, 20);
-        assertEquals(1, pasture.getProductionOf(horsesType));
-        assertEquals(2, pasture.getMaximumProduction());
-        assertEquals(1, colony.getProductionNetOf(horsesType));
-        colony.addGoods(horsesType, 20);
+        colony.addGoods(horsesType, 50);
         assertEquals(2, pasture.getProductionOf(horsesType));
-        assertEquals(4, pasture.getMaximumProduction());
-        assertEquals(2, colony.getProductionNetOf(horsesType));
+        assertEquals(2, pasture.getMaximumProduction());
+        assertEquals(2, colony.getNetProductionOf(horsesType));
 
+        colony.addGoods(horsesType, 1);
+        assertEquals(4, pasture.getProductionOf(horsesType));
+        assertEquals(4, pasture.getMaximumProduction());
+        assertEquals(4, colony.getNetProductionOf(horsesType));
+
+        pasture.upgrade();
+        colony.removeGoods(horsesType);
+
+        colony.addGoods(horsesType, 25);
+        assertEquals(2, pasture.getProductionOf(horsesType));
+        assertEquals(2, pasture.getMaximumProduction());
+        assertEquals(2, colony.getNetProductionOf(horsesType));
+
+        colony.addGoods(horsesType, 1);
+        assertEquals(4, pasture.getProductionOf(horsesType));
+        assertEquals(4, pasture.getMaximumProduction());
+        assertEquals(4, colony.getNetProductionOf(horsesType));
+
+        colony.addGoods(horsesType, 24);
+        assertEquals(4, pasture.getProductionOf(horsesType));
+        assertEquals(4, pasture.getMaximumProduction());
+        assertEquals(4, colony.getNetProductionOf(horsesType));
+
+        colony.addGoods(horsesType, 1);
+        assertEquals(6, pasture.getProductionOf(horsesType));
+        assertEquals(6, pasture.getMaximumProduction());
+        assertEquals("Horse production should equal food surplus.",
+                     colony.getNetProductionOf(foodType),
+                     colony.getNetProductionOf(horsesType));
     }
 
     public void testTownhallProduction() {
     	Game game = getGame();
-    	game.setMap(getTestMap(plainsType,true));
-        
+    	game.setMap(getTestMap(true));
+
         Colony colony = getStandardColony(6);
         Player owner = colony.getOwner();
-        UnitType statesmanType = spec().getUnitType("model.unit.elderStatesman");
         Unit colonist = colony.getUnitList().get(0);
         Unit statesman = colony.getUnitList().get(1);
-        Unit statesman2 = colony.getUnitList().get(2);
-        statesman.setType(statesmanType);
-        statesman2.setType(statesmanType);
+        statesman.setType(spec().getUnitType("model.unit.elderStatesman"));
 
-        BuildingType townHall = spec().getBuildingType("model.building.TownHall");
+        BuildingType townHall = spec().getBuildingType("model.building.townHall");
         Building building = colony.getBuilding(townHall);
+        GoodsType bellsType = spec().getGoodsType("model.goods.bells");
 
         Set<Modifier> modifiers = colony.getModifierSet("model.goods.bells");
         assertEquals(1, modifiers.size());
@@ -417,117 +454,96 @@ public class BuildingTest extends FreeColTestCase {
 
         building.add(colonist);
         // 3 from the colonist
-        assertEquals(3, colonist.getProductionOf(Goods.BELLS, townHall.getBasicProduction()));
+        assertEquals(3, colonist.getProductionOf(bellsType, townHall.getBasicProduction()));
         assertEquals(3, building.getUnitProductivity(colonist));
         // 3 from the colonist + 1
         assertEquals("Wrong bell production", 4, building.getProduction());
 
-        owner.addFather(spec().getFoundingFather("model.foundingFather.thomasJefferson"));
-        // 3 from the colonist + 50% = 5
-        assertEquals(5, colonist.getProductionOf(Goods.BELLS, townHall.getBasicProduction()));
-        assertEquals(5, building.getUnitProductivity(colonist));
-        // 3 from the colonist + 50% + 1 = 6
-        assertEquals("Wrong bell production with Jefferson", 6, building.getProduction());
+        FoundingFather jefferson = spec().getFoundingFather("model.foundingFather.thomasJefferson");
+        modifiers = jefferson.getModifierSet("model.goods.bells");
+        assertEquals(1, modifiers.size());
+        bellsModifier = modifiers.iterator().next();
+        owner.addFather(jefferson);
+        assertTrue(colony.getOwner().getFeatureContainer().getModifierSet("model.goods.bells")
+                   .contains(bellsModifier));
+        assertTrue(colony.getModifierSet("model.goods.bells").contains(bellsModifier));
+
+        assertEquals(3, building.getUnitProductivity(colonist));
+        // 3 from the colonist + 50% + 1 = 5.5
+        assertEquals("Wrong bell production with Jefferson", 5, building.getProduction());
 
         building.add(statesman);
-        // 3 from colonist + 50% = 5
-        assertEquals(5, colonist.getProductionOf(Goods.BELLS, townHall.getBasicProduction()));
-        assertEquals(5, building.getUnitProductivity(colonist));
-        // 2*3 from statesman + 50% = 9
-        assertEquals(9, statesman.getProductionOf(Goods.BELLS, townHall.getBasicProduction()));
-        assertEquals(9, building.getUnitProductivity(statesman));
-        // 5 + 9 + 1 = 15
-        assertEquals("Wrong bell production with Jefferson", 15, building.getProduction());
+        assertEquals(3, building.getUnitProductivity(colonist));
+        assertEquals(6, building.getUnitProductivity(statesman));
+        // 3 + 6 + 50% + 1 = 14
+        assertEquals("Wrong bell production with Jefferson", 14, building.getProduction());
 
         setProductionBonus(colony, 2);
-        // 3 from colonist + 50% + 2 = 7
-        assertEquals(5, colonist.getProductionOf(Goods.BELLS, townHall.getBasicProduction()));
-        assertEquals(7, building.getUnitProductivity(colonist));
-        // 2*3 from statesman + 50% + 2 = 11
-        assertEquals(9, statesman.getProductionOf(Goods.BELLS, townHall.getBasicProduction()));
-        assertEquals(11, building.getUnitProductivity(statesman));
-        // 7 + 11 + 1 = 19
+        assertEquals(5, building.getUnitProductivity(colonist));
+        assertEquals(10, building.getUnitProductivity(statesman));
+        // 5 + 10 + 50% + 1 = 23
         assertEquals("Wrong bell production with Jefferson and +2 production bonus",
-                     19, building.getProduction());
-        
-        Building newspaper = new Building(getGame(), colony, newspaperType);
+                     23, building.getProduction());
+
+        Building newspaper = new ServerBuilding(getGame(), colony, newspaperType);
         colony.addBuilding(newspaper);
-
-        // 3 from colonist + 50% + 2 = 7
-        assertEquals(5, colonist.getProductionOf(Goods.BELLS, townHall.getBasicProduction()));
-        assertEquals(7, building.getUnitProductivity(colonist));
-        // 2*3 from statesman + 50% + 2 = 11
-        assertEquals(9, statesman.getProductionOf(Goods.BELLS, townHall.getBasicProduction()));
-        assertEquals(11, building.getUnitProductivity(statesman));
-        // 7 + 11 + 1 + 100% = 38
+        assertEquals(5, building.getUnitProductivity(colonist));
+        assertEquals(10, building.getUnitProductivity(statesman));
+        // 5 + 10 + 50% + 1 + 100% = 47
         assertEquals("Wrong bell production with Jefferson, newspaper and +2 production bonus",
-                     38, building.getProduction());
+                     47, building.getProduction());
 
-        colonist.setType(statesmanType);
-        building.add(statesman2);
-        setProductionBonus(colony, -2);
-
-        // 2*3 from statesman + 50% - 2 = 7
-        assertEquals(9, colonist.getProductionOf(Goods.BELLS, townHall.getBasicProduction()));
-        assertEquals(7, building.getUnitProductivity(colonist));
-        assertEquals(9, statesman.getProductionOf(Goods.BELLS, townHall.getBasicProduction()));
-        assertEquals(7, building.getUnitProductivity(statesman));
-        assertEquals(9, statesman2.getProductionOf(Goods.BELLS, townHall.getBasicProduction()));
-        assertEquals(7, building.getUnitProductivity(statesman2));
-        // 7 + 7 + 7 + 1 + 100% = 44
-        assertEquals("Wrong bell production with Jefferson, newspaper and -2 production bonus",
-                     44, building.getProduction());
 
     }
-    
+
     public void testPrintingPressBonus() {
         Game game = getGame();
-        game.setMap(getTestMap(plainsType,true));
-        
+        game.setMap(getTestMap(true));
+
         Colony colony = getStandardColony(6);
         Unit unit = colony.getRandomUnit();
-        Building building = colony.getBuilding(spec().getBuildingType("model.building.TownHall"));
-        
+        Building building = colony.getBuilding(spec().getBuildingType("model.building.townHall"));
+
         int bellProduction = building.getProduction();
         int expectBellProd = 1;
         assertEquals("Wrong initial bell production",expectBellProd,bellProduction);
-        
-        Building printingPress = new Building(getGame(), colony, printingPressType);
+
+        Building printingPress = new ServerBuilding(getGame(), colony, printingPressType);
         colony.addBuilding(printingPress);
-        
+
         bellProduction = building.getProduction();
-        expectBellProd = 2;
+        expectBellProd = 1;
         assertEquals("Wrong bell production with printing press",expectBellProd,bellProduction);
-        
+
         building.add(unit);
         bellProduction = building.getProduction();
         expectBellProd = 6; // 1 initial plus 3 from the colonist + 2 from printing press
-        assertEquals("Wrong final bell production",expectBellProd,bellProduction); 
+        assertEquals("Wrong final bell production",expectBellProd,bellProduction);
     }
-    
+
     public void testNewspaperBonus() {
         Game game = getGame();
-        game.setMap(getTestMap(plainsType,true));
-        
+        game.setMap(getTestMap(true));
+
         Colony colony = getStandardColony(6);
         Unit unit = colony.getRandomUnit();
-        Building building = colony.getBuilding(spec().getBuildingType("model.building.TownHall"));
-        
+        Building building = colony.getBuilding(spec().getBuildingType("model.building.townHall"));
+
         int bellProduction = building.getProduction();
         int expectBellProd = 1;
         assertEquals("Wrong initial bell production",expectBellProd,bellProduction);
-        
-        Building newspaper = new Building(getGame(), colony, newspaperType);
+
+        Building newspaper = new ServerBuilding(getGame(), colony, newspaperType);
         colony.addBuilding(newspaper);
-        
+
         bellProduction = building.getProduction();
         expectBellProd = 2;
         assertEquals("Wrong bell production with newspaper",expectBellProd,bellProduction);
-        
+
         building.add(unit);
         bellProduction = building.getProduction();
         expectBellProd = 8; // 1 initial plus 3 from the colonist + 4 from newspaper
-        assertEquals("Wrong final bell production",expectBellProd,bellProduction); 
+        assertEquals("Wrong final bell production",expectBellProd,bellProduction);
     }
-    
+
 }
