@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -23,6 +23,7 @@ import java.util.ArrayList;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Random;
 import java.util.Set;
 
 import net.sf.freecol.common.FreeColException;
@@ -33,11 +34,15 @@ import net.sf.freecol.common.model.Unit.UnitState;
 import net.sf.freecol.common.model.pathfinding.CostDecider;
 import net.sf.freecol.common.model.pathfinding.CostDeciders;
 import net.sf.freecol.common.model.pathfinding.GoalDecider;
+import net.sf.freecol.server.model.ServerUnit;
 import net.sf.freecol.util.test.FreeColTestCase;
 import net.sf.freecol.util.test.FreeColTestUtils;
 
+
 public class MapTest extends FreeColTestCase {
     TileType oceanType = spec().getTileType("model.tile.ocean");
+    TileType plainsType = spec().getTileType("model.tile.plains");
+
     UnitType colonistType = spec().getUnitType("model.unit.freeColonist");
     
     private Map getSingleLandPathMap(Game game){
@@ -105,9 +110,13 @@ public class MapTest extends FreeColTestCase {
 
         MapBuilder builder = new MapBuilder(game);
         Map map = builder.setDimensions(10, 15).build();
+        game.setMap(map);
 
         // Check in the middle
-        List<Tile> surroundingTiles = map.getSurroundingTiles(map.getTile(4,8), 1);
+        List<Tile> surroundingTiles = new ArrayList<Tile>();
+        for (Tile t: map.getTile(4,8).getSurroundingTiles(1))
+            surroundingTiles.add(t);
+
 
         assertEquals(8, surroundingTiles.size());
         assertTrue(surroundingTiles.contains(map.getTile(4, 6)));
@@ -120,7 +129,9 @@ public class MapTest extends FreeColTestCase {
         assertTrue(surroundingTiles.contains(map.getTile(4, 9)));
 
         // Check on sides
-        surroundingTiles = map.getSurroundingTiles(map.getTile(0, 0), 1);
+        surroundingTiles = new ArrayList<Tile>();
+        for (Tile t: map.getTile(0, 0).getSurroundingTiles(1))
+            surroundingTiles.add(t);
 
         assertEquals(3, surroundingTiles.size());
         assertTrue(surroundingTiles.contains(map.getTile(0, 2)));
@@ -128,12 +139,17 @@ public class MapTest extends FreeColTestCase {
         assertTrue(surroundingTiles.contains(map.getTile(0, 1)));
 
         // Check larger range
-        surroundingTiles = map.getSurroundingTiles(map.getTile(4,8), 2);
+        surroundingTiles = new ArrayList<Tile>();
+        for (Tile t: map.getTile(4, 8).getSurroundingTiles(2))
+            surroundingTiles.add(t);
 
         assertEquals(25 - 1, surroundingTiles.size());
 
         // Check that all tiles are returned
-        surroundingTiles = map.getSurroundingTiles(map.getTile(4,8), 10);
+        surroundingTiles = new ArrayList<Tile>();
+        for (Tile t: map.getTile(4, 8).getSurroundingTiles(10))
+            surroundingTiles.add(t);
+
         assertEquals(150 - 1, surroundingTiles.size());
     }
 
@@ -259,9 +275,8 @@ public class MapTest extends FreeColTestCase {
     public void testRandomDirection() {
         Game game = getStandardGame();
         MapBuilder builder = new MapBuilder(game);
-        Map map = builder.setDimensions(10, 15).build();
-        
-        Direction[] dirs = map.getRandomDirectionArray();
+        builder.setDimensions(10, 15).build();
+        Direction[] dirs = Direction.getRandomDirectionArray(new Random(1));
         assertNotNull(dirs);
     }
     
@@ -277,13 +292,14 @@ public class MapTest extends FreeColTestCase {
         // set obstructing indian camp
         Tile settlementTile = map.getTile(2,10);
         FreeColTestCase.IndianSettlementBuilder builder = new FreeColTestCase.IndianSettlementBuilder(game);
-        IndianSettlement settlement = builder.settlementTile(settlementTile).build();
+        builder.settlementTile(settlementTile).build();
 
         // set unit
         Player dutchPlayer = game.getPlayer("model.nation.dutch");
         Tile unitTile = map.getTile(1, 11);
         Tile destinationTile = map.getTile(3,7);
-        Unit colonist = new Unit(game, unitTile, dutchPlayer, colonistType, UnitState.ACTIVE);
+        Unit colonist = new ServerUnit(game, unitTile, dutchPlayer,
+                                       colonistType, UnitState.ACTIVE);
         colonist.setDestination(destinationTile);
         
         PathNode path = map.findPath(colonist, colonist.getTile(), destinationTile);
@@ -302,7 +318,7 @@ public class MapTest extends FreeColTestCase {
         // set obstructing french colony
         Player frenchPlayer = game.getPlayer("model.nation.french");
         Tile settlementTile = map.getTile(2,10);
-        Colony frenchColony = FreeColTestUtils.getColonyBuilder().player(frenchPlayer)
+        FreeColTestUtils.getColonyBuilder().player(frenchPlayer)
                                                                  .colonyTile(settlementTile)
                                                                  .build();
         assertTrue("French colony was not set properly on the map",settlementTile.getSettlement() != null);
@@ -310,7 +326,8 @@ public class MapTest extends FreeColTestCase {
         Player dutchPlayer = game.getPlayer("model.nation.dutch");
         Tile unitTile = map.getTile(1, 11);
         Tile destinationTile = map.getTile(3,7);
-        Unit colonist = new Unit(game, unitTile, dutchPlayer, colonistType, UnitState.ACTIVE);
+        Unit colonist = new ServerUnit(game, unitTile, dutchPlayer,
+                                       colonistType, UnitState.ACTIVE);
         colonist.setDestination(destinationTile);
         
         PathNode path = map.findPath(colonist, colonist.getTile(), destinationTile);
@@ -320,17 +337,19 @@ public class MapTest extends FreeColTestCase {
     public void testMoveThroughTileWithEnemyUnit() {
         final UnitType pioneerType = spec().getUnitType("model.unit.hardyPioneer");
         Game game = getStandardGame();
-        Map map = getTestMap(plainsType);
+        Map map = getTestMap();
         game.setMap(map);
         
         //Setup
         Tile enemyUnitTile = map.getTile(2,1);
         Player frenchPlayer = game.getPlayer("model.nation.french");
-        new Unit(game, enemyUnitTile, frenchPlayer, pioneerType, UnitState.ACTIVE);
+        new ServerUnit(game, enemyUnitTile, frenchPlayer, pioneerType,
+                       UnitState.ACTIVE);
         
         Tile unitTile = map.getTile(1, 1);
         Player dutchPlayer = game.getPlayer("model.nation.dutch");
-        Unit unit = new Unit(game, unitTile, dutchPlayer, pioneerType, UnitState.ACTIVE);
+        Unit unit = new ServerUnit(game, unitTile, dutchPlayer, pioneerType,
+                                   UnitState.ACTIVE);
         // unit is going somewhere else
         Tile unitDestination = map.getTile(3, 1);
         unit.setDestination(unitDestination);
@@ -355,16 +374,18 @@ public class MapTest extends FreeColTestCase {
         // set obstructing unit
         Tile unitObstructionTile = map.getTile(2,10);
         Player frenchPlayer = game.getPlayer("model.nation.french");
-        new Unit(game, unitObstructionTile, frenchPlayer, colonistType, UnitState.ACTIVE);
+        new ServerUnit(game, unitObstructionTile, frenchPlayer, colonistType,
+                       UnitState.ACTIVE);
         
         // set unit
         Player dutchPlayer = game.getPlayer("model.nation.dutch");
         Tile unitTile = map.getTile(1, 11);
         Tile destinationTile = map.getTile(3,7);
-        Unit colonist = new Unit(game, unitTile, dutchPlayer, colonistType, UnitState.ACTIVE);
+        Unit colonist = new ServerUnit(game, unitTile, dutchPlayer,
+                                       colonistType, UnitState.ACTIVE);
         colonist.setDestination(destinationTile);
         
-        PathNode path = map.findPath(colonist, colonist.getTile(), destinationTile, CostDeciders.avoidSettlementsAndBlockingUnits());
+        PathNode path = map.findPath(colonist, colonist.getTile(), destinationTile, null, CostDeciders.avoidSettlementsAndBlockingUnits());
         assertNull("No path should be available",path);
     }
     
@@ -381,7 +402,8 @@ public class MapTest extends FreeColTestCase {
         // set unit
         Player dutchPlayer = game.getPlayer("model.nation.dutch");
         Tile unitTile = map.getTile(1, 11);
-        Unit colonist = new Unit(game, unitTile, dutchPlayer, colonistType, UnitState.ACTIVE);
+        Unit colonist = new ServerUnit(game, unitTile, dutchPlayer,
+                                       colonistType, UnitState.ACTIVE);
         Tile destinationTile = map.getTile(3,7);
         colonist.setDestination(destinationTile);
         
@@ -391,7 +413,7 @@ public class MapTest extends FreeColTestCase {
 
     public void testSearchForColony() {
         Game game = getStandardGame();
-        Map map = getCoastTestMap(plainsType, true);
+        Map map = getCoastTestMap(spec().getTileType("model.tile.plains"), true);
         game.setMap(map);
 
         Player dutchPlayer = game.getPlayer("model.nation.dutch");
@@ -400,16 +422,18 @@ public class MapTest extends FreeColTestCase {
         UnitType galleonType = spec().getUnitType("model.unit.galleon");
         Tile unitTile = map.getTile(15, 5);
         Tile colonyTile = map.getTile(9, 9); // should be on coast
-        Unit galleon = new Unit(game, unitTile, dutchPlayer, galleonType,
-                                UnitState.ACTIVE);
-        Unit artillery = new Unit(game, galleon, dutchPlayer, artilleryType, UnitState.ACTIVE);
-        Colony frenchColony = FreeColTestUtils.getColonyBuilder()
+        Unit galleon = new ServerUnit(game, unitTile, dutchPlayer, galleonType,
+                                      UnitState.ACTIVE);
+        Unit artillery = new ServerUnit(game, galleon, dutchPlayer,
+                                        artilleryType, UnitState.ACTIVE);
+        FreeColTestUtils.getColonyBuilder()
             .player(frenchPlayer)
             .colonyTile(colonyTile)
             .build();
         assertTrue("French colony not on the map",
                    colonyTile.getSettlement() != null);
-        dutchPlayer.changeRelationWithPlayer(frenchPlayer, Stance.WAR);
+        dutchPlayer.setStance(frenchPlayer, Stance.WAR);
+        frenchPlayer.setStance(dutchPlayer, Stance.WAR);
 
         // Test a GoalDecider with subgoals.
         // The scoring function is deliberately simple.
@@ -442,20 +466,24 @@ public class MapTest extends FreeColTestCase {
                 }
             };
 
-        PathNode path = map.search(artillery, gd, Integer.MAX_VALUE, galleon);
+        PathNode path = map.search(artillery, unitTile,
+                                   gd, CostDeciders.avoidIllegal(),
+                                   Integer.MAX_VALUE, galleon);
         assertTrue("Should find the French colony via a drop off",
                    path != null && path.getTransportDropNode() != null
                    && path.getLastNode().getTile() == colonyTile);
 
         // Add another colony
         Tile colonyTile2 = map.getTile(5, 5); // should score less
-        Colony frenchColony2 = FreeColTestUtils.getColonyBuilder()
+        FreeColTestUtils.getColonyBuilder()
             .player(frenchPlayer)
             .colonyTile(colonyTile2)
             .build();
         assertTrue("French colony not on the map",
                    colonyTile2.getSettlement() != null);
-        path = map.search(artillery, gd, Integer.MAX_VALUE, galleon);
+        path = map.search(artillery, unitTile,
+                          gd, CostDeciders.avoidIllegal(),
+                          Integer.MAX_VALUE, galleon);
         assertTrue("Should still find the first French colony via a drop off",
                    path != null && path.getTransportDropNode() != null
                    && path.getLastNode().getTile() == colonyTile);
