@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -23,48 +23,103 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Set;
 
+import net.sf.freecol.client.gui.GUI;
 import net.sf.freecol.util.test.FreeColTestCase;
 import net.sf.freecol.util.test.FreeColTestUtils;
 
 public class TileTest extends FreeColTestCase {
 
-    TileType plains = spec().getTileType("model.tile.plains");
-    TileType desert = spec().getTileType("model.tile.desert");
-    TileType grassland = spec().getTileType("model.tile.grassland");
-    TileType prairie = spec().getTileType("model.tile.prairie");
-    TileType tundra = spec().getTileType("model.tile.tundra");
-    TileType savannah = spec().getTileType("model.tile.savannah");
-    TileType marsh = spec().getTileType("model.tile.marsh");
-    TileType swamp = spec().getTileType("model.tile.swamp");
-    TileType arctic = spec().getTileType("model.tile.arctic");
-    TileType hills = spec().getTileType("model.tile.hills");
-    TileType mountains = spec().getTileType("model.tile.mountains");
-    TileType ocean = spec().getTileType("model.tile.ocean");
-    TileType highSeas = spec().getTileType("model.tile.highSeas");
+    private static final BuildingType townHallType
+        = spec().getBuildingType("model.building.townHall");
 
-    TileType plainsForest = spec().getTileType("model.tile.mixedForest");
-    TileType desertForest = spec().getTileType("model.tile.scrubForest");
-    TileType grasslandForest = spec().getTileType("model.tile.coniferForest");
-    TileType prairieForest = spec().getTileType("model.tile.broadleafForest");
-    TileType tundraForest = spec().getTileType("model.tile.borealForest");
-    TileType savannahForest = spec().getTileType("model.tile.tropicalForest");
-    TileType marshForest = spec().getTileType("model.tile.wetlandForest");
-    TileType swampForest = spec().getTileType("model.tile.rainForest");
+    private static final GoodsType cotton
+        = spec().getGoodsType("model.goods.cotton");
+    private static final GoodsType fish
+        = spec().getGoodsType("model.goods.fish");
+    private static final GoodsType food
+        = spec().getPrimaryFoodType();
+    private static final GoodsType furs
+        = spec().getGoodsType("model.goods.furs");
+    private static final GoodsType grain
+        = spec().getGoodsType("model.goods.grain");
+    private static final GoodsType lumber
+        = spec().getGoodsType("model.goods.lumber");
+    private static final GoodsType ore
+        = spec().getGoodsType("model.goods.ore");
+    private static final GoodsType silver
+        = spec().getGoodsType("model.goods.silver");
+    private static final GoodsType sugar
+        = spec().getGoodsType("model.goods.sugar");
+    private static final GoodsType tobacco
+        = spec().getGoodsType("model.goods.tobacco");
 
-    TileImprovementType plow = spec().getTileImprovementType("model.improvement.Plow");
-    TileImprovementType river = spec().getTileImprovementType("model.improvement.River");
-    TileImprovementType road = spec().getTileImprovementType("model.improvement.Road");
-    TileImprovementType clearForest = spec().getTileImprovementType("model.improvement.ClearForest");
-    TileImprovementType fishBonusLand = spec().getTileImprovementType("model.improvement.fishBonusLand");
-    TileImprovementType fishBonusRiver = spec().getTileImprovementType("model.improvement.fishBonusRiver");
+    private static final ResourceType grainResource
+        = spec().getResourceType("model.resource.grain");
+    private static final ResourceType mineralsResource
+        = spec().getResourceType("model.resource.minerals");
+    private static final ResourceType silverResource
+        = spec().getResourceType("model.resource.silver");
+    private static final ResourceType sugarResource
+        = spec().getResourceType("model.resource.sugar");
 
-    GoodsType food = spec().getGoodsType("model.goods.food");
-    GoodsType fish = spec().getGoodsType("model.goods.fish");
-    GoodsType sugar = spec().getGoodsType("model.goods.sugar");
-    GoodsType cotton = spec().getGoodsType("model.goods.cotton");
-    GoodsType tobacco = spec().getGoodsType("model.goods.tobacco");
-    GoodsType lumber = spec().getGoodsType("model.goods.lumber");
-    GoodsType ore = spec().getGoodsType("model.goods.ore");
+    private static final TileImprovementType clearForest
+        = spec().getTileImprovementType("model.improvement.clearForest");
+    private static final TileImprovementType fishBonusLand
+        = spec().getTileImprovementType("model.improvement.fishBonusLand");
+    private static final TileImprovementType fishBonusRiver
+        = spec().getTileImprovementType("model.improvement.fishBonusRiver");
+    private static final TileImprovementType plow
+        = spec().getTileImprovementType("model.improvement.plow");
+    private static final TileImprovementType river
+        = spec().getTileImprovementType("model.improvement.river");
+    private static final TileImprovementType road
+        = spec().getTileImprovementType("model.improvement.road");
+
+    private static final TileType arctic
+        = spec().getTileType("model.tile.arctic");
+    private static final TileType desert
+        = spec().getTileType("model.tile.desert");
+    private static final TileType desertForest
+        = spec().getTileType("model.tile.scrubForest");
+    private static final TileType grassland
+        = spec().getTileType("model.tile.grassland");
+    private static final TileType grasslandForest
+        = spec().getTileType("model.tile.coniferForest");
+    private static final TileType highSeas
+        = spec().getTileType("model.tile.highSeas");
+    private static final TileType hills
+        = spec().getTileType("model.tile.hills");
+    private static final TileType marsh
+        = spec().getTileType("model.tile.marsh");
+    private static final TileType marshForest
+        = spec().getTileType("model.tile.wetlandForest");
+    private static final TileType mountains
+        = spec().getTileType("model.tile.mountains");
+    private static final TileType ocean
+        = spec().getTileType("model.tile.ocean");
+    private static final TileType plains
+        = spec().getTileType("model.tile.plains");
+    private static final TileType plainsForest
+        = spec().getTileType("model.tile.mixedForest");
+    private static final TileType prairie
+        = spec().getTileType("model.tile.prairie");
+    private static final TileType prairieForest
+        = spec().getTileType("model.tile.broadleafForest");
+    private static final TileType savannah
+        = spec().getTileType("model.tile.savannah");
+    private static final TileType savannahForest
+        = spec().getTileType("model.tile.tropicalForest");
+    private static final TileType swamp
+        = spec().getTileType("model.tile.swamp");
+    private static final TileType swampForest
+        = spec().getTileType("model.tile.rainForest");
+    private static final TileType tundra
+        = spec().getTileType("model.tile.tundra");
+    private static final TileType tundraForest
+        = spec().getTileType("model.tile.borealForest");
+
+    private static final UnitType colonistType
+        = spec().getUnitType("model.unit.freeColonist");
 
 
     public void testGetWorkAmount() {
@@ -80,7 +135,7 @@ public class TileTest extends FreeColTestCase {
         assertNotNull( marsh );
         assertNotNull( swamp );
         assertNotNull( arctic );
-        
+
         assertNotNull( plainsForest );
         assertNotNull( desertForest );
         assertNotNull( grasslandForest );
@@ -89,17 +144,17 @@ public class TileTest extends FreeColTestCase {
         assertNotNull( savannahForest );
         assertNotNull( marshForest );
         assertNotNull( swampForest );
-        
-        
+
+
         assertEquals(2, plow.getAddWorkTurns());
         assertEquals(0, road.getAddWorkTurns());
         assertEquals(2, clearForest.getAddWorkTurns());
-        
-        
+
+
         assertNotNull(plow);
         assertNotNull(road);
         assertNotNull(clearForest);
-        
+
         java.util.Map<TileType, int[]> cost = new HashMap<TileType, int[]>();
         cost.put(plains, new int[] { 5, 3 });
         cost.put(desert, new int[] { 5, 3 });
@@ -111,17 +166,17 @@ public class TileTest extends FreeColTestCase {
         cost.put(swamp, new int[] { 9, 7 });
         // TODO: fix test
         //cost.put(arctic, new int[] { 6, 4 });
-        
+
         for (java.util.Map.Entry<TileType, int[]> entry : cost.entrySet()){
             Tile tile = new Tile(game, entry.getKey(), 0, 0);
             assertTrue(tile.getType().toString(), plow.isTileAllowed(tile));
             assertTrue(tile.getType().toString(), road.isTileAllowed(tile));
             assertFalse(tile.getType().toString(), clearForest.isTileAllowed(tile));
-            
+
             assertEquals(tile.getType().toString(), entry.getValue()[0], tile.getWorkAmount(plow));
             assertEquals(tile.getType().toString(), entry.getValue()[1], tile.getWorkAmount(road));
         }
-        
+
         // Now check the forests
         cost.clear();
         cost.put(tundraForest, new int[] { 6, 4 });
@@ -132,80 +187,111 @@ public class TileTest extends FreeColTestCase {
         cost.put(marshForest, new int[] { 8, 6 });
         cost.put(swampForest, new int[] { 9, 7});
         cost.put(plainsForest, new int[] { 6, 4});
-        
+
         for (java.util.Map.Entry<TileType, int[]> entry : cost.entrySet()){
             Tile tile = new Tile(game, entry.getKey(), 0, 0);
             assertFalse(tile.getType().toString(), plow.isTileAllowed(tile));
             assertTrue(tile.getType().toString(), road.isTileAllowed(tile));
             assertTrue(tile.getType().toString(), clearForest.isTileAllowed(tile));
-            
+
             assertEquals(tile.getType().toString(), entry.getValue()[0], tile.getWorkAmount(clearForest));
             assertEquals(tile.getType().toString(), entry.getValue()[1], tile.getWorkAmount(road));
         }
-        
+
     }
-    
+
     public void testPrimarySecondaryGoods() {
-        
+
         Game game = getStandardGame();
-        
-        Tile tile = new Tile(game, spec().getTileType("model.tile.prairie"), 0, 0);
-        assertEquals(spec().getGoodsType("model.goods.food"),tile.primaryGoods());
-        assertEquals(spec().getGoodsType("model.goods.cotton"),tile.secondaryGoods());
-        
-        Tile tile2 = new Tile(game, spec().getTileType("model.tile.mixedForest"), 0, 0);
-        assertEquals(spec().getGoodsType("model.goods.food"),tile2.primaryGoods());
-        assertEquals(spec().getGoodsType("model.goods.furs"),tile2.secondaryGoods());
-        
+        game.setMap(getTestMap(true));
+        Colony colony = getStandardColony();
+        Tile tile = colony.getTile();
+        ColonyTile center = colony.getColonyTile(tile);
+
+        List<AbstractGoods> production = center.getProduction();
+        assertEquals(2, production.size());
+        AbstractGoods primaryProduction = production.get(0);
+        AbstractGoods secondaryProduction = production.get(1);
+        assertEquals(grain, primaryProduction.getType());
+        assertEquals(5, primaryProduction.getAmount());
+        assertEquals(cotton, secondaryProduction.getType());
+        assertEquals(2, secondaryProduction.getAmount());
+
+        TileImprovement ti = new TileImprovement(game, tile, spec().getTileImprovementType("model.improvement.plow"));
+        ti.setTurnsToComplete(0);
+        tile.add(ti);
+
+        production = center.getProduction();
+        assertEquals(2, production.size());
+        primaryProduction = production.get(0);
+        secondaryProduction = production.get(1);
+        assertEquals(grain, primaryProduction.getType());
+        assertEquals(6, primaryProduction.getAmount());
+        assertEquals(cotton, secondaryProduction.getType());
+        assertEquals(2, secondaryProduction.getAmount());
+
+        tile.setType(plainsForest);
+        production = center.getProduction();
+        assertEquals(2, production.size());
+        primaryProduction = production.get(0);
+        secondaryProduction = production.get(1);
+        assertEquals(grain, primaryProduction.getType());
+        assertEquals(3, primaryProduction.getAmount());
+        assertEquals(furs, secondaryProduction.getType());
+        assertEquals(3, secondaryProduction.getAmount());
+
+        ti = new TileImprovement(game, tile, spec().getTileImprovementType("model.improvement.road"));
+        ti.setTurnsToComplete(0);
+        tile.add(ti);
+
+        production = center.getProduction();
+        assertEquals(2, production.size());
+        primaryProduction = production.get(0);
+        secondaryProduction = production.get(1);
+        assertEquals(grain, primaryProduction.getType());
+        assertEquals(3, primaryProduction.getAmount());
+        assertEquals(furs, secondaryProduction.getType());
+        assertEquals(3, secondaryProduction.getAmount());
+
     }
 
     public void testPotential() {
         Game game = getStandardGame();
-        Tile tile = new Tile(game, spec().getTileType("model.tile.mountains"), 0, 0);
-        assertEquals(0,tile.potential(spec().getGoodsType("model.goods.food"), null));
-        assertEquals(1,tile.potential(spec().getGoodsType("model.goods.silver"), null));
-        tile.setResource(spec().getResourceType("model.resource.Silver"));
-        assertEquals(0,tile.potential(spec().getGoodsType("model.goods.food"), null));
-        assertEquals(3,tile.potential(spec().getGoodsType("model.goods.silver"), null));
+        Tile tile = new Tile(game, mountains, 0, 0);
+        assertEquals(0,tile.potential(food, null));
+        assertEquals(1,tile.potential(silver, null));
+        tile.addResource(new Resource(game, tile, silverResource));
+        assertEquals(0,tile.potential(food, null));
+        assertEquals(3,tile.potential(silver, null));
     }
 
     public void testMaximumPotential() {
         Game game = getStandardGame();
 
-        Tile tile1 = new Tile(game, spec().getTileType("model.tile.mountains"), 0, 0);
-        assertEquals(0, tile1.potential(spec().getGoodsType("model.goods.food"), null));
-        assertEquals(0, tile1.getMaximumPotential(spec().getGoodsType("model.goods.food"), null));
-        assertEquals(1, tile1.potential(spec().getGoodsType("model.goods.silver"), null));
-        assertEquals(2, tile1.getMaximumPotential(spec().getGoodsType("model.goods.silver"), null));
-        tile1.setResource(spec().getResourceType("model.resource.Silver"));
-        assertEquals(0, tile1.potential(spec().getGoodsType("model.goods.food"), null));
-        assertEquals(3, tile1.potential(spec().getGoodsType("model.goods.silver"), null));
-        assertEquals(4, tile1.getMaximumPotential(spec().getGoodsType("model.goods.silver"), null));
+        Tile tile1 = new Tile(game, mountains, 0, 0);
+        assertEquals(0, tile1.potential(food, null));
+        assertEquals(0, tile1.getMaximumPotential(food, null));
+        assertEquals(1, tile1.potential(silver, null));
+        assertEquals(2, tile1.getMaximumPotential(silver, null));
+        tile1.addResource(new Resource(game, tile1, silverResource));
+        assertEquals(0, tile1.potential(food, null));
+        assertEquals(3, tile1.potential(silver, null));
+        assertEquals(4, tile1.getMaximumPotential(silver, null));
 
-        Tile tile2 = new Tile(game, spec().getTileType("model.tile.plains"), 0, 1);
-        assertEquals(5, tile2.potential(spec().getGoodsType("model.goods.food"), null));
-        assertEquals(6, tile2.getMaximumPotential(spec().getGoodsType("model.goods.food"), null));
-        tile2.setResource(spec().getResourceType("model.resource.Grain"));
+        Tile tile2 = new Tile(game, plains, 0, 1);
+        assertEquals(5, tile2.potential(grain, null));
+        assertEquals(6, tile2.getMaximumPotential(grain, null));
+        tile2.addResource(new Resource(game, tile2, grainResource));
         // potential assumes expert
-        assertEquals(9, tile2.potential(spec().getGoodsType("model.goods.food"), null));
-        assertEquals(10, tile2.getMaximumPotential(spec().getGoodsType("model.goods.food"), null));
-
-        Tile tile3 = new Tile(game, spec().getTileType("model.tile.mixedForest"), 1, 1);
-        assertEquals(3, tile3.potential(spec().getGoodsType("model.goods.food"), null));
-        assertEquals(6, tile3.getMaximumPotential(spec().getGoodsType("model.goods.food"), null));
+        assertEquals(9, tile2.potential(grain, null));
+        assertEquals(10, tile2.getMaximumPotential(grain, null));
 
+        Tile tile3 = new Tile(game, plainsForest, 1, 1);
+        assertEquals(3, tile3.potential(grain, null));
+        assertEquals(6, tile3.getMaximumPotential(grain, null));
     }
 
-    public void testMovement() {
-        Game game = getStandardGame();
-        Tile tile1 = new Tile(game, spec().getTileType("model.tile.plains"), 0, 0);
-        Tile tile2 = new Tile(game, spec().getTileType("model.tile.plains"), 0, 1);
-        assertEquals(3, tile1.getMoveCost(tile2));
-    }
-
-
     public void testCanHaveImprovement() {
-
         for (TileType tileType : spec().getTileTypeList()) {
 
             if (tileType.isWater()) {
@@ -246,7 +332,6 @@ public class TileTest extends FreeColTestCase {
 
 
     public void testImprovements() throws Exception {
-
         Game game = getStandardGame();
         Map map = getTestMap(plains);
         game.setMap(map);
@@ -280,15 +365,14 @@ public class TileTest extends FreeColTestCase {
         tile2.setType(hills);
         assertTrue(tile2.hasRoad());
         assertFalse(tile2.hasRiver());
-
     }
 
     public void testProductionModifiers() throws Exception {
-    	Game game = getGame();
-    	game.setMap(getTestMap(plainsType,true));
-    	
+        Game game = getGame();
+        game.setMap(getTestMap(true));
+
         Colony colony = getStandardColony();
-        
+
         List<ColonyTile> colonyTiles = colony.getColonyTiles();
 
         ColonyTile colonyTile1 = colonyTiles.get(0);
@@ -317,11 +401,11 @@ public class TileTest extends FreeColTestCase {
         assertTrue(tile2.hasRoad());
         assertTrue(tile2.hasRiver());
 
-        tile1.setType(spec().getTileType("model.tile.savannah"));
+        tile1.setType(savannah);
         assertTrue(tile1.hasRoad());
         assertTrue(tile1.hasRiver());
 
-        tile2.setType(spec().getTileType("model.tile.hills"));
+        tile2.setType(hills);
         assertTrue(tile2.hasRoad());
         assertFalse(tile2.hasRiver());
 
@@ -330,8 +414,7 @@ public class TileTest extends FreeColTestCase {
         assertFalse(hasBonusFromSource(tile2.getProductionBonus(sugar, null), road2.getType()));
         assertTrue(hasBonusFromSource(tile2.getProductionBonus(ore, null), road2.getType()));
 
-        ResourceType sugarResource = spec().getResourceType("model.resource.Sugar");
-        tile1.setResource(sugarResource);
+        tile1.addResource(new Resource(game, tile1, sugarResource));
 
         assertTrue(hasBonusFromSource(tile1.getProductionBonus(sugar, null), savannah));
         assertTrue(hasBonusFromSource(tile1.getProductionBonus(sugar, null), river1.getType()));
@@ -354,7 +437,7 @@ public class TileTest extends FreeColTestCase {
         Game game = getStandardGame();
         Map map = getTestMap(plains);
         game.setMap(map);
-        
+
         Colony colony = FreeColTestUtils.getColonyBuilder().build();
 
         assertTrue(colony.getTile().hasRoad());
@@ -372,35 +455,35 @@ public class TileTest extends FreeColTestCase {
         Tile tile1 = map.getTile(5, 8);
 
         tile1.setType(savannah);
-        assertEquals(plow, TileImprovement.findBestTileImprovementType(tile1, food));
+        assertEquals(plow, TileImprovement.findBestTileImprovementType(tile1, grain));
         assertEquals(plow, TileImprovement.findBestTileImprovementType(tile1, sugar));
         assertEquals(null, TileImprovement.findBestTileImprovementType(tile1, tobacco));
         assertEquals(null, TileImprovement.findBestTileImprovementType(tile1, lumber));
         assertEquals(null, TileImprovement.findBestTileImprovementType(tile1, ore));
 
         tile1.setType(marsh);
-        assertEquals(plow, TileImprovement.findBestTileImprovementType(tile1, food));
+        assertEquals(plow, TileImprovement.findBestTileImprovementType(tile1, grain));
         assertEquals(null, TileImprovement.findBestTileImprovementType(tile1, sugar));
         assertEquals(plow, TileImprovement.findBestTileImprovementType(tile1, tobacco));
         assertEquals(null, TileImprovement.findBestTileImprovementType(tile1, lumber));
         assertEquals(road, TileImprovement.findBestTileImprovementType(tile1, ore));
 
         tile1.setType(savannahForest);
-        assertEquals(clearForest, TileImprovement.findBestTileImprovementType(tile1, food));
+        assertEquals(clearForest, TileImprovement.findBestTileImprovementType(tile1, grain));
         assertEquals(clearForest, TileImprovement.findBestTileImprovementType(tile1, sugar));
         assertEquals(null, TileImprovement.findBestTileImprovementType(tile1, tobacco));
         assertEquals(road, TileImprovement.findBestTileImprovementType(tile1, lumber));
         assertEquals(null, TileImprovement.findBestTileImprovementType(tile1, ore));
 
         tile1.setType(hills);
-        assertEquals(null, TileImprovement.findBestTileImprovementType(tile1, food));
+        assertEquals(null, TileImprovement.findBestTileImprovementType(tile1, grain));
         assertEquals(null, TileImprovement.findBestTileImprovementType(tile1, sugar));
         assertEquals(null, TileImprovement.findBestTileImprovementType(tile1, tobacco));
         assertEquals(null, TileImprovement.findBestTileImprovementType(tile1, lumber));
         assertEquals(road, TileImprovement.findBestTileImprovementType(tile1, ore));
 
         tile1.setType(arctic);
-        assertEquals(null, TileImprovement.findBestTileImprovementType(tile1, food));
+        assertEquals(null, TileImprovement.findBestTileImprovementType(tile1, grain));
         assertEquals(null, TileImprovement.findBestTileImprovementType(tile1, sugar));
         assertEquals(null, TileImprovement.findBestTileImprovementType(tile1, tobacco));
         assertEquals(null, TileImprovement.findBestTileImprovementType(tile1, lumber));
@@ -426,7 +509,7 @@ public class TileTest extends FreeColTestCase {
 
         List<AbstractGoods> sortedPotential = tile1.getSortedPotential();
         // savannah produces more food than sugar
-        assertEquals(food, sortedPotential.get(0).getType());
+        assertEquals(grain, sortedPotential.get(0).getType());
         assertEquals(4, sortedPotential.get(0).getAmount());
         assertEquals(sugar, sortedPotential.get(1).getType());
         assertEquals(3, sortedPotential.get(1).getAmount());
@@ -439,17 +522,15 @@ public class TileTest extends FreeColTestCase {
         assertEquals(6, sortedPotential.get(0).getAmount());
 
         sortedPotential = tile1.getSortedPotential(farmer, dutch);
-        assertEquals(food, sortedPotential.get(0).getType());
+        assertEquals(grain, sortedPotential.get(0).getType());
         assertEquals(7, sortedPotential.get(0).getAmount());
-        assertTrue(market.getSalePrice(food, 7) > market.getSalePrice(sugar, 3));
+        assertTrue(market.getSalePrice(grain, 7) > market.getSalePrice(sugar, 3));
 
         tile1.setType(plains);
-        // make sure 2 cotton is more expensive than 5 food
-        market.getMarketData(cotton).setPaidForSale(3);
 
         // plains produces more food than sugar
-        assertEquals(food, tile1.getSortedPotential().get(0).getType());
-        assertEquals(food, tile1.getSortedPotential(farmer, dutch).get(0).getType());
+        assertEquals(grain, tile1.getSortedPotential().get(0).getType());
+        assertEquals(grain, tile1.getSortedPotential(farmer, dutch).get(0).getType());
         assertEquals(cotton, tile1.getSortedPotential(cottonPlanter, dutch).get(0).getType());
 
         tile1.setType(ocean);
@@ -460,7 +541,75 @@ public class TileTest extends FreeColTestCase {
         sortedPotential = tile1.getSortedPotential(farmer, null);
         assertEquals(1, sortedPotential.size());
         assertEquals(fish, sortedPotential.get(0).getType());
-        
+
     }
     */
+
+    public void testArctic() {
+        arctic.applyDifficultyLevel(spec().getDifficultyLevel("model.difficulty.veryEasy"));
+        assertTrue(arctic.canSettle());
+        assertEquals(2, arctic.getPrimaryGoods().getAmount());
+        assertNull(arctic.getSecondaryGoods());
+
+        arctic.applyDifficultyLevel(spec().getDifficultyLevel("model.difficulty.easy"));
+        assertTrue(arctic.canSettle());
+        assertEquals(1, arctic.getPrimaryGoods().getAmount());
+        assertNull(arctic.getSecondaryGoods());
+
+        arctic.applyDifficultyLevel(spec().getDifficultyLevel("model.difficulty.medium"));
+        assertTrue(arctic.canSettle());
+        assertNull(arctic.getPrimaryGoods());
+        assertNull(arctic.getSecondaryGoods());
+
+        arctic.applyDifficultyLevel(spec().getDifficultyLevel("model.difficulty.hard"));
+        assertTrue(arctic.canSettle());
+        assertNull(arctic.getPrimaryGoods());
+        assertNull(arctic.getSecondaryGoods());
+
+        arctic.applyDifficultyLevel(spec().getDifficultyLevel("model.difficulty.veryHard"));
+        assertTrue(arctic.canSettle());
+        assertNull(arctic.getPrimaryGoods());
+        assertNull(arctic.getSecondaryGoods());
+    }
+
+    public void testMinerals() {
+        Game game = getGame();
+        Map map = getTestMap(tundra);
+        game.setMap(map);
+
+        Colony colony = getStandardColony();
+        Tile tile = colony.getTile().getNeighbourOrNull(Map.Direction.N);
+        ColonyTile colonyTile = colony.getColonyTile(tile);
+        tile.addResource(new Resource(game, tile, mineralsResource));
+        if (colonyTile.getUnit() != null) {
+            colonyTile.getUnit().setLocation(colony.getBuilding(townHallType));
+        }
+        assertNull(colonyTile.getUnit());
+
+        Unit unit = colony.getUnitList().get(0);
+        assertEquals(colonistType, unit.getType());
+        assertTrue(silver.isFarmed());
+        assertEquals(0, tundra.getProductionOf(silver, colonistType));
+        assertEquals(1, tile.potential(silver, colonistType));
+
+        Set<Modifier> modifiers = tile.getProductionBonus(silver, unit.getType());
+        assertFalse(modifiers.isEmpty());
+
+        assertEquals(1, colonyTile.getProductionOf(unit, silver));
+        assertEquals(colonyTile.getWorkTile().getOwningSettlement(), colony);
+        assertTrue(colonyTile.canBeWorked());
+        assertTrue(colonyTile.canAdd(unit));
+        assertEquals(colonyTile, colony.getVacantColonyTileFor(unit, false, silver));
+    }
+
+    public void testZIndex() {
+        assertTrue(GUI.OVERLAY_INDEX < GUI.FOREST_INDEX);
+        assertTrue(GUI.FOREST_INDEX < TileItem.RESOURCE_ZINDEX);
+        assertTrue(TileItem.RESOURCE_ZINDEX < TileItem.RUMOUR_ZINDEX);
+        assertTrue(plow.getZIndex() < river.getZIndex());
+        assertTrue(river.getZIndex() < road.getZIndex());
+        assertTrue(GUI.FOREST_INDEX < road.getZIndex());
+        assertTrue(road.getZIndex() < TileItem.RESOURCE_ZINDEX);
+    }
+
 }
