@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2008  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -26,7 +26,11 @@ public class AllTests {
 	public static Test suite() {
 		TestSuite suite = new TestSuite("Test for net.sf.freecol.server.model");
 		//$JUnit-BEGIN$
+		suite.addTestSuite(ServerBuildingTest.class);
+    suite.addTestSuite(ServerColonyTest.class);
+		suite.addTestSuite(ServerIndianSettlementTest.class);
 		suite.addTestSuite(ServerPlayerTest.class);
+		suite.addTestSuite(ServerUnitTest.class);
 		//$JUnit-END$
 		return suite;
 	}
