@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2011  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -20,44 +20,49 @@
 package net.sf.freecol.util.test;
 
 import java.lang.reflect.Field;
-
+import java.util.ArrayList;
+import java.util.HashMap;
 import java.util.HashSet;
-import java.util.Set;
+import java.util.List;
 import java.util.Locale;
+import java.util.Set;
 
 import junit.framework.TestCase;
-import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.gui.i18n.Messages;
-import net.sf.freecol.common.Specification;
+import net.sf.freecol.common.io.FreeColTcFile;
 import net.sf.freecol.common.model.Colony;
+import net.sf.freecol.common.model.CombatModel;
+import net.sf.freecol.common.model.CombatModel.CombatOdds;
+import net.sf.freecol.common.model.CombatModel.CombatResult;
+import net.sf.freecol.common.model.FreeColGameObject;
 import net.sf.freecol.common.model.Game;
-import net.sf.freecol.common.model.GameOptions;
 import net.sf.freecol.common.model.IndianSettlement;
 import net.sf.freecol.common.model.Map;
 import net.sf.freecol.common.model.Nation;
 import net.sf.freecol.common.model.NationOptions;
+import net.sf.freecol.common.model.NationOptions.Advantages;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Region;
+import net.sf.freecol.common.model.Specification;
 import net.sf.freecol.common.model.Tile;
-import net.sf.freecol.common.model.TileImprovement;
-import net.sf.freecol.common.model.TileImprovementType;
 import net.sf.freecol.common.model.TileType;
 import net.sf.freecol.common.model.Unit;
-import net.sf.freecol.common.model.UnitType;
 import net.sf.freecol.common.model.Unit.UnitState;
+import net.sf.freecol.common.model.UnitType;
 import net.sf.freecol.server.model.ServerGame;
+import net.sf.freecol.server.model.ServerIndianSettlement;
+import net.sf.freecol.server.model.ServerPlayer;
+import net.sf.freecol.server.model.ServerUnit;
+
 
 /**
  * The base class for all FreeCol tests. Contains useful methods used by the
  * individual tests.
- * 
- * @author $Author: burschik $
- * @version $Revision: 6360 $ ($Date: 2010-01-24 22:46:03 +1030 (Sun, 24 Jan 2010) $)
- * 
  */
 public class FreeColTestCase extends TestCase {
 
-    public static final TileType plainsType = FreeCol.getSpecification().getTileType("model.tile.plains");
+    private static java.util.Map<String, Specification> specifications =
+        new HashMap<String, Specification>();
 
     /**
      * use getGame to access this.
@@ -65,7 +70,7 @@ public class FreeColTestCase extends TestCase {
     static Game game;
 
     static boolean updateLocale = true;
-    
+
     @Override
     protected void setUp() throws Exception {
         if (updateLocale) {
@@ -76,18 +81,18 @@ public class FreeColTestCase extends TestCase {
 
     @Override
     protected void tearDown() throws Exception {
-    	// If a game has been created destroy it.
-    	game = null;
+        // If a game has been created destroy it.
+        game = null;
     }
 
     /**
      * Get a game pseudo-singleton, i.e. the same instance will be returned
      * until getStandardGame() is called, which resets the singleton to a new
      * value.
-     * 
+     *
      * Calling this method repetitively without calling getStandardGame() will
      * result in the same Game being returned.
-     * 
+     *
      * @return The game singleton.
      */
     public static Game getGame() {
@@ -96,53 +101,84 @@ public class FreeColTestCase extends TestCase {
         }
         return game;
     }
-    
+
     /**
-     * Specifically sets the game instance to run with
-     * Necessary for server tests that create their own game instances
-     * Allows for same interface for accessing the game instance for all types of tests
+     * Specifically sets the game instance to run with.  Necessary for
+     * server tests that create their own game instances.  Allows for
+     * same interface for accessing the game instance for all types of
+     * tests.
+     *
      * @param newGame Game instance to work with
      */
     public static void setGame(Game newGame) {
         game = newGame;
     }
-    
-    public static Specification spec(){
-    	return FreeCol.getSpecification();
+
+    public static Specification spec() {
+        return getSpecification("freecol");
+    }
+
+    public static Specification getSpecification(String name) {
+        Specification result = specifications.get(name);
+        if (result == null) {
+            try {
+                FreeColTcFile tc = new FreeColTcFile(name);
+                result = tc.getSpecification();
+                result.applyDifficultyLevel("model.difficulty.medium");
+                specifications.put(name, result);
+            } catch(Exception e) {
+                e.printStackTrace();
+                return null;
+            }
+        }
+        return result;
+    }
+
+
+    /**
+     * Returns a new game, with all players set.
+     *
+     * As a side effect this call will reset the singleton game value that can
+     * be accessed using getGame().
+     *
+     * @return A new game with with players for each nation added.
+     */
+    public static Game getStandardGame() {
+        return getStandardGame("freecol");
     }
 
     /**
      * Returns a new game, with all players set.
-     * 
+     *
      * As a side effect this call will reset the singleton game value that can
      * be accessed using getGame().
-     * 
+     *
+     * @param specName a <code>String</code> value
      * @return A new game with with players for each nation added.
      */
-    public static Game getStandardGame() {
-        game = new ServerGame(new MockModelController());
-        game.setNationOptions(NationOptions.getDefaults());
+    public static Game getStandardGame(String specName) {
+        Specification specification = getSpecification(specName);
+        game = new ServerGame(specification);
+        game.setNationOptions(new NationOptions(specification, Advantages.SELECTABLE));
 
-        Specification.getSpecification().applyDifficultyLevel(game.getGameOptions().getInteger(GameOptions.DIFFICULTY));
-        for (Nation n : FreeCol.getSpecification().getNations()) {
-            Player p;
-            if (n.getType().isEuropean() && !n.getType().isREF()){
-                p = new Player(game, n.getRulerName(), false, n);
-            } else {
-                p = new Player(game, n.getRulerName(), false, true, n);
-            }
+        specification.applyDifficultyLevel("model.difficulty.medium");
+        for (Nation n : specification.getNations()) {
+            Player p = new ServerPlayer(game, n.getRulerNameKey(), false, n,
+                                        null, null);
+            p.setAI(!n.getType().isEuropean() || n.getType().isREF());
             game.addPlayer(p);
         }
         return game;
     }
-    
+
     /**
      * Creates a standardized map on which all fields have the plains type.
-     * 
+     *
      * Uses the getGame() method to access the currently running game.
-     * 
-     * Does not call Game.setMap(Map) with the returned map. The map is unexplored.
-     * 
+     *
+     * Does not call Game.setMap(Map) with the returned map. The map
+     * is unexplored.
+     *
      * @return The map created as described above.
      */
     public static Map getTestMap() {
@@ -152,13 +188,14 @@ public class FreeColTestCase extends TestCase {
 
     /**
      * Creates a standardized map on which all fields have the same given type.
-     * 
+     *
      * Uses the getGame() method to access the currently running game.
-     * 
-     * Does not call Game.setMap(Map) with the returned map. The map is unexplored.
-     * 
+     *
+     * Does not call Game.setMap(Map) with the returned map. The map
+     * is unexplored.
+     *
      * @param type The type of land with which to initialize the map.
-     * 
+     *
      * @return The map created as described above.
      */
     public static Map getTestMap(TileType tileType) {
@@ -166,18 +203,17 @@ public class FreeColTestCase extends TestCase {
         builder.setBaseTileType(tileType);
         return builder.build();
     }
-    
+
     /**
      * Creates a standardized map on which all fields have the same given type.
-     * 
+     *
      * Uses the getGame() method to access the currently running game.
-     * 
+     *
      * Does not call Game.setMap(Map) with the returned map.
-     * 
+     *
      * @param type The type of land with which to initialize the map.
-     * 
-     * @param explored Set to true if you want all the tiles on the map to have been explored by all players.
-     * 
+     * @param explored Set to true if you want all the tiles on the
+     *     map to have been explored by all players.
      * @return The map created as described above.
      */
     public static Map getTestMap(TileType tileType, boolean explored) {
@@ -185,31 +221,37 @@ public class FreeColTestCase extends TestCase {
         builder.setBaseTileType(tileType).setExploredByAll(explored);
         return builder.build();
     }
-    
+
+    public static Map getTestMap(boolean explored) {
+        MapBuilder builder = new MapBuilder(getGame());
+        builder.setExploredByAll(explored);
+        return builder.build();
+    }
+
     public static Map getCoastTestMap(TileType landTileType) {
         return getCoastTestMap(landTileType, false);
     }
 
     /**
      * Creates a standardized map, half land (left), half sea (right)
-     * 
+     *
      * The land half has the same given type.
-     * 
+     *
      * Uses the getGame() method to access the currently running game.
-     * 
+     *
      * Does not call Game.setMap(Map) with the returned map.
-     * 
+     *
      * @param type The type of land with which to initialize the map.
-     * 
+     *
      * @param explored Set to true if you want all the tiles on the map to have been explored by all players.
-     * 
+     *
      * @return The map created as described above.
      */
     public static Map getCoastTestMap(TileType landTileType, boolean explored) {
         int totalWidth = 20;
         int totalHeight = 15;
         TileType oceanType = spec().getTileType("model.tile.ocean");
-        
+
         MapBuilder builder = new MapBuilder(getGame());
         builder.setDimensions(totalWidth, totalHeight).setBaseTileType(oceanType);
         if (explored) {
@@ -226,10 +268,10 @@ public class FreeColTestCase extends TestCase {
 
         return builder.build();
     }
-    
+
     /**
      * Get a standard colony at the location 5,8 with one free colonist
-     * 
+     *
      * @return
      */
     public Colony getStandardColony() {
@@ -238,10 +280,10 @@ public class FreeColTestCase extends TestCase {
 
     /**
      * Get a colony with the given number of settlers
-     * 
+     *
      * @param numberOfSettlers The number of settlers to put into the colony.
      *            Must be >= 1.
-     * 
+     *
      * @return
      */
     public Colony getStandardColony(int numberOfSettlers) {
@@ -250,12 +292,12 @@ public class FreeColTestCase extends TestCase {
 
     /**
      * Get a colony with the given number of settlers
-     * 
+     *
      * @param numberOfSettlers The number of settlers to put into the colony.
      *            Must be >= 1.
      * @param tileX Coordinate of tile for the colony.
      * @param tileY Coordinate of tile for the colony.
-     * 
+     *
      * @return
      */
     public Colony getStandardColony(int numberOfSettlers, int tileX, int tileY) {
@@ -267,31 +309,31 @@ public class FreeColTestCase extends TestCase {
 
         FreeColTestUtils.ColonyBuilder builder = FreeColTestUtils.getColonyBuilder();
         builder.colonyTile(tile).initialColonists(numberOfSettlers);
-        
+
         return builder.build();
     }
-    
+
     public static class MapBuilder{
-        
+
         // Required parameter
         private final Game game;
-        
-        private TileType[][] tiles = null; 
+
+        private TileType[][] tiles = null;
         private int width;
         private int height;
         private TileType baseTile;
         private boolean exploredByAll;
         private boolean initiated;
-        
+
         public MapBuilder(Game game){
             this.game = game;
             setStartingParams();
         }
-        
+
         private void setStartingParams(){
             width = 20;
             height = 15;
-            baseTile = plainsType;
+            baseTile = spec().getTileType("model.tile.plains");
             exploredByAll = false;
             initiated = false;
             // set empty grid
@@ -304,7 +346,7 @@ public class FreeColTestCase extends TestCase {
                 }
             }
         }
-        
+
         public MapBuilder setBaseTileType(TileType baseType){
             if(baseType == null){
                 throw new NullPointerException("Base tile type cannot be null");
@@ -312,7 +354,7 @@ public class FreeColTestCase extends TestCase {
             this.baseTile = baseType;
             return this;
         }
-        
+
         public MapBuilder setDimensions(int width, int heigth){
             if(width <= 0){
                 throw new IllegalArgumentException("Width must be positive");
@@ -327,12 +369,12 @@ public class FreeColTestCase extends TestCase {
             this.height = heigth;
             return this;
         }
-        
+
         public MapBuilder setExploredByAll(boolean exploredByAll){
             this.exploredByAll = exploredByAll;
             return this;
         }
-        
+
         public MapBuilder setTile(int x, int y, TileType tileType){
             if(x < 0 || y < 0){
                 throw new IllegalArgumentException("Coordenates cannot be negative");
@@ -343,16 +385,16 @@ public class FreeColTestCase extends TestCase {
             if(tileType == null){
                 throw new NullPointerException("Tile type cannot be null");
             }
-            
+
             tiles[x][y]= tileType;
             initiated = true;
-            
+
             return this;
         }
-        
+
         // Implementation method, completes grid by setting uninitialized tiles
         //to the base tile type
-        private void completeWorkingGrid(){      
+        private void completeWorkingGrid(){
             for (int x = 0; x < width; x++) {
                 for (int y = 0; y < height; y++) {
                     // Already manually set by the tester
@@ -364,7 +406,7 @@ public class FreeColTestCase extends TestCase {
             }
             initiated=true;
         }
-        
+
         public Map build(){
             completeWorkingGrid();
             Tile[][] mapTiles = new Tile[width][height];
@@ -377,7 +419,7 @@ public class FreeColTestCase extends TestCase {
                     mapTiles[x][y].setRegion(region);
                 }
             }
-            
+
             Map m = new Map(game, mapTiles);
             if (exploredByAll) {
                 for (Player player : game.getPlayers()) {
@@ -388,147 +430,159 @@ public class FreeColTestCase extends TestCase {
             }
             return m;
         }
-        
+
         public MapBuilder reset() {
             setStartingParams();
-            
+
             return this;
         }
     }
 
     public static class IndianSettlementBuilder{
-    	
-    	// Required parameter
-    	private final Game game;
-    	
-    	private Player indianPlayer;
-    	private final String defaultIndianPlayer = "model.nation.tupi";
-    	private String skillTaught;
-    	private int initialBravesInCamp;
-    	private Tile settlementTile;
-    	
-    	private boolean isCapital;
+
+        // Required parameter
+        private final Game game;
+
+        private Player indianPlayer;
+        private final String defaultIndianPlayer = "model.nation.tupi";
+        private String skillTaught;
+        private int initialBravesInCamp;
+        private Tile settlementTile;
+        private static int settlementNumber = 1;
+
+        private boolean isCapital;
         private Set<Player> isVisited;
         private Unit residentMissionary;
-    	
-    	public IndianSettlementBuilder(Game game){
-    		this.game = game;
-    		setStartingParams();
-    	}
-    	
-    	private void setStartingParams(){
-    		// Some params can only be set in build(), because the default values 
-    		//may not be valid for the game set
-    		// However, the tester himself may set them to valid values later, 
-    		//so they are set to null for now
-    		indianPlayer = null;
-        	initialBravesInCamp = 1;
-        	settlementTile = null;
-        	skillTaught = "model.unit.masterCottonPlanter";
-        	isCapital = false;
+
+        public IndianSettlementBuilder(Game game){
+            this.game = game;
+            setStartingParams();
+        }
+
+        private void setStartingParams(){
+            // Some params can only be set in build(), because the default values
+            //may not be valid for the game set
+            // However, the tester himself may set them to valid values later,
+            //so they are set to null for now
+            indianPlayer = null;
+            initialBravesInCamp = 1;
+            settlementTile = null;
+            skillTaught = "model.unit.masterCottonPlanter";
+            isCapital = false;
             isVisited = new HashSet<Player>();
             residentMissionary = null;
-    	}
-    	
-    	public IndianSettlementBuilder player(Player player){
-    		this.indianPlayer = player;
-    		
-			if(indianPlayer == null || !game.getPlayers().contains(player)){
-				throw new IllegalArgumentException("Indian player not in game");
-			}
-    		
-    		return this;
-    	}
-    	
-    	public IndianSettlementBuilder initialBravesInCamp(int nBraves){
-    		if(nBraves <= 0){
-    			throw new IllegalArgumentException("Number of braves must be positive");
-    		}
-    		this.initialBravesInCamp = nBraves;
-    		return this;
-    	}
-    	
-    	public IndianSettlementBuilder settlementTile(Tile tile){
-    		Tile tileOnMap = this.game.getMap().getTile(tile.getPosition());
-    		if(tile != tileOnMap){
-    			throw new IllegalArgumentException("Given tile not on map");
-    		}
-    		this.settlementTile = tile;
-    		return this;
-    	}
-    	
-    	public IndianSettlementBuilder capital(boolean isCapital){
-    		this.isCapital = isCapital;
-    		
-    		return this;
-    	}
+        }
 
-    	public IndianSettlementBuilder isVisitedByPlayer(Player player, boolean isVisited){
-    		if (player != null) {
-                    if (isVisited) {
-                        this.isVisited.add(player);
-                    } else {
-                        this.isVisited.remove(player);
-                    }
-    		}
-    		
-    		return this;
-    	}
-    	
-    	public IndianSettlementBuilder missionary(Unit missionary){
-    		this.residentMissionary = missionary;
-    		
-    		return this;
-    	}
-    	
-    	public IndianSettlementBuilder skillToTeach(String skill){
-    		this.skillTaught = skill;
-    		
-    		return this;
-    	}
-    	
-    	public IndianSettlement build(){
-    		UnitType skillToTeach = null;
-    		
-    		if(skillTaught != null){
-    			skillToTeach = FreeCol.getSpecification().getUnitType(skillTaught);
-    		}
-    			
-    		UnitType indianBraveType = FreeCol.getSpecification().getUnitType("model.unit.brave");
-    		
-    		// indianPlayer not set, get default
-    		if(indianPlayer == null){
-    			indianPlayer = game.getPlayer(defaultIndianPlayer);
-    			if(indianPlayer == null){
-    				throw new IllegalArgumentException("Default Indian player " + defaultIndianPlayer + " not in game");
-    			}
-    		}
-    		
-    		// settlement tile no set, get default
-    		if(settlementTile == null){
-    			settlementTile = game.getMap().getTile(5, 8);
-    			if(settlementTile == null){
-    				throw new IllegalArgumentException("Default tile not in game");
-    			}
-    		}
-    		
-    		IndianSettlement camp = new IndianSettlement(game, indianPlayer, settlementTile, indianPlayer.getDefaultSettlementName(isCapital), isCapital, skillToTeach, isVisited, residentMissionary);
-            
-    		// Add braves
+        public IndianSettlementBuilder player(Player player){
+            this.indianPlayer = player;
+
+            if(indianPlayer == null || !game.getPlayers().contains(player)){
+                throw new IllegalArgumentException("Indian player not in game");
+            }
+
+            return this;
+        }
+
+        public IndianSettlementBuilder initialBravesInCamp(int nBraves){
+            if(nBraves <= 0){
+                throw new IllegalArgumentException("Number of braves must be positive");
+            }
+            this.initialBravesInCamp = nBraves;
+            return this;
+        }
+
+        public IndianSettlementBuilder settlementTile(Tile tile){
+            Tile tileOnMap = this.game.getMap().getTile(tile.getPosition());
+            if(tile != tileOnMap){
+                throw new IllegalArgumentException("Given tile not on map");
+            }
+            this.settlementTile = tile;
+            return this;
+        }
+
+        public IndianSettlementBuilder capital(boolean isCapital){
+            this.isCapital = isCapital;
+
+            return this;
+        }
+
+        public IndianSettlementBuilder isVisitedByPlayer(Player player, boolean isVisited){
+            if (player != null) {
+                if (isVisited) {
+                    this.isVisited.add(player);
+                } else {
+                    this.isVisited.remove(player);
+                }
+            }
+
+            return this;
+        }
+
+        public IndianSettlementBuilder missionary(Unit missionary){
+            this.residentMissionary = missionary;
+
+            return this;
+        }
+
+        public IndianSettlementBuilder skillToTeach(String skill){
+            this.skillTaught = skill;
+
+            return this;
+        }
+
+        private String getSimpleName(Player player, boolean isCapital) {
+            return (isCapital) ? player.getName() + "-capital"
+                : "Settlement-" + settlementNumber++;
+        }
+
+        public IndianSettlement build(){
+            UnitType skillToTeach = null;
+
+            if(skillTaught != null){
+                skillToTeach = spec().getUnitType(skillTaught);
+            }
+
+            UnitType indianBraveType = spec().getUnitType("model.unit.brave");
+
+            // indianPlayer not set, get default
+            if(indianPlayer == null){
+                indianPlayer = game.getPlayer(defaultIndianPlayer);
+                if(indianPlayer == null){
+                    throw new IllegalArgumentException("Default Indian player " + defaultIndianPlayer + " not in game");
+                }
+            }
+
+            // settlement tile no set, get default
+            if(settlementTile == null){
+                settlementTile = game.getMap().getTile(5, 8);
+                if(settlementTile == null){
+                    throw new IllegalArgumentException("Default tile not in game");
+                }
+            }
+
+            IndianSettlement camp
+                = new ServerIndianSettlement(game, indianPlayer,
+                                             getSimpleName(indianPlayer, isCapital),
+                                             settlementTile, isCapital,
+                                             skillToTeach, isVisited,
+                                             residentMissionary);
+
+            // Add braves
             for(int i=0; i < initialBravesInCamp; i++){
-            	Unit brave = new Unit(game, camp, indianPlayer, indianBraveType, UnitState.ACTIVE,
-                    indianBraveType.getDefaultEquipment());
-            	camp.addOwnedUnit(brave);
+                Unit brave = new ServerUnit(game, camp, indianPlayer,
+                                            indianBraveType, UnitState.ACTIVE,
+                                            indianBraveType.getDefaultEquipment());
+                camp.addOwnedUnit(brave);
             }
             camp.placeSettlement();
             return camp;
-    	}
-    	
-    	public IndianSettlementBuilder reset() {
-    		setStartingParams();
-    		
-    		return this;
-    	}
+        }
+
+        public IndianSettlementBuilder reset() {
+            setStartingParams();
+
+            return this;
+        }
     }
 
 
@@ -549,10 +603,50 @@ public class FreeColTestCase extends TestCase {
         }
     }
 
-    public void addImprovement(Tile tile, String id) {
-        TileImprovementType type = spec().getTileImprovementType(id);
-        TileImprovement improvement = new TileImprovement(game, tile, type);
-        improvement.setTurnsToComplete(0);
-        tile.add(improvement);
+    /**
+     * Build/place a colony with a unit, without requiring the server.
+     */
+    public void nonServerBuildColony(Unit builder, Colony colony) {
+        colony.placeSettlement();
+        nonServerJoinColony(builder, colony);
     }
+
+    /**
+     * Join a colony with a unit, without requiring the server.
+     */
+    public void nonServerJoinColony(Unit builder, Colony colony) {
+        builder.setState(UnitState.IN_COLONY);
+        builder.setLocation(colony);
+        builder.setMovesLeft(0);
+    }
+
+    /**
+     * Repeatedly ask the CombatModel for an attack result until it
+     * gives the primary one we want (WIN, LOSE, NO_RESULT).
+     */
+    public List<CombatResult> fakeAttackResult(CombatResult result,
+                                               FreeColGameObject attacker,
+                                               FreeColGameObject defender)
+    {
+        List<CombatResult> crs;
+        final float delta = 0.02f;
+        CombatModel combatModel = getGame().getCombatModel();
+        CombatOdds combatOdds = combatModel.calculateCombatOdds(attacker, defender);
+        float f = combatOdds.win;
+        MockPseudoRandom mr = new MockPseudoRandom();
+        List<Integer> number = new ArrayList<Integer>();
+        number.add(-1);
+        do {
+            f += (result == CombatResult.WIN) ? -delta : delta;
+            if (f < 0.0f || f >= 1.0f) {
+                throw new IllegalStateException("f out of range: "
+                                                + Float.toString(f));
+            }
+            number.set(0, new Integer((int)(Integer.MAX_VALUE * f)));
+            mr.setNextNumbers(number, true);
+            crs = combatModel.generateAttackResult(mr, attacker, defender);
+        } while (crs.get(0) != result);
+        return crs;
+    }
+
 }
